!----------------------------------------------------------------------
!
!  File:   pp_defs.f90
!  Author: Ian Ross, University of Bristol
!  Date:   17 Feb 2005
!
!----------------------------------------------------------------------
!
!  Basic type and constant definitions used in MOTIF/PMIP2 file post-
!  processor.
!
!----------------------------------------------------------------------

MODULE pp_defs

  IMPLICIT NONE


  ! String length definitions.

  INTEGER, PARAMETER :: VAR_NAME_LEN = 20
  INTEGER, PARAMETER :: AXIS_NAME_LEN = 15
  INTEGER, PARAMETER :: UNIT_NAME_LEN = 32
  INTEGER, PARAMETER :: ARG_LEN = 512
  INTEGER, PARAMETER :: ARG_KEY_LEN = 32


  ! Maximum number of entries in the configuration file.

  INTEGER, PARAMETER :: MAX_CFG_ARGS = 32


  ! Error codes.

  INTEGER, PARAMETER :: ERR_BAD_COMMAND_LINE     = 1
  INTEGER, PARAMETER :: ERR_INVALID_TIME_RANGE   = 2
  INTEGER, PARAMETER :: ERR_EMPTY_TIME_RANGE     = 3
  INTEGER, PARAMETER :: ERR_BAD_CFG_FILE         = 4
  INTEGER, PARAMETER :: ERR_CFG_MISSING          = 5
  INTEGER, PARAMETER :: ERR_INVALID_CFG          = 6
  INTEGER, PARAMETER :: ERR_DUPLICATE_CFG        = 7
  INTEGER, PARAMETER :: ERR_TOO_MANY_CFG_ENTRIES = 8
  INTEGER, PARAMETER :: ERR_CMOR_SETUP           = 9
  INTEGER, PARAMETER :: ERR_CMOR_DATASET         = 10
  INTEGER, PARAMETER :: ERR_CMOR_AXIS            = 11
  INTEGER, PARAMETER :: ERR_CMOR_VAR             = 12
  INTEGER, PARAMETER :: ERR_CMOR_WRITE           = 13
  INTEGER, PARAMETER :: ERR_CMOR_CLOSE           = 14
  INTEGER, PARAMETER :: ERR_NON_2D_FIXED         = 15
  INTEGER, PARAMETER :: ERR_UNKNOWN_AXIS         = 16
  INTEGER, PARAMETER :: ERR_UNKNOWN_VAR          = 17
  INTEGER, PARAMETER :: ERR_SINGLETON_AXIS       = 18
  INTEGER, PARAMETER :: ERR_MODEL_FILE_OPEN      = 19
  INTEGER, PARAMETER :: ERR_MODEL_FILE_READ      = 20
  INTEGER, PARAMETER :: ERR_MODEL_FILE_CLOSE     = 21
  INTEGER, PARAMETER :: ERR_MODEL_INVALID_VAR    = 22
  INTEGER, PARAMETER :: ERR_UNSUPPORTED          = 23
  INTEGER, PARAMETER :: ERR_INTERNAL             = 24
  INTEGER, PARAMETER :: ERR_OUT_OF_MEMORY        = 25


  ! Model domains that variables can be part of.

  CHARACTER, PARAMETER :: DOMAIN_ATMOS   = 'A'
  CHARACTER, PARAMETER :: DOMAIN_OCEAN   = 'O'
  CHARACTER, PARAMETER :: DOMAIN_SURFACE = 'S'
  CHARACTER, PARAMETER :: DOMAIN_ICE     = 'I'


  ! Output frequencies.

  CHARACTER(LEN=2), PARAMETER :: FREQ_FIXED =        'FX'
  CHARACTER(LEN=2), PARAMETER :: FREQ_ANNUAL =       'AN'
  CHARACTER(LEN=2), PARAMETER :: FREQ_SEASONAL =     'SE'
  CHARACTER(LEN=2), PARAMETER :: FREQ_MONTHLY =      'MO'
  CHARACTER(LEN=2), PARAMETER :: FREQ_DAILY =        'DA'
  CHARACTER(LEN=2), PARAMETER :: FREQ_EXTRA_DAILY =  'X1'
  CHARACTER(LEN=2), PARAMETER :: FREQ_EXTRA_12HRLY = 'X2'
  CHARACTER(LEN=2), PARAMETER :: FREQ_EXTRA_6HRLY =  'X4'


  ! Axis names.

  CHARACTER(LEN=AXIS_NAME_LEN), PARAMETER :: AXIS_LAT    = 'latitude'
  CHARACTER(LEN=AXIS_NAME_LEN), PARAMETER :: AXIS_LON    = 'longitude'
  CHARACTER(LEN=AXIS_NAME_LEN), PARAMETER :: AXIS_P17    = 'pressure'
  CHARACTER(LEN=AXIS_NAME_LEN), PARAMETER :: AXIS_P3     = 'pressure_p3'
  CHARACTER(LEN=AXIS_NAME_LEN), PARAMETER :: AXIS_Pbt    = 'pressure_pbt'
  CHARACTER(LEN=AXIS_NAME_LEN), PARAMETER :: AXIS_Pbm    = 'pressure_pbm'
  CHARACTER(LEN=AXIS_NAME_LEN), PARAMETER :: AXIS_Pb     = 'pressure_pb'
  CHARACTER(LEN=AXIS_NAME_LEN), PARAMETER :: AXIS_Pm     = 'pressure_pm'
  CHARACTER(LEN=AXIS_NAME_LEN), PARAMETER :: AXIS_DEPTH  = 'depth'
  CHARACTER(LEN=AXIS_NAME_LEN), PARAMETER :: AXIS_PFT    = 'pft'
  CHARACTER(LEN=AXIS_NAME_LEN), PARAMETER :: AXIS_REGION = 'region'
  CHARACTER(LEN=AXIS_NAME_LEN), PARAMETER :: AXIS_HEMI   = 'hemisphere'
  CHARACTER(LEN=AXIS_NAME_LEN), PARAMETER :: AXIS_HGT1   = 'height1'
  CHARACTER(LEN=AXIS_NAME_LEN), PARAMETER :: AXIS_HGT2   = 'height2'
  CHARACTER(LEN=AXIS_NAME_LEN), PARAMETER :: AXIS_TIME   = 'time'


  ! Hemisphere indices and names.

  INTEGER, PARAMETER :: NORTHERN_HEMI = 1
  INTEGER, PARAMETER :: SOUTHERN_HEMI = 2
  CHARACTER(LEN=ARG_LEN), DIMENSION(2) :: hemi_names = &
       (/ 'northern_hemisphere', 'southern_hemisphere' /)


  ! Region indices and names.

  INTEGER, PARAMETER :: REGION_ATLANTIC = 1
  INTEGER, PARAMETER :: REGION_INDIAN   = 2
  INTEGER, PARAMETER :: REGION_PACIFIC  = 3
  INTEGER, PARAMETER :: REGION_GLOBAL   = 4
  CHARACTER(LEN=ARG_LEN), DIMENSION(4) :: region_names = &
       (/ 'atlantic_ocean', 'indian_ocean', 'pacific_ocean', 'global_ocean' /)


  ! Type used for defining characteristics of an axis.

  TYPE axis_def
     INTEGER                           :: cmor_id
     CHARACTER                         :: domain
     CHARACTER(LEN=AXIS_NAME_LEN)      :: name
     CHARACTER(LEN=UNIT_NAME_LEN)      :: units
     REAL, DIMENSION(:), ALLOCATABLE   :: values
     REAL, DIMENSION(:,:), ALLOCATABLE :: bounds
  END TYPE axis_def


  ! Type used for defining characteristics of a variable.

  TYPE variable_def
     CHARACTER                   :: domain
     CHARACTER(LEN=2)            :: freq
     CHARACTER(LEN=VAR_NAME_LEN) :: name
  END TYPE variable_def


  ! Configuration file entry.

  TYPE cfg_arg
     CHARACTER(LEN=ARG_KEY_LEN) :: key
     CHARACTER(LEN=ARG_LEN) :: value
  END TYPE cfg_arg


  ! Calendar information.

  TYPE calendar_info
     CHARACTER(LEN=ARG_LEN) :: name
     INTEGER, DIMENSION(12) :: mon_lens
  END TYPE calendar_info

END MODULE pp_defs

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!----------------------------------------------------------------------
