MODULE cmor_users_functions

  ! This work was produced at the University of California, Lawrence
  ! Livermore National Laboratory (UC LLNL) under contract 
  ! no. W-7405-ENG-48 (Contract 48) between the U.S. Department of 
  ! Energy (DOE) and The Regents of the University of California 
  ! (University) for the operation of UC LLNL. The rights of the 
  ! Federal Government are reserved under Contract 48 subject to 
  ! the restrictions agreed upon by the DOE and University as allowed 
  ! under DOE Acquisition Letter 97-1.

  ! DISCLAIMER

  ! This work was prepared as an account of work sponsored by an agency 
  ! of the United States Government. Neither the United States 
  ! Government nor the University of California nor any of their 
  ! employees, makes any warranty, express or implied, or assumes any 
  ! liability or responsibility for the accuracy, completeness, or 
  ! usefulness of any information, apparatus, product, or process 
  ! disclosed, or represents that its use would not infringe privately-
  ! owned rights.  Reference herein to any specific commercial products, 
  ! process, or service by trade name, trademark, manufacturer or 
  ! otherwise does not necessarily constitute or imply its endorsement, 
  ! recommendation, or favoring by the United States Government or the 
  ! University of California. The views and opinions of authors 
  ! expressed herein do not necessarily state or reflect those of the 
  ! United States Government or the University of California, and shall 
  ! not be used for advertising or product endorsement purposes.

  ! NOTIFICATION OF COMMERCIAL USE

  ! Commercialization of this product is prohibited without notifying the 
  ! Department of Energy (DOE) or Lawrence Livermore National Laboratory 
  ! (LLNL).


  !  ???  need to treat ok_mean_abs_min and max
  !  ??? in MIP input file specify units as "days since ?" avoid 1600???
  !  ??? write to output file(s) #of eror encountered (fatal, non-fatal,
  !                 warning) during  initialization and write phases.
  !  treat new suffix
  !  table entry for a zfactor_bounds in input table must be "zfactor_bnds",
  !      where "zfactor" is the table entry for zfactor.  
  !      (.e.g., "b" and "b_bnds")
  !  zfactor_bounds must only be a function of level
  !  ??? treat alternative type/rank arguments  (e.g. , for bounds 1-d)
  !  ??? add log file
  !  ??? add error messages
  !  add time-coordinate generator
  !  ??? develop a few other examples
  !  ??? magic number check
  !  ??? "climatology" treatment
  !  ??? change time coordinate vlaues to be beginning of interval ?
  !  ??? develop test cases
  !        check that cmor_write can handle time-independent fields
  !        check that cmor_write can handle case when no field is passed.
  !        check that units transformation works
  !        check sigma and alternate hybrid-sigma work
  !        check missing value
  !        check: user supplies data from west to east, rather than east to
  !                   west (which is what the table specifies for the output
  !                   direction.
  !        check: user supplies data from -180 to 180, rather than 0 to 360
  !                   (which is what the table specifies for the output range).
  !        check: user supplies a missing value of 1.e30, and there *are* 
  !                    missing values in the array passed to CMOR.
  !        When output fields are split  across several files (by time slice),
  !                    check that cdscan can handle the results. 
  !  ??? error should not be called from cmor_setup because output
  !        directory has not yet been defined.
  !  ??? current version only works if A's and B's (and their bounds) are 
  !        output as double
  !  ??? "append" option won't work unless when file is opened, times_written,
  !        %ncid, and %varid are also defined

  use cmor_internal_tables
  USE cmor_getinput, ONLY : similar, read_table, cmor_getTime, cmor_getDate
  use netcdf

  PRIVATE
  PUBLIC cmor_setup, cmor_dataset, cmor_close, &
       cmor_time_and_bounds, cmor_write, cmor_variable, cmor_zfactor, cmor_axis
  
  interface cmor_write
     module procedure cmor_write_1d_i
     module procedure cmor_write_1d_r
     module procedure cmor_write_1d_d
     module procedure cmor_write_2d_i
     module procedure cmor_write_2d_r
     module procedure cmor_write_2d_d
     module procedure cmor_write_3d_i
     module procedure cmor_write_3d_r
     module procedure cmor_write_3d_d
     module procedure cmor_write_4d_i
     module procedure cmor_write_4d_r
     module procedure cmor_write_4d_d
     module procedure cmor_write_5d_i
     module procedure cmor_write_5d_r
     module procedure cmor_write_5d_d
     module procedure cmor_write_6d_i
     module procedure cmor_write_6d_r
     module procedure cmor_write_6d_d
     module procedure cmor_write_7d_i
     module procedure cmor_write_7d_r
     module procedure cmor_write_7d_d
  end interface
  
  INTERFACE cmor_variable
     module procedure  cmor_variable_r
     module procedure  cmor_variable_d
     module procedure  cmor_variable_i
     module procedure cmor_variable_no_missing
  END INTERFACE
  
     !   in the following, r=real, d=double, c=character refer to
     !      the type of the coord_vals and possibly cell_bounds
     !      0 indicates refers to no cell_bounds argument (or character string)
     !      1 indicates a 1-d array with cell_bounds
     !      2 indicates a 2-d array with cell_bounds
     !   in the case of character coord_vals, no cell bounds are allowed
     !   The "novals" case is when neither the coord_vals or cell_bounds 
     !      arguments are present.
  INTERFACE cmor_axis
     module procedure cmor_axis_r0
     module procedure cmor_axis_r1
     module procedure cmor_axis_r2
     module procedure cmor_axis_d0
     module procedure cmor_axis_d1
     module procedure cmor_axis_d2
     module procedure cmor_axis_c
     module procedure cmor_axis_novals
  END INTERFACE
  
  INTERFACE cmor_zfactor
     !  double or single precision values and/or bounds
     !  no values/bounds (for time-dependent fields)
     !  scalar, 1-d, 2-d, or 3-d (time-independent) values passed
     !  bounds have same dimension as scalar, or an extra dimension of size 2
     
     !  double precision values and bounds included
!!$     module procedure cmor_zfactor_r_3d_none
!!$     module procedure cmor_zfactor_r_2d_none
     module procedure cmor_zfactor_r_1d_b2d
     module procedure cmor_zfactor_r_1d_b1d
     module procedure cmor_zfactor_r_1d_none
     module procedure cmor_zfactor_r_0d_none
!!$     module procedure cmor_zfactor_d_3d_none
!!$     module procedure cmor_zfactor_d_2d_none
     module procedure cmor_zfactor_d_1d_b2d
     module procedure cmor_zfactor_d_1d_b1d
     module procedure cmor_zfactor_d_1d_none
     module procedure cmor_zfactor_d_0d_none
     module procedure cmor_zfactor_none_none

  END INTERFACE

CONTAINS
  
  subroutine add_times_values(varid,time_id,ntimes_passed,time_vals)
   !  this subroutine is not currently used by CMOR
    implicit none
    double precision, allocatable :: tmp(:)
    integer time_id,ntimes_passed,varid
    double precision time_vals(:)

    if (associated(adim(time_id)%values)) then
       allocate(tmp(var(varid)%times_written))
       tmp=adim(time_id)%values
       deallocate(adim(time_id)%values)
       allocate(adim(time_id)%values(var(varid)%times_written+ntimes_passed))
       adim(time_id)%values(1:var(varid)%times_written)=tmp
       deallocate(tmp)
       adim(time_id)%values(var(varid)%times_written+1:) = time_vals(1:ntimes_passed)
    else
       allocate(adim(time_id)%values(ntimes_passed))
       adim(time_id)%values=time_vals(1:ntimes_passed)
    endif
  end subroutine add_times_values

  subroutine add_times_bounds(varid,time_id,nbounds_passed,bounds_vals)
   !  this subroutine is not currently used by CMOR
    implicit none
    double precision, ALLOCATABLE :: tmp(:,:)
    integer time_id,nbounds_passed,varid
    double precision bounds_vals(:,:)
    if (associated(adim(time_id)%bounds)) then
       allocate(tmp(2,var(varid)%times_written))
       tmp=adim(time_id)%bounds
       deallocate(adim(time_id)%bounds)
       allocate(adim(time_id)%bounds(2,var(varid)%times_written+nbounds_passed))
       adim(time_id)%bounds(:,1:var(varid)%times_written)=tmp
       deallocate(tmp)
       adim(time_id)%bounds(:,var(varid)%times_written+1:) = bounds_vals(:,1:nbounds_passed)
    else
       allocate(adim(time_id)%bounds(2,nbounds_passed))
       adim(time_id)%bounds=bounds_vals(:,1:nbounds_passed)
    endif
  end subroutine add_times_bounds



  subroutine cmor_process_time(ncid, varid, ntimes_passed, dims_len, &
       start, a_time_slice, time_vals, time_bnds, store_with)

    implicit none

    integer, intent(in) :: ncid
    integer, intent(in) :: varid
    integer, intent(in), OPTIONAL ::  ntimes_passed
    integer, intent(out) :: dims_len(7)
    integer, intent(out) :: start(7)
    integer, intent(out) :: a_time_slice
    double precision, intent(in), OPTIONAL :: time_vals(:)
    double precision, intent(in), OPTIONAL :: time_bnds(:,:)
    integer, intent(in), OPTIONAL :: store_with

    double precision, ALLOCATABLE :: tmp_tvals(:)
    double precision, ALLOCATABLE :: tmp_tbnds(:,:)
    integer ierr
    integer time_id, iaxis, i
        
    time_id = -1
    i = 1
    a_time_slice = 1
    start = 1
    dims_len = 1

    do while ((var(varid)%dim_idxs(i).ne.0).and.(i.le.max_rank)) 
       if (axis(adim(var(varid)%dim_idxs(i))%axis_idx)%axis == 'T') then
          time_id = var(varid)%dim_idxs(i)
          if (present(ntimes_passed)) then
             dims_len(i) = ntimes_passed
          else
             dims_len(i)=-1
          endif
          if (present(store_with)) then
             start(i) = var(varid)%zfac_times_written(store_with)+1
          else
             start(i) = var(varid)%times_written+1
          endif
       else
          dims_len(i) = adim(var(varid)%dim_idxs(i))%lendim
          a_time_slice=a_time_slice*dims_len(i)
       endif
       i = i+1
    end do

    IF (.not. PRESENT(store_with)) THEN
       if (PRESENT(time_vals) .and. (time_id > 0)) then
          
          iaxis = adim(time_id)%axis_idx
          ALLOCATE(tmp_tvals( SIZE(time_vals)))
          tmp_tvals = time_vals

          IF (PRESENT(time_bnds)) THEN
             ALLOCATE(tmp_tbnds(2, SIZE(time_vals)))
             tmp_tbnds = time_bnds

             call scale_time(adim(time_id)%users_units,   &
                  axis(iaxis)%out_units, ntimes_passed, tmp_tvals, tmp_tbnds, &
                  ierr=ierr)

          ELSE
             
             call scale_time(adim(time_id)%users_units,   &
                  axis(iaxis)%out_units, ntimes_passed, tmp_tvals,  &
                  ierr=ierr)
             
          END IF          

          ierr = NF90_PUT_VAR(ncid,&
               var(varid)%nctimid,&
               tmp_tvals,&
               count = (/ntimes_passed/),&
               start=(/var(varid)%times_written+1/)&
               )
          DEALLOCATE(tmp_tvals)
          if (ierr.ne.NF90_NOERR) then
             PRINT*, 'Error encountered in CMOR: cmor_process_time'
             PRINT*, 'Attempting to write the following ', ntimes_passed, &
                  ' time values:'
             PRINT*, time_vals(var(varid)%times_written+1:var(varid)%times_written+ntimes_passed)
             PRINT*, ' for ', axis(adim(time_id)%axis_idx)%table_entry, &
                  ' axis, which is associated with the variable, ', var(varid)%table_entry
             print*, TRIM(nf90_strerror(ierr))
             STOP
          endif

       endif
       
       if (present(time_bnds) .and. (time_id > 0)) then

          ierr = NF90_PUT_VAR(ncid, &
               var(varid)%nctim_bndsid, &
               tmp_tbnds,&
               count = (/2,ntimes_passed/),&
               start=(/1,var(varid)%times_written+1/)&
               )
          DEALLOCATE(tmp_tbnds)

          if (ierr.ne.NF90_NOERR) then
             PRINT*, 'Error encountered in CMOR: cmor_process_time'
             PRINT*, 'Attempting to write the following ', ntimes_passed, &
                  ' time bounds:'
             PRINT*, time_bnds(1:2, var(varid)%times_written+1:var(varid)%times_written+ntimes_passed)
             PRINT*, ' for ', axis(adim(time_id)%axis_idx)%table_entry, &
                     ' axis, which is associated with the variable, ', var(varid)%table_entry
             print*, TRIM(nf90_strerror(ierr))
             STOP
          endif

       endif

    endif

  end subroutine cmor_process_time




  FUNCTION check_args(var_id, ncid, ntimes_passed, time_vals, time_bnds, &
       store_with, file_suffix) RESULT(ierr)

    ! check arguments passed to cmor_write, possibly open a new file, 
    ! return the ncid.

    IMPLICIT NONE

    INTEGER, INTENT(in) ::  var_id
    INTEGER , INTENT(out) :: ncid
    INTEGER, INTENT(in), OPTIONAL :: ntimes_passed
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_vals(:)
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_bnds(:,:)
    INTEGER , INTENT(in), OPTIONAL :: store_with
    CHARACTER(len=*), INTENT(in), OPTIONAL :: file_suffix
    
    INTEGER ierr

    INTEGER i
    CHARACTER(len=a64) :: suffix_copy
    CHARACTER(len=6) :: str1
    CHARACTER(len=13) :: str2

    ierr=0

    SELECT CASE(var(var_id)%def_missing)
    CASE(0)
       CONTINUE
    CASE(1)
       IF (var(var_id)%users_type .NE. 'real') THEN
          PRINT*, 'Invalid missing_value type', var(var_id)%def_missing
          ierr=-1012
       ENDIF
    CASE(2)
       IF (var(var_id)%users_type .NE. 'double') THEN
          PRINT*, 'Invalid missing_value type', var(var_id)%def_missing
          ierr=-1012
       ENDIF
    CASE(3)
       IF (var(var_id)%users_type .NE. 'integer') THEN
          PRINT*, 'Invalid missing_value type', var(var_id)%def_missing
          ierr=-1012
       ENDIF
    END SELECT

    IF (var_id.GT.num_vars) THEN
       PRINT*,'Error, variable id, is greater than actual number of variable (',var_id,'>', num_vars,')'
       ierr=-1000
       STOP
       RETURN
    END IF
    IF (var_id .LT. 1) THEN
       PRINT *, 'Error, variable id has to be greater than 1, you passed:',var_id
       ierr=-1001
       STOP
       RETURN  
    ENDIF

    PRINT*, 'CMOR is writing to netCDF  variable ', var(var_id)%table_entry
    IF (PRESENT(ntimes_passed)) THEN
       IF (PRESENT(time_vals)) THEN
          WRITE(str1, '(i6)') ntimes_passed
          WRITE(str2, '(ES13.6)') time_vals(1)
          PRINT*, str1 // &
               ' time slice(s) will be written starting with time ' // str2
       ELSE
          PRINT*, '   ', ntimes_passed, ' time slices will be written.'
       ENDIF
    ENDIF
    
    IF (PRESENT(time_vals)) THEN
       IF (PRESENT(ntimes_passed)) THEN
          IF (ntimes_passed .NE. SIZE(time_vals)) THEN
             ierr=-1002
             PRINT*, 'Error, ntime_passed (',ntimes_passed,  &
                  ') does not match length of time_vals (',SIZE(time_vals),')'
             STOP
             RETURN
          ENDIF
       ENDIF
       IF (PRESENT(time_bnds)) THEN
          IF (SIZE(time_vals) .NE. SIZE(time_bnds,2)) THEN
             ierr=-1003
             PRINT*, 'Error: time_vals and time_bounds have different lengths: ', &
                  ntimes_passed,SIZE(time_bnds,2)
             STOP
             RETURN
          ENDIF
       ENDIF
       DO i=1,SIZE(time_vals)
          IF (i .NE. SIZE(time_vals)) THEN
             IF (time_vals(i) .GE. time_vals(i+1)) THEN
                ierr=-1004
                PRINT*, 'Error: time value passed to cmor_write at index', i
                PRINT*, '   is greater than following value, i.e., '
                PRINT*, '   ', time_vals(i), ' is greater than ', time_vals(i+1)
                STOP
                RETURN
             ENDIF
          ENDIF
          IF (PRESENT(time_bnds)) THEN
             IF (time_bnds(1,i) .GE. time_bnds(2,i)) THEN
                ierr=-1005
                PRINT*, 'Error: pair of time_bnds passed to cmor_write at index', i
                PRINT*, '  are not chronologically ordered: ', time_bnds(:,i)
                STOP
                RETURN
             ENDIF
             IF ((time_bnds(1,i) .GT. time_vals(i)) .OR.    &
                  (time_bnds(2,i) .LT. time_vals(i))) THEN
                ierr=-1006
                PRINT*,'Error: time_vals, ', time_vals(i), &
                     ' passed to cmor_write at, index', i 
                PRINT*, '  is not contained between its bounds: ',time_bnds(:,i)
                STOP
                RETURN
             ENDIF
          ENDIF
       ENDDO
    ENDIF
    IF (PRESENT(store_with)) THEN
       IF (store_with .GT. num_vars) THEN
          PRINT*,'Error, store_with id, is greater than actual number of variables (', &
               store_with,'>',num_vars,')'
          ierr=-1007
          STOP
          RETURN
       END IF
    ENDIF

    IF (PRESENT(file_suffix)) THEN
       IF (LEN_TRIM(ADJUSTL(file_suffix)) <= a64) THEN
          suffix_copy = TRIM(ADJUSTL(file_suffix))
       ELSE
          ierr=-1009
          PRINT*, 'CMOR Error file suffix too long (>',a64,') value:',TRIM(ADJUSTL(file_suffix))
          STOP
          RETURN
       END IF
    ELSE
       suffix_copy = init_undef_char
    END IF

    IF (PRESENT(store_with)) THEN
       IF (store_with.LT.1) THEN
          PRINT *, 'Error, store_with id has to be greater than 1, you passed:',store_with
          ierr=-1008
          STOP
          RETURN  
       ENDIF
    ENDIF

    !  possibly open a file and  write metadata
    IF (PRESENT(store_with)) THEN
       IF (store_with /= 0) THEN
          ierr = initialize_file(store_with, suffix_copy)
       ELSE
          ierr = initialize_file(var_id, suffix_copy)
       END IF
    ELSE
       ierr = initialize_file(var_id, suffix_copy)
    ENDIF

    IF (PRESENT(store_with)) THEN
       ncid = var(store_with)%ncid
    ELSE
       ncid = var(var_id)%ncid
    ENDIF

    RETURN

  END FUNCTION check_args




  FUNCTION offset_or_reverse_dim(var_id,dim,dim_len) RESULT(indx)

    IMPLICIT NONE    
    INTEGER, INTENT(in) :: var_id
    INTEGER, INTENT(in) :: dim        ! input dimension number
    INTEGER, INTENT(in) :: dim_len    ! output length of dimension

    INTEGER indx(dim_len)
    INTEGER jj

    !  This function converts the location to match potential reversing, 
    !     offsetting, reordering etc...

    !  It generates an index array, indx(jj), containing the location
    !    in the output array corresponding to input location jj.

    IF (dim_len == 1) THEN

       indx(1) = 1

    ELSE

       IF (adim(var(var_id)%dim_idxs(dim))%reverse) THEN
          
          DO jj = 1, dim_len
             indx(jj)=1 + adim(var(var_id)%dim_idxs(dim))%begin_out - jj
             IF (indx(jj) < 1)  indx(jj) = indx(jj) + dim_len
          END DO
          
       ELSE
          
          DO jj = 1, dim_len
             indx(jj)=1 - adim(var(var_id)%dim_idxs(dim))%begin_out + jj
             IF (indx(jj) < 1)  indx(jj) = indx(jj) + dim_len
          END DO
          
       END IF

    END IF

  END FUNCTION offset_or_reverse_dim




  subroutine check_time_and_dim_lens(data_size, var_id, dims_len, ntimes, &
       ierr, ntimes_passed)

    implicit none

    integer, intent(in) :: var_id
    integer, intent(in) ::  data_size(7)
    integer, intent(inout) :: dims_len(7)
    integer, intent(out) :: ierr, ntimes
    integer, intent(in), OPTIONAL :: ntimes_passed

    integer i, j, rk

    ierr = 0

    !  determine rank of user-passed data array
    rk=7
    DO i=1,7
       IF (data_size(i).eq.0) THEN
          rk=i-1
          EXIT
       END IF
    END DO

    ntimes = 0
    if (.not.present(ntimes_passed)) then
       do i = 1, 7
          if (dims_len(i).eq.-1) then 
             do j = 1, 7
                if (var(var_id)%dim_order(j).eq.i) then
                   ntimes=data_size(j)
                   ! if array passed does not have time:
                   if (ntimes.eq.0) ntimes=1
                   dims_len(i)=ntimes
                   EXIT
                endif
             enddo
          endif
       end do
    else
       ntimes = ntimes_passed
    endif

    !  check that the array passed by the user has dimensions >= those
    !  expected according to dims_len

    ! loop through input dimensions
    DO i=1,rk
       ! j = position of output dimension
       j = var(var_id)%dim_order(i)
       IF (data_size(i) < dims_len(j)) THEN
          print*, 'Error: dimension ', i,   &
               'of the array passed by the user to cmor_write is too short.'
          print*, 'The length of this dimension was ', data_size(i)
          print*, 'but should be at least ', dims_len(j)
          ierr=-1052
          stop
       END IF
    END DO

    ! check that the input array includes all the necessary input
    !  dimensions.
    IF (rk < 7) THEN
       DO i=rk+1,7
          j = var(var_id)%dim_order(i)
          IF (j > 0) THEN
             IF (dims_len(j) > 1) THEN
                print*, 'Error: Rank of array passed by user is ', rk 
                print*, 'but should be at least ', rk+1, ' to accommodate'
                print*, 'the ', &
                     axis(var(var_id)%axis_idxs(j))%table_entry, &
                     ' dimension, which should be length ', &
                     adim(var(var_id)%dim_idxs(j))%lendim
                STOP
             END IF
          END IF
       END DO
    END IF

    RETURN

  end subroutine check_time_and_dim_lens




  subroutine errmsg_r(minmax, irank, var_id, valin, rval, ival, dval, &
       i1, i2, i3, i4, i5, i6, i7)
    
    IMPLICIT none
    CHARACTER(len=*), INTENT(in) :: minmax
    INTEGER, INTENT(in) :: irank
    INTEGER, INTENT(in) :: var_id
    REAL, INTENT(in) :: valin
    REAL, INTENT(in) :: rval
    INTEGER, INTENT(in) :: ival
    DOUBLE PRECISION, INTENT(in) :: dval
    INTEGER, INTENT(in) :: i1, i2, i3, i4, i5, i6, i7
    
    INTEGER :: i, indx(irank)

    indx(1) = i1
    indx(2) = i2
    indx(3) = i3
    indx(4) = i4
    indx(5) = i5
    indx(6) = i6
    indx(7) = i7

    SELECT CASE(minmax)
       
    CASE('min')
       
       PRINT*, 'data value:', valin, 'passed to CMOR '
       PRINT*, 'was multiplied by ', var(var_id)%scale_factor 
       PRINT*, 'and offset by ', var(var_id)%offset

       SELECT CASE(TRIM(var(var_id)%type))
       CASE('real')
          PRINT*, 'yielding a value, ', rval, 'that is less than' 
       CASE('integer')
          PRINT*, 'yielding a value, ', ival, 'that is less than' 
       CASE('double')
          PRINT*, 'yielding a value, ', dval, 'that is less than' 
       END SELECT
       
       PRINT*, 'minimum value allowed: ', var(var_id)%valid_min
       PRINT*, 'data value found in user-passed array at location: ', &
            (indx(i), i=1,irank)
       
    CASE('max')
       
       PRINT*, 'data value:', valin, 'passed to CMOR '
       PRINT*, 'was multiplied by ', var(var_id)%scale_factor 
       PRINT*, 'and offset by ', var(var_id)%offset

       SELECT CASE(TRIM(var(var_id)%type))
       CASE('real')
          PRINT*, 'yielding a value, ', rval, 'that is greater than' 
       CASE('integer')
          PRINT*, 'yielding a value, ', ival, 'that is greater than' 
       CASE('double')
          PRINT*, 'yielding a value, ', dval, 'that is greater than' 
       END SELECT

       PRINT*, 'maximum value allowed: ',  var(var_id)%valid_max
       PRINT*, 'data value found in user-passed array at location: ', &
            (indx(i), i=1,irank)

    END SELECT
    
    STOP

  END subroutine errmsg_r



  subroutine errmsg_i(minmax, irank, var_id, valin, rval, ival, dval, &
       i1, i2, i3, i4, i5, i6, i7)
    
    IMPLICIT none
    CHARACTER(len=*), INTENT(in) :: minmax
    INTEGER, INTENT(in) :: irank
    INTEGER, INTENT(in) :: var_id
    INTEGER, INTENT(in) :: valin
    REAL, INTENT(in) :: rval
    INTEGER, INTENT(in) :: ival
    DOUBLE PRECISION, INTENT(in) :: dval
    INTEGER, INTENT(in) :: i1, i2, i3, i4, i5, i6, i7
    
    INTEGER :: i, indx(irank)

    indx(1) = i1
    indx(2) = i2
    indx(3) = i3
    indx(4) = i4
    indx(5) = i5
    indx(6) = i6
    indx(7) = i7

    SELECT CASE(minmax)
       
    CASE('min')
       
       PRINT*, 'data value:', valin, 'passed to CMOR '
       PRINT*, 'was multiplied by ', var(var_id)%scale_factor 
       PRINT*, 'and offset by ', var(var_id)%offset

       SELECT CASE(TRIM(var(var_id)%type))
       CASE('real')
          PRINT*, 'yielding a value, ', rval, 'that is less than' 
       CASE('integer')
          PRINT*, 'yielding a value, ', ival, 'that is less than' 
       CASE('double')
          PRINT*, 'yielding a value, ', dval, 'that is less than' 
       END SELECT
       
       PRINT*, 'minimum value allowed: ', var(var_id)%valid_min
       PRINT*, 'data value found in user-passed array at location: ', &
            (indx(i), i=1,irank)
       
    CASE('max')
       
       PRINT*, 'data value:', valin, 'passed to CMOR '
       PRINT*, 'was multiplied by ', var(var_id)%scale_factor 
       PRINT*, 'and offset by ', var(var_id)%offset

       SELECT CASE(TRIM(var(var_id)%type))
       CASE('real')
          PRINT*, 'yielding a value, ', rval, 'that is greater than' 
       CASE('integer')
          PRINT*, 'yielding a value, ', ival, 'that is greater than' 
       CASE('double')
          PRINT*, 'yielding a value, ', dval, 'that is greater than' 
       END SELECT

       PRINT*, 'maximum value allowed: ',  var(var_id)%valid_max
       PRINT*, 'data value found in user-passed array at location: ', &
            (indx(i), i=1,irank)

    END SELECT
    
    STOP

  END subroutine errmsg_i



  subroutine errmsg_d(minmax, irank, var_id, valin, rval, ival, dval, &
       i1, i2, i3, i4, i5, i6, i7)
    
    IMPLICIT none
    CHARACTER(len=*), INTENT(in) :: minmax
    INTEGER, INTENT(in) :: irank
    INTEGER, INTENT(in) :: var_id
    DOUBLE PRECISION, INTENT(in) :: valin
    REAL, INTENT(in) :: rval
    INTEGER, INTENT(in) :: ival
    DOUBLE PRECISION, INTENT(in) :: dval
    INTEGER, INTENT(in) :: i1, i2, i3, i4, i5, i6, i7
    
    INTEGER :: i, indx(irank)

    indx(1) = i1
    indx(2) = i2
    indx(3) = i3
    indx(4) = i4
    indx(5) = i5
    indx(6) = i6
    indx(7) = i7

    SELECT CASE(minmax)
       
    CASE('min')
       
       PRINT*, 'data value:', valin, 'passed to CMOR '
       PRINT*, 'was multiplied by ', var(var_id)%scale_factor 
       PRINT*, 'and offset by ', var(var_id)%offset

       SELECT CASE(TRIM(var(var_id)%type))
       CASE('real')
          PRINT*, 'yielding a value, ', rval, 'that is less than' 
       CASE('integer')
          PRINT*, 'yielding a value, ', ival, 'that is less than' 
       CASE('double')
          PRINT*, 'yielding a value, ', dval, 'that is less than' 
       END SELECT
       
       PRINT*, 'minimum value allowed: ', var(var_id)%valid_min
       PRINT*, 'data value found in user-passed array at location: ', &
            (indx(i), i=1,irank)
       
    CASE('max')
       
       PRINT*, 'data value:', valin, 'passed to CMOR '
       PRINT*, 'was multiplied by ', var(var_id)%scale_factor 
       PRINT*, 'and offset by ', var(var_id)%offset

       SELECT CASE(TRIM(var(var_id)%type))
       CASE('real')
          PRINT*, 'yielding a value, ', rval, 'that is greater than' 
       CASE('integer')
          PRINT*, 'yielding a value, ', ival, 'that is greater than' 
       CASE('double')
          PRINT*, 'yielding a value, ', dval, 'that is greater than' 
       END SELECT

       PRINT*, 'maximum value allowed: ',  var(var_id)%valid_max
       PRINT*, 'data value found in user-passed array at location: ', &
            (indx(i), i=1,irank)

    END SELECT
    
    STOP

  END subroutine errmsg_d




  FUNCTION cmor_write_1d_r(var_id, data, file_suffix, ntimes_passed,  &
       time_vals, time_bnds, store_with) RESULT(ierr)

    IMPLICIT NONE

    REAL, INTENT(in):: data(:)
    INTEGER, INTENT(in) ::var_id
    CHARACTER(len=*), INTENT(in), OPTIONAL :: file_suffix
    INTEGER, INTENT(in), OPTIONAL :: ntimes_passed
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_vals(:)
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_bnds(:,:)
    INTEGER, INTENT(in), OPTIONAL :: store_with

    INTEGER j, time_id
    INTEGER ipt1, ipt2, ipt3, ipt4, ipt5, ipt6, ipt7

    INTEGER, ALLOCATABLE, DIMENSION(:) :: indx1, indx2, indx3, indx4, &
         indx5, indx6, indx7
    INTEGER, ALLOCATABLE :: data_out_i(:,:,:,:,:,:,:)
    REAL, ALLOCATABLE :: data_out_r(:,:,:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: data_out_d(:,:,:,:,:,:,:)
    INTEGER, DIMENSION(max_rank) :: inout, loc, dims_len, start, data_size
    INTEGER :: i1=1, i2=1, i3=1, i4=1, i5=1, i6=1, i7=1
    INTEGER :: ival, istat, ncid, ntimes, a_time_slice, i, irank, ierr
    REAL :: rval
    DOUBLE PRECISION :: dval

    REAL :: valin

    var(var_id)%users_type = 'real'

    ierr = 0

    time_id = -1
    data_size = 0

    ! loop through input dimension
    irank = 7
    a_time_slice = 1
    DO j = 1, 7

       i = var(var_id)%dim_order(j)
       IF (i == 0) EXIT
       irank = j

       IF (axis(var(var_id)%axis_idxs(i))%axis == 'T') THEN
          time_id = j
       ELSE
          data_size(j) =  adim(var(var_id)%dim_idxs(i))%lendim
          a_time_slice=a_time_slice*data_size(i)
       END IF

    END DO

    j = 1

    IF (time_id > 0) THEN

       IF (PRESENT(ntimes_passed)) THEN

          data_size(time_id) = ntimes_passed

          IF (data_size(time_id) == 0) THEN
             PRINT*, 'Error encountered in cmor_write:'
             PRINT*, '  user passed ntimes_passed = 0 but this field'
             PRINT*, '  should have a time dimension.'
             STOP
          END IF

       ELSE

          data_size(time_id) = SIZE(DATA)/a_time_slice
          IF (data_size(time_id) == 0) THEN
             PRINT*, 'Error encountered in cmor_write:'
             PRINT*, '  user passed a data array that should be larger, '
             PRINT*, '  it cannot hold even a single time-slice of data.'
             STOP
          END IF

       END IF

       j = data_size(time_id)

    END IF

    IF (a_time_slice*j > SIZE(data)) THEN

       PRINT*, 'Error encountered in cmor_write:'
       PRINT*, '  The data array passed by user has fewer elements than it should'
       PRINT*, '  The axes defined indicate the following dimension lengths: ',&
            data_size
       PRINT*, '  for a total size of ',  &
            a_time_slice*j
       PRINT*,  ' but the users array is length: ', SIZE(data)
       STOP

    END IF

    ierr = check_args(var_id, ncid, ntimes_passed, time_vals, time_bnds, &
         store_with, file_suffix)
    IF (ierr < 0) RETURN

    ! Process time stuff, define dims_len and start, and for use in the 
    !     write_1d case (only??)  also defines a_time_slice

    CALL cmor_process_time(ncid, var_id, ntimes_passed, dims_len, start, &
         a_time_slice, time_vals, time_bnds, store_with)


    ! besides checking, the following defines ntimes

    CALL check_time_and_dim_lens(data_size, var_id, dims_len, ntimes, ierr, &
         ntimes_passed)
    IF (ierr.LT.0) RETURN


    ! dim_order(i) stores the output array index that corresponds to 
    !   input array index i; it stores 0 for output dimensions > the
    !   rank of the output array.
    !   Singleton (scalar) dimensions that the user hasn't overridden are
    !   omitted.

    ! inout is similar to dim_order, but stores i for output dimensions > the
    !   rank of the output array.

    DO i=1,max_rank
       inout(i) = var(var_id)%dim_order(i)
       if (inout(i) == 0) inout(i) = i
    END DO

    ALLOCATE(indx1(dims_len(inout(1))), STAT=istat)
    ALLOCATE(indx2(dims_len(inout(2))), STAT=istat)
    ALLOCATE(indx3(dims_len(inout(3))), STAT=istat)
    ALLOCATE(indx4(dims_len(inout(4))), STAT=istat)
    ALLOCATE(indx5(dims_len(inout(5))), STAT=istat)
    ALLOCATE(indx6(dims_len(inout(6))), STAT=istat)
    ALLOCATE(indx7(dims_len(inout(7))), STAT=istat)

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*,  '   Unable to allocate memory for one of the "idx?" arrays,' 
       PRINT*,  '   which have one of the following dimensions: ', dims_len
       STOP
    END IF

    ! The following generates an index array, indx?(i), containing the location
    !    in the output array corresponding to input location i.
    ! indx1 is index for input array dimension 1, 
    !    indx2 for input dimension 2, etc.

    indx1 = offset_or_reverse_dim(var_id,1,dims_len(inout(1)))
    indx2 = offset_or_reverse_dim(var_id,2,dims_len(inout(2)))
    indx3 = offset_or_reverse_dim(var_id,3,dims_len(inout(3)))
    indx4 = offset_or_reverse_dim(var_id,4,dims_len(inout(4)))
    indx5 = offset_or_reverse_dim(var_id,5,dims_len(inout(5)))
    indx6 = offset_or_reverse_dim(var_id,6,dims_len(inout(6)))
    indx7 = offset_or_reverse_dim(var_id,7,dims_len(inout(7)))

    ! now assign input data to appropriate positions in output array.

    SELECT CASE(TRIM(var(var_id)%type))

    CASE("real")

       ALLOCATE(data_out_r(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("integer")

       ALLOCATE(data_out_i(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("double")

       ALLOCATE(data_out_d(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    END SELECT

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*, '   Unable to allocate memory for one of the "data_out",'
       PRINT*, '   which has the following dimensions: ', dims_len
       STOP
    END IF
    
    loc = 1
 
    DO i7 = 1, dims_len(inout(7))
       loc(inout(7)) = indx7(i7)
       ipt7 = (i7-1)*data_size(6)
       DO i6 = 1, dims_len(inout(6))
          loc(inout(6)) = indx6(i6)
          ipt6 = (ipt7 + i6-1)*data_size(5)
          DO i5 = 1, dims_len(inout(5))
             loc(inout(5)) = indx5(i5)
             ipt5 = (ipt6 + i5-1)*data_size(4)
             DO i4 = 1, dims_len(inout(4))
                loc(inout(4)) = indx4(i4)
                ipt4 = (ipt5 + i4-1)*data_size(3)
                DO i3 = 1, dims_len(inout(3))
                   loc(inout(3)) = indx3(i3)
                   ipt3 = (ipt4 + i3-1)*data_size(2)
                   DO i2 = 1, dims_len(inout(2))
                      loc(inout(2)) = indx2(i2)
                      ipt2 = (ipt3 + i2-1)*data_size(1)
                      DO i1 = 1, dims_len(inout(1))
                         loc(inout(1)) = indx1(i1)
                         ipt1 = ipt2 + i1
                            
                         valin = data(ipt1)
                         
                         SELECT CASE(TRIM(var(var_id)%type))

                         CASE("real")
                            
                            IF ((var(var_id)%def_missing > 0) .and.  &
                                 (ABS(valin-var(var_id)%users_missing) &
                                 < ABS(var(var_id)%users_missing * &
                                 var(var_id)%miss_val_tolerance))) THEN

                               i = var(var_id)%mip_table_idx
                               rval = mip_tbl(i)%single_missing

                            ELSE
                               
                               rval = var(var_id)%offset +       &
                                    valin*var(var_id)%scale_factor
                               
                               IF ((var(var_id)%valid_min /=     &
                                    init_undef_single) .and. &
                                    (rval < var(var_id)%valid_min)) THEN
                                  call errmsg_r('min', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               ELSE IF ((var(var_id)%valid_max /=    &
                                    init_undef_single) .and. &
                                    (rval > var(var_id)%valid_max)) THEN
                                  call errmsg_r('max', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               END IF
                               
                            END IF
                            
                            data_out_r(loc(1),loc(2),loc(3),loc(4),  &
                                 loc(5),loc(6),loc(7)) = rval

                         CASE("integer")
                            
                            IF ((var(var_id)%def_missing > 0) .and.  &
                                 (ABS(valin-var(var_id)%users_missing) &
                                 < ABS(var(var_id)%users_missing * &
                                 var(var_id)%miss_val_tolerance))) THEN

                               i = var(var_id)%mip_table_idx
                               ival = mip_tbl(i)%integer_missing

                            ELSE
                               
                               ival = var(var_id)%offset +       &
                                    valin*var(var_id)%scale_factor
                               
                               IF ((var(var_id)%valid_min /=     &
                                    init_undef_single) .and. &
                                    (ival < var(var_id)%valid_min)) THEN
                                  call errmsg_r('min', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               ELSE IF ((var(var_id)%valid_max /=    &
                                    init_undef_single) .and. &
                                    (ival > var(var_id)%valid_max)) THEN
                                  call errmsg_r('max', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               END IF

                               
                            END IF
                            
                            data_out_i(loc(1),loc(2),loc(3),loc(4),  &
                                 loc(5),loc(6),loc(7)) = ival

                         CASE("double")
                            
                            IF ((var(var_id)%def_missing > 0) .and.  &
                                 (ABS(valin-var(var_id)%users_missing) &
                                 < ABS(var(var_id)%users_missing * &
                                 var(var_id)%miss_val_tolerance))) THEN
                               i = var(var_id)%mip_table_idx
                               dval = mip_tbl(i)%integer_missing
                            ELSE
                               
                               dval = var(var_id)%offset +       &
                                    valin*var(var_id)%scale_factor
                               
                               IF ((var(var_id)%valid_min /=     &
                                    init_undef_single) .and. &
                                    (dval < var(var_id)%valid_min)) THEN
                                  call errmsg_r('min', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               ELSE IF ((var(var_id)%valid_max /=    &
                                    init_undef_single) .and. &
                                    (dval > var(var_id)%valid_max)) THEN
                                  call errmsg_r('max', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               END IF
                               
                            END IF
                            
                            data_out_d(loc(1),loc(2),loc(3),loc(4),  &
                                 loc(5),loc(6),loc(7)) = dval
                            
                         END SELECT
                         
                      END DO
                   END DO
                END DO
             END DO
          END DO
       END DO
    END DO

    SELECT CASE(TRIM(var(var_id)%type))

    CASE('real')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_r, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_r)
       
    CASE('integer')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_i, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_i)
       
    CASE('double')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_d, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_d)
       
    END SELECT
           
    DEALLOCATE(indx1)
    DEALLOCATE(indx2)
    DEALLOCATE(indx3)
    DEALLOCATE(indx4)
    DEALLOCATE(indx5)
    DEALLOCATE(indx6)
    DEALLOCATE(indx7)
        
    IF (ierr == NF90_NOERR) THEN
       IF (ntimes /= 0) THEN
          IF (PRESENT(store_with)) THEN
             var(var_id)%zfac_times_written(store_with) =   &
                  var(var_id)%zfac_times_written(store_with)+ntimes
          ELSE
             var(var_id)%times_written=var(var_id)%times_written+ntimes
          END IF
       END IF
    ELSE
       PRINT*, 'Error writing NetCDF out data: error number =  ', ierr
       PRINT*, '   netCDF error message: ',  TRIM(nf90_strerror(ierr))
       STOP
    END IF
    
    RETURN
  END FUNCTION cmor_write_1d_r




  FUNCTION cmor_write_2d_r(var_id, data, file_suffix, ntimes_passed,  &
       time_vals, time_bnds, store_with) RESULT(ierr)

    IMPLICIT NONE

    REAL, INTENT(in):: data(:,:)
    INTEGER, INTENT(in) ::var_id
    CHARACTER(len=*), INTENT(in), OPTIONAL :: file_suffix
    INTEGER, INTENT(in), OPTIONAL :: ntimes_passed
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_vals(:)
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_bnds(:,:)
    INTEGER, INTENT(in), OPTIONAL :: store_with

    INTEGER, ALLOCATABLE, DIMENSION(:) :: indx1, indx2
    INTEGER, ALLOCATABLE :: data_out_i(:,:,:,:,:,:,:)
    REAL, ALLOCATABLE :: data_out_r(:,:,:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: data_out_d(:,:,:,:,:,:,:)
    INTEGER, DIMENSION(max_rank) :: inout, loc, dims_len, start, data_size
    INTEGER :: i1=1, i2=1, i3=1, i4=1, i5=1, i6=1, i7=1
    INTEGER :: ival, istat, ncid, ntimes, a_time_slice, i, irank, ierr
    REAL :: rval
    DOUBLE PRECISION :: dval

    REAL :: valin

    var(var_id)%users_type = 'real'
    irank = 2

    ierr = 0

    ! store input array's dimension lengths
    data_size = 0
    DO i =1 , irank
       data_size(i)=SIZE(data,i)
    END DO

    ierr = check_args(var_id, ncid, ntimes_passed, time_vals, time_bnds, &
         store_with, file_suffix)
    IF (ierr < 0) RETURN

    ! Process time stuff, define dims_len and start, and for use in the 
    !     write_1d case (only??)  also defines a_time_slice

    CALL cmor_process_time(ncid, var_id, ntimes_passed, dims_len, start, &
         a_time_slice, time_vals, time_bnds, store_with)


    ! besides checking, the following defines ntimes

    CALL check_time_and_dim_lens(data_size, var_id, dims_len, ntimes, ierr, &
         ntimes_passed)
    IF (ierr.LT.0) RETURN


    ! dim_order(i) stores the output array index that corresponds to 
    !   input array index i; it stores 0 for output dimensions > the
    !   rank of the output array.
    !   Singleton (scalar) dimensions that the user hasn't overridden are
    !   omitted.

    ! inout is similar to dim_order, but stores i for output dimensions > the
    !   rank of the output array.

    DO i=1,max_rank
       inout(i) = var(var_id)%dim_order(i)
       if (inout(i) == 0) inout(i) = i
    END DO

    ALLOCATE(indx1(dims_len(inout(1))), STAT=istat)
    ALLOCATE(indx2(dims_len(inout(2))), STAT=istat)

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*,  '   Unable to allocate memory for one of the "idx?" arrays,' 
       PRINT*,  '   which have one of the following dimensions: ', dims_len
       STOP
    END IF

    ! The following generates an index array, indx?(i), containing the location
    !    in the output array corresponding to input location i.
    ! indx1 is index for input array dimension 1, 
    !    indx2 for input dimension 2, etc.

    indx1 = offset_or_reverse_dim(var_id,1,dims_len(inout(1)))
    indx2 = offset_or_reverse_dim(var_id,2,dims_len(inout(2)))

    ! now assign input data to appropriate positions in output array.

    SELECT CASE(TRIM(var(var_id)%type))

    CASE("real")

       ALLOCATE(data_out_r(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("integer")

       ALLOCATE(data_out_i(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("double")

       ALLOCATE(data_out_d(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    END SELECT

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*, '   Unable to allocate memory for one of the "data_out",'
       PRINT*, '   which has the following dimensions: ', dims_len
       STOP
    END IF
    
    loc = 1
    
    DO i2 = 1, dims_len(inout(2))
       loc(inout(2)) = indx2(i2)
       DO i1 = 1, dims_len(inout(1))
          loc(inout(1)) = indx1(i1)

          valin = data(i1,i2)

          SELECT CASE(TRIM(var(var_id)%type))

          CASE("real")

             IF ((var(var_id)%def_missing > 0) .and.  &
                  (ABS(valin-var(var_id)%users_missing) &
                  < ABS(var(var_id)%users_missing * &
                  var(var_id)%miss_val_tolerance))) THEN

                i = var(var_id)%mip_table_idx
                rval = mip_tbl(i)%single_missing

             ELSE

                rval = var(var_id)%offset +       &
                     valin*var(var_id)%scale_factor

                IF ((var(var_id)%valid_min /=     &
                     init_undef_single) .and. &
                     (rval < var(var_id)%valid_min)) THEN
                   call errmsg_r('min', irank, var_id, valin, &
                        rval, ival, dval, i1, i2, i3, i4, i5, &
                        i6, i7)
                ELSE IF ((var(var_id)%valid_max /=    &
                     init_undef_single) .and. &
                     (rval > var(var_id)%valid_max)) THEN
                   call errmsg_r('max', irank, var_id, valin, &
                        rval, ival, dval, i1, i2, i3, i4, i5, &
                        i6, i7)
                END IF

             END IF

             data_out_r(loc(1),loc(2),loc(3),loc(4),  &
                  loc(5),loc(6),loc(7)) = rval

          CASE("integer")

             IF ((var(var_id)%def_missing > 0) .and.  &
                  (ABS(valin-var(var_id)%users_missing) &
                  < ABS(var(var_id)%users_missing * &
                  var(var_id)%miss_val_tolerance))) THEN

                i = var(var_id)%mip_table_idx
                ival = mip_tbl(i)%integer_missing

             ELSE

                ival = var(var_id)%offset +       &
                     valin*var(var_id)%scale_factor

                IF ((var(var_id)%valid_min /=     &
                     init_undef_single) .and. &
                     (ival < var(var_id)%valid_min)) THEN
                   call errmsg_r('min', irank, var_id, valin, &
                        rval, ival, dval, i1, i2, i3, i4, i5, &
                        i6, i7)
                ELSE IF ((var(var_id)%valid_max /=    &
                     init_undef_single) .and. &
                     (ival > var(var_id)%valid_max)) THEN
                   call errmsg_r('max', irank, var_id, valin, &
                        rval, ival, dval, i1, i2, i3, i4, i5, &
                        i6, i7)
                END IF


             END IF

             data_out_i(loc(1),loc(2),loc(3),loc(4),  &
                  loc(5),loc(6),loc(7)) = ival

          CASE("double")

             IF ((var(var_id)%def_missing > 0) .and.  &
                  (ABS(valin-var(var_id)%users_missing) &
                  < ABS(var(var_id)%users_missing * &
                  var(var_id)%miss_val_tolerance))) THEN
                i = var(var_id)%mip_table_idx
                dval = mip_tbl(i)%integer_missing
             ELSE

                dval = var(var_id)%offset +       &
                     valin*var(var_id)%scale_factor

                IF ((var(var_id)%valid_min /=     &
                     init_undef_single) .and. &
                     (dval < var(var_id)%valid_min)) THEN
                   call errmsg_r('min', irank, var_id, valin, &
                        rval, ival, dval, i1, i2, i3, i4, i5, &
                        i6, i7)
                ELSE IF ((var(var_id)%valid_max /=    &
                     init_undef_single) .and. &
                     (dval > var(var_id)%valid_max)) THEN
                   call errmsg_r('max', irank, var_id, valin, &
                        rval, ival, dval, i1, i2, i3, i4, i5, &
                        i6, i7)
                END IF

             END IF

             data_out_d(loc(1),loc(2),loc(3),loc(4),  &
                  loc(5),loc(6),loc(7)) = dval

          END SELECT

       END DO
    END DO

    SELECT CASE(TRIM(var(var_id)%type))

    CASE('real')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_r, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_r)
       
    CASE('integer')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_i, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_i)
       
    CASE('double')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_d, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_d)
       
    END SELECT
           
    DEALLOCATE(indx1)
    DEALLOCATE(indx2)
    
    IF (ierr == NF90_NOERR) THEN
       IF (ntimes /= 0) THEN
          IF (PRESENT(store_with)) THEN
             var(var_id)%zfac_times_written(store_with) =   &
                  var(var_id)%zfac_times_written(store_with)+ntimes
          ELSE
             var(var_id)%times_written=var(var_id)%times_written+ntimes
          END IF
       END IF
    ELSE
       PRINT*, 'Error writing NetCDF out data: error number =  ', ierr
       PRINT*, '   netCDF error message: ',  TRIM(nf90_strerror(ierr))
       STOP
    END IF
    
    RETURN
  END FUNCTION cmor_write_2d_r



  FUNCTION cmor_write_3d_r(var_id, data, file_suffix, ntimes_passed,  &
       time_vals, time_bnds, store_with) RESULT(ierr)

    IMPLICIT NONE

    REAL, INTENT(in):: data(:,:,:)
    INTEGER, INTENT(in) ::var_id
    CHARACTER(len=*), INTENT(in), OPTIONAL :: file_suffix
    INTEGER, INTENT(in), OPTIONAL :: ntimes_passed
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_vals(:)
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_bnds(:,:)
    INTEGER, INTENT(in), OPTIONAL :: store_with

    INTEGER, ALLOCATABLE, DIMENSION(:) :: indx1, indx2, indx3
    INTEGER, ALLOCATABLE :: data_out_i(:,:,:,:,:,:,:)
    REAL, ALLOCATABLE :: data_out_r(:,:,:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: data_out_d(:,:,:,:,:,:,:)
    INTEGER, DIMENSION(max_rank) :: inout, loc, dims_len, start, data_size
    INTEGER :: i1=1, i2=1, i3=1, i4=1, i5=1, i6=1, i7=1
    INTEGER :: ival, istat, ncid, ntimes, a_time_slice, i, irank, ierr
    REAL :: rval
    DOUBLE PRECISION :: dval

    REAL :: valin

    var(var_id)%users_type = 'real'
    irank = 3

    ierr = 0

    ! store input array's dimension lengths
    data_size = 0
    DO i =1 , irank
       data_size(i)=SIZE(data,i)
    END DO

    ierr = check_args(var_id, ncid, ntimes_passed, time_vals, time_bnds, &
         store_with, file_suffix)
    IF (ierr < 0) RETURN

    ! Process time stuff, define dims_len and start, and for use in the 
    !     write_1d case (only??)  also defines a_time_slice

    CALL cmor_process_time(ncid, var_id, ntimes_passed, dims_len, start, &
         a_time_slice, time_vals, time_bnds, store_with)


    ! besides checking, the following defines ntimes

    CALL check_time_and_dim_lens(data_size, var_id, dims_len, ntimes, ierr, &
         ntimes_passed)
    IF (ierr.LT.0) RETURN


    ! dim_order(i) stores the output array index that corresponds to 
    !   input array index i; it stores 0 for output dimensions > the
    !   rank of the output array.
    !   Singleton (scalar) dimensions that the user hasn't overridden are
    !   omitted.

    ! inout is similar to dim_order, but stores i for output dimensions > the
    !   rank of the output array.

    DO i=1,max_rank
       inout(i) = var(var_id)%dim_order(i)
       if (inout(i) == 0) inout(i) = i
    END DO

    ALLOCATE(indx1(dims_len(inout(1))), STAT=istat)
    ALLOCATE(indx2(dims_len(inout(2))), STAT=istat)
    ALLOCATE(indx3(dims_len(inout(3))), STAT=istat)

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*,  '   Unable to allocate memory for one of the "idx?" arrays,' 
       PRINT*,  '   which have one of the following dimensions: ', dims_len
       STOP
    END IF

    ! The following generates an index array, indx?(i), containing the location
    !    in the output array corresponding to input location i.
    ! indx1 is index for input array dimension 1, 
    !    indx2 for input dimension 2, etc.

    indx1 = offset_or_reverse_dim(var_id,1,dims_len(inout(1)))
    indx2 = offset_or_reverse_dim(var_id,2,dims_len(inout(2)))
    indx3 = offset_or_reverse_dim(var_id,3,dims_len(inout(3)))

    ! now assign input data to appropriate positions in output array.

    SELECT CASE(TRIM(var(var_id)%type))

    CASE("real")

       ALLOCATE(data_out_r(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("integer")

       ALLOCATE(data_out_i(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("double")

       ALLOCATE(data_out_d(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    END SELECT

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*, '   Unable to allocate memory for one of the "data_out",'
       PRINT*, '   which has the following dimensions: ', dims_len
       STOP
    END IF
    
    loc = 1
    
    DO i3 = 1, dims_len(inout(3))
       loc(inout(3)) = indx3(i3)
       DO i2 = 1, dims_len(inout(2))
          loc(inout(2)) = indx2(i2)
          DO i1 = 1, dims_len(inout(1))
             loc(inout(1)) = indx1(i1)

             valin = data(i1,i2,i3)

             SELECT CASE(TRIM(var(var_id)%type))

             CASE("real")

                IF ((var(var_id)%def_missing > 0) .and.  &
                     (ABS(valin-var(var_id)%users_missing) &
                     < ABS(var(var_id)%users_missing * &
                     var(var_id)%miss_val_tolerance))) THEN

                   i = var(var_id)%mip_table_idx
                   rval = mip_tbl(i)%single_missing

                ELSE

                   rval = var(var_id)%offset +       &
                        valin*var(var_id)%scale_factor

                   IF ((var(var_id)%valid_min /=     &
                        init_undef_single) .and. &
                        (rval < var(var_id)%valid_min)) THEN
                      call errmsg_r('min', irank, var_id, valin, &
                           rval, ival, dval, i1, i2, i3, i4, i5, &
                           i6, i7)
                   ELSE IF ((var(var_id)%valid_max /=    &
                        init_undef_single) .and. &
                        (rval > var(var_id)%valid_max)) THEN
                      call errmsg_r('max', irank, var_id, valin, &
                           rval, ival, dval, i1, i2, i3, i4, i5, &
                           i6, i7)
                   END IF

                END IF

                data_out_r(loc(1),loc(2),loc(3),loc(4),  &
                     loc(5),loc(6),loc(7)) = rval

             CASE("integer")

                IF ((var(var_id)%def_missing > 0) .and.  &
                     (ABS(valin-var(var_id)%users_missing) &
                     < ABS(var(var_id)%users_missing * &
                     var(var_id)%miss_val_tolerance))) THEN

                   i = var(var_id)%mip_table_idx
                   ival = mip_tbl(i)%integer_missing

                ELSE

                   ival = var(var_id)%offset +       &
                        valin*var(var_id)%scale_factor

                   IF ((var(var_id)%valid_min /=     &
                        init_undef_single) .and. &
                        (ival < var(var_id)%valid_min)) THEN
                      call errmsg_r('min', irank, var_id, valin, &
                           rval, ival, dval, i1, i2, i3, i4, i5, &
                           i6, i7)
                   ELSE IF ((var(var_id)%valid_max /=    &
                        init_undef_single) .and. &
                        (ival > var(var_id)%valid_max)) THEN
                      call errmsg_r('max', irank, var_id, valin, &
                           rval, ival, dval, i1, i2, i3, i4, i5, &
                           i6, i7)
                   END IF


                END IF

                data_out_i(loc(1),loc(2),loc(3),loc(4),  &
                     loc(5),loc(6),loc(7)) = ival

             CASE("double")

                IF ((var(var_id)%def_missing > 0) .and.  &
                     (ABS(valin-var(var_id)%users_missing) &
                     < ABS(var(var_id)%users_missing * &
                     var(var_id)%miss_val_tolerance))) THEN
                   i = var(var_id)%mip_table_idx
                   dval = mip_tbl(i)%integer_missing
                ELSE

                   dval = var(var_id)%offset +       &
                        valin*var(var_id)%scale_factor

                   IF ((var(var_id)%valid_min /=     &
                        init_undef_single) .and. &
                        (dval < var(var_id)%valid_min)) THEN
                      call errmsg_r('min', irank, var_id, valin, &
                           rval, ival, dval, i1, i2, i3, i4, i5, &
                           i6, i7)
                   ELSE IF ((var(var_id)%valid_max /=    &
                        init_undef_single) .and. &
                        (dval > var(var_id)%valid_max)) THEN
                      call errmsg_r('max', irank, var_id, valin, &
                           rval, ival, dval, i1, i2, i3, i4, i5, &
                           i6, i7)
                   END IF

                END IF

                data_out_d(loc(1),loc(2),loc(3),loc(4),  &
                     loc(5),loc(6),loc(7)) = dval

             END SELECT

          END DO
       END DO
    END DO

    SELECT CASE(TRIM(var(var_id)%type))

    CASE('real')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_r, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_r)
       
    CASE('integer')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_i, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_i)
       
    CASE('double')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_d, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_d)
       
    END SELECT
           
    DEALLOCATE(indx1)
    DEALLOCATE(indx2)
    DEALLOCATE(indx3)
    
    IF (ierr == NF90_NOERR) THEN
       IF (ntimes /= 0) THEN
          IF (PRESENT(store_with)) THEN
             var(var_id)%zfac_times_written(store_with) =   &
                  var(var_id)%zfac_times_written(store_with)+ntimes
          ELSE
             var(var_id)%times_written=var(var_id)%times_written+ntimes
          END IF
       END IF
    ELSE
       PRINT*, 'Error writing NetCDF out data: error number =  ', ierr
       PRINT*, '   netCDF error message: ',  TRIM(nf90_strerror(ierr))
       STOP
    END IF
    
    RETURN
  END FUNCTION cmor_write_3d_r



  FUNCTION cmor_write_4d_r(var_id, data, file_suffix, ntimes_passed,  &
       time_vals, time_bnds, store_with) RESULT(ierr)

    IMPLICIT NONE

    REAL, INTENT(in):: data(:,:,:,:)
    INTEGER, INTENT(in) ::var_id
    CHARACTER(len=*), INTENT(in), OPTIONAL :: file_suffix
    INTEGER, INTENT(in), OPTIONAL :: ntimes_passed
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_vals(:)
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_bnds(:,:)
    INTEGER, INTENT(in), OPTIONAL :: store_with

    INTEGER, ALLOCATABLE, DIMENSION(:) :: indx1, indx2, indx3, indx4
    INTEGER, ALLOCATABLE :: data_out_i(:,:,:,:,:,:,:)
    REAL, ALLOCATABLE :: data_out_r(:,:,:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: data_out_d(:,:,:,:,:,:,:)
    INTEGER, DIMENSION(max_rank) :: inout, loc, dims_len, start, data_size
    INTEGER :: i1=1, i2=1, i3=1, i4=1, i5=1, i6=1, i7=1
    INTEGER :: ival, istat, ncid, ntimes, a_time_slice, i, irank, ierr
    REAL :: rval
    DOUBLE PRECISION :: dval

    REAL :: valin

    var(var_id)%users_type = 'real'
    irank = 4

    ierr = 0

    ! store input array's dimension lengths
    data_size = 0
    DO i =1 , irank
       data_size(i)=SIZE(data,i)
    END DO

    ierr = check_args(var_id, ncid, ntimes_passed, time_vals, time_bnds, &
         store_with, file_suffix)
    IF (ierr < 0) RETURN

    ! Process time stuff, define dims_len and start, and for use in the 
    !     write_1d case (only??)  also defines a_time_slice

    CALL cmor_process_time(ncid, var_id, ntimes_passed, dims_len, start, &
         a_time_slice, time_vals, time_bnds, store_with)


    ! besides checking, the following defines ntimes

    CALL check_time_and_dim_lens(data_size, var_id, dims_len, ntimes, ierr, &
         ntimes_passed)
    IF (ierr.LT.0) RETURN


    ! dim_order(i) stores the output array index that corresponds to 
    !   input array index i; it stores 0 for output dimensions > the
    !   rank of the output array.
    !   Singleton (scalar) dimensions that the user hasn't overridden are
    !   omitted.

    ! inout is similar to dim_order, but stores i for output dimensions > the
    !   rank of the output array.

    DO i=1,max_rank
       inout(i) = var(var_id)%dim_order(i)
       if (inout(i) == 0) inout(i) = i
    END DO

    ALLOCATE(indx1(dims_len(inout(1))), STAT=istat)
    ALLOCATE(indx2(dims_len(inout(2))), STAT=istat)
    ALLOCATE(indx3(dims_len(inout(3))), STAT=istat)
    ALLOCATE(indx4(dims_len(inout(4))), STAT=istat)

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*,  '   Unable to allocate memory for one of the "idx?" arrays,' 
       PRINT*,  '   which have one of the following dimensions: ', dims_len
       STOP
    END IF

    ! The following generates an index array, indx?(i), containing the location
    !    in the output array corresponding to input location i.
    ! indx1 is index for input array dimension 1, 
    !    indx2 for input dimension 2, etc.

    indx1 = offset_or_reverse_dim(var_id,1,dims_len(inout(1)))
    indx2 = offset_or_reverse_dim(var_id,2,dims_len(inout(2)))
    indx3 = offset_or_reverse_dim(var_id,3,dims_len(inout(3)))
    indx4 = offset_or_reverse_dim(var_id,4,dims_len(inout(4)))

    ! now assign input data to appropriate positions in output array.

    SELECT CASE(TRIM(var(var_id)%type))

    CASE("real")

       ALLOCATE(data_out_r(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("integer")

       ALLOCATE(data_out_i(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("double")

       ALLOCATE(data_out_d(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    END SELECT

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*, '   Unable to allocate memory for one of the "data_out",'
       PRINT*, '   which has the following dimensions: ', dims_len
       STOP
    END IF
    
    loc = 1
    
    DO i4 = 1, dims_len(inout(4))
       loc(inout(4)) = indx4(i4)
       DO i3 = 1, dims_len(inout(3))
          loc(inout(3)) = indx3(i3)
          DO i2 = 1, dims_len(inout(2))
             loc(inout(2)) = indx2(i2)
             DO i1 = 1, dims_len(inout(1))
                loc(inout(1)) = indx1(i1)

                valin = data(i1,i2,i3,i4)

                SELECT CASE(TRIM(var(var_id)%type))

                CASE("real")

                   IF ((var(var_id)%def_missing > 0) .and.  &
                        (ABS(valin-var(var_id)%users_missing) &
                        < ABS(var(var_id)%users_missing * &
                        var(var_id)%miss_val_tolerance))) THEN

                      i = var(var_id)%mip_table_idx
                      rval = mip_tbl(i)%single_missing

                   ELSE

                      rval = var(var_id)%offset +       &
                           valin*var(var_id)%scale_factor

                      IF ((var(var_id)%valid_min /=     &
                           init_undef_single) .and. &
                           (rval < var(var_id)%valid_min)) THEN
                         call errmsg_r('min', irank, var_id, valin, &
                              rval, ival, dval, i1, i2, i3, i4, i5, &
                              i6, i7)
                      ELSE IF ((var(var_id)%valid_max /=    &
                           init_undef_single) .and. &
                           (rval > var(var_id)%valid_max)) THEN
                         call errmsg_r('max', irank, var_id, valin, &
                              rval, ival, dval, i1, i2, i3, i4, i5, &
                              i6, i7)
                      END IF

                   END IF

                   data_out_r(loc(1),loc(2),loc(3),loc(4),  &
                        loc(5),loc(6),loc(7)) = rval

                CASE("integer")

                   IF ((var(var_id)%def_missing > 0) .and.  &
                        (ABS(valin-var(var_id)%users_missing) &
                        < ABS(var(var_id)%users_missing * &
                        var(var_id)%miss_val_tolerance))) THEN

                      i = var(var_id)%mip_table_idx
                      ival = mip_tbl(i)%integer_missing

                   ELSE

                      ival = var(var_id)%offset +       &
                           valin*var(var_id)%scale_factor

                      IF ((var(var_id)%valid_min /=     &
                           init_undef_single) .and. &
                           (ival < var(var_id)%valid_min)) THEN
                         call errmsg_r('min', irank, var_id, valin, &
                              rval, ival, dval, i1, i2, i3, i4, i5, &
                              i6, i7)
                      ELSE IF ((var(var_id)%valid_max /=    &
                           init_undef_single) .and. &
                           (ival > var(var_id)%valid_max)) THEN
                         call errmsg_r('max', irank, var_id, valin, &
                              rval, ival, dval, i1, i2, i3, i4, i5, &
                              i6, i7)
                      END IF


                   END IF

                   data_out_i(loc(1),loc(2),loc(3),loc(4),  &
                        loc(5),loc(6),loc(7)) = ival

                CASE("double")

                   IF ((var(var_id)%def_missing > 0) .and.  &
                        (ABS(valin-var(var_id)%users_missing) &
                        < ABS(var(var_id)%users_missing * &
                        var(var_id)%miss_val_tolerance))) THEN
                      i = var(var_id)%mip_table_idx
                      dval = mip_tbl(i)%integer_missing
                   ELSE

                      dval = var(var_id)%offset +       &
                           valin*var(var_id)%scale_factor

                      IF ((var(var_id)%valid_min /=     &
                           init_undef_single) .and. &
                           (dval < var(var_id)%valid_min)) THEN
                         call errmsg_r('min', irank, var_id, valin, &
                              rval, ival, dval, i1, i2, i3, i4, i5, &
                              i6, i7)
                      ELSE IF ((var(var_id)%valid_max /=    &
                           init_undef_single) .and. &
                           (dval > var(var_id)%valid_max)) THEN
                         call errmsg_r('max', irank, var_id, valin, &
                              rval, ival, dval, i1, i2, i3, i4, i5, &
                              i6, i7)
                      END IF

                   END IF

                   data_out_d(loc(1),loc(2),loc(3),loc(4),  &
                        loc(5),loc(6),loc(7)) = dval

                END SELECT

             END DO
          END DO
       END DO
    END DO

    SELECT CASE(TRIM(var(var_id)%type))

    CASE('real')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_r, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_r)
       
    CASE('integer')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_i, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_i)
       
    CASE('double')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_d, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_d)
       
    END SELECT
           
    DEALLOCATE(indx1)
    DEALLOCATE(indx2)
    DEALLOCATE(indx3)
    DEALLOCATE(indx4)
    
    IF (ierr == NF90_NOERR) THEN
       IF (ntimes /= 0) THEN
          IF (PRESENT(store_with)) THEN
             var(var_id)%zfac_times_written(store_with) =   &
                  var(var_id)%zfac_times_written(store_with)+ntimes
          ELSE
             var(var_id)%times_written=var(var_id)%times_written+ntimes
          END IF
       END IF
    ELSE
       PRINT*, 'Error writing NetCDF out data: error number =  ', ierr
       PRINT*, '   netCDF error message: ',  TRIM(nf90_strerror(ierr))
       STOP
    END IF
    
    RETURN
  END FUNCTION cmor_write_4d_r



 


  FUNCTION cmor_write_5d_r(var_id, data, file_suffix, ntimes_passed,  &
       time_vals, time_bnds, store_with) RESULT(ierr)

    IMPLICIT NONE

    REAL, INTENT(in):: data(:,:,:,:,:)
    INTEGER, INTENT(in) ::var_id
    CHARACTER(len=*), INTENT(in), OPTIONAL :: file_suffix
    INTEGER, INTENT(in), OPTIONAL :: ntimes_passed
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_vals(:)
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_bnds(:,:)
    INTEGER, INTENT(in), OPTIONAL :: store_with

    INTEGER, ALLOCATABLE, DIMENSION(:) :: indx1, indx2, indx3, indx4, &
         indx5
    INTEGER, ALLOCATABLE :: data_out_i(:,:,:,:,:,:,:)
    REAL, ALLOCATABLE :: data_out_r(:,:,:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: data_out_d(:,:,:,:,:,:,:)
    INTEGER, DIMENSION(max_rank) :: inout, loc, dims_len, start, data_size
    INTEGER :: i1=1, i2=1, i3=1, i4=1, i5=1, i6=1, i7=1
    INTEGER :: ival, istat, ncid, ntimes, a_time_slice, i, irank, ierr
    REAL :: rval
    DOUBLE PRECISION :: dval

    REAL :: valin

    var(var_id)%users_type = 'real'
    irank = 5

    ierr = 0

    ! store input array's dimension lengths
    data_size = 0
    DO i =1 , irank
       data_size(i)=SIZE(data,i)
    END DO

    ierr = check_args(var_id, ncid, ntimes_passed, time_vals, time_bnds, &
         store_with, file_suffix)
    IF (ierr < 0) RETURN

    ! Process time stuff, define dims_len and start, and for use in the 
    !     write_1d case (only??)  also defines a_time_slice

    CALL cmor_process_time(ncid, var_id, ntimes_passed, dims_len, start, &
         a_time_slice, time_vals, time_bnds, store_with)


    ! besides checking, the following defines ntimes

    CALL check_time_and_dim_lens(data_size, var_id, dims_len, ntimes, ierr, &
         ntimes_passed)
    IF (ierr.LT.0) RETURN


    ! dim_order(i) stores the output array index that corresponds to 
    !   input array index i; it stores 0 for output dimensions > the
    !   rank of the output array.
    !   Singleton (scalar) dimensions that the user hasn't overridden are
    !   omitted.

    ! inout is similar to dim_order, but stores i for output dimensions > the
    !   rank of the output array.

    DO i=1,max_rank
       inout(i) = var(var_id)%dim_order(i)
       if (inout(i) == 0) inout(i) = i
    END DO

    ALLOCATE(indx1(dims_len(inout(1))), STAT=istat)
    ALLOCATE(indx2(dims_len(inout(2))), STAT=istat)
    ALLOCATE(indx3(dims_len(inout(3))), STAT=istat)
    ALLOCATE(indx4(dims_len(inout(4))), STAT=istat)
    ALLOCATE(indx5(dims_len(inout(5))), STAT=istat)

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*,  '   Unable to allocate memory for one of the "idx?" arrays,' 
       PRINT*,  '   which have one of the following dimensions: ', dims_len
       STOP
    END IF

    ! The following generates an index array, indx?(i), containing the location
    !    in the output array corresponding to input location i.
    ! indx1 is index for input array dimension 1, 
    !    indx2 for input dimension 2, etc.

    indx1 = offset_or_reverse_dim(var_id,1,dims_len(inout(1)))
    indx2 = offset_or_reverse_dim(var_id,2,dims_len(inout(2)))
    indx3 = offset_or_reverse_dim(var_id,3,dims_len(inout(3)))
    indx4 = offset_or_reverse_dim(var_id,4,dims_len(inout(4)))
    indx5 = offset_or_reverse_dim(var_id,5,dims_len(inout(5)))

    ! now assign input data to appropriate positions in output array.

    SELECT CASE(TRIM(var(var_id)%type))

    CASE("real")

       ALLOCATE(data_out_r(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("integer")

       ALLOCATE(data_out_i(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("double")

       ALLOCATE(data_out_d(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    END SELECT

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*, '   Unable to allocate memory for one of the "data_out",'
       PRINT*, '   which has the following dimensions: ', dims_len
       STOP
    END IF
    
    loc = 1
    
    DO i5 = 1, dims_len(inout(5))
       loc(inout(5)) = indx5(i5)
       DO i4 = 1, dims_len(inout(4))
          loc(inout(4)) = indx4(i4)
          DO i3 = 1, dims_len(inout(3))
             loc(inout(3)) = indx3(i3)
             DO i2 = 1, dims_len(inout(2))
                loc(inout(2)) = indx2(i2)
                DO i1 = 1, dims_len(inout(1))
                   loc(inout(1)) = indx1(i1)

                   valin = data(i1,i2,i3,i4,i5)

                   SELECT CASE(TRIM(var(var_id)%type))

                   CASE("real")

                      IF ((var(var_id)%def_missing > 0) .and.  &
                           (ABS(valin-var(var_id)%users_missing) &
                           < ABS(var(var_id)%users_missing * &
                           var(var_id)%miss_val_tolerance))) THEN

                         i = var(var_id)%mip_table_idx
                         rval = mip_tbl(i)%single_missing

                      ELSE

                         rval = var(var_id)%offset +       &
                              valin*var(var_id)%scale_factor

                         IF ((var(var_id)%valid_min /=     &
                              init_undef_single) .and. &
                              (rval < var(var_id)%valid_min)) THEN
                            call errmsg_r('min', irank, var_id, valin, &
                                 rval, ival, dval, i1, i2, i3, i4, i5, &
                                 i6, i7)
                         ELSE IF ((var(var_id)%valid_max /=    &
                              init_undef_single) .and. &
                              (rval > var(var_id)%valid_max)) THEN
                            call errmsg_r('max', irank, var_id, valin, &
                                 rval, ival, dval, i1, i2, i3, i4, i5, &
                                 i6, i7)
                         END IF

                      END IF

                      data_out_r(loc(1),loc(2),loc(3),loc(4),  &
                           loc(5),loc(6),loc(7)) = rval

                   CASE("integer")

                      IF ((var(var_id)%def_missing > 0) .and.  &
                           (ABS(valin-var(var_id)%users_missing) &
                           < ABS(var(var_id)%users_missing * &
                           var(var_id)%miss_val_tolerance))) THEN

                         i = var(var_id)%mip_table_idx
                         ival = mip_tbl(i)%integer_missing

                      ELSE

                         ival = var(var_id)%offset +       &
                              valin*var(var_id)%scale_factor

                         IF ((var(var_id)%valid_min /=     &
                              init_undef_single) .and. &
                              (ival < var(var_id)%valid_min)) THEN
                            call errmsg_r('min', irank, var_id, valin, &
                                 rval, ival, dval, i1, i2, i3, i4, i5, &
                                 i6, i7)
                         ELSE IF ((var(var_id)%valid_max /=    &
                              init_undef_single) .and. &
                              (ival > var(var_id)%valid_max)) THEN
                            call errmsg_r('max', irank, var_id, valin, &
                                 rval, ival, dval, i1, i2, i3, i4, i5, &
                                 i6, i7)
                         END IF


                      END IF

                      data_out_i(loc(1),loc(2),loc(3),loc(4),  &
                           loc(5),loc(6),loc(7)) = ival

                   CASE("double")

                      IF ((var(var_id)%def_missing > 0) .and.  &
                           (ABS(valin-var(var_id)%users_missing) &
                           < ABS(var(var_id)%users_missing * &
                           var(var_id)%miss_val_tolerance))) THEN
                         i = var(var_id)%mip_table_idx
                         dval = mip_tbl(i)%integer_missing
                      ELSE

                         dval = var(var_id)%offset +       &
                              valin*var(var_id)%scale_factor

                         IF ((var(var_id)%valid_min /=     &
                              init_undef_single) .and. &
                              (dval < var(var_id)%valid_min)) THEN
                            call errmsg_r('min', irank, var_id, valin, &
                                 rval, ival, dval, i1, i2, i3, i4, i5, &
                                 i6, i7)
                         ELSE IF ((var(var_id)%valid_max /=    &
                              init_undef_single) .and. &
                              (dval > var(var_id)%valid_max)) THEN
                            call errmsg_r('max', irank, var_id, valin, &
                                 rval, ival, dval, i1, i2, i3, i4, i5, &
                                 i6, i7)
                         END IF

                      END IF

                      data_out_d(loc(1),loc(2),loc(3),loc(4),  &
                           loc(5),loc(6),loc(7)) = dval

                   END SELECT

                END DO
             END DO
          END DO
       END DO
    END DO

    SELECT CASE(TRIM(var(var_id)%type))

    CASE('real')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_r, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_r)
       
    CASE('integer')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_i, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_i)
       
    CASE('double')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_d, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_d)
       
    END SELECT
           
    DEALLOCATE(indx1)
    DEALLOCATE(indx2)
    DEALLOCATE(indx3)
    DEALLOCATE(indx4)
    DEALLOCATE(indx5)
        
    IF (ierr == NF90_NOERR) THEN
       IF (ntimes /= 0) THEN
          IF (PRESENT(store_with)) THEN
             var(var_id)%zfac_times_written(store_with) =   &
                  var(var_id)%zfac_times_written(store_with)+ntimes
          ELSE
             var(var_id)%times_written=var(var_id)%times_written+ntimes
          END IF
       END IF
    ELSE
       PRINT*, 'Error writing NetCDF out data: error number =  ', ierr
       PRINT*, '   netCDF error message: ',  TRIM(nf90_strerror(ierr))
       STOP
    END IF
    
    RETURN
  END FUNCTION cmor_write_5d_r




  FUNCTION cmor_write_6d_r(var_id, data, file_suffix, ntimes_passed,  &
       time_vals, time_bnds, store_with) RESULT(ierr)

    IMPLICIT NONE

    REAL, INTENT(in):: data(:,:,:,:,:,:)
    INTEGER, INTENT(in) ::var_id
    CHARACTER(len=*), INTENT(in), OPTIONAL :: file_suffix
    INTEGER, INTENT(in), OPTIONAL :: ntimes_passed
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_vals(:)
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_bnds(:,:)
    INTEGER, INTENT(in), OPTIONAL :: store_with

    INTEGER, ALLOCATABLE, DIMENSION(:) :: indx1, indx2, indx3, indx4, &
         indx5, indx6
    INTEGER, ALLOCATABLE :: data_out_i(:,:,:,:,:,:,:)
    REAL, ALLOCATABLE :: data_out_r(:,:,:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: data_out_d(:,:,:,:,:,:,:)
    INTEGER, DIMENSION(max_rank) :: inout, loc, dims_len, start, data_size
    INTEGER :: i1=1, i2=1, i3=1, i4=1, i5=1, i6=1, i7=1
    INTEGER :: ival, istat, ncid, ntimes, a_time_slice, i, irank, ierr
    REAL :: rval
    DOUBLE PRECISION :: dval

    REAL :: valin

    var(var_id)%users_type = 'real'
    irank = 6

    ierr = 0

    ! store input array's dimension lengths
    data_size = 0
    DO i =1 , irank
       data_size(i)=SIZE(data,i)
    END DO

    ierr = check_args(var_id, ncid, ntimes_passed, time_vals, time_bnds, &
         store_with, file_suffix)
    IF (ierr < 0) RETURN

    ! Process time stuff, define dims_len and start, and for use in the 
    !     write_1d case (only??)  also defines a_time_slice

    CALL cmor_process_time(ncid, var_id, ntimes_passed, dims_len, start, &
         a_time_slice, time_vals, time_bnds, store_with)


    ! besides checking, the following defines ntimes

    CALL check_time_and_dim_lens(data_size, var_id, dims_len, ntimes, ierr, &
         ntimes_passed)
    IF (ierr.LT.0) RETURN


    ! dim_order(i) stores the output array index that corresponds to 
    !   input array index i; it stores 0 for output dimensions > the
    !   rank of the output array.
    !   Singleton (scalar) dimensions that the user hasn't overridden are
    !   omitted.

    ! inout is similar to dim_order, but stores i for output dimensions > the
    !   rank of the output array.

    DO i=1,max_rank
       inout(i) = var(var_id)%dim_order(i)
       if (inout(i) == 0) inout(i) = i
    END DO

    ALLOCATE(indx1(dims_len(inout(1))), STAT=istat)
    ALLOCATE(indx2(dims_len(inout(2))), STAT=istat)
    ALLOCATE(indx3(dims_len(inout(3))), STAT=istat)
    ALLOCATE(indx4(dims_len(inout(4))), STAT=istat)
    ALLOCATE(indx5(dims_len(inout(5))), STAT=istat)
    ALLOCATE(indx6(dims_len(inout(6))), STAT=istat)

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*,  '   Unable to allocate memory for one of the "idx?" arrays,' 
       PRINT*,  '   which have one of the following dimensions: ', dims_len
       STOP
    END IF

    ! The following generates an index array, indx?(i), containing the location
    !    in the output array corresponding to input location i.
    ! indx1 is index for input array dimension 1, 
    !    indx2 for input dimension 2, etc.

    indx1 = offset_or_reverse_dim(var_id,1,dims_len(inout(1)))
    indx2 = offset_or_reverse_dim(var_id,2,dims_len(inout(2)))
    indx3 = offset_or_reverse_dim(var_id,3,dims_len(inout(3)))
    indx4 = offset_or_reverse_dim(var_id,4,dims_len(inout(4)))
    indx5 = offset_or_reverse_dim(var_id,5,dims_len(inout(5)))
    indx6 = offset_or_reverse_dim(var_id,6,dims_len(inout(6)))

    ! now assign input data to appropriate positions in output array.

    SELECT CASE(TRIM(var(var_id)%type))

    CASE("real")

       ALLOCATE(data_out_r(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("integer")

       ALLOCATE(data_out_i(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("double")

       ALLOCATE(data_out_d(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    END SELECT

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*, '   Unable to allocate memory for one of the "data_out",'
       PRINT*, '   which has the following dimensions: ', dims_len
       STOP
    END IF
    
    loc = 1
    
    DO i6 = 1, dims_len(inout(6))
       loc(inout(6)) = indx6(i6)
       DO i5 = 1, dims_len(inout(5))
          loc(inout(5)) = indx5(i5)
          DO i4 = 1, dims_len(inout(4))
             loc(inout(4)) = indx4(i4)
             DO i3 = 1, dims_len(inout(3))
                loc(inout(3)) = indx3(i3)
                DO i2 = 1, dims_len(inout(2))
                   loc(inout(2)) = indx2(i2)
                   DO i1 = 1, dims_len(inout(1))
                      loc(inout(1)) = indx1(i1)

                      valin = data(i1,i2,i3,i4,i5,i6)

                      SELECT CASE(TRIM(var(var_id)%type))

                      CASE("real")

                         IF ((var(var_id)%def_missing > 0) .and.  &
                              (ABS(valin-var(var_id)%users_missing) &
                              < ABS(var(var_id)%users_missing * &
                              var(var_id)%miss_val_tolerance))) THEN

                            i = var(var_id)%mip_table_idx
                            rval = mip_tbl(i)%single_missing

                         ELSE

                            rval = var(var_id)%offset +       &
                                 valin*var(var_id)%scale_factor

                            IF ((var(var_id)%valid_min /=     &
                                 init_undef_single) .and. &
                                 (rval < var(var_id)%valid_min)) THEN
                               call errmsg_r('min', irank, var_id, valin, &
                                    rval, ival, dval, i1, i2, i3, i4, i5, &
                                    i6, i7)
                            ELSE IF ((var(var_id)%valid_max /=    &
                                 init_undef_single) .and. &
                                 (rval > var(var_id)%valid_max)) THEN
                               call errmsg_r('max', irank, var_id, valin, &
                                    rval, ival, dval, i1, i2, i3, i4, i5, &
                                    i6, i7)
                            END IF

                         END IF

                         data_out_r(loc(1),loc(2),loc(3),loc(4),  &
                              loc(5),loc(6),loc(7)) = rval

                      CASE("integer")

                         IF ((var(var_id)%def_missing > 0) .and.  &
                              (ABS(valin-var(var_id)%users_missing) &
                              < ABS(var(var_id)%users_missing * &
                              var(var_id)%miss_val_tolerance))) THEN

                            i = var(var_id)%mip_table_idx
                            ival = mip_tbl(i)%integer_missing

                         ELSE

                            ival = var(var_id)%offset +       &
                                 valin*var(var_id)%scale_factor

                            IF ((var(var_id)%valid_min /=     &
                                 init_undef_single) .and. &
                                 (ival < var(var_id)%valid_min)) THEN
                               call errmsg_r('min', irank, var_id, valin, &
                                    rval, ival, dval, i1, i2, i3, i4, i5, &
                                    i6, i7)
                            ELSE IF ((var(var_id)%valid_max /=    &
                                 init_undef_single) .and. &
                                 (ival > var(var_id)%valid_max)) THEN
                               call errmsg_r('max', irank, var_id, valin, &
                                    rval, ival, dval, i1, i2, i3, i4, i5, &
                                    i6, i7)
                            END IF


                         END IF

                         data_out_i(loc(1),loc(2),loc(3),loc(4),  &
                              loc(5),loc(6),loc(7)) = ival

                      CASE("double")

                         IF ((var(var_id)%def_missing > 0) .and.  &
                              (ABS(valin-var(var_id)%users_missing) &
                              < ABS(var(var_id)%users_missing * &
                              var(var_id)%miss_val_tolerance))) THEN
                            i = var(var_id)%mip_table_idx
                            dval = mip_tbl(i)%integer_missing
                         ELSE

                            dval = var(var_id)%offset +       &
                                 valin*var(var_id)%scale_factor

                            IF ((var(var_id)%valid_min /=     &
                                 init_undef_single) .and. &
                                 (dval < var(var_id)%valid_min)) THEN
                               call errmsg_r('min', irank, var_id, valin, &
                                    rval, ival, dval, i1, i2, i3, i4, i5, &
                                    i6, i7)
                            ELSE IF ((var(var_id)%valid_max /=    &
                                 init_undef_single) .and. &
                                 (dval > var(var_id)%valid_max)) THEN
                               call errmsg_r('max', irank, var_id, valin, &
                                    rval, ival, dval, i1, i2, i3, i4, i5, &
                                    i6, i7)
                            END IF

                         END IF

                         data_out_d(loc(1),loc(2),loc(3),loc(4),  &
                              loc(5),loc(6),loc(7)) = dval

                      END SELECT

                   END DO
                END DO
             END DO
          END DO
       END DO
    END DO

    SELECT CASE(TRIM(var(var_id)%type))

    CASE('real')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_r, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_r)
       
    CASE('integer')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_i, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_i)
       
    CASE('double')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_d, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_d)
       
    END SELECT
           
    DEALLOCATE(indx1)
    DEALLOCATE(indx2)
    DEALLOCATE(indx3)
    DEALLOCATE(indx4)
    DEALLOCATE(indx5)
    DEALLOCATE(indx6)
        
    IF (ierr == NF90_NOERR) THEN
       IF (ntimes /= 0) THEN
          IF (PRESENT(store_with)) THEN
             var(var_id)%zfac_times_written(store_with) =   &
                  var(var_id)%zfac_times_written(store_with)+ntimes
          ELSE
             var(var_id)%times_written=var(var_id)%times_written+ntimes
          END IF
       END IF
    ELSE
       PRINT*, 'Error writing NetCDF out data: error number =  ', ierr
       PRINT*, '   netCDF error message: ',  TRIM(nf90_strerror(ierr))
       STOP
    END IF
    
    RETURN
  END FUNCTION cmor_write_6d_r




  FUNCTION cmor_write_7d_r(var_id, data, file_suffix, ntimes_passed,  &
       time_vals, time_bnds, store_with) RESULT(ierr)

    IMPLICIT NONE

    REAL, INTENT(in):: data(:,:,:,:,:,:,:)
    INTEGER, INTENT(in) ::var_id
    CHARACTER(len=*), INTENT(in), OPTIONAL :: file_suffix
    INTEGER, INTENT(in), OPTIONAL :: ntimes_passed
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_vals(:)
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_bnds(:,:)
    INTEGER, INTENT(in), OPTIONAL :: store_with

    INTEGER, ALLOCATABLE, DIMENSION(:) :: indx1, indx2, indx3, indx4, &
         indx5, indx6, indx7
    INTEGER, ALLOCATABLE :: data_out_i(:,:,:,:,:,:,:)
    REAL, ALLOCATABLE :: data_out_r(:,:,:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: data_out_d(:,:,:,:,:,:,:)
    INTEGER, DIMENSION(max_rank) :: inout, loc, dims_len, start, data_size
    INTEGER :: i1=1, i2=1, i3=1, i4=1, i5=1, i6=1, i7=1
    INTEGER :: ival, istat, ncid, ntimes, a_time_slice, i, irank, ierr
    REAL :: rval
    DOUBLE PRECISION :: dval

    REAL :: valin

    var(var_id)%users_type = 'real'
    irank = 7

    ierr = 0

    ! store input array's dimension lengths
    data_size = 0
    DO i =1 , irank
       data_size(i)=SIZE(data,i)
    END DO

    ierr = check_args(var_id, ncid, ntimes_passed, time_vals, time_bnds, &
         store_with, file_suffix)
    IF (ierr < 0) RETURN

    ! Process time stuff, define dims_len and start, and for use in the 
    !     write_1d case (only??)  also defines a_time_slice

    CALL cmor_process_time(ncid, var_id, ntimes_passed, dims_len, start, &
         a_time_slice, time_vals, time_bnds, store_with)


    ! besides checking, the following defines ntimes

    CALL check_time_and_dim_lens(data_size, var_id, dims_len, ntimes, ierr, &
         ntimes_passed)
    IF (ierr.LT.0) RETURN


    ! dim_order(i) stores the output array index that corresponds to 
    !   input array index i; it stores 0 for output dimensions > the
    !   rank of the output array.
    !   Singleton (scalar) dimensions that the user hasn't overridden are
    !   omitted.

    ! inout is similar to dim_order, but stores i for output dimensions > the
    !   rank of the output array.

    DO i=1,max_rank
       inout(i) = var(var_id)%dim_order(i)
       if (inout(i) == 0) inout(i) = i
    END DO

    ALLOCATE(indx1(dims_len(inout(1))), STAT=istat)
    ALLOCATE(indx2(dims_len(inout(2))), STAT=istat)
    ALLOCATE(indx3(dims_len(inout(3))), STAT=istat)
    ALLOCATE(indx4(dims_len(inout(4))), STAT=istat)
    ALLOCATE(indx5(dims_len(inout(5))), STAT=istat)
    ALLOCATE(indx6(dims_len(inout(6))), STAT=istat)
    ALLOCATE(indx7(dims_len(inout(7))), STAT=istat)

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*,  '   Unable to allocate memory for one of the "idx?" arrays,' 
       PRINT*,  '   which have one of the following dimensions: ', dims_len
       STOP
    END IF

    ! The following generates an index array, indx?(i), containing the location
    !    in the output array corresponding to input location i.
    ! indx1 is index for input array dimension 1, 
    !    indx2 for input dimension 2, etc.

    indx1 = offset_or_reverse_dim(var_id,1,dims_len(inout(1)))
    indx2 = offset_or_reverse_dim(var_id,2,dims_len(inout(2)))
    indx3 = offset_or_reverse_dim(var_id,3,dims_len(inout(3)))
    indx4 = offset_or_reverse_dim(var_id,4,dims_len(inout(4)))
    indx5 = offset_or_reverse_dim(var_id,5,dims_len(inout(5)))
    indx6 = offset_or_reverse_dim(var_id,6,dims_len(inout(6)))
    indx7 = offset_or_reverse_dim(var_id,7,dims_len(inout(7)))

    ! now assign input data to appropriate positions in output array.

    SELECT CASE(TRIM(var(var_id)%type))

    CASE("real")

       ALLOCATE(data_out_r(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("integer")

       ALLOCATE(data_out_i(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("double")

       ALLOCATE(data_out_d(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    END SELECT

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*, '   Unable to allocate memory for one of the "data_out",'
       PRINT*, '   which has the following dimensions: ', dims_len
       STOP
    END IF
    
    loc = 1
    
    DO i7 = 1, dims_len(inout(7))
       loc(inout(7)) = indx7(i7)
       DO i6 = 1, dims_len(inout(6))
          loc(inout(6)) = indx6(i6)
          DO i5 = 1, dims_len(inout(5))
             loc(inout(5)) = indx5(i5)
             DO i4 = 1, dims_len(inout(4))
                loc(inout(4)) = indx4(i4)
                DO i3 = 1, dims_len(inout(3))
                   loc(inout(3)) = indx3(i3)
                   DO i2 = 1, dims_len(inout(2))
                      loc(inout(2)) = indx2(i2)
                      DO i1 = 1, dims_len(inout(1))
                         loc(inout(1)) = indx1(i1)
                         
                         valin = data(i1,i2,i3,i4,i5,i6,i7)
                         
                         SELECT CASE(TRIM(var(var_id)%type))

                         CASE("real")
                            
                            IF ((var(var_id)%def_missing > 0) .and.  &
                                 (ABS(valin-var(var_id)%users_missing) &
                                 < ABS(var(var_id)%users_missing * &
                                 var(var_id)%miss_val_tolerance))) THEN

                               i = var(var_id)%mip_table_idx
                               rval = mip_tbl(i)%single_missing

                            ELSE
                               
                               rval = var(var_id)%offset +       &
                                    valin*var(var_id)%scale_factor
                               
                               IF ((var(var_id)%valid_min /=     &
                                    init_undef_single) .and. &
                                    (rval < var(var_id)%valid_min)) THEN
                                  call errmsg_r('min', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               ELSE IF ((var(var_id)%valid_max /=    &
                                    init_undef_single) .and. &
                                    (rval > var(var_id)%valid_max)) THEN
                                  call errmsg_r('max', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               END IF
                               
                            END IF
                            
                            data_out_r(loc(1),loc(2),loc(3),loc(4),  &
                                 loc(5),loc(6),loc(7)) = rval

                         CASE("integer")
                            
                            IF ((var(var_id)%def_missing > 0) .and.  &
                                 (ABS(valin-var(var_id)%users_missing) &
                                 < ABS(var(var_id)%users_missing * &
                                 var(var_id)%miss_val_tolerance))) THEN

                               i = var(var_id)%mip_table_idx
                               ival = mip_tbl(i)%integer_missing

                            ELSE
                               
                               ival = var(var_id)%offset +       &
                                    valin*var(var_id)%scale_factor
                               
                               IF ((var(var_id)%valid_min /=     &
                                    init_undef_single) .and. &
                                    (ival < var(var_id)%valid_min)) THEN
                                  call errmsg_r('min', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               ELSE IF ((var(var_id)%valid_max /=    &
                                    init_undef_single) .and. &
                                    (ival > var(var_id)%valid_max)) THEN
                                  call errmsg_r('max', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               END IF

                               
                            END IF
                            
                            data_out_i(loc(1),loc(2),loc(3),loc(4),  &
                                 loc(5),loc(6),loc(7)) = ival

                         CASE("double")
                            
                            IF ((var(var_id)%def_missing > 0) .and.  &
                                 (ABS(valin-var(var_id)%users_missing) &
                                 < ABS(var(var_id)%users_missing * &
                                 var(var_id)%miss_val_tolerance))) THEN
                               i = var(var_id)%mip_table_idx
                               dval = mip_tbl(i)%integer_missing
                            ELSE
                               
                               dval = var(var_id)%offset +       &
                                    valin*var(var_id)%scale_factor
                               
                               IF ((var(var_id)%valid_min /=     &
                                    init_undef_single) .and. &
                                    (dval < var(var_id)%valid_min)) THEN
                                  call errmsg_r('min', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               ELSE IF ((var(var_id)%valid_max /=    &
                                    init_undef_single) .and. &
                                    (dval > var(var_id)%valid_max)) THEN
                                  call errmsg_r('max', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               END IF
                               
                            END IF
                            
                            data_out_d(loc(1),loc(2),loc(3),loc(4),  &
                                 loc(5),loc(6),loc(7)) = dval
                            
                         END SELECT
                         
                      END DO
                   END DO
                END DO
             END DO
          END DO
       END DO
    END DO

    SELECT CASE(TRIM(var(var_id)%type))

    CASE('real')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_r, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_r)
       
    CASE('integer')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_i, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_i)
       
    CASE('double')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_d, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_d)
       
    END SELECT
           
    DEALLOCATE(indx1)
    DEALLOCATE(indx2)
    DEALLOCATE(indx3)
    DEALLOCATE(indx4)
    DEALLOCATE(indx5)
    DEALLOCATE(indx6)
    DEALLOCATE(indx7)
        
    IF (ierr == NF90_NOERR) THEN
       IF (ntimes /= 0) THEN
          IF (PRESENT(store_with)) THEN
             var(var_id)%zfac_times_written(store_with) =   &
                  var(var_id)%zfac_times_written(store_with)+ntimes
          ELSE
             var(var_id)%times_written=var(var_id)%times_written+ntimes
          END IF
       END IF
    ELSE
       PRINT*, 'Error writing NetCDF out data: error number =  ', ierr
       PRINT*, '   netCDF error message: ',  TRIM(nf90_strerror(ierr))
       STOP
    END IF
    
    RETURN
  END FUNCTION cmor_write_7d_r




  FUNCTION cmor_write_1d_i(var_id, data, file_suffix, ntimes_passed,  &
       time_vals, time_bnds, store_with) RESULT(ierr)

    IMPLICIT NONE

    INTEGER, INTENT(in):: data(:)
    INTEGER, INTENT(in) ::var_id
    CHARACTER(len=*), INTENT(in), OPTIONAL :: file_suffix
    INTEGER, INTENT(in), OPTIONAL :: ntimes_passed
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_vals(:)
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_bnds(:,:)
    INTEGER, INTENT(in), OPTIONAL :: store_with

    INTEGER j, time_id
    INTEGER ipt1, ipt2, ipt3, ipt4, ipt5, ipt6, ipt7

    INTEGER, ALLOCATABLE, DIMENSION(:) :: indx1, indx2, indx3, indx4, &
         indx5, indx6, indx7
    INTEGER, ALLOCATABLE :: data_out_i(:,:,:,:,:,:,:)
    REAL, ALLOCATABLE :: data_out_r(:,:,:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: data_out_d(:,:,:,:,:,:,:)
    INTEGER, DIMENSION(max_rank) :: inout, loc, dims_len, start, data_size
    INTEGER :: i1=1, i2=1, i3=1, i4=1, i5=1, i6=1, i7=1
    INTEGER :: ival, istat, ncid, ntimes, a_time_slice, i, irank, ierr
    REAL :: rval
    DOUBLE PRECISION :: dval

    INTEGER :: valin

    var(var_id)%users_type = 'integer'

    ierr = 0

    time_id = -1
    data_size = 0

    ! loop through input dimension
    irank = 7
    a_time_slice = 1
    DO j = 1, 7

       i = var(var_id)%dim_order(j)
       IF (i == 0) EXIT
       irank = j

       IF (axis(var(var_id)%axis_idxs(i))%axis == 'T') THEN
          time_id = j
       ELSE
          data_size(j) =  adim(var(var_id)%dim_idxs(i))%lendim
          a_time_slice=a_time_slice*data_size(i)
       END IF

    END DO


    j = 1

    IF (time_id > 0) THEN

       IF (PRESENT(ntimes_passed)) THEN

          data_size(time_id) = ntimes_passed

          IF (data_size(time_id) == 0) THEN
             PRINT*, 'Error encountered in cmor_write:'
             PRINT*, '  user passed ntimes_passed = 0 but this field'
             PRINT*, '  should have a time dimension.'
             STOP
          END IF

       ELSE

          data_size(time_id) = SIZE(DATA)/a_time_slice
          IF (data_size(time_id) == 0) THEN
             PRINT*, 'Error encountered in cmor_write:'
             PRINT*, '  user passed a data array that should be larger, '
             PRINT*, '  it cannot hold even a single time-slice of data.'
             STOP
          END IF

       END IF

       j = data_size(time_id)

    END IF

    IF (a_time_slice*j > SIZE(data)) THEN

       PRINT*, 'Error encountered in cmor_write:'
       PRINT*, '  The data array passed by user has fewer elements than it should'
       PRINT*, '  The axes defined indicate the following dimension lengths: ',&
            data_size
       PRINT*, '  for a total size of ',  &
            a_time_slice*j
       PRINT*,  ' but the users array is length: ', SIZE(data)
       STOP

    END IF

    ierr = check_args(var_id, ncid, ntimes_passed, time_vals, time_bnds, &
         store_with, file_suffix)
    IF (ierr < 0) RETURN

    ! Process time stuff, define dims_len and start, and for use in the 
    !     write_1d case (only??)  also defines a_time_slice

    CALL cmor_process_time(ncid, var_id, ntimes_passed, dims_len, start, &
         a_time_slice, time_vals, time_bnds, store_with)


    ! besides checking, the following defines ntimes

    CALL check_time_and_dim_lens(data_size, var_id, dims_len, ntimes, ierr, &
         ntimes_passed)
    IF (ierr.LT.0) RETURN


    ! dim_order(i) stores the output array index that corresponds to 
    !   input array index i; it stores 0 for output dimensions > the
    !   rank of the output array.
    !   Singleton (scalar) dimensions that the user hasn't overridden are
    !   omitted.

    ! inout is similar to dim_order, but stores i for output dimensions > the
    !   rank of the output array.

    DO i=1,max_rank
       inout(i) = var(var_id)%dim_order(i)
       if (inout(i) == 0) inout(i) = i
    END DO

    ALLOCATE(indx1(dims_len(inout(1))), STAT=istat)
    ALLOCATE(indx2(dims_len(inout(2))), STAT=istat)
    ALLOCATE(indx3(dims_len(inout(3))), STAT=istat)
    ALLOCATE(indx4(dims_len(inout(4))), STAT=istat)
    ALLOCATE(indx5(dims_len(inout(5))), STAT=istat)
    ALLOCATE(indx6(dims_len(inout(6))), STAT=istat)
    ALLOCATE(indx7(dims_len(inout(7))), STAT=istat)

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*,  '   Unable to allocate memory for one of the "idx?" arrays,' 
       PRINT*,  '   which have one of the following dimensions: ', dims_len
       STOP
    END IF

    ! The following generates an index array, indx?(i), containing the location
    !    in the output array corresponding to input location i.
    ! indx1 is index for input array dimension 1, 
    !    indx2 for input dimension 2, etc.

    indx1 = offset_or_reverse_dim(var_id,1,dims_len(inout(1)))
    indx2 = offset_or_reverse_dim(var_id,2,dims_len(inout(2)))
    indx3 = offset_or_reverse_dim(var_id,3,dims_len(inout(3)))
    indx4 = offset_or_reverse_dim(var_id,4,dims_len(inout(4)))
    indx5 = offset_or_reverse_dim(var_id,5,dims_len(inout(5)))
    indx6 = offset_or_reverse_dim(var_id,6,dims_len(inout(6)))
    indx7 = offset_or_reverse_dim(var_id,7,dims_len(inout(7)))

    ! now assign input data to appropriate positions in output array.

    SELECT CASE(TRIM(var(var_id)%type))

    CASE("real")

       ALLOCATE(data_out_r(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("integer")

       ALLOCATE(data_out_i(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("double")

       ALLOCATE(data_out_d(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    END SELECT

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*, '   Unable to allocate memory for one of the "data_out",'
       PRINT*, '   which has the following dimensions: ', dims_len
       STOP
    END IF
    
    loc = 1
 
    DO i7 = 1, dims_len(inout(7))
       loc(inout(7)) = indx7(i7)
       ipt7 = (i7-1)*data_size(6)
       DO i6 = 1, dims_len(inout(6))
          loc(inout(6)) = indx6(i6)
          ipt6 = (ipt7 + i6-1)*data_size(5)
          DO i5 = 1, dims_len(inout(5))
             loc(inout(5)) = indx5(i5)
             ipt5 = (ipt6 + i5-1)*data_size(4)
             DO i4 = 1, dims_len(inout(4))
                loc(inout(4)) = indx4(i4)
                ipt4 = (ipt5 + i4-1)*data_size(3)
                DO i3 = 1, dims_len(inout(3))
                   loc(inout(3)) = indx3(i3)
                   ipt3 = (ipt4 + i3-1)*data_size(2)
                   DO i2 = 1, dims_len(inout(2))
                      loc(inout(2)) = indx2(i2)
                      ipt2 = (ipt3 + i2-1)*data_size(1)
                      DO i1 = 1, dims_len(inout(1))
                         loc(inout(1)) = indx1(i1)
                         ipt1 = ipt2 + i1
                            
                         valin = data(ipt1)
                         
                         SELECT CASE(TRIM(var(var_id)%type))

                         CASE("real")
                            
                            IF ((var(var_id)%def_missing > 0) .and.  &
                                 (ABS(valin-var(var_id)%users_missing_int) &
                                 < ABS(var(var_id)%users_missing_int * &
                                 var(var_id)%miss_val_tolerance))) THEN

                               i = var(var_id)%mip_table_idx
                               rval = mip_tbl(i)%single_missing

                            ELSE
                               
                               rval = var(var_id)%offset +       &
                                    valin*var(var_id)%scale_factor
                               
                               IF ((var(var_id)%valid_min /=     &
                                    init_undef_single) .and. &
                                    (rval < var(var_id)%valid_min)) THEN
                                  call errmsg_i('min', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               ELSE IF ((var(var_id)%valid_max /=    &
                                    init_undef_single) .and. &
                                    (rval > var(var_id)%valid_max)) THEN
                                  call errmsg_i('max', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               END IF
                               
                            END IF
                            
                            data_out_r(loc(1),loc(2),loc(3),loc(4),  &
                                 loc(5),loc(6),loc(7)) = rval

                         CASE("integer")
                            
                            IF ((var(var_id)%def_missing > 0) .and.  &
                                 (ABS(valin-var(var_id)%users_missing_int) &
                                 < ABS(var(var_id)%users_missing_int * &
                                 var(var_id)%miss_val_tolerance))) THEN

                               i = var(var_id)%mip_table_idx
                               ival = mip_tbl(i)%integer_missing

                            ELSE
                               
                               ival = var(var_id)%offset +       &
                                    valin*var(var_id)%scale_factor
                               
                               IF ((var(var_id)%valid_min /=     &
                                    init_undef_single) .and. &
                                    (ival < var(var_id)%valid_min)) THEN
                                  call errmsg_i('min', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               ELSE IF ((var(var_id)%valid_max /=    &
                                    init_undef_single) .and. &
                                    (ival > var(var_id)%valid_max)) THEN
                                  call errmsg_i('max', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               END IF

                               
                            END IF
                            
                            data_out_i(loc(1),loc(2),loc(3),loc(4),  &
                                 loc(5),loc(6),loc(7)) = ival

                         CASE("double")
                            
                            IF ((var(var_id)%def_missing > 0) .and.  &
                                 (ABS(valin-var(var_id)%users_missing_int) &
                                 < ABS(var(var_id)%users_missing_int * &
                                 var(var_id)%miss_val_tolerance))) THEN
                               i = var(var_id)%mip_table_idx
                               dval = mip_tbl(i)%integer_missing
                            ELSE
                               
                               dval = var(var_id)%offset +       &
                                    valin*var(var_id)%scale_factor
                               
                               IF ((var(var_id)%valid_min /=     &
                                    init_undef_single) .and. &
                                    (dval < var(var_id)%valid_min)) THEN
                                  call errmsg_i('min', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               ELSE IF ((var(var_id)%valid_max /=    &
                                    init_undef_single) .and. &
                                    (dval > var(var_id)%valid_max)) THEN
                                  call errmsg_i('max', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               END IF
                               
                            END IF
                            
                            data_out_d(loc(1),loc(2),loc(3),loc(4),  &
                                 loc(5),loc(6),loc(7)) = dval
                            
                         END SELECT
                         
                      END DO
                   END DO
                END DO
             END DO
          END DO
       END DO
    END DO

    SELECT CASE(TRIM(var(var_id)%type))

    CASE('real')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_r, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_r)
       
    CASE('integer')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_i, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_i)
       
    CASE('double')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_d, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_d)
       
    END SELECT
           
    DEALLOCATE(indx1)
    DEALLOCATE(indx2)
    DEALLOCATE(indx3)
    DEALLOCATE(indx4)
    DEALLOCATE(indx5)
    DEALLOCATE(indx6)
    DEALLOCATE(indx7)
        
    IF (ierr == NF90_NOERR) THEN
       IF (ntimes /= 0) THEN
          IF (PRESENT(store_with)) THEN
             var(var_id)%zfac_times_written(store_with) =   &
                  var(var_id)%zfac_times_written(store_with)+ntimes
          ELSE
             var(var_id)%times_written=var(var_id)%times_written+ntimes
          END IF
       END IF
    ELSE
       PRINT*, 'Error writing NetCDF out data: error number =  ', ierr
       PRINT*, '   netCDF error message: ',  TRIM(nf90_strerror(ierr))
       STOP
    END IF
    
    RETURN
  END FUNCTION cmor_write_1d_i




  FUNCTION cmor_write_2d_i(var_id, data, file_suffix, ntimes_passed,  &
       time_vals, time_bnds, store_with) RESULT(ierr)

    IMPLICIT NONE

    INTEGER, INTENT(in):: data(:,:)
    INTEGER, INTENT(in) ::var_id
    CHARACTER(len=*), INTENT(in), OPTIONAL :: file_suffix
    INTEGER, INTENT(in), OPTIONAL :: ntimes_passed
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_vals(:)
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_bnds(:,:)
    INTEGER, INTENT(in), OPTIONAL :: store_with

    INTEGER, ALLOCATABLE, DIMENSION(:) :: indx1, indx2
    INTEGER, ALLOCATABLE :: data_out_i(:,:,:,:,:,:,:)
    REAL, ALLOCATABLE :: data_out_r(:,:,:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: data_out_d(:,:,:,:,:,:,:)
    INTEGER, DIMENSION(max_rank) :: inout, loc, dims_len, start, data_size
    INTEGER :: i1=1, i2=1, i3=1, i4=1, i5=1, i6=1, i7=1
    INTEGER :: ival, istat, ncid, ntimes, a_time_slice, i, irank, ierr
    REAL :: rval
    DOUBLE PRECISION :: dval

    INTEGER :: valin

    var(var_id)%users_type = 'integer'
    irank = 2

    ierr = 0

    ! store input array's dimension lengths
    data_size = 0
    DO i =1 , irank
       data_size(i)=SIZE(data,i)
    END DO

    ierr = check_args(var_id, ncid, ntimes_passed, time_vals, time_bnds, &
         store_with, file_suffix)
    IF (ierr < 0) RETURN

    ! Process time stuff, define dims_len and start, and for use in the 
    !     write_1d case (only??)  also defines a_time_slice

    CALL cmor_process_time(ncid, var_id, ntimes_passed, dims_len, start, &
         a_time_slice, time_vals, time_bnds, store_with)


    ! besides checking, the following defines ntimes

    CALL check_time_and_dim_lens(data_size, var_id, dims_len, ntimes, ierr, &
         ntimes_passed)
    IF (ierr.LT.0) RETURN


    ! dim_order(i) stores the output array index that corresponds to 
    !   input array index i; it stores 0 for output dimensions > the
    !   rank of the output array.
    !   Singleton (scalar) dimensions that the user hasn't overridden are
    !   omitted.

    ! inout is similar to dim_order, but stores i for output dimensions > the
    !   rank of the output array.

    DO i=1,max_rank
       inout(i) = var(var_id)%dim_order(i)
       if (inout(i) == 0) inout(i) = i
    END DO

    ALLOCATE(indx1(dims_len(inout(1))), STAT=istat)
    ALLOCATE(indx2(dims_len(inout(2))), STAT=istat)

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*,  '   Unable to allocate memory for one of the "idx?" arrays,' 
       PRINT*,  '   which have one of the following dimensions: ', dims_len
       STOP
    END IF

    ! The following generates an index array, indx?(i), containing the location
    !    in the output array corresponding to input location i.
    ! indx1 is index for input array dimension 1, 
    !    indx2 for input dimension 2, etc.

    indx1 = offset_or_reverse_dim(var_id,1,dims_len(inout(1)))
    indx2 = offset_or_reverse_dim(var_id,2,dims_len(inout(2)))

    ! now assign input data to appropriate positions in output array.

    SELECT CASE(TRIM(var(var_id)%type))

    CASE("real")

       ALLOCATE(data_out_r(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("integer")

       ALLOCATE(data_out_i(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("double")

       ALLOCATE(data_out_d(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    END SELECT

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*, '   Unable to allocate memory for one of the "data_out",'
       PRINT*, '   which has the following dimensions: ', dims_len
       STOP
    END IF
    
    loc = 1
    
    DO i2 = 1, dims_len(inout(2))
       loc(inout(2)) = indx2(i2)
       DO i1 = 1, dims_len(inout(1))
          loc(inout(1)) = indx1(i1)

          valin = data(i1,i2)

          SELECT CASE(TRIM(var(var_id)%type))

          CASE("real")

             IF ((var(var_id)%def_missing > 0) .and.  &
                  (ABS(valin-var(var_id)%users_missing_int) &
                  < ABS(var(var_id)%users_missing_int * &
                  var(var_id)%miss_val_tolerance))) THEN

                i = var(var_id)%mip_table_idx
                rval = mip_tbl(i)%single_missing

             ELSE

                rval = var(var_id)%offset +       &
                     valin*var(var_id)%scale_factor

                IF ((var(var_id)%valid_min /=     &
                     init_undef_single) .and. &
                     (rval < var(var_id)%valid_min)) THEN
                   call errmsg_i('min', irank, var_id, valin, &
                        rval, ival, dval, i1, i2, i3, i4, i5, &
                        i6, i7)
                ELSE IF ((var(var_id)%valid_max /=    &
                     init_undef_single) .and. &
                     (rval > var(var_id)%valid_max)) THEN
                   call errmsg_i('max', irank, var_id, valin, &
                        rval, ival, dval, i1, i2, i3, i4, i5, &
                        i6, i7)
                END IF

             END IF

             data_out_r(loc(1),loc(2),loc(3),loc(4),  &
                  loc(5),loc(6),loc(7)) = rval

          CASE("integer")

             IF ((var(var_id)%def_missing > 0) .and.  &
                  (ABS(valin-var(var_id)%users_missing_int) &
                  < ABS(var(var_id)%users_missing_int * &
                  var(var_id)%miss_val_tolerance))) THEN

                i = var(var_id)%mip_table_idx
                ival = mip_tbl(i)%integer_missing

             ELSE

                ival = var(var_id)%offset +       &
                     valin*var(var_id)%scale_factor

                IF ((var(var_id)%valid_min /=     &
                     init_undef_single) .and. &
                     (ival < var(var_id)%valid_min)) THEN
                   call errmsg_i('min', irank, var_id, valin, &
                        rval, ival, dval, i1, i2, i3, i4, i5, &
                        i6, i7)
                ELSE IF ((var(var_id)%valid_max /=    &
                     init_undef_single) .and. &
                     (ival > var(var_id)%valid_max)) THEN
                   call errmsg_i('max', irank, var_id, valin, &
                        rval, ival, dval, i1, i2, i3, i4, i5, &
                        i6, i7)
                END IF


             END IF

             data_out_i(loc(1),loc(2),loc(3),loc(4),  &
                  loc(5),loc(6),loc(7)) = ival

          CASE("double")

             IF ((var(var_id)%def_missing > 0) .and.  &
                  (ABS(valin-var(var_id)%users_missing_int) &
                  < ABS(var(var_id)%users_missing_int * &
                  var(var_id)%miss_val_tolerance))) THEN
                i = var(var_id)%mip_table_idx
                dval = mip_tbl(i)%integer_missing
             ELSE

                dval = var(var_id)%offset +       &
                     valin*var(var_id)%scale_factor

                IF ((var(var_id)%valid_min /=     &
                     init_undef_single) .and. &
                     (dval < var(var_id)%valid_min)) THEN
                   call errmsg_i('min', irank, var_id, valin, &
                        rval, ival, dval, i1, i2, i3, i4, i5, &
                        i6, i7)
                ELSE IF ((var(var_id)%valid_max /=    &
                     init_undef_single) .and. &
                     (dval > var(var_id)%valid_max)) THEN
                   call errmsg_i('max', irank, var_id, valin, &
                        rval, ival, dval, i1, i2, i3, i4, i5, &
                        i6, i7)
                END IF

             END IF

             data_out_d(loc(1),loc(2),loc(3),loc(4),  &
                  loc(5),loc(6),loc(7)) = dval

          END SELECT

       END DO
    END DO

    SELECT CASE(TRIM(var(var_id)%type))

    CASE('real')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_r, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_r)
       
    CASE('integer')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_i, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_i)
       
    CASE('double')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_d, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_d)
       
    END SELECT
           
    DEALLOCATE(indx1)
    DEALLOCATE(indx2)
    
    IF (ierr == NF90_NOERR) THEN
       IF (ntimes /= 0) THEN
          IF (PRESENT(store_with)) THEN
             var(var_id)%zfac_times_written(store_with) =   &
                  var(var_id)%zfac_times_written(store_with)+ntimes
          ELSE
             var(var_id)%times_written=var(var_id)%times_written+ntimes
          END IF
       END IF
    ELSE
       PRINT*, 'Error writing NetCDF out data: error number =  ', ierr
       PRINT*, '   netCDF error message: ',  TRIM(nf90_strerror(ierr))
       STOP
    END IF
    
    RETURN
  END FUNCTION cmor_write_2d_i



  FUNCTION cmor_write_3d_i(var_id, data, file_suffix, ntimes_passed,  &
       time_vals, time_bnds, store_with) RESULT(ierr)

    IMPLICIT NONE

    INTEGER, INTENT(in):: data(:,:,:)
    INTEGER, INTENT(in) ::var_id
    CHARACTER(len=*), INTENT(in), OPTIONAL :: file_suffix
    INTEGER, INTENT(in), OPTIONAL :: ntimes_passed
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_vals(:)
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_bnds(:,:)
    INTEGER, INTENT(in), OPTIONAL :: store_with

    INTEGER, ALLOCATABLE, DIMENSION(:) :: indx1, indx2, indx3
    INTEGER, ALLOCATABLE :: data_out_i(:,:,:,:,:,:,:)
    REAL, ALLOCATABLE :: data_out_r(:,:,:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: data_out_d(:,:,:,:,:,:,:)
    INTEGER, DIMENSION(max_rank) :: inout, loc, dims_len, start, data_size
    INTEGER :: i1=1, i2=1, i3=1, i4=1, i5=1, i6=1, i7=1
    INTEGER :: ival, istat, ncid, ntimes, a_time_slice, i, irank, ierr
    REAL :: rval
    DOUBLE PRECISION :: dval

    INTEGER :: valin

    var(var_id)%users_type = 'integer'
    irank = 3

    ierr = 0

    ! store input array's dimension lengths
    data_size = 0
    DO i =1 , irank
       data_size(i)=SIZE(data,i)
    END DO

    ierr = check_args(var_id, ncid, ntimes_passed, time_vals, time_bnds, &
         store_with, file_suffix)
    IF (ierr < 0) RETURN

    ! Process time stuff, define dims_len and start, and for use in the 
    !     write_1d case (only??)  also defines a_time_slice

    CALL cmor_process_time(ncid, var_id, ntimes_passed, dims_len, start, &
         a_time_slice, time_vals, time_bnds, store_with)


    ! besides checking, the following defines ntimes

    CALL check_time_and_dim_lens(data_size, var_id, dims_len, ntimes, ierr, &
         ntimes_passed)
    IF (ierr.LT.0) RETURN


    ! dim_order(i) stores the output array index that corresponds to 
    !   input array index i; it stores 0 for output dimensions > the
    !   rank of the output array.
    !   Singleton (scalar) dimensions that the user hasn't overridden are
    !   omitted.

    ! inout is similar to dim_order, but stores i for output dimensions > the
    !   rank of the output array.

    DO i=1,max_rank
       inout(i) = var(var_id)%dim_order(i)
       if (inout(i) == 0) inout(i) = i
    END DO

    ALLOCATE(indx1(dims_len(inout(1))), STAT=istat)
    ALLOCATE(indx2(dims_len(inout(2))), STAT=istat)
    ALLOCATE(indx3(dims_len(inout(3))), STAT=istat)

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*,  '   Unable to allocate memory for one of the "idx?" arrays,' 
       PRINT*,  '   which have one of the following dimensions: ', dims_len
       STOP
    END IF

    ! The following generates an index array, indx?(i), containing the location
    !    in the output array corresponding to input location i.
    ! indx1 is index for input array dimension 1, 
    !    indx2 for input dimension 2, etc.

    indx1 = offset_or_reverse_dim(var_id,1,dims_len(inout(1)))
    indx2 = offset_or_reverse_dim(var_id,2,dims_len(inout(2)))
    indx3 = offset_or_reverse_dim(var_id,3,dims_len(inout(3)))

    ! now assign input data to appropriate positions in output array.

    SELECT CASE(TRIM(var(var_id)%type))

    CASE("real")

       ALLOCATE(data_out_r(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("integer")

       ALLOCATE(data_out_i(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("double")

       ALLOCATE(data_out_d(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    END SELECT

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*, '   Unable to allocate memory for one of the "data_out",'
       PRINT*, '   which has the following dimensions: ', dims_len
       STOP
    END IF
    
    loc = 1
    
    DO i3 = 1, dims_len(inout(3))
       loc(inout(3)) = indx3(i3)
       DO i2 = 1, dims_len(inout(2))
          loc(inout(2)) = indx2(i2)
          DO i1 = 1, dims_len(inout(1))
             loc(inout(1)) = indx1(i1)

             valin = data(i1,i2,i3)

             SELECT CASE(TRIM(var(var_id)%type))

             CASE("real")

                IF ((var(var_id)%def_missing > 0) .and.  &
                     (ABS(valin-var(var_id)%users_missing_int) &
                     < ABS(var(var_id)%users_missing_int * &
                     var(var_id)%miss_val_tolerance))) THEN

                   i = var(var_id)%mip_table_idx
                   rval = mip_tbl(i)%single_missing

                ELSE

                   rval = var(var_id)%offset +       &
                        valin*var(var_id)%scale_factor

                   IF ((var(var_id)%valid_min /=     &
                        init_undef_single) .and. &
                        (rval < var(var_id)%valid_min)) THEN
                      call errmsg_i('min', irank, var_id, valin, &
                           rval, ival, dval, i1, i2, i3, i4, i5, &
                           i6, i7)
                   ELSE IF ((var(var_id)%valid_max /=    &
                        init_undef_single) .and. &
                        (rval > var(var_id)%valid_max)) THEN
                      call errmsg_i('max', irank, var_id, valin, &
                           rval, ival, dval, i1, i2, i3, i4, i5, &
                           i6, i7)
                   END IF

                END IF

                data_out_r(loc(1),loc(2),loc(3),loc(4),  &
                     loc(5),loc(6),loc(7)) = rval

             CASE("integer")

                IF ((var(var_id)%def_missing > 0) .and.  &
                     (ABS(valin-var(var_id)%users_missing_int) &
                     < ABS(var(var_id)%users_missing_int * &
                     var(var_id)%miss_val_tolerance))) THEN

                   i = var(var_id)%mip_table_idx
                   ival = mip_tbl(i)%integer_missing

                ELSE

                   ival = var(var_id)%offset +       &
                        valin*var(var_id)%scale_factor

                   IF ((var(var_id)%valid_min /=     &
                        init_undef_single) .and. &
                        (ival < var(var_id)%valid_min)) THEN
                      call errmsg_i('min', irank, var_id, valin, &
                           rval, ival, dval, i1, i2, i3, i4, i5, &
                           i6, i7)
                   ELSE IF ((var(var_id)%valid_max /=    &
                        init_undef_single) .and. &
                        (ival > var(var_id)%valid_max)) THEN
                      call errmsg_i('max', irank, var_id, valin, &
                           rval, ival, dval, i1, i2, i3, i4, i5, &
                           i6, i7)
                   END IF


                END IF

                data_out_i(loc(1),loc(2),loc(3),loc(4),  &
                     loc(5),loc(6),loc(7)) = ival

             CASE("double")

                IF ((var(var_id)%def_missing > 0) .and.  &
                     (ABS(valin-var(var_id)%users_missing_int) &
                     < ABS(var(var_id)%users_missing_int * &
                     var(var_id)%miss_val_tolerance))) THEN
                   i = var(var_id)%mip_table_idx
                   dval = mip_tbl(i)%integer_missing
                ELSE

                   dval = var(var_id)%offset +       &
                        valin*var(var_id)%scale_factor

                   IF ((var(var_id)%valid_min /=     &
                        init_undef_single) .and. &
                        (dval < var(var_id)%valid_min)) THEN
                      call errmsg_i('min', irank, var_id, valin, &
                           rval, ival, dval, i1, i2, i3, i4, i5, &
                           i6, i7)
                   ELSE IF ((var(var_id)%valid_max /=    &
                        init_undef_single) .and. &
                        (dval > var(var_id)%valid_max)) THEN
                      call errmsg_i('max', irank, var_id, valin, &
                           rval, ival, dval, i1, i2, i3, i4, i5, &
                           i6, i7)
                   END IF

                END IF

                data_out_d(loc(1),loc(2),loc(3),loc(4),  &
                     loc(5),loc(6),loc(7)) = dval

             END SELECT

          END DO
       END DO
    END DO

    SELECT CASE(TRIM(var(var_id)%type))

    CASE('real')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_r, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_r)
       
    CASE('integer')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_i, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_i)
       
    CASE('double')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_d, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_d)
       
    END SELECT
           
    DEALLOCATE(indx1)
    DEALLOCATE(indx2)
    DEALLOCATE(indx3)
    
    IF (ierr == NF90_NOERR) THEN
       IF (ntimes /= 0) THEN
          IF (PRESENT(store_with)) THEN
             var(var_id)%zfac_times_written(store_with) =   &
                  var(var_id)%zfac_times_written(store_with)+ntimes
          ELSE
             var(var_id)%times_written=var(var_id)%times_written+ntimes
          END IF
       END IF
    ELSE
       PRINT*, 'Error writing NetCDF out data: error number =  ', ierr
       PRINT*, '   netCDF error message: ',  TRIM(nf90_strerror(ierr))
       STOP
    END IF
    
    RETURN
  END FUNCTION cmor_write_3d_i



  FUNCTION cmor_write_4d_i(var_id, data, file_suffix, ntimes_passed,  &
       time_vals, time_bnds, store_with) RESULT(ierr)

    IMPLICIT NONE

    INTEGER, INTENT(in):: data(:,:,:,:)
    INTEGER, INTENT(in) ::var_id
    CHARACTER(len=*), INTENT(in), OPTIONAL :: file_suffix
    INTEGER, INTENT(in), OPTIONAL :: ntimes_passed
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_vals(:)
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_bnds(:,:)
    INTEGER, INTENT(in), OPTIONAL :: store_with

    INTEGER, ALLOCATABLE, DIMENSION(:) :: indx1, indx2, indx3, indx4
    INTEGER, ALLOCATABLE :: data_out_i(:,:,:,:,:,:,:)
    REAL, ALLOCATABLE :: data_out_r(:,:,:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: data_out_d(:,:,:,:,:,:,:)
    INTEGER, DIMENSION(max_rank) :: inout, loc, dims_len, start, data_size
    INTEGER :: i1=1, i2=1, i3=1, i4=1, i5=1, i6=1, i7=1
    INTEGER :: ival, istat, ncid, ntimes, a_time_slice, i, irank, ierr
    REAL :: rval
    DOUBLE PRECISION :: dval

    INTEGER :: valin

    var(var_id)%users_type = 'integer'
    irank = 4

    ierr = 0

    ! store input array's dimension lengths
    data_size = 0
    DO i =1 , irank
       data_size(i)=SIZE(data,i)
    END DO

    ierr = check_args(var_id, ncid, ntimes_passed, time_vals, time_bnds, &
         store_with, file_suffix)
    IF (ierr < 0) RETURN

    ! Process time stuff, define dims_len and start, and for use in the 
    !     write_1d case (only??)  also defines a_time_slice

    CALL cmor_process_time(ncid, var_id, ntimes_passed, dims_len, start, &
         a_time_slice, time_vals, time_bnds, store_with)


    ! besides checking, the following defines ntimes

    CALL check_time_and_dim_lens(data_size, var_id, dims_len, ntimes, ierr, &
         ntimes_passed)
    IF (ierr.LT.0) RETURN


    ! dim_order(i) stores the output array index that corresponds to 
    !   input array index i; it stores 0 for output dimensions > the
    !   rank of the output array.
    !   Singleton (scalar) dimensions that the user hasn't overridden are
    !   omitted.

    ! inout is similar to dim_order, but stores i for output dimensions > the
    !   rank of the output array.

    DO i=1,max_rank
       inout(i) = var(var_id)%dim_order(i)
       if (inout(i) == 0) inout(i) = i
    END DO

    ALLOCATE(indx1(dims_len(inout(1))), STAT=istat)
    ALLOCATE(indx2(dims_len(inout(2))), STAT=istat)
    ALLOCATE(indx3(dims_len(inout(3))), STAT=istat)
    ALLOCATE(indx4(dims_len(inout(4))), STAT=istat)

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*,  '   Unable to allocate memory for one of the "idx?" arrays,' 
       PRINT*,  '   which have one of the following dimensions: ', dims_len
       STOP
    END IF

    ! The following generates an index array, indx?(i), containing the location
    !    in the output array corresponding to input location i.
    ! indx1 is index for input array dimension 1, 
    !    indx2 for input dimension 2, etc.

    indx1 = offset_or_reverse_dim(var_id,1,dims_len(inout(1)))
    indx2 = offset_or_reverse_dim(var_id,2,dims_len(inout(2)))
    indx3 = offset_or_reverse_dim(var_id,3,dims_len(inout(3)))
    indx4 = offset_or_reverse_dim(var_id,4,dims_len(inout(4)))

    ! now assign input data to appropriate positions in output array.

    SELECT CASE(TRIM(var(var_id)%type))

    CASE("real")

       ALLOCATE(data_out_r(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("integer")

       ALLOCATE(data_out_i(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("double")

       ALLOCATE(data_out_d(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    END SELECT

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*, '   Unable to allocate memory for one of the "data_out",'
       PRINT*, '   which has the following dimensions: ', dims_len
       STOP
    END IF
    
    loc = 1
    
    DO i4 = 1, dims_len(inout(4))
       loc(inout(4)) = indx4(i4)
       DO i3 = 1, dims_len(inout(3))
          loc(inout(3)) = indx3(i3)
          DO i2 = 1, dims_len(inout(2))
             loc(inout(2)) = indx2(i2)
             DO i1 = 1, dims_len(inout(1))
                loc(inout(1)) = indx1(i1)

                valin = data(i1,i2,i3,i4)

                SELECT CASE(TRIM(var(var_id)%type))

                CASE("real")

                   IF ((var(var_id)%def_missing > 0) .and.  &
                        (ABS(valin-var(var_id)%users_missing_int) &
                        < ABS(var(var_id)%users_missing_int * &
                        var(var_id)%miss_val_tolerance))) THEN

                      i = var(var_id)%mip_table_idx
                      rval = mip_tbl(i)%single_missing

                   ELSE

                      rval = var(var_id)%offset +       &
                           valin*var(var_id)%scale_factor

                      IF ((var(var_id)%valid_min /=     &
                           init_undef_single) .and. &
                           (rval < var(var_id)%valid_min)) THEN
                         call errmsg_i('min', irank, var_id, valin, &
                              rval, ival, dval, i1, i2, i3, i4, i5, &
                              i6, i7)
                      ELSE IF ((var(var_id)%valid_max /=    &
                           init_undef_single) .and. &
                           (rval > var(var_id)%valid_max)) THEN
                         call errmsg_i('max', irank, var_id, valin, &
                              rval, ival, dval, i1, i2, i3, i4, i5, &
                              i6, i7)
                      END IF

                   END IF

                   data_out_r(loc(1),loc(2),loc(3),loc(4),  &
                        loc(5),loc(6),loc(7)) = rval

                CASE("integer")

                   IF ((var(var_id)%def_missing > 0) .and.  &
                        (ABS(valin-var(var_id)%users_missing_int) &
                        < ABS(var(var_id)%users_missing_int * &
                        var(var_id)%miss_val_tolerance))) THEN

                      i = var(var_id)%mip_table_idx
                      ival = mip_tbl(i)%integer_missing

                   ELSE

                      ival = var(var_id)%offset +       &
                           valin*var(var_id)%scale_factor

                      IF ((var(var_id)%valid_min /=     &
                           init_undef_single) .and. &
                           (ival < var(var_id)%valid_min)) THEN
                         call errmsg_i('min', irank, var_id, valin, &
                              rval, ival, dval, i1, i2, i3, i4, i5, &
                              i6, i7)
                      ELSE IF ((var(var_id)%valid_max /=    &
                           init_undef_single) .and. &
                           (ival > var(var_id)%valid_max)) THEN
                         call errmsg_i('max', irank, var_id, valin, &
                              rval, ival, dval, i1, i2, i3, i4, i5, &
                              i6, i7)
                      END IF


                   END IF

                   data_out_i(loc(1),loc(2),loc(3),loc(4),  &
                        loc(5),loc(6),loc(7)) = ival

                CASE("double")

                   IF ((var(var_id)%def_missing > 0) .and.  &
                        (ABS(valin-var(var_id)%users_missing_int) &
                        < ABS(var(var_id)%users_missing_int * &
                        var(var_id)%miss_val_tolerance))) THEN
                      i = var(var_id)%mip_table_idx
                      dval = mip_tbl(i)%integer_missing
                   ELSE

                      dval = var(var_id)%offset +       &
                           valin*var(var_id)%scale_factor

                      IF ((var(var_id)%valid_min /=     &
                           init_undef_single) .and. &
                           (dval < var(var_id)%valid_min)) THEN
                         call errmsg_i('min', irank, var_id, valin, &
                              rval, ival, dval, i1, i2, i3, i4, i5, &
                              i6, i7)
                      ELSE IF ((var(var_id)%valid_max /=    &
                           init_undef_single) .and. &
                           (dval > var(var_id)%valid_max)) THEN
                         call errmsg_i('max', irank, var_id, valin, &
                              rval, ival, dval, i1, i2, i3, i4, i5, &
                              i6, i7)
                      END IF

                   END IF

                   data_out_d(loc(1),loc(2),loc(3),loc(4),  &
                        loc(5),loc(6),loc(7)) = dval

                END SELECT

             END DO
          END DO
       END DO
    END DO

    SELECT CASE(TRIM(var(var_id)%type))

    CASE('real')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_r, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_r)
       
    CASE('integer')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_i, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_i)
       
    CASE('double')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_d, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_d)
       
    END SELECT
           
    DEALLOCATE(indx1)
    DEALLOCATE(indx2)
    DEALLOCATE(indx3)
    DEALLOCATE(indx4)
    
    IF (ierr == NF90_NOERR) THEN
       IF (ntimes /= 0) THEN
          IF (PRESENT(store_with)) THEN
             var(var_id)%zfac_times_written(store_with) =   &
                  var(var_id)%zfac_times_written(store_with)+ntimes
          ELSE
             var(var_id)%times_written=var(var_id)%times_written+ntimes
          END IF
       END IF
    ELSE
       PRINT*, 'Error writing NetCDF out data: error number =  ', ierr
       PRINT*, '   netCDF error message: ',  TRIM(nf90_strerror(ierr))
       STOP
    END IF
    
    RETURN
  END FUNCTION cmor_write_4d_i



 


  FUNCTION cmor_write_5d_i(var_id, data, file_suffix, ntimes_passed,  &
       time_vals, time_bnds, store_with) RESULT(ierr)

    IMPLICIT NONE

    INTEGER, INTENT(in):: data(:,:,:,:,:)
    INTEGER, INTENT(in) ::var_id
    CHARACTER(len=*), INTENT(in), OPTIONAL :: file_suffix
    INTEGER, INTENT(in), OPTIONAL :: ntimes_passed
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_vals(:)
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_bnds(:,:)
    INTEGER, INTENT(in), OPTIONAL :: store_with

    INTEGER, ALLOCATABLE, DIMENSION(:) :: indx1, indx2, indx3, indx4, &
         indx5
    INTEGER, ALLOCATABLE :: data_out_i(:,:,:,:,:,:,:)
    REAL, ALLOCATABLE :: data_out_r(:,:,:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: data_out_d(:,:,:,:,:,:,:)
    INTEGER, DIMENSION(max_rank) :: inout, loc, dims_len, start, data_size
    INTEGER :: i1=1, i2=1, i3=1, i4=1, i5=1, i6=1, i7=1
    INTEGER :: ival, istat, ncid, ntimes, a_time_slice, i, irank, ierr
    REAL :: rval
    DOUBLE PRECISION :: dval

    INTEGER :: valin

    var(var_id)%users_type = 'integer'
    irank = 5

    ierr = 0

    ! store input array's dimension lengths
    data_size = 0
    DO i =1 , irank
       data_size(i)=SIZE(data,i)
    END DO

    ierr = check_args(var_id, ncid, ntimes_passed, time_vals, time_bnds, &
         store_with, file_suffix)
    IF (ierr < 0) RETURN

    ! Process time stuff, define dims_len and start, and for use in the 
    !     write_1d case (only??)  also defines a_time_slice

    CALL cmor_process_time(ncid, var_id, ntimes_passed, dims_len, start, &
         a_time_slice, time_vals, time_bnds, store_with)


    ! besides checking, the following defines ntimes

    CALL check_time_and_dim_lens(data_size, var_id, dims_len, ntimes, ierr, &
         ntimes_passed)
    IF (ierr.LT.0) RETURN


    ! dim_order(i) stores the output array index that corresponds to 
    !   input array index i; it stores 0 for output dimensions > the
    !   rank of the output array.
    !   Singleton (scalar) dimensions that the user hasn't overridden are
    !   omitted.

    ! inout is similar to dim_order, but stores i for output dimensions > the
    !   rank of the output array.

    DO i=1,max_rank
       inout(i) = var(var_id)%dim_order(i)
       if (inout(i) == 0) inout(i) = i
    END DO

    ALLOCATE(indx1(dims_len(inout(1))), STAT=istat)
    ALLOCATE(indx2(dims_len(inout(2))), STAT=istat)
    ALLOCATE(indx3(dims_len(inout(3))), STAT=istat)
    ALLOCATE(indx4(dims_len(inout(4))), STAT=istat)
    ALLOCATE(indx5(dims_len(inout(5))), STAT=istat)

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*,  '   Unable to allocate memory for one of the "idx?" arrays,' 
       PRINT*,  '   which have one of the following dimensions: ', dims_len
       STOP
    END IF

    ! The following generates an index array, indx?(i), containing the location
    !    in the output array corresponding to input location i.
    ! indx1 is index for input array dimension 1, 
    !    indx2 for input dimension 2, etc.

    indx1 = offset_or_reverse_dim(var_id,1,dims_len(inout(1)))
    indx2 = offset_or_reverse_dim(var_id,2,dims_len(inout(2)))
    indx3 = offset_or_reverse_dim(var_id,3,dims_len(inout(3)))
    indx4 = offset_or_reverse_dim(var_id,4,dims_len(inout(4)))
    indx5 = offset_or_reverse_dim(var_id,5,dims_len(inout(5)))

    ! now assign input data to appropriate positions in output array.

    SELECT CASE(TRIM(var(var_id)%type))

    CASE("real")

       ALLOCATE(data_out_r(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("integer")

       ALLOCATE(data_out_i(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("double")

       ALLOCATE(data_out_d(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    END SELECT

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*, '   Unable to allocate memory for one of the "data_out",'
       PRINT*, '   which has the following dimensions: ', dims_len
       STOP
    END IF
    
    loc = 1
    
    DO i5 = 1, dims_len(inout(5))
       loc(inout(5)) = indx5(i5)
       DO i4 = 1, dims_len(inout(4))
          loc(inout(4)) = indx4(i4)
          DO i3 = 1, dims_len(inout(3))
             loc(inout(3)) = indx3(i3)
             DO i2 = 1, dims_len(inout(2))
                loc(inout(2)) = indx2(i2)
                DO i1 = 1, dims_len(inout(1))
                   loc(inout(1)) = indx1(i1)

                   valin = data(i1,i2,i3,i4,i5)

                   SELECT CASE(TRIM(var(var_id)%type))

                   CASE("real")

                      IF ((var(var_id)%def_missing > 0) .and.  &
                           (ABS(valin-var(var_id)%users_missing_int) &
                           < ABS(var(var_id)%users_missing_int * &
                           var(var_id)%miss_val_tolerance))) THEN

                         i = var(var_id)%mip_table_idx
                         rval = mip_tbl(i)%single_missing

                      ELSE

                         rval = var(var_id)%offset +       &
                              valin*var(var_id)%scale_factor

                         IF ((var(var_id)%valid_min /=     &
                              init_undef_single) .and. &
                              (rval < var(var_id)%valid_min)) THEN
                            call errmsg_i('min', irank, var_id, valin, &
                                 rval, ival, dval, i1, i2, i3, i4, i5, &
                                 i6, i7)
                         ELSE IF ((var(var_id)%valid_max /=    &
                              init_undef_single) .and. &
                              (rval > var(var_id)%valid_max)) THEN
                            call errmsg_i('max', irank, var_id, valin, &
                                 rval, ival, dval, i1, i2, i3, i4, i5, &
                                 i6, i7)
                         END IF

                      END IF

                      data_out_r(loc(1),loc(2),loc(3),loc(4),  &
                           loc(5),loc(6),loc(7)) = rval

                   CASE("integer")

                      IF ((var(var_id)%def_missing > 0) .and.  &
                           (ABS(valin-var(var_id)%users_missing_int) &
                           < ABS(var(var_id)%users_missing_int * &
                           var(var_id)%miss_val_tolerance))) THEN

                         i = var(var_id)%mip_table_idx
                         ival = mip_tbl(i)%integer_missing

                      ELSE

                         ival = var(var_id)%offset +       &
                              valin*var(var_id)%scale_factor

                         IF ((var(var_id)%valid_min /=     &
                              init_undef_single) .and. &
                              (ival < var(var_id)%valid_min)) THEN
                            call errmsg_i('min', irank, var_id, valin, &
                                 rval, ival, dval, i1, i2, i3, i4, i5, &
                                 i6, i7)
                         ELSE IF ((var(var_id)%valid_max /=    &
                              init_undef_single) .and. &
                              (ival > var(var_id)%valid_max)) THEN
                            call errmsg_i('max', irank, var_id, valin, &
                                 rval, ival, dval, i1, i2, i3, i4, i5, &
                                 i6, i7)
                         END IF


                      END IF

                      data_out_i(loc(1),loc(2),loc(3),loc(4),  &
                           loc(5),loc(6),loc(7)) = ival

                   CASE("double")

                      IF ((var(var_id)%def_missing > 0) .and.  &
                           (ABS(valin-var(var_id)%users_missing_int) &
                           < ABS(var(var_id)%users_missing_int * &
                           var(var_id)%miss_val_tolerance))) THEN
                         i = var(var_id)%mip_table_idx
                         dval = mip_tbl(i)%integer_missing
                      ELSE

                         dval = var(var_id)%offset +       &
                              valin*var(var_id)%scale_factor

                         IF ((var(var_id)%valid_min /=     &
                              init_undef_single) .and. &
                              (dval < var(var_id)%valid_min)) THEN
                            call errmsg_i('min', irank, var_id, valin, &
                                 rval, ival, dval, i1, i2, i3, i4, i5, &
                                 i6, i7)
                         ELSE IF ((var(var_id)%valid_max /=    &
                              init_undef_single) .and. &
                              (dval > var(var_id)%valid_max)) THEN
                            call errmsg_i('max', irank, var_id, valin, &
                                 rval, ival, dval, i1, i2, i3, i4, i5, &
                                 i6, i7)
                         END IF

                      END IF

                      data_out_d(loc(1),loc(2),loc(3),loc(4),  &
                           loc(5),loc(6),loc(7)) = dval

                   END SELECT

                END DO
             END DO
          END DO
       END DO
    END DO

    SELECT CASE(TRIM(var(var_id)%type))

    CASE('real')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_r, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_r)
       
    CASE('integer')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_i, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_i)
       
    CASE('double')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_d, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_d)
       
    END SELECT
           
    DEALLOCATE(indx1)
    DEALLOCATE(indx2)
    DEALLOCATE(indx3)
    DEALLOCATE(indx4)
    DEALLOCATE(indx5)
        
    IF (ierr == NF90_NOERR) THEN
       IF (ntimes /= 0) THEN
          IF (PRESENT(store_with)) THEN
             var(var_id)%zfac_times_written(store_with) =   &
                  var(var_id)%zfac_times_written(store_with)+ntimes
          ELSE
             var(var_id)%times_written=var(var_id)%times_written+ntimes
          END IF
       END IF
    ELSE
       PRINT*, 'Error writing NetCDF out data: error number =  ', ierr
       PRINT*, '   netCDF error message: ',  TRIM(nf90_strerror(ierr))
       STOP
    END IF
    
    RETURN
  END FUNCTION cmor_write_5d_i




  FUNCTION cmor_write_6d_i(var_id, data, file_suffix, ntimes_passed,  &
       time_vals, time_bnds, store_with) RESULT(ierr)

    IMPLICIT NONE

    INTEGER, INTENT(in):: data(:,:,:,:,:,:)
    INTEGER, INTENT(in) ::var_id
    CHARACTER(len=*), INTENT(in), OPTIONAL :: file_suffix
    INTEGER, INTENT(in), OPTIONAL :: ntimes_passed
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_vals(:)
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_bnds(:,:)
    INTEGER, INTENT(in), OPTIONAL :: store_with

    INTEGER, ALLOCATABLE, DIMENSION(:) :: indx1, indx2, indx3, indx4, &
         indx5, indx6
    INTEGER, ALLOCATABLE :: data_out_i(:,:,:,:,:,:,:)
    REAL, ALLOCATABLE :: data_out_r(:,:,:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: data_out_d(:,:,:,:,:,:,:)
    INTEGER, DIMENSION(max_rank) :: inout, loc, dims_len, start, data_size
    INTEGER :: i1=1, i2=1, i3=1, i4=1, i5=1, i6=1, i7=1
    INTEGER :: ival, istat, ncid, ntimes, a_time_slice, i, irank, ierr
    REAL :: rval
    DOUBLE PRECISION :: dval

    INTEGER :: valin

    var(var_id)%users_type = 'integer'
    irank = 6

    ierr = 0

    ! store input array's dimension lengths
    data_size = 0
    DO i =1 , irank
       data_size(i)=SIZE(data,i)
    END DO

    ierr = check_args(var_id, ncid, ntimes_passed, time_vals, time_bnds, &
         store_with, file_suffix)
    IF (ierr < 0) RETURN

    ! Process time stuff, define dims_len and start, and for use in the 
    !     write_1d case (only??)  also defines a_time_slice

    CALL cmor_process_time(ncid, var_id, ntimes_passed, dims_len, start, &
         a_time_slice, time_vals, time_bnds, store_with)


    ! besides checking, the following defines ntimes

    CALL check_time_and_dim_lens(data_size, var_id, dims_len, ntimes, ierr, &
         ntimes_passed)
    IF (ierr.LT.0) RETURN


    ! dim_order(i) stores the output array index that corresponds to 
    !   input array index i; it stores 0 for output dimensions > the
    !   rank of the output array.
    !   Singleton (scalar) dimensions that the user hasn't overridden are
    !   omitted.

    ! inout is similar to dim_order, but stores i for output dimensions > the
    !   rank of the output array.

    DO i=1,max_rank
       inout(i) = var(var_id)%dim_order(i)
       if (inout(i) == 0) inout(i) = i
    END DO

    ALLOCATE(indx1(dims_len(inout(1))), STAT=istat)
    ALLOCATE(indx2(dims_len(inout(2))), STAT=istat)
    ALLOCATE(indx3(dims_len(inout(3))), STAT=istat)
    ALLOCATE(indx4(dims_len(inout(4))), STAT=istat)
    ALLOCATE(indx5(dims_len(inout(5))), STAT=istat)
    ALLOCATE(indx6(dims_len(inout(6))), STAT=istat)

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*,  '   Unable to allocate memory for one of the "idx?" arrays,' 
       PRINT*,  '   which have one of the following dimensions: ', dims_len
       STOP
    END IF

    ! The following generates an index array, indx?(i), containing the location
    !    in the output array corresponding to input location i.
    ! indx1 is index for input array dimension 1, 
    !    indx2 for input dimension 2, etc.

    indx1 = offset_or_reverse_dim(var_id,1,dims_len(inout(1)))
    indx2 = offset_or_reverse_dim(var_id,2,dims_len(inout(2)))
    indx3 = offset_or_reverse_dim(var_id,3,dims_len(inout(3)))
    indx4 = offset_or_reverse_dim(var_id,4,dims_len(inout(4)))
    indx5 = offset_or_reverse_dim(var_id,5,dims_len(inout(5)))
    indx6 = offset_or_reverse_dim(var_id,6,dims_len(inout(6)))

    ! now assign input data to appropriate positions in output array.

    SELECT CASE(TRIM(var(var_id)%type))

    CASE("real")

       ALLOCATE(data_out_r(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("integer")

       ALLOCATE(data_out_i(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("double")

       ALLOCATE(data_out_d(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    END SELECT

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*, '   Unable to allocate memory for one of the "data_out",'
       PRINT*, '   which has the following dimensions: ', dims_len
       STOP
    END IF
    
    loc = 1
    
    DO i6 = 1, dims_len(inout(6))
       loc(inout(6)) = indx6(i6)
       DO i5 = 1, dims_len(inout(5))
          loc(inout(5)) = indx5(i5)
          DO i4 = 1, dims_len(inout(4))
             loc(inout(4)) = indx4(i4)
             DO i3 = 1, dims_len(inout(3))
                loc(inout(3)) = indx3(i3)
                DO i2 = 1, dims_len(inout(2))
                   loc(inout(2)) = indx2(i2)
                   DO i1 = 1, dims_len(inout(1))
                      loc(inout(1)) = indx1(i1)

                      valin = data(i1,i2,i3,i4,i5,i6)

                      SELECT CASE(TRIM(var(var_id)%type))

                      CASE("real")

                         IF ((var(var_id)%def_missing > 0) .and.  &
                              (ABS(valin-var(var_id)%users_missing_int) &
                              < ABS(var(var_id)%users_missing_int * &
                              var(var_id)%miss_val_tolerance))) THEN

                            i = var(var_id)%mip_table_idx
                            rval = mip_tbl(i)%single_missing

                         ELSE

                            rval = var(var_id)%offset +       &
                                 valin*var(var_id)%scale_factor

                            IF ((var(var_id)%valid_min /=     &
                                 init_undef_single) .and. &
                                 (rval < var(var_id)%valid_min)) THEN
                               call errmsg_i('min', irank, var_id, valin, &
                                    rval, ival, dval, i1, i2, i3, i4, i5, &
                                    i6, i7)
                            ELSE IF ((var(var_id)%valid_max /=    &
                                 init_undef_single) .and. &
                                 (rval > var(var_id)%valid_max)) THEN
                               call errmsg_i('max', irank, var_id, valin, &
                                    rval, ival, dval, i1, i2, i3, i4, i5, &
                                    i6, i7)
                            END IF

                         END IF

                         data_out_r(loc(1),loc(2),loc(3),loc(4),  &
                              loc(5),loc(6),loc(7)) = rval

                      CASE("integer")

                         IF ((var(var_id)%def_missing > 0) .and.  &
                              (ABS(valin-var(var_id)%users_missing_int) &
                              < ABS(var(var_id)%users_missing_int * &
                              var(var_id)%miss_val_tolerance))) THEN

                            i = var(var_id)%mip_table_idx
                            ival = mip_tbl(i)%integer_missing

                         ELSE

                            ival = var(var_id)%offset +       &
                                 valin*var(var_id)%scale_factor

                            IF ((var(var_id)%valid_min /=     &
                                 init_undef_single) .and. &
                                 (ival < var(var_id)%valid_min)) THEN
                               call errmsg_i('min', irank, var_id, valin, &
                                    rval, ival, dval, i1, i2, i3, i4, i5, &
                                    i6, i7)
                            ELSE IF ((var(var_id)%valid_max /=    &
                                 init_undef_single) .and. &
                                 (ival > var(var_id)%valid_max)) THEN
                               call errmsg_i('max', irank, var_id, valin, &
                                    rval, ival, dval, i1, i2, i3, i4, i5, &
                                    i6, i7)
                            END IF


                         END IF

                         data_out_i(loc(1),loc(2),loc(3),loc(4),  &
                              loc(5),loc(6),loc(7)) = ival

                      CASE("double")

                         IF ((var(var_id)%def_missing > 0) .and.  &
                              (ABS(valin-var(var_id)%users_missing_int) &
                              < ABS(var(var_id)%users_missing_int * &
                              var(var_id)%miss_val_tolerance))) THEN
                            i = var(var_id)%mip_table_idx
                            dval = mip_tbl(i)%integer_missing
                         ELSE

                            dval = var(var_id)%offset +       &
                                 valin*var(var_id)%scale_factor

                            IF ((var(var_id)%valid_min /=     &
                                 init_undef_single) .and. &
                                 (dval < var(var_id)%valid_min)) THEN
                               call errmsg_i('min', irank, var_id, valin, &
                                    rval, ival, dval, i1, i2, i3, i4, i5, &
                                    i6, i7)
                            ELSE IF ((var(var_id)%valid_max /=    &
                                 init_undef_single) .and. &
                                 (dval > var(var_id)%valid_max)) THEN
                               call errmsg_i('max', irank, var_id, valin, &
                                    rval, ival, dval, i1, i2, i3, i4, i5, &
                                    i6, i7)
                            END IF

                         END IF

                         data_out_d(loc(1),loc(2),loc(3),loc(4),  &
                              loc(5),loc(6),loc(7)) = dval

                      END SELECT

                   END DO
                END DO
             END DO
          END DO
       END DO
    END DO

    SELECT CASE(TRIM(var(var_id)%type))

    CASE('real')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_r, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_r)
       
    CASE('integer')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_i, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_i)
       
    CASE('double')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_d, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_d)
       
    END SELECT
           
    DEALLOCATE(indx1)
    DEALLOCATE(indx2)
    DEALLOCATE(indx3)
    DEALLOCATE(indx4)
    DEALLOCATE(indx5)
    DEALLOCATE(indx6)
        
    IF (ierr == NF90_NOERR) THEN
       IF (ntimes /= 0) THEN
          IF (PRESENT(store_with)) THEN
             var(var_id)%zfac_times_written(store_with) =   &
                  var(var_id)%zfac_times_written(store_with)+ntimes
          ELSE
             var(var_id)%times_written=var(var_id)%times_written+ntimes
          END IF
       END IF
    ELSE
       PRINT*, 'Error writing NetCDF out data: error number =  ', ierr
       PRINT*, '   netCDF error message: ',  TRIM(nf90_strerror(ierr))
       STOP
    END IF
    
    RETURN
  END FUNCTION cmor_write_6d_i




  FUNCTION cmor_write_7d_i(var_id, data, file_suffix, ntimes_passed,  &
       time_vals, time_bnds, store_with) RESULT(ierr)

    IMPLICIT NONE

    INTEGER, INTENT(in):: data(:,:,:,:,:,:,:)
    INTEGER, INTENT(in) ::var_id
    CHARACTER(len=*), INTENT(in), OPTIONAL :: file_suffix
    INTEGER, INTENT(in), OPTIONAL :: ntimes_passed
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_vals(:)
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_bnds(:,:)
    INTEGER, INTENT(in), OPTIONAL :: store_with

    INTEGER, ALLOCATABLE, DIMENSION(:) :: indx1, indx2, indx3, indx4, &
         indx5, indx6, indx7
    INTEGER, ALLOCATABLE :: data_out_i(:,:,:,:,:,:,:)
    REAL, ALLOCATABLE :: data_out_r(:,:,:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: data_out_d(:,:,:,:,:,:,:)
    INTEGER, DIMENSION(max_rank) :: inout, loc, dims_len, start, data_size
    INTEGER :: i1=1, i2=1, i3=1, i4=1, i5=1, i6=1, i7=1
    INTEGER :: ival, istat, ncid, ntimes, a_time_slice, i, irank, ierr
    REAL :: rval
    DOUBLE PRECISION :: dval

    INTEGER :: valin

    var(var_id)%users_type = 'integer'
    irank = 7

    ierr = 0

    ! store input array's dimension lengths
    data_size = 0
    DO i =1 , irank
       data_size(i)=SIZE(data,i)
    END DO

    ierr = check_args(var_id, ncid, ntimes_passed, time_vals, time_bnds, &
         store_with, file_suffix)
    IF (ierr < 0) RETURN

    ! Process time stuff, define dims_len and start, and for use in the 
    !     write_1d case (only??)  also defines a_time_slice

    CALL cmor_process_time(ncid, var_id, ntimes_passed, dims_len, start, &
         a_time_slice, time_vals, time_bnds, store_with)


    ! besides checking, the following defines ntimes

    CALL check_time_and_dim_lens(data_size, var_id, dims_len, ntimes, ierr, &
         ntimes_passed)
    IF (ierr.LT.0) RETURN


    ! dim_order(i) stores the output array index that corresponds to 
    !   input array index i; it stores 0 for output dimensions > the
    !   rank of the output array.
    !   Singleton (scalar) dimensions that the user hasn't overridden are
    !   omitted.

    ! inout is similar to dim_order, but stores i for output dimensions > the
    !   rank of the output array.

    DO i=1,max_rank
       inout(i) = var(var_id)%dim_order(i)
       if (inout(i) == 0) inout(i) = i
    END DO

    ALLOCATE(indx1(dims_len(inout(1))), STAT=istat)
    ALLOCATE(indx2(dims_len(inout(2))), STAT=istat)
    ALLOCATE(indx3(dims_len(inout(3))), STAT=istat)
    ALLOCATE(indx4(dims_len(inout(4))), STAT=istat)
    ALLOCATE(indx5(dims_len(inout(5))), STAT=istat)
    ALLOCATE(indx6(dims_len(inout(6))), STAT=istat)
    ALLOCATE(indx7(dims_len(inout(7))), STAT=istat)

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*,  '   Unable to allocate memory for one of the "idx?" arrays,' 
       PRINT*,  '   which have one of the following dimensions: ', dims_len
       STOP
    END IF

    ! The following generates an index array, indx?(i), containing the location
    !    in the output array corresponding to input location i.
    ! indx1 is index for input array dimension 1, 
    !    indx2 for input dimension 2, etc.

    indx1 = offset_or_reverse_dim(var_id,1,dims_len(inout(1)))
    indx2 = offset_or_reverse_dim(var_id,2,dims_len(inout(2)))
    indx3 = offset_or_reverse_dim(var_id,3,dims_len(inout(3)))
    indx4 = offset_or_reverse_dim(var_id,4,dims_len(inout(4)))
    indx5 = offset_or_reverse_dim(var_id,5,dims_len(inout(5)))
    indx6 = offset_or_reverse_dim(var_id,6,dims_len(inout(6)))
    indx7 = offset_or_reverse_dim(var_id,7,dims_len(inout(7)))

    ! now assign input data to appropriate positions in output array.

    SELECT CASE(TRIM(var(var_id)%type))

    CASE("real")

       ALLOCATE(data_out_r(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("integer")

       ALLOCATE(data_out_i(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("double")

       ALLOCATE(data_out_d(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    END SELECT

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*, '   Unable to allocate memory for one of the "data_out",'
       PRINT*, '   which has the following dimensions: ', dims_len
       STOP
    END IF
    
    loc = 1
    
    DO i7 = 1, dims_len(inout(7))
       loc(inout(7)) = indx7(i7)
       DO i6 = 1, dims_len(inout(6))
          loc(inout(6)) = indx6(i6)
          DO i5 = 1, dims_len(inout(5))
             loc(inout(5)) = indx5(i5)
             DO i4 = 1, dims_len(inout(4))
                loc(inout(4)) = indx4(i4)
                DO i3 = 1, dims_len(inout(3))
                   loc(inout(3)) = indx3(i3)
                   DO i2 = 1, dims_len(inout(2))
                      loc(inout(2)) = indx2(i2)
                      DO i1 = 1, dims_len(inout(1))
                         loc(inout(1)) = indx1(i1)
                         
                         valin = data(i1,i2,i3,i4,i5,i6,i7)
                         
                         SELECT CASE(TRIM(var(var_id)%type))

                         CASE("real")
                            
                            IF ((var(var_id)%def_missing > 0) .and.  &
                                 (ABS(valin-var(var_id)%users_missing_int) &
                                 < ABS(var(var_id)%users_missing_int * &
                                 var(var_id)%miss_val_tolerance))) THEN

                               i = var(var_id)%mip_table_idx
                               rval = mip_tbl(i)%single_missing

                            ELSE
                               
                               rval = var(var_id)%offset +       &
                                    valin*var(var_id)%scale_factor
                               
                               IF ((var(var_id)%valid_min /=     &
                                    init_undef_single) .and. &
                                    (rval < var(var_id)%valid_min)) THEN
                                  call errmsg_i('min', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               ELSE IF ((var(var_id)%valid_max /=    &
                                    init_undef_single) .and. &
                                    (rval > var(var_id)%valid_max)) THEN
                                  call errmsg_i('max', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               END IF
                               
                            END IF
                            
                            data_out_r(loc(1),loc(2),loc(3),loc(4),  &
                                 loc(5),loc(6),loc(7)) = rval

                         CASE("integer")
                            
                            IF ((var(var_id)%def_missing > 0) .and.  &
                                 (ABS(valin-var(var_id)%users_missing_int) &
                                 < ABS(var(var_id)%users_missing_int * &
                                 var(var_id)%miss_val_tolerance))) THEN

                               i = var(var_id)%mip_table_idx
                               ival = mip_tbl(i)%integer_missing

                            ELSE
                               
                               ival = var(var_id)%offset +       &
                                    valin*var(var_id)%scale_factor
                               
                               IF ((var(var_id)%valid_min /=     &
                                    init_undef_single) .and. &
                                    (ival < var(var_id)%valid_min)) THEN
                                  call errmsg_i('min', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               ELSE IF ((var(var_id)%valid_max /=    &
                                    init_undef_single) .and. &
                                    (ival > var(var_id)%valid_max)) THEN
                                  call errmsg_i('max', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               END IF

                               
                            END IF
                            
                            data_out_i(loc(1),loc(2),loc(3),loc(4),  &
                                 loc(5),loc(6),loc(7)) = ival

                         CASE("double")
                            
                            IF ((var(var_id)%def_missing > 0) .and.  &
                                 (ABS(valin-var(var_id)%users_missing_int) &
                                 < ABS(var(var_id)%users_missing_int * &
                                 var(var_id)%miss_val_tolerance))) THEN
                               i = var(var_id)%mip_table_idx
                               dval = mip_tbl(i)%integer_missing
                            ELSE
                               
                               dval = var(var_id)%offset +       &
                                    valin*var(var_id)%scale_factor
                               
                               IF ((var(var_id)%valid_min /=     &
                                    init_undef_single) .and. &
                                    (dval < var(var_id)%valid_min)) THEN
                                  call errmsg_i('min', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               ELSE IF ((var(var_id)%valid_max /=    &
                                    init_undef_single) .and. &
                                    (dval > var(var_id)%valid_max)) THEN
                                  call errmsg_i('max', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               END IF
                               
                            END IF
                            
                            data_out_d(loc(1),loc(2),loc(3),loc(4),  &
                                 loc(5),loc(6),loc(7)) = dval
                            
                         END SELECT
                         
                      END DO
                   END DO
                END DO
             END DO
          END DO
       END DO
    END DO

    SELECT CASE(TRIM(var(var_id)%type))

    CASE('real')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_r, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_r)
       
    CASE('integer')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_i, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_i)
       
    CASE('double')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_d, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_d)
       
    END SELECT
           
    DEALLOCATE(indx1)
    DEALLOCATE(indx2)
    DEALLOCATE(indx3)
    DEALLOCATE(indx4)
    DEALLOCATE(indx5)
    DEALLOCATE(indx6)
    DEALLOCATE(indx7)
        
    IF (ierr == NF90_NOERR) THEN
       IF (ntimes /= 0) THEN
          IF (PRESENT(store_with)) THEN
             var(var_id)%zfac_times_written(store_with) =   &
                  var(var_id)%zfac_times_written(store_with)+ntimes
          ELSE
             var(var_id)%times_written=var(var_id)%times_written+ntimes
          END IF
       END IF
    ELSE
       PRINT*, 'Error writing NetCDF out data: error number =  ', ierr
       PRINT*, '   netCDF error message: ',  TRIM(nf90_strerror(ierr))
       STOP
    END IF
    
    RETURN
  END FUNCTION cmor_write_7d_i



  FUNCTION cmor_write_1d_d(var_id, data, file_suffix, ntimes_passed,  &
       time_vals, time_bnds, store_with) RESULT(ierr)

    IMPLICIT NONE

    DOUBLE PRECISION, INTENT(in):: data(:)
    INTEGER, INTENT(in) ::var_id
    CHARACTER(len=*), INTENT(in), OPTIONAL :: file_suffix
    INTEGER, INTENT(in), OPTIONAL :: ntimes_passed
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_vals(:)
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_bnds(:,:)
    INTEGER, INTENT(in), OPTIONAL :: store_with

    INTEGER j, time_id
    INTEGER ipt1, ipt2, ipt3, ipt4, ipt5, ipt6, ipt7

    INTEGER, ALLOCATABLE, DIMENSION(:) :: indx1, indx2, indx3, indx4, &
         indx5, indx6, indx7
    INTEGER, ALLOCATABLE :: data_out_i(:,:,:,:,:,:,:)
    REAL, ALLOCATABLE :: data_out_r(:,:,:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: data_out_d(:,:,:,:,:,:,:)
    INTEGER, DIMENSION(max_rank) :: inout, loc, dims_len, start, data_size
    INTEGER :: i1=1, i2=1, i3=1, i4=1, i5=1, i6=1, i7=1
    INTEGER :: ival, istat, ncid, ntimes, a_time_slice, i, irank, ierr
    REAL :: rval
    DOUBLE PRECISION :: dval

    DOUBLE PRECISION :: valin

    var(var_id)%users_type = 'double'

    ierr = 0

    time_id = -1
    data_size = 0

    ! loop through input dimension
    irank = 7
    a_time_slice = 1
    DO j = 1, 7

       i = var(var_id)%dim_order(j)
       IF (i == 0) EXIT
       irank = j

       IF (axis(var(var_id)%axis_idxs(i))%axis == 'T') THEN
          time_id = j
       ELSE
          data_size(j) =  adim(var(var_id)%dim_idxs(i))%lendim
          a_time_slice=a_time_slice*data_size(i)
       END IF

    END DO


    j = 1

    IF (time_id > 0) THEN

       IF (PRESENT(ntimes_passed)) THEN

          data_size(time_id) = ntimes_passed

          IF (data_size(time_id) == 0) THEN
             PRINT*, 'Error encountered in cmor_write:'
             PRINT*, '  user passed ntimes_passed = 0 but this field'
             PRINT*, '  should have a time dimension.'
             STOP
          END IF

       ELSE

          data_size(time_id) = SIZE(DATA)/a_time_slice
          IF (data_size(time_id) == 0) THEN
             PRINT*, 'Error encountered in cmor_write:'
             PRINT*, '  user passed a data array that should be larger, '
             PRINT*, '  it cannot hold even a single time-slice of data.'
             STOP
          END IF

       END IF

       j = data_size(time_id)

    END IF

    IF (a_time_slice*j > SIZE(data)) THEN

       PRINT*, 'Error encountered in cmor_write:'
       PRINT*, '  The data array passed by user has fewer elements than it should'
       PRINT*, '  The axes defined indicate the following dimension lengths: ',&
            data_size
       PRINT*, '  for a total size of ',  &
            a_time_slice*j
       PRINT*,  ' but the users array is length: ', SIZE(data)
       STOP

    END IF

    ierr = check_args(var_id, ncid, ntimes_passed, time_vals, time_bnds, &
         store_with, file_suffix)
    IF (ierr < 0) RETURN

    ! Process time stuff, define dims_len and start, and for use in the 
    !     write_1d case (only??)  also defines a_time_slice

    CALL cmor_process_time(ncid, var_id, ntimes_passed, dims_len, start, &
         a_time_slice, time_vals, time_bnds, store_with)


    ! besides checking, the following defines ntimes

    CALL check_time_and_dim_lens(data_size, var_id, dims_len, ntimes, ierr, &
         ntimes_passed)
    IF (ierr.LT.0) RETURN


    ! dim_order(i) stores the output array index that corresponds to 
    !   input array index i; it stores 0 for output dimensions > the
    !   rank of the output array.
    !   Singleton (scalar) dimensions that the user hasn't overridden are
    !   omitted.

    ! inout is similar to dim_order, but stores i for output dimensions > the
    !   rank of the output array.

    DO i=1,max_rank
       inout(i) = var(var_id)%dim_order(i)
       if (inout(i) == 0) inout(i) = i
    END DO

    ALLOCATE(indx1(dims_len(inout(1))), STAT=istat)
    ALLOCATE(indx2(dims_len(inout(2))), STAT=istat)
    ALLOCATE(indx3(dims_len(inout(3))), STAT=istat)
    ALLOCATE(indx4(dims_len(inout(4))), STAT=istat)
    ALLOCATE(indx5(dims_len(inout(5))), STAT=istat)
    ALLOCATE(indx6(dims_len(inout(6))), STAT=istat)
    ALLOCATE(indx7(dims_len(inout(7))), STAT=istat)

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*,  '   Unable to allocate memory for one of the "idx?" arrays,' 
       PRINT*,  '   which have one of the following dimensions: ', dims_len
       STOP
    END IF

    ! The following generates an index array, indx?(i), containing the location
    !    in the output array corresponding to input location i.
    ! indx1 is index for input array dimension 1, 
    !    indx2 for input dimension 2, etc.

    indx1 = offset_or_reverse_dim(var_id,1,dims_len(inout(1)))
    indx2 = offset_or_reverse_dim(var_id,2,dims_len(inout(2)))
    indx3 = offset_or_reverse_dim(var_id,3,dims_len(inout(3)))
    indx4 = offset_or_reverse_dim(var_id,4,dims_len(inout(4)))
    indx5 = offset_or_reverse_dim(var_id,5,dims_len(inout(5)))
    indx6 = offset_or_reverse_dim(var_id,6,dims_len(inout(6)))
    indx7 = offset_or_reverse_dim(var_id,7,dims_len(inout(7)))

    ! now assign input data to appropriate positions in output array.

    SELECT CASE(TRIM(var(var_id)%type))

    CASE("real")

       ALLOCATE(data_out_r(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("integer")

       ALLOCATE(data_out_i(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("double")

       ALLOCATE(data_out_d(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    END SELECT

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*, '   Unable to allocate memory for one of the "data_out",'
       PRINT*, '   which has the following dimensions: ', dims_len
       STOP
    END IF
    
    loc = 1
 
    DO i7 = 1, dims_len(inout(7))
       loc(inout(7)) = indx7(i7)
       ipt7 = (i7-1)*data_size(6)
       DO i6 = 1, dims_len(inout(6))
          loc(inout(6)) = indx6(i6)
          ipt6 = (ipt7 + i6-1)*data_size(5)
          DO i5 = 1, dims_len(inout(5))
             loc(inout(5)) = indx5(i5)
             ipt5 = (ipt6 + i5-1)*data_size(4)
             DO i4 = 1, dims_len(inout(4))
                loc(inout(4)) = indx4(i4)
                ipt4 = (ipt5 + i4-1)*data_size(3)
                DO i3 = 1, dims_len(inout(3))
                   loc(inout(3)) = indx3(i3)
                   ipt3 = (ipt4 + i3-1)*data_size(2)
                   DO i2 = 1, dims_len(inout(2))
                      loc(inout(2)) = indx2(i2)
                      ipt2 = (ipt3 + i2-1)*data_size(1)
                      DO i1 = 1, dims_len(inout(1))
                         loc(inout(1)) = indx1(i1)
                         ipt1 = ipt2 + i1
                            
                         valin = data(ipt1)
                         
                         SELECT CASE(TRIM(var(var_id)%type))

                         CASE("real")
                            
                            IF ((var(var_id)%def_missing > 0) .and.  &
                                 (ABS(valin-var(var_id)%users_missing_dbl) &
                                 < ABS(var(var_id)%users_missing_dbl * &
                                 var(var_id)%miss_val_tolerance))) THEN

                               i = var(var_id)%mip_table_idx
                               rval = mip_tbl(i)%single_missing

                            ELSE
                               
                               rval = var(var_id)%offset +       &
                                    valin*var(var_id)%scale_factor
                               
                               IF ((var(var_id)%valid_min /=     &
                                    init_undef_single) .and. &
                                    (rval < var(var_id)%valid_min)) THEN
                                  call errmsg_d('min', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               ELSE IF ((var(var_id)%valid_max /=    &
                                    init_undef_single) .and. &
                                    (rval > var(var_id)%valid_max)) THEN
                                  call errmsg_d('max', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               END IF
                               
                            END IF
                            
                            data_out_r(loc(1),loc(2),loc(3),loc(4),  &
                                 loc(5),loc(6),loc(7)) = rval

                         CASE("integer")
                            
                            IF ((var(var_id)%def_missing > 0) .and.  &
                                 (ABS(valin-var(var_id)%users_missing_dbl) &
                                 < ABS(var(var_id)%users_missing_dbl * &
                                 var(var_id)%miss_val_tolerance))) THEN

                               i = var(var_id)%mip_table_idx
                               ival = mip_tbl(i)%integer_missing

                            ELSE
                               
                               ival = var(var_id)%offset +       &
                                    valin*var(var_id)%scale_factor
                               
                               IF ((var(var_id)%valid_min /=     &
                                    init_undef_single) .and. &
                                    (ival < var(var_id)%valid_min)) THEN
                                  call errmsg_d('min', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               ELSE IF ((var(var_id)%valid_max /=    &
                                    init_undef_single) .and. &
                                    (ival > var(var_id)%valid_max)) THEN
                                  call errmsg_d('max', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               END IF

                               
                            END IF
                            
                            data_out_i(loc(1),loc(2),loc(3),loc(4),  &
                                 loc(5),loc(6),loc(7)) = ival

                         CASE("double")
                            
                            IF ((var(var_id)%def_missing > 0) .and.  &
                                 (ABS(valin-var(var_id)%users_missing_dbl) &
                                 < ABS(var(var_id)%users_missing_dbl * &
                                 var(var_id)%miss_val_tolerance))) THEN
                               i = var(var_id)%mip_table_idx
                               dval = mip_tbl(i)%integer_missing
                            ELSE
                               
                               dval = var(var_id)%offset +       &
                                    valin*var(var_id)%scale_factor
                               
                               IF ((var(var_id)%valid_min /=     &
                                    init_undef_single) .and. &
                                    (dval < var(var_id)%valid_min)) THEN
                                  call errmsg_d('min', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               ELSE IF ((var(var_id)%valid_max /=    &
                                    init_undef_single) .and. &
                                    (dval > var(var_id)%valid_max)) THEN
                                  call errmsg_d('max', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               END IF
                               
                            END IF
                            
                            data_out_d(loc(1),loc(2),loc(3),loc(4),  &
                                 loc(5),loc(6),loc(7)) = dval
                            
                         END SELECT
                         
                      END DO
                   END DO
                END DO
             END DO
          END DO
       END DO
    END DO

    SELECT CASE(TRIM(var(var_id)%type))

    CASE('real')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_r, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_r)
       
    CASE('integer')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_i, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_i)
       
    CASE('double')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_d, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_d)
       
    END SELECT
           
    DEALLOCATE(indx1)
    DEALLOCATE(indx2)
    DEALLOCATE(indx3)
    DEALLOCATE(indx4)
    DEALLOCATE(indx5)
    DEALLOCATE(indx6)
    DEALLOCATE(indx7)
        
    IF (ierr == NF90_NOERR) THEN
       IF (ntimes /= 0) THEN
          IF (PRESENT(store_with)) THEN
             var(var_id)%zfac_times_written(store_with) =   &
                  var(var_id)%zfac_times_written(store_with)+ntimes
          ELSE
             var(var_id)%times_written=var(var_id)%times_written+ntimes
          END IF
       END IF
    ELSE
       PRINT*, 'Error writing NetCDF out data: error number =  ', ierr
       PRINT*, '   netCDF error message: ',  TRIM(nf90_strerror(ierr))
       STOP
    END IF
    
    RETURN
  END FUNCTION cmor_write_1d_d




  FUNCTION cmor_write_2d_d(var_id, data, file_suffix, ntimes_passed,  &
       time_vals, time_bnds, store_with) RESULT(ierr)

    IMPLICIT NONE

    DOUBLE PRECISION, INTENT(in):: data(:,:)
    INTEGER, INTENT(in) ::var_id
    CHARACTER(len=*), INTENT(in), OPTIONAL :: file_suffix
    INTEGER, INTENT(in), OPTIONAL :: ntimes_passed
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_vals(:)
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_bnds(:,:)
    INTEGER, INTENT(in), OPTIONAL :: store_with

    INTEGER, ALLOCATABLE, DIMENSION(:) :: indx1, indx2
    INTEGER, ALLOCATABLE :: data_out_i(:,:,:,:,:,:,:)
    REAL, ALLOCATABLE :: data_out_r(:,:,:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: data_out_d(:,:,:,:,:,:,:)
    INTEGER, DIMENSION(max_rank) :: inout, loc, dims_len, start, data_size
    INTEGER :: i1=1, i2=1, i3=1, i4=1, i5=1, i6=1, i7=1
    INTEGER :: ival, istat, ncid, ntimes, a_time_slice, i, irank, ierr
    REAL :: rval
    DOUBLE PRECISION :: dval

    DOUBLE PRECISION :: valin

    var(var_id)%users_type = 'double'
    irank = 2

    ierr = 0

    ! store input array's dimension lengths
    data_size = 0
    DO i =1 , irank
       data_size(i)=SIZE(data,i)
    END DO

    ierr = check_args(var_id, ncid, ntimes_passed, time_vals, time_bnds, &
         store_with, file_suffix)
    IF (ierr < 0) RETURN

    ! Process time stuff, define dims_len and start, and for use in the 
    !     write_1d case (only??)  also defines a_time_slice

    CALL cmor_process_time(ncid, var_id, ntimes_passed, dims_len, start, &
         a_time_slice, time_vals, time_bnds, store_with)


    ! besides checking, the following defines ntimes

    CALL check_time_and_dim_lens(data_size, var_id, dims_len, ntimes, ierr, &
         ntimes_passed)
    IF (ierr.LT.0) RETURN


    ! dim_order(i) stores the output array index that corresponds to 
    !   input array index i; it stores 0 for output dimensions > the
    !   rank of the output array.
    !   Singleton (scalar) dimensions that the user hasn't overridden are
    !   omitted.

    ! inout is similar to dim_order, but stores i for output dimensions > the
    !   rank of the output array.

    DO i=1,max_rank
       inout(i) = var(var_id)%dim_order(i)
       if (inout(i) == 0) inout(i) = i
    END DO

    ALLOCATE(indx1(dims_len(inout(1))), STAT=istat)
    ALLOCATE(indx2(dims_len(inout(2))), STAT=istat)

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*,  '   Unable to allocate memory for one of the "idx?" arrays,' 
       PRINT*,  '   which have one of the following dimensions: ', dims_len
       STOP
    END IF

    ! The following generates an index array, indx?(i), containing the location
    !    in the output array corresponding to input location i.
    ! indx1 is index for input array dimension 1, 
    !    indx2 for input dimension 2, etc.

    indx1 = offset_or_reverse_dim(var_id,1,dims_len(inout(1)))
    indx2 = offset_or_reverse_dim(var_id,2,dims_len(inout(2)))

    ! now assign input data to appropriate positions in output array.

    SELECT CASE(TRIM(var(var_id)%type))

    CASE("real")

       ALLOCATE(data_out_r(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("integer")

       ALLOCATE(data_out_i(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("double")

       ALLOCATE(data_out_d(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    END SELECT

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*, '   Unable to allocate memory for one of the "data_out",'
       PRINT*, '   which has the following dimensions: ', dims_len
       STOP
    END IF
    
    loc = 1
    
    DO i2 = 1, dims_len(inout(2))
       loc(inout(2)) = indx2(i2)
       DO i1 = 1, dims_len(inout(1))
          loc(inout(1)) = indx1(i1)

          valin = data(i1,i2)

          SELECT CASE(TRIM(var(var_id)%type))

          CASE("real")

             IF ((var(var_id)%def_missing > 0) .and.  &
                  (ABS(valin-var(var_id)%users_missing_dbl) &
                  < ABS(var(var_id)%users_missing_dbl * &
                  var(var_id)%miss_val_tolerance))) THEN

                i = var(var_id)%mip_table_idx
                rval = mip_tbl(i)%single_missing

             ELSE

                rval = var(var_id)%offset +       &
                     valin*var(var_id)%scale_factor

                IF ((var(var_id)%valid_min /=     &
                     init_undef_single) .and. &
                     (rval < var(var_id)%valid_min)) THEN
                   call errmsg_d('min', irank, var_id, valin, &
                        rval, ival, dval, i1, i2, i3, i4, i5, &
                        i6, i7)
                ELSE IF ((var(var_id)%valid_max /=    &
                     init_undef_single) .and. &
                     (rval > var(var_id)%valid_max)) THEN
                   call errmsg_d('max', irank, var_id, valin, &
                        rval, ival, dval, i1, i2, i3, i4, i5, &
                        i6, i7)
                END IF

             END IF

             data_out_r(loc(1),loc(2),loc(3),loc(4),  &
                  loc(5),loc(6),loc(7)) = rval

          CASE("integer")

             IF ((var(var_id)%def_missing > 0) .and.  &
                  (ABS(valin-var(var_id)%users_missing_dbl) &
                  < ABS(var(var_id)%users_missing_dbl * &
                  var(var_id)%miss_val_tolerance))) THEN

                i = var(var_id)%mip_table_idx
                ival = mip_tbl(i)%integer_missing

             ELSE

                ival = var(var_id)%offset +       &
                     valin*var(var_id)%scale_factor

                IF ((var(var_id)%valid_min /=     &
                     init_undef_single) .and. &
                     (ival < var(var_id)%valid_min)) THEN
                   call errmsg_d('min', irank, var_id, valin, &
                        rval, ival, dval, i1, i2, i3, i4, i5, &
                        i6, i7)
                ELSE IF ((var(var_id)%valid_max /=    &
                     init_undef_single) .and. &
                     (ival > var(var_id)%valid_max)) THEN
                   call errmsg_d('max', irank, var_id, valin, &
                        rval, ival, dval, i1, i2, i3, i4, i5, &
                        i6, i7)
                END IF


             END IF

             data_out_i(loc(1),loc(2),loc(3),loc(4),  &
                  loc(5),loc(6),loc(7)) = ival

          CASE("double")

             IF ((var(var_id)%def_missing > 0) .and.  &
                  (ABS(valin-var(var_id)%users_missing_dbl) &
                  < ABS(var(var_id)%users_missing_dbl * &
                  var(var_id)%miss_val_tolerance))) THEN
                i = var(var_id)%mip_table_idx
                dval = mip_tbl(i)%integer_missing
             ELSE

                dval = var(var_id)%offset +       &
                     valin*var(var_id)%scale_factor

                IF ((var(var_id)%valid_min /=     &
                     init_undef_single) .and. &
                     (dval < var(var_id)%valid_min)) THEN
                   call errmsg_d('min', irank, var_id, valin, &
                        rval, ival, dval, i1, i2, i3, i4, i5, &
                        i6, i7)
                ELSE IF ((var(var_id)%valid_max /=    &
                     init_undef_single) .and. &
                     (dval > var(var_id)%valid_max)) THEN
                   call errmsg_d('max', irank, var_id, valin, &
                        rval, ival, dval, i1, i2, i3, i4, i5, &
                        i6, i7)
                END IF

             END IF

             data_out_d(loc(1),loc(2),loc(3),loc(4),  &
                  loc(5),loc(6),loc(7)) = dval

          END SELECT

       END DO
    END DO

    SELECT CASE(TRIM(var(var_id)%type))

    CASE('real')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_r, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_r)
       
    CASE('integer')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_i, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_i)
       
    CASE('double')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_d, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_d)
       
    END SELECT
           
    DEALLOCATE(indx1)
    DEALLOCATE(indx2)
    
    IF (ierr == NF90_NOERR) THEN
       IF (ntimes /= 0) THEN
          IF (PRESENT(store_with)) THEN
             var(var_id)%zfac_times_written(store_with) =   &
                  var(var_id)%zfac_times_written(store_with)+ntimes
          ELSE
             var(var_id)%times_written=var(var_id)%times_written+ntimes
          END IF
       END IF
    ELSE
       PRINT*, 'Error writing NetCDF out data: error number =  ', ierr
       PRINT*, '   netCDF error message: ',  TRIM(nf90_strerror(ierr))
       STOP
    END IF
    
    RETURN
  END FUNCTION cmor_write_2d_d



  FUNCTION cmor_write_3d_d(var_id, data, file_suffix, ntimes_passed,  &
       time_vals, time_bnds, store_with) RESULT(ierr)

    IMPLICIT NONE

    DOUBLE PRECISION, INTENT(in):: data(:,:,:)
    INTEGER, INTENT(in) ::var_id
    CHARACTER(len=*), INTENT(in), OPTIONAL :: file_suffix
    INTEGER, INTENT(in), OPTIONAL :: ntimes_passed
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_vals(:)
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_bnds(:,:)
    INTEGER, INTENT(in), OPTIONAL :: store_with

    INTEGER, ALLOCATABLE, DIMENSION(:) :: indx1, indx2, indx3
    INTEGER, ALLOCATABLE :: data_out_i(:,:,:,:,:,:,:)
    REAL, ALLOCATABLE :: data_out_r(:,:,:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: data_out_d(:,:,:,:,:,:,:)
    INTEGER, DIMENSION(max_rank) :: inout, loc, dims_len, start, data_size
    INTEGER :: i1=1, i2=1, i3=1, i4=1, i5=1, i6=1, i7=1
    INTEGER :: ival, istat, ncid, ntimes, a_time_slice, i, irank, ierr
    REAL :: rval
    DOUBLE PRECISION :: dval

    DOUBLE PRECISION :: valin

    var(var_id)%users_type = 'double'
    irank = 3

    ierr = 0

    ! store input array's dimension lengths
    data_size = 0
    DO i =1 , irank
       data_size(i)=SIZE(data,i)
    END DO

    ierr = check_args(var_id, ncid, ntimes_passed, time_vals, time_bnds, &
         store_with, file_suffix)
    IF (ierr < 0) RETURN

    ! Process time stuff, define dims_len and start, and for use in the 
    !     write_1d case (only??)  also defines a_time_slice

    CALL cmor_process_time(ncid, var_id, ntimes_passed, dims_len, start, &
         a_time_slice, time_vals, time_bnds, store_with)


    ! besides checking, the following defines ntimes

    CALL check_time_and_dim_lens(data_size, var_id, dims_len, ntimes, ierr, &
         ntimes_passed)
    IF (ierr.LT.0) RETURN


    ! dim_order(i) stores the output array index that corresponds to 
    !   input array index i; it stores 0 for output dimensions > the
    !   rank of the output array.
    !   Singleton (scalar) dimensions that the user hasn't overridden are
    !   omitted.

    ! inout is similar to dim_order, but stores i for output dimensions > the
    !   rank of the output array.

    DO i=1,max_rank
       inout(i) = var(var_id)%dim_order(i)
       if (inout(i) == 0) inout(i) = i
    END DO

    ALLOCATE(indx1(dims_len(inout(1))), STAT=istat)
    ALLOCATE(indx2(dims_len(inout(2))), STAT=istat)
    ALLOCATE(indx3(dims_len(inout(3))), STAT=istat)

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*,  '   Unable to allocate memory for one of the "idx?" arrays,' 
       PRINT*,  '   which have one of the following dimensions: ', dims_len
       STOP
    END IF

    ! The following generates an index array, indx?(i), containing the location
    !    in the output array corresponding to input location i.
    ! indx1 is index for input array dimension 1, 
    !    indx2 for input dimension 2, etc.

    indx1 = offset_or_reverse_dim(var_id,1,dims_len(inout(1)))
    indx2 = offset_or_reverse_dim(var_id,2,dims_len(inout(2)))
    indx3 = offset_or_reverse_dim(var_id,3,dims_len(inout(3)))

    ! now assign input data to appropriate positions in output array.

    SELECT CASE(TRIM(var(var_id)%type))

    CASE("real")

       ALLOCATE(data_out_r(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("integer")

       ALLOCATE(data_out_i(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("double")

       ALLOCATE(data_out_d(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    END SELECT

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*, '   Unable to allocate memory for one of the "data_out",'
       PRINT*, '   which has the following dimensions: ', dims_len
       STOP
    END IF
    
    loc = 1
    
    DO i3 = 1, dims_len(inout(3))
       loc(inout(3)) = indx3(i3)
       DO i2 = 1, dims_len(inout(2))
          loc(inout(2)) = indx2(i2)
          DO i1 = 1, dims_len(inout(1))
             loc(inout(1)) = indx1(i1)

             valin = data(i1,i2,i3)

             SELECT CASE(TRIM(var(var_id)%type))

             CASE("real")

                IF ((var(var_id)%def_missing > 0) .and.  &
                     (ABS(valin-var(var_id)%users_missing_dbl) &
                     < ABS(var(var_id)%users_missing_dbl * &
                     var(var_id)%miss_val_tolerance))) THEN

                   i = var(var_id)%mip_table_idx
                   rval = mip_tbl(i)%single_missing

                ELSE

                   rval = var(var_id)%offset +       &
                        valin*var(var_id)%scale_factor

                   IF ((var(var_id)%valid_min /=     &
                        init_undef_single) .and. &
                        (rval < var(var_id)%valid_min)) THEN
                      call errmsg_d('min', irank, var_id, valin, &
                           rval, ival, dval, i1, i2, i3, i4, i5, &
                           i6, i7)
                   ELSE IF ((var(var_id)%valid_max /=    &
                        init_undef_single) .and. &
                        (rval > var(var_id)%valid_max)) THEN
                      call errmsg_d('max', irank, var_id, valin, &
                           rval, ival, dval, i1, i2, i3, i4, i5, &
                           i6, i7)
                   END IF

                END IF

                data_out_r(loc(1),loc(2),loc(3),loc(4),  &
                     loc(5),loc(6),loc(7)) = rval

             CASE("integer")

                IF ((var(var_id)%def_missing > 0) .and.  &
                     (ABS(valin-var(var_id)%users_missing_dbl) &
                     < ABS(var(var_id)%users_missing_dbl * &
                     var(var_id)%miss_val_tolerance))) THEN

                   i = var(var_id)%mip_table_idx
                   ival = mip_tbl(i)%integer_missing

                ELSE

                   ival = var(var_id)%offset +       &
                        valin*var(var_id)%scale_factor

                   IF ((var(var_id)%valid_min /=     &
                        init_undef_single) .and. &
                        (ival < var(var_id)%valid_min)) THEN
                      call errmsg_d('min', irank, var_id, valin, &
                           rval, ival, dval, i1, i2, i3, i4, i5, &
                           i6, i7)
                   ELSE IF ((var(var_id)%valid_max /=    &
                        init_undef_single) .and. &
                        (ival > var(var_id)%valid_max)) THEN
                      call errmsg_d('max', irank, var_id, valin, &
                           rval, ival, dval, i1, i2, i3, i4, i5, &
                           i6, i7)
                   END IF


                END IF

                data_out_i(loc(1),loc(2),loc(3),loc(4),  &
                     loc(5),loc(6),loc(7)) = ival

             CASE("double")

                IF ((var(var_id)%def_missing > 0) .and.  &
                     (ABS(valin-var(var_id)%users_missing_dbl) &
                     < ABS(var(var_id)%users_missing_dbl * &
                     var(var_id)%miss_val_tolerance))) THEN
                   i = var(var_id)%mip_table_idx
                   dval = mip_tbl(i)%integer_missing
                ELSE

                   dval = var(var_id)%offset +       &
                        valin*var(var_id)%scale_factor

                   IF ((var(var_id)%valid_min /=     &
                        init_undef_single) .and. &
                        (dval < var(var_id)%valid_min)) THEN
                      call errmsg_d('min', irank, var_id, valin, &
                           rval, ival, dval, i1, i2, i3, i4, i5, &
                           i6, i7)
                   ELSE IF ((var(var_id)%valid_max /=    &
                        init_undef_single) .and. &
                        (dval > var(var_id)%valid_max)) THEN
                      call errmsg_d('max', irank, var_id, valin, &
                           rval, ival, dval, i1, i2, i3, i4, i5, &
                           i6, i7)
                   END IF

                END IF

                data_out_d(loc(1),loc(2),loc(3),loc(4),  &
                     loc(5),loc(6),loc(7)) = dval

             END SELECT

          END DO
       END DO
    END DO

    SELECT CASE(TRIM(var(var_id)%type))

    CASE('real')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_r, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_r)
       
    CASE('integer')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_i, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_i)
       
    CASE('double')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_d, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_d)
       
    END SELECT
           
    DEALLOCATE(indx1)
    DEALLOCATE(indx2)
    DEALLOCATE(indx3)
    
    IF (ierr == NF90_NOERR) THEN
       IF (ntimes /= 0) THEN
          IF (PRESENT(store_with)) THEN
             var(var_id)%zfac_times_written(store_with) =   &
                  var(var_id)%zfac_times_written(store_with)+ntimes
          ELSE
             var(var_id)%times_written=var(var_id)%times_written+ntimes
          END IF
       END IF
    ELSE
       PRINT*, 'Error writing NetCDF out data: error number =  ', ierr
       PRINT*, '   netCDF error message: ',  TRIM(nf90_strerror(ierr))
       STOP
    END IF
    
    RETURN
  END FUNCTION cmor_write_3d_d



  FUNCTION cmor_write_4d_d(var_id, data, file_suffix, ntimes_passed,  &
       time_vals, time_bnds, store_with) RESULT(ierr)

    IMPLICIT NONE

    DOUBLE PRECISION, INTENT(in):: data(:,:,:,:)
    INTEGER, INTENT(in) ::var_id
    CHARACTER(len=*), INTENT(in), OPTIONAL :: file_suffix
    INTEGER, INTENT(in), OPTIONAL :: ntimes_passed
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_vals(:)
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_bnds(:,:)
    INTEGER, INTENT(in), OPTIONAL :: store_with

    INTEGER, ALLOCATABLE, DIMENSION(:) :: indx1, indx2, indx3, indx4
    INTEGER, ALLOCATABLE :: data_out_i(:,:,:,:,:,:,:)
    REAL, ALLOCATABLE :: data_out_r(:,:,:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: data_out_d(:,:,:,:,:,:,:)
    INTEGER, DIMENSION(max_rank) :: inout, loc, dims_len, start, data_size
    INTEGER :: i1=1, i2=1, i3=1, i4=1, i5=1, i6=1, i7=1
    INTEGER :: ival, istat, ncid, ntimes, a_time_slice, i, irank, ierr
    REAL :: rval
    DOUBLE PRECISION :: dval

    DOUBLE PRECISION :: valin

    var(var_id)%users_type = 'double'
    irank = 4

    ierr = 0

    ! store input array's dimension lengths
    data_size = 0
    DO i =1 , irank
       data_size(i)=SIZE(data,i)
    END DO

    ierr = check_args(var_id, ncid, ntimes_passed, time_vals, time_bnds, &
         store_with, file_suffix)
    IF (ierr < 0) RETURN

    ! Process time stuff, define dims_len and start, and for use in the 
    !     write_1d case (only??)  also defines a_time_slice

    CALL cmor_process_time(ncid, var_id, ntimes_passed, dims_len, start, &
         a_time_slice, time_vals, time_bnds, store_with)


    ! besides checking, the following defines ntimes

    CALL check_time_and_dim_lens(data_size, var_id, dims_len, ntimes, ierr, &
         ntimes_passed)
    IF (ierr.LT.0) RETURN


    ! dim_order(i) stores the output array index that corresponds to 
    !   input array index i; it stores 0 for output dimensions > the
    !   rank of the output array.
    !   Singleton (scalar) dimensions that the user hasn't overridden are
    !   omitted.

    ! inout is similar to dim_order, but stores i for output dimensions > the
    !   rank of the output array.

    DO i=1,max_rank
       inout(i) = var(var_id)%dim_order(i)
       if (inout(i) == 0) inout(i) = i
    END DO

    ALLOCATE(indx1(dims_len(inout(1))), STAT=istat)
    ALLOCATE(indx2(dims_len(inout(2))), STAT=istat)
    ALLOCATE(indx3(dims_len(inout(3))), STAT=istat)
    ALLOCATE(indx4(dims_len(inout(4))), STAT=istat)

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*,  '   Unable to allocate memory for one of the "idx?" arrays,' 
       PRINT*,  '   which have one of the following dimensions: ', dims_len
       STOP
    END IF

    ! The following generates an index array, indx?(i), containing the location
    !    in the output array corresponding to input location i.
    ! indx1 is index for input array dimension 1, 
    !    indx2 for input dimension 2, etc.

    indx1 = offset_or_reverse_dim(var_id,1,dims_len(inout(1)))
    indx2 = offset_or_reverse_dim(var_id,2,dims_len(inout(2)))
    indx3 = offset_or_reverse_dim(var_id,3,dims_len(inout(3)))
    indx4 = offset_or_reverse_dim(var_id,4,dims_len(inout(4)))

    ! now assign input data to appropriate positions in output array.

    SELECT CASE(TRIM(var(var_id)%type))

    CASE("real")

       ALLOCATE(data_out_r(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("integer")

       ALLOCATE(data_out_i(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("double")

       ALLOCATE(data_out_d(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    END SELECT

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*, '   Unable to allocate memory for one of the "data_out",'
       PRINT*, '   which has the following dimensions: ', dims_len
       STOP
    END IF
    
    loc = 1
    
    DO i4 = 1, dims_len(inout(4))
       loc(inout(4)) = indx4(i4)
       DO i3 = 1, dims_len(inout(3))
          loc(inout(3)) = indx3(i3)
          DO i2 = 1, dims_len(inout(2))
             loc(inout(2)) = indx2(i2)
             DO i1 = 1, dims_len(inout(1))
                loc(inout(1)) = indx1(i1)

                valin = data(i1,i2,i3,i4)

                SELECT CASE(TRIM(var(var_id)%type))

                CASE("real")

                   IF ((var(var_id)%def_missing > 0) .and.  &
                        (ABS(valin-var(var_id)%users_missing_dbl) &
                        < ABS(var(var_id)%users_missing_dbl * &
                        var(var_id)%miss_val_tolerance))) THEN

                      i = var(var_id)%mip_table_idx
                      rval = mip_tbl(i)%single_missing

                   ELSE

                      rval = var(var_id)%offset +       &
                           valin*var(var_id)%scale_factor

                      IF ((var(var_id)%valid_min /=     &
                           init_undef_single) .and. &
                           (rval < var(var_id)%valid_min)) THEN
                         call errmsg_d('min', irank, var_id, valin, &
                              rval, ival, dval, i1, i2, i3, i4, i5, &
                              i6, i7)
                      ELSE IF ((var(var_id)%valid_max /=    &
                           init_undef_single) .and. &
                           (rval > var(var_id)%valid_max)) THEN
                         call errmsg_d('max', irank, var_id, valin, &
                              rval, ival, dval, i1, i2, i3, i4, i5, &
                              i6, i7)
                      END IF

                   END IF

                   data_out_r(loc(1),loc(2),loc(3),loc(4),  &
                        loc(5),loc(6),loc(7)) = rval

                CASE("integer")

                   IF ((var(var_id)%def_missing > 0) .and.  &
                        (ABS(valin-var(var_id)%users_missing_dbl) &
                        < ABS(var(var_id)%users_missing_dbl * &
                        var(var_id)%miss_val_tolerance))) THEN

                      i = var(var_id)%mip_table_idx
                      ival = mip_tbl(i)%integer_missing

                   ELSE

                      ival = var(var_id)%offset +       &
                           valin*var(var_id)%scale_factor

                      IF ((var(var_id)%valid_min /=     &
                           init_undef_single) .and. &
                           (ival < var(var_id)%valid_min)) THEN
                         call errmsg_d('min', irank, var_id, valin, &
                              rval, ival, dval, i1, i2, i3, i4, i5, &
                              i6, i7)
                      ELSE IF ((var(var_id)%valid_max /=    &
                           init_undef_single) .and. &
                           (ival > var(var_id)%valid_max)) THEN
                         call errmsg_d('max', irank, var_id, valin, &
                              rval, ival, dval, i1, i2, i3, i4, i5, &
                              i6, i7)
                      END IF


                   END IF

                   data_out_i(loc(1),loc(2),loc(3),loc(4),  &
                        loc(5),loc(6),loc(7)) = ival

                CASE("double")

                   IF ((var(var_id)%def_missing > 0) .and.  &
                        (ABS(valin-var(var_id)%users_missing_dbl) &
                        < ABS(var(var_id)%users_missing_dbl * &
                        var(var_id)%miss_val_tolerance))) THEN
                      i = var(var_id)%mip_table_idx
                      dval = mip_tbl(i)%integer_missing
                   ELSE

                      dval = var(var_id)%offset +       &
                           valin*var(var_id)%scale_factor

                      IF ((var(var_id)%valid_min /=     &
                           init_undef_single) .and. &
                           (dval < var(var_id)%valid_min)) THEN
                         call errmsg_d('min', irank, var_id, valin, &
                              rval, ival, dval, i1, i2, i3, i4, i5, &
                              i6, i7)
                      ELSE IF ((var(var_id)%valid_max /=    &
                           init_undef_single) .and. &
                           (dval > var(var_id)%valid_max)) THEN
                         call errmsg_d('max', irank, var_id, valin, &
                              rval, ival, dval, i1, i2, i3, i4, i5, &
                              i6, i7)
                      END IF

                   END IF

                   data_out_d(loc(1),loc(2),loc(3),loc(4),  &
                        loc(5),loc(6),loc(7)) = dval

                END SELECT

             END DO
          END DO
       END DO
    END DO

    SELECT CASE(TRIM(var(var_id)%type))

    CASE('real')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_r, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_r)
       
    CASE('integer')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_i, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_i)
       
    CASE('double')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_d, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_d)
       
    END SELECT
           
    DEALLOCATE(indx1)
    DEALLOCATE(indx2)
    DEALLOCATE(indx3)
    DEALLOCATE(indx4)
    
    IF (ierr == NF90_NOERR) THEN
       IF (ntimes /= 0) THEN
          IF (PRESENT(store_with)) THEN
             var(var_id)%zfac_times_written(store_with) =   &
                  var(var_id)%zfac_times_written(store_with)+ntimes
          ELSE
             var(var_id)%times_written=var(var_id)%times_written+ntimes
          END IF
       END IF
    ELSE
       PRINT*, 'Error writing NetCDF out data: error number =  ', ierr
       PRINT*, '   netCDF error message: ',  TRIM(nf90_strerror(ierr))
       STOP
    END IF
    
    RETURN
  END FUNCTION cmor_write_4d_d



 


  FUNCTION cmor_write_5d_d(var_id, data, file_suffix, ntimes_passed,  &
       time_vals, time_bnds, store_with) RESULT(ierr)

    IMPLICIT NONE

    DOUBLE PRECISION, INTENT(in):: data(:,:,:,:,:)
    INTEGER, INTENT(in) ::var_id
    CHARACTER(len=*), INTENT(in), OPTIONAL :: file_suffix
    INTEGER, INTENT(in), OPTIONAL :: ntimes_passed
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_vals(:)
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_bnds(:,:)
    INTEGER, INTENT(in), OPTIONAL :: store_with

    INTEGER, ALLOCATABLE, DIMENSION(:) :: indx1, indx2, indx3, indx4, &
         indx5
    INTEGER, ALLOCATABLE :: data_out_i(:,:,:,:,:,:,:)
    REAL, ALLOCATABLE :: data_out_r(:,:,:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: data_out_d(:,:,:,:,:,:,:)
    INTEGER, DIMENSION(max_rank) :: inout, loc, dims_len, start, data_size
    INTEGER :: i1=1, i2=1, i3=1, i4=1, i5=1, i6=1, i7=1
    INTEGER :: ival, istat, ncid, ntimes, a_time_slice, i, irank, ierr
    REAL :: rval
    DOUBLE PRECISION :: dval

    DOUBLE PRECISION :: valin

    var(var_id)%users_type = 'double'
    irank = 5

    ierr = 0

    ! store input array's dimension lengths
    data_size = 0
    DO i =1 , irank
       data_size(i)=SIZE(data,i)
    END DO

    ierr = check_args(var_id, ncid, ntimes_passed, time_vals, time_bnds, &
         store_with, file_suffix)
    IF (ierr < 0) RETURN

    ! Process time stuff, define dims_len and start, and for use in the 
    !     write_1d case (only??)  also defines a_time_slice

    CALL cmor_process_time(ncid, var_id, ntimes_passed, dims_len, start, &
         a_time_slice, time_vals, time_bnds, store_with)


    ! besides checking, the following defines ntimes

    CALL check_time_and_dim_lens(data_size, var_id, dims_len, ntimes, ierr, &
         ntimes_passed)
    IF (ierr.LT.0) RETURN


    ! dim_order(i) stores the output array index that corresponds to 
    !   input array index i; it stores 0 for output dimensions > the
    !   rank of the output array.
    !   Singleton (scalar) dimensions that the user hasn't overridden are
    !   omitted.

    ! inout is similar to dim_order, but stores i for output dimensions > the
    !   rank of the output array.

    DO i=1,max_rank
       inout(i) = var(var_id)%dim_order(i)
       if (inout(i) == 0) inout(i) = i
    END DO

    ALLOCATE(indx1(dims_len(inout(1))), STAT=istat)
    ALLOCATE(indx2(dims_len(inout(2))), STAT=istat)
    ALLOCATE(indx3(dims_len(inout(3))), STAT=istat)
    ALLOCATE(indx4(dims_len(inout(4))), STAT=istat)
    ALLOCATE(indx5(dims_len(inout(5))), STAT=istat)

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*,  '   Unable to allocate memory for one of the "idx?" arrays,' 
       PRINT*,  '   which have one of the following dimensions: ', dims_len
       STOP
    END IF

    ! The following generates an index array, indx?(i), containing the location
    !    in the output array corresponding to input location i.
    ! indx1 is index for input array dimension 1, 
    !    indx2 for input dimension 2, etc.

    indx1 = offset_or_reverse_dim(var_id,1,dims_len(inout(1)))
    indx2 = offset_or_reverse_dim(var_id,2,dims_len(inout(2)))
    indx3 = offset_or_reverse_dim(var_id,3,dims_len(inout(3)))
    indx4 = offset_or_reverse_dim(var_id,4,dims_len(inout(4)))
    indx5 = offset_or_reverse_dim(var_id,5,dims_len(inout(5)))

    ! now assign input data to appropriate positions in output array.

    SELECT CASE(TRIM(var(var_id)%type))

    CASE("real")

       ALLOCATE(data_out_r(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("integer")

       ALLOCATE(data_out_i(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("double")

       ALLOCATE(data_out_d(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    END SELECT

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*, '   Unable to allocate memory for one of the "data_out",'
       PRINT*, '   which has the following dimensions: ', dims_len
       STOP
    END IF
    
    loc = 1
    
    DO i5 = 1, dims_len(inout(5))
       loc(inout(5)) = indx5(i5)
       DO i4 = 1, dims_len(inout(4))
          loc(inout(4)) = indx4(i4)
          DO i3 = 1, dims_len(inout(3))
             loc(inout(3)) = indx3(i3)
             DO i2 = 1, dims_len(inout(2))
                loc(inout(2)) = indx2(i2)
                DO i1 = 1, dims_len(inout(1))
                   loc(inout(1)) = indx1(i1)

                   valin = data(i1,i2,i3,i4,i5)

                   SELECT CASE(TRIM(var(var_id)%type))

                   CASE("real")

                      IF ((var(var_id)%def_missing > 0) .and.  &
                           (ABS(valin-var(var_id)%users_missing_dbl) &
                           < ABS(var(var_id)%users_missing_dbl * &
                           var(var_id)%miss_val_tolerance))) THEN

                         i = var(var_id)%mip_table_idx
                         rval = mip_tbl(i)%single_missing

                      ELSE

                         rval = var(var_id)%offset +       &
                              valin*var(var_id)%scale_factor

                         IF ((var(var_id)%valid_min /=     &
                              init_undef_single) .and. &
                              (rval < var(var_id)%valid_min)) THEN
                            call errmsg_d('min', irank, var_id, valin, &
                                 rval, ival, dval, i1, i2, i3, i4, i5, &
                                 i6, i7)
                         ELSE IF ((var(var_id)%valid_max /=    &
                              init_undef_single) .and. &
                              (rval > var(var_id)%valid_max)) THEN
                            call errmsg_d('max', irank, var_id, valin, &
                                 rval, ival, dval, i1, i2, i3, i4, i5, &
                                 i6, i7)
                         END IF

                      END IF

                      data_out_r(loc(1),loc(2),loc(3),loc(4),  &
                           loc(5),loc(6),loc(7)) = rval

                   CASE("integer")

                      IF ((var(var_id)%def_missing > 0) .and.  &
                           (ABS(valin-var(var_id)%users_missing_dbl) &
                           < ABS(var(var_id)%users_missing_dbl * &
                           var(var_id)%miss_val_tolerance))) THEN

                         i = var(var_id)%mip_table_idx
                         ival = mip_tbl(i)%integer_missing

                      ELSE

                         ival = var(var_id)%offset +       &
                              valin*var(var_id)%scale_factor

                         IF ((var(var_id)%valid_min /=     &
                              init_undef_single) .and. &
                              (ival < var(var_id)%valid_min)) THEN
                            call errmsg_d('min', irank, var_id, valin, &
                                 rval, ival, dval, i1, i2, i3, i4, i5, &
                                 i6, i7)
                         ELSE IF ((var(var_id)%valid_max /=    &
                              init_undef_single) .and. &
                              (ival > var(var_id)%valid_max)) THEN
                            call errmsg_d('max', irank, var_id, valin, &
                                 rval, ival, dval, i1, i2, i3, i4, i5, &
                                 i6, i7)
                         END IF


                      END IF

                      data_out_i(loc(1),loc(2),loc(3),loc(4),  &
                           loc(5),loc(6),loc(7)) = ival

                   CASE("double")

                      IF ((var(var_id)%def_missing > 0) .and.  &
                           (ABS(valin-var(var_id)%users_missing_dbl) &
                           < ABS(var(var_id)%users_missing_dbl * &
                           var(var_id)%miss_val_tolerance))) THEN
                         i = var(var_id)%mip_table_idx
                         dval = mip_tbl(i)%integer_missing
                      ELSE

                         dval = var(var_id)%offset +       &
                              valin*var(var_id)%scale_factor

                         IF ((var(var_id)%valid_min /=     &
                              init_undef_single) .and. &
                              (dval < var(var_id)%valid_min)) THEN
                            call errmsg_d('min', irank, var_id, valin, &
                                 rval, ival, dval, i1, i2, i3, i4, i5, &
                                 i6, i7)
                         ELSE IF ((var(var_id)%valid_max /=    &
                              init_undef_single) .and. &
                              (dval > var(var_id)%valid_max)) THEN
                            call errmsg_d('max', irank, var_id, valin, &
                                 rval, ival, dval, i1, i2, i3, i4, i5, &
                                 i6, i7)
                         END IF

                      END IF

                      data_out_d(loc(1),loc(2),loc(3),loc(4),  &
                           loc(5),loc(6),loc(7)) = dval

                   END SELECT

                END DO
             END DO
          END DO
       END DO
    END DO

    SELECT CASE(TRIM(var(var_id)%type))

    CASE('real')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_r, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_r)
       
    CASE('integer')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_i, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_i)
       
    CASE('double')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_d, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_d)
       
    END SELECT
           
    DEALLOCATE(indx1)
    DEALLOCATE(indx2)
    DEALLOCATE(indx3)
    DEALLOCATE(indx4)
    DEALLOCATE(indx5)
        
    IF (ierr == NF90_NOERR) THEN
       IF (ntimes /= 0) THEN
          IF (PRESENT(store_with)) THEN
             var(var_id)%zfac_times_written(store_with) =   &
                  var(var_id)%zfac_times_written(store_with)+ntimes
          ELSE
             var(var_id)%times_written=var(var_id)%times_written+ntimes
          END IF
       END IF
    ELSE
       PRINT*, 'Error writing NetCDF out data: error number =  ', ierr
       PRINT*, '   netCDF error message: ',  TRIM(nf90_strerror(ierr))
       STOP
    END IF
    
    RETURN
  END FUNCTION cmor_write_5d_d




  FUNCTION cmor_write_6d_d(var_id, data, file_suffix, ntimes_passed,  &
       time_vals, time_bnds, store_with) RESULT(ierr)

    IMPLICIT NONE

    DOUBLE PRECISION, INTENT(in):: data(:,:,:,:,:,:)
    INTEGER, INTENT(in) ::var_id
    CHARACTER(len=*), INTENT(in), OPTIONAL :: file_suffix
    INTEGER, INTENT(in), OPTIONAL :: ntimes_passed
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_vals(:)
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_bnds(:,:)
    INTEGER, INTENT(in), OPTIONAL :: store_with

    INTEGER, ALLOCATABLE, DIMENSION(:) :: indx1, indx2, indx3, indx4, &
         indx5, indx6
    INTEGER, ALLOCATABLE :: data_out_i(:,:,:,:,:,:,:)
    REAL, ALLOCATABLE :: data_out_r(:,:,:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: data_out_d(:,:,:,:,:,:,:)
    INTEGER, DIMENSION(max_rank) :: inout, loc, dims_len, start, data_size
    INTEGER :: i1=1, i2=1, i3=1, i4=1, i5=1, i6=1, i7=1
    INTEGER :: ival, istat, ncid, ntimes, a_time_slice, i, irank, ierr
    REAL :: rval
    DOUBLE PRECISION :: dval

    DOUBLE PRECISION :: valin

    var(var_id)%users_type = 'double'
    irank = 6

    ierr = 0

    ! store input array's dimension lengths
    data_size = 0
    DO i =1 , irank
       data_size(i)=SIZE(data,i)
    END DO

    ierr = check_args(var_id, ncid, ntimes_passed, time_vals, time_bnds, &
         store_with, file_suffix)
    IF (ierr < 0) RETURN

    ! Process time stuff, define dims_len and start, and for use in the 
    !     write_1d case (only??)  also defines a_time_slice

    CALL cmor_process_time(ncid, var_id, ntimes_passed, dims_len, start, &
         a_time_slice, time_vals, time_bnds, store_with)


    ! besides checking, the following defines ntimes

    CALL check_time_and_dim_lens(data_size, var_id, dims_len, ntimes, ierr, &
         ntimes_passed)
    IF (ierr.LT.0) RETURN


    ! dim_order(i) stores the output array index that corresponds to 
    !   input array index i; it stores 0 for output dimensions > the
    !   rank of the output array.
    !   Singleton (scalar) dimensions that the user hasn't overridden are
    !   omitted.

    ! inout is similar to dim_order, but stores i for output dimensions > the
    !   rank of the output array.

    DO i=1,max_rank
       inout(i) = var(var_id)%dim_order(i)
       if (inout(i) == 0) inout(i) = i
    END DO

    ALLOCATE(indx1(dims_len(inout(1))), STAT=istat)
    ALLOCATE(indx2(dims_len(inout(2))), STAT=istat)
    ALLOCATE(indx3(dims_len(inout(3))), STAT=istat)
    ALLOCATE(indx4(dims_len(inout(4))), STAT=istat)
    ALLOCATE(indx5(dims_len(inout(5))), STAT=istat)
    ALLOCATE(indx6(dims_len(inout(6))), STAT=istat)

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*,  '   Unable to allocate memory for one of the "idx?" arrays,' 
       PRINT*,  '   which have one of the following dimensions: ', dims_len
       STOP
    END IF

    ! The following generates an index array, indx?(i), containing the location
    !    in the output array corresponding to input location i.
    ! indx1 is index for input array dimension 1, 
    !    indx2 for input dimension 2, etc.

    indx1 = offset_or_reverse_dim(var_id,1,dims_len(inout(1)))
    indx2 = offset_or_reverse_dim(var_id,2,dims_len(inout(2)))
    indx3 = offset_or_reverse_dim(var_id,3,dims_len(inout(3)))
    indx4 = offset_or_reverse_dim(var_id,4,dims_len(inout(4)))
    indx5 = offset_or_reverse_dim(var_id,5,dims_len(inout(5)))
    indx6 = offset_or_reverse_dim(var_id,6,dims_len(inout(6)))

    ! now assign input data to appropriate positions in output array.

    SELECT CASE(TRIM(var(var_id)%type))

    CASE("real")

       ALLOCATE(data_out_r(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("integer")

       ALLOCATE(data_out_i(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("double")

       ALLOCATE(data_out_d(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    END SELECT

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*, '   Unable to allocate memory for one of the "data_out",'
       PRINT*, '   which has the following dimensions: ', dims_len
       STOP
    END IF
    
    loc = 1
    
    DO i6 = 1, dims_len(inout(6))
       loc(inout(6)) = indx6(i6)
       DO i5 = 1, dims_len(inout(5))
          loc(inout(5)) = indx5(i5)
          DO i4 = 1, dims_len(inout(4))
             loc(inout(4)) = indx4(i4)
             DO i3 = 1, dims_len(inout(3))
                loc(inout(3)) = indx3(i3)
                DO i2 = 1, dims_len(inout(2))
                   loc(inout(2)) = indx2(i2)
                   DO i1 = 1, dims_len(inout(1))
                      loc(inout(1)) = indx1(i1)

                      valin = data(i1,i2,i3,i4,i5,i6)

                      SELECT CASE(TRIM(var(var_id)%type))

                      CASE("real")

                         IF ((var(var_id)%def_missing > 0) .and.  &
                              (ABS(valin-var(var_id)%users_missing_dbl) &
                              < ABS(var(var_id)%users_missing_dbl * &
                              var(var_id)%miss_val_tolerance))) THEN

                            i = var(var_id)%mip_table_idx
                            rval = mip_tbl(i)%single_missing

                         ELSE

                            rval = var(var_id)%offset +       &
                                 valin*var(var_id)%scale_factor

                            IF ((var(var_id)%valid_min /=     &
                                 init_undef_single) .and. &
                                 (rval < var(var_id)%valid_min)) THEN
                               call errmsg_d('min', irank, var_id, valin, &
                                    rval, ival, dval, i1, i2, i3, i4, i5, &
                                    i6, i7)
                            ELSE IF ((var(var_id)%valid_max /=    &
                                 init_undef_single) .and. &
                                 (rval > var(var_id)%valid_max)) THEN
                               call errmsg_d('max', irank, var_id, valin, &
                                    rval, ival, dval, i1, i2, i3, i4, i5, &
                                    i6, i7)
                            END IF

                         END IF

                         data_out_r(loc(1),loc(2),loc(3),loc(4),  &
                              loc(5),loc(6),loc(7)) = rval

                      CASE("integer")

                         IF ((var(var_id)%def_missing > 0) .and.  &
                              (ABS(valin-var(var_id)%users_missing_dbl) &
                              < ABS(var(var_id)%users_missing_dbl * &
                              var(var_id)%miss_val_tolerance))) THEN

                            i = var(var_id)%mip_table_idx
                            ival = mip_tbl(i)%integer_missing

                         ELSE

                            ival = var(var_id)%offset +       &
                                 valin*var(var_id)%scale_factor

                            IF ((var(var_id)%valid_min /=     &
                                 init_undef_single) .and. &
                                 (ival < var(var_id)%valid_min)) THEN
                               call errmsg_d('min', irank, var_id, valin, &
                                    rval, ival, dval, i1, i2, i3, i4, i5, &
                                    i6, i7)
                            ELSE IF ((var(var_id)%valid_max /=    &
                                 init_undef_single) .and. &
                                 (ival > var(var_id)%valid_max)) THEN
                               call errmsg_d('max', irank, var_id, valin, &
                                    rval, ival, dval, i1, i2, i3, i4, i5, &
                                    i6, i7)
                            END IF


                         END IF

                         data_out_i(loc(1),loc(2),loc(3),loc(4),  &
                              loc(5),loc(6),loc(7)) = ival

                      CASE("double")

                         IF ((var(var_id)%def_missing > 0) .and.  &
                              (ABS(valin-var(var_id)%users_missing_dbl) &
                              < ABS(var(var_id)%users_missing_dbl * &
                              var(var_id)%miss_val_tolerance))) THEN
                            i = var(var_id)%mip_table_idx
                            dval = mip_tbl(i)%integer_missing
                         ELSE

                            dval = var(var_id)%offset +       &
                                 valin*var(var_id)%scale_factor

                            IF ((var(var_id)%valid_min /=     &
                                 init_undef_single) .and. &
                                 (dval < var(var_id)%valid_min)) THEN
                               call errmsg_d('min', irank, var_id, valin, &
                                    rval, ival, dval, i1, i2, i3, i4, i5, &
                                    i6, i7)
                            ELSE IF ((var(var_id)%valid_max /=    &
                                 init_undef_single) .and. &
                                 (dval > var(var_id)%valid_max)) THEN
                               call errmsg_d('max', irank, var_id, valin, &
                                    rval, ival, dval, i1, i2, i3, i4, i5, &
                                    i6, i7)
                            END IF

                         END IF

                         data_out_d(loc(1),loc(2),loc(3),loc(4),  &
                              loc(5),loc(6),loc(7)) = dval

                      END SELECT

                   END DO
                END DO
             END DO
          END DO
       END DO
    END DO

    SELECT CASE(TRIM(var(var_id)%type))

    CASE('real')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_r, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_r)
       
    CASE('integer')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_i, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_i)
       
    CASE('double')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_d, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_d)
       
    END SELECT
           
    DEALLOCATE(indx1)
    DEALLOCATE(indx2)
    DEALLOCATE(indx3)
    DEALLOCATE(indx4)
    DEALLOCATE(indx5)
    DEALLOCATE(indx6)
        
    IF (ierr == NF90_NOERR) THEN
       IF (ntimes /= 0) THEN
          IF (PRESENT(store_with)) THEN
             var(var_id)%zfac_times_written(store_with) =   &
                  var(var_id)%zfac_times_written(store_with)+ntimes
          ELSE
             var(var_id)%times_written=var(var_id)%times_written+ntimes
          END IF
       END IF
    ELSE
       PRINT*, 'Error writing NetCDF out data: error number =  ', ierr
       PRINT*, '   netCDF error message: ',  TRIM(nf90_strerror(ierr))
       STOP
    END IF
    
    RETURN
  END FUNCTION cmor_write_6d_d




  FUNCTION cmor_write_7d_d(var_id, data, file_suffix, ntimes_passed,  &
       time_vals, time_bnds, store_with) RESULT(ierr)

    IMPLICIT NONE

    DOUBLE PRECISION, INTENT(in):: data(:,:,:,:,:,:,:)
    INTEGER, INTENT(in) ::var_id
    CHARACTER(len=*), INTENT(in), OPTIONAL :: file_suffix
    INTEGER, INTENT(in), OPTIONAL :: ntimes_passed
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_vals(:)
    DOUBLE PRECISION, INTENT(in), OPTIONAL :: time_bnds(:,:)
    INTEGER, INTENT(in), OPTIONAL :: store_with

    INTEGER, ALLOCATABLE, DIMENSION(:) :: indx1, indx2, indx3, indx4, &
         indx5, indx6, indx7
    INTEGER, ALLOCATABLE :: data_out_i(:,:,:,:,:,:,:)
    REAL, ALLOCATABLE :: data_out_r(:,:,:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: data_out_d(:,:,:,:,:,:,:)
    INTEGER, DIMENSION(max_rank) :: inout, loc, dims_len, start, data_size
    INTEGER :: i1=1, i2=1, i3=1, i4=1, i5=1, i6=1, i7=1
    INTEGER :: ival, istat, ncid, ntimes, a_time_slice, i, irank, ierr
    REAL :: rval
    DOUBLE PRECISION :: dval

    DOUBLE PRECISION :: valin

    var(var_id)%users_type = 'double'
    irank = 7

    ierr = 0

    ! store input array's dimension lengths
    data_size = 0
    DO i =1 , irank
       data_size(i)=SIZE(data,i)
    END DO

    ierr = check_args(var_id, ncid, ntimes_passed, time_vals, time_bnds, &
         store_with, file_suffix)
    IF (ierr < 0) RETURN

    ! Process time stuff, define dims_len and start, and for use in the 
    !     write_1d case (only??)  also defines a_time_slice

    CALL cmor_process_time(ncid, var_id, ntimes_passed, dims_len, start, &
         a_time_slice, time_vals, time_bnds, store_with)


    ! besides checking, the following defines ntimes

    CALL check_time_and_dim_lens(data_size, var_id, dims_len, ntimes, ierr, &
         ntimes_passed)
    IF (ierr.LT.0) RETURN


    ! dim_order(i) stores the output array index that corresponds to 
    !   input array index i; it stores 0 for output dimensions > the
    !   rank of the output array.
    !   Singleton (scalar) dimensions that the user hasn't overridden are
    !   omitted.

    ! inout is similar to dim_order, but stores i for output dimensions > the
    !   rank of the output array.

    DO i=1,max_rank
       inout(i) = var(var_id)%dim_order(i)
       if (inout(i) == 0) inout(i) = i
    END DO

    ALLOCATE(indx1(dims_len(inout(1))), STAT=istat)
    ALLOCATE(indx2(dims_len(inout(2))), STAT=istat)
    ALLOCATE(indx3(dims_len(inout(3))), STAT=istat)
    ALLOCATE(indx4(dims_len(inout(4))), STAT=istat)
    ALLOCATE(indx5(dims_len(inout(5))), STAT=istat)
    ALLOCATE(indx6(dims_len(inout(6))), STAT=istat)
    ALLOCATE(indx7(dims_len(inout(7))), STAT=istat)

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*,  '   Unable to allocate memory for one of the "idx?" arrays,' 
       PRINT*,  '   which have one of the following dimensions: ', dims_len
       STOP
    END IF

    ! The following generates an index array, indx?(i), containing the location
    !    in the output array corresponding to input location i.
    ! indx1 is index for input array dimension 1, 
    !    indx2 for input dimension 2, etc.

    indx1 = offset_or_reverse_dim(var_id,1,dims_len(inout(1)))
    indx2 = offset_or_reverse_dim(var_id,2,dims_len(inout(2)))
    indx3 = offset_or_reverse_dim(var_id,3,dims_len(inout(3)))
    indx4 = offset_or_reverse_dim(var_id,4,dims_len(inout(4)))
    indx5 = offset_or_reverse_dim(var_id,5,dims_len(inout(5)))
    indx6 = offset_or_reverse_dim(var_id,6,dims_len(inout(6)))
    indx7 = offset_or_reverse_dim(var_id,7,dims_len(inout(7)))

    ! now assign input data to appropriate positions in output array.

    SELECT CASE(TRIM(var(var_id)%type))

    CASE("real")

       ALLOCATE(data_out_r(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("integer")

       ALLOCATE(data_out_i(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    CASE("double")

       ALLOCATE(data_out_d(dims_len(1),dims_len(2),dims_len(3),dims_len(4), &
            dims_len(5),dims_len(6),dims_len(7)), STAT=istat)

    END SELECT

    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR:  rearrange_r_and_write'
       PRINT*, '   Unable to allocate memory for one of the "data_out",'
       PRINT*, '   which has the following dimensions: ', dims_len
       STOP
    END IF
    
    loc = 1
    
    DO i7 = 1, dims_len(inout(7))
       loc(inout(7)) = indx7(i7)
       DO i6 = 1, dims_len(inout(6))
          loc(inout(6)) = indx6(i6)
          DO i5 = 1, dims_len(inout(5))
             loc(inout(5)) = indx5(i5)
             DO i4 = 1, dims_len(inout(4))
                loc(inout(4)) = indx4(i4)
                DO i3 = 1, dims_len(inout(3))
                   loc(inout(3)) = indx3(i3)
                   DO i2 = 1, dims_len(inout(2))
                      loc(inout(2)) = indx2(i2)
                      DO i1 = 1, dims_len(inout(1))
                         loc(inout(1)) = indx1(i1)
                         
                         valin = data(i1,i2,i3,i4,i5,i6,i7)
                         
                         SELECT CASE(TRIM(var(var_id)%type))

                         CASE("real")
                            
                            IF ((var(var_id)%def_missing > 0) .and.  &
                                 (ABS(valin-var(var_id)%users_missing_dbl) &
                                 < ABS(var(var_id)%users_missing_dbl * &
                                 var(var_id)%miss_val_tolerance))) THEN

                               i = var(var_id)%mip_table_idx
                               rval = mip_tbl(i)%single_missing

                            ELSE
                               
                               rval = var(var_id)%offset +       &
                                    valin*var(var_id)%scale_factor
                               
                               IF ((var(var_id)%valid_min /=     &
                                    init_undef_single) .and. &
                                    (rval < var(var_id)%valid_min)) THEN
                                  call errmsg_d('min', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               ELSE IF ((var(var_id)%valid_max /=    &
                                    init_undef_single) .and. &
                                    (rval > var(var_id)%valid_max)) THEN
                                  call errmsg_d('max', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               END IF
                               
                            END IF
                            
                            data_out_r(loc(1),loc(2),loc(3),loc(4),  &
                                 loc(5),loc(6),loc(7)) = rval

                         CASE("integer")
                            
                            IF ((var(var_id)%def_missing > 0) .and.  &
                                 (ABS(valin-var(var_id)%users_missing_dbl) &
                                 < ABS(var(var_id)%users_missing_dbl * &
                                 var(var_id)%miss_val_tolerance))) THEN

                               i = var(var_id)%mip_table_idx
                               ival = mip_tbl(i)%integer_missing

                            ELSE
                               
                               ival = var(var_id)%offset +       &
                                    valin*var(var_id)%scale_factor
                               
                               IF ((var(var_id)%valid_min /=     &
                                    init_undef_single) .and. &
                                    (ival < var(var_id)%valid_min)) THEN
                                  call errmsg_d('min', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               ELSE IF ((var(var_id)%valid_max /=    &
                                    init_undef_single) .and. &
                                    (ival > var(var_id)%valid_max)) THEN
                                  call errmsg_d('max', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               END IF

                               
                            END IF
                            
                            data_out_i(loc(1),loc(2),loc(3),loc(4),  &
                                 loc(5),loc(6),loc(7)) = ival

                         CASE("double")
                            
                            IF ((var(var_id)%def_missing > 0) .and.  &
                                 (ABS(valin-var(var_id)%users_missing_dbl) &
                                 < ABS(var(var_id)%users_missing_dbl * &
                                 var(var_id)%miss_val_tolerance))) THEN
                               i = var(var_id)%mip_table_idx
                               dval = mip_tbl(i)%integer_missing
                            ELSE
                               
                               dval = var(var_id)%offset +       &
                                    valin*var(var_id)%scale_factor
                               
                               IF ((var(var_id)%valid_min /=     &
                                    init_undef_single) .and. &
                                    (dval < var(var_id)%valid_min)) THEN
                                  call errmsg_d('min', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               ELSE IF ((var(var_id)%valid_max /=    &
                                    init_undef_single) .and. &
                                    (dval > var(var_id)%valid_max)) THEN
                                  call errmsg_d('max', irank, var_id, valin, &
                                       rval, ival, dval, i1, i2, i3, i4, i5, &
                                       i6, i7)
                               END IF
                               
                            END IF
                            
                            data_out_d(loc(1),loc(2),loc(3),loc(4),  &
                                 loc(5),loc(6),loc(7)) = dval
                            
                         END SELECT
                         
                      END DO
                   END DO
                END DO
             END DO
          END DO
       END DO
    END DO

    SELECT CASE(TRIM(var(var_id)%type))

    CASE('real')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_r, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_r)
       
    CASE('integer')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_i, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_i)
       
    CASE('double')
       
       ierr = NF90_PUT_VAR(ncid, var(var_id)%varid, &
            data_out_d, &
            start = start , &
            count = dims_len)
       
       DEALLOCATE(data_out_d)
       
    END SELECT
           
    DEALLOCATE(indx1)
    DEALLOCATE(indx2)
    DEALLOCATE(indx3)
    DEALLOCATE(indx4)
    DEALLOCATE(indx5)
    DEALLOCATE(indx6)
    DEALLOCATE(indx7)
        
    IF (ierr == NF90_NOERR) THEN
       IF (ntimes /= 0) THEN
          IF (PRESENT(store_with)) THEN
             var(var_id)%zfac_times_written(store_with) =   &
                  var(var_id)%zfac_times_written(store_with)+ntimes
          ELSE
             var(var_id)%times_written=var(var_id)%times_written+ntimes
          END IF
       END IF
    ELSE
       PRINT*, 'Error writing NetCDF out data: error number =  ', ierr
       PRINT*, '   netCDF error message: ',  TRIM(nf90_strerror(ierr))
       STOP
    END IF
    
    RETURN
  END FUNCTION cmor_write_7d_d




  FUNCTION cmor_type_to_nf90(type) result(nf90type)

    IMPLICIT NONE

    CHARACTER(len=*), INTENT(IN) :: type
    INTEGER :: nf90type

    INTEGER :: ierr
    CHARACTER(len=16) :: tmp
    CHARACTER(len=128), DIMENSION(2) :: e=''

    ! Convert cmor string type to NF90 integer type

    tmp = TRIM(type)
    SELECT CASE(tmp)

    CASE('char')
       nf90type=NF90_CHAR
    CASE('character')
       nf90type=NF90_CHAR
    CASE('int')
       nf90type=NF90_INT
    CASE('integer')
       nf90type=NF90_INT
    CASE('float')
       nf90type=NF90_FLOAT
    CASE('real')
       nf90type=NF90_FLOAT
    CASE('double')
       nf90type=NF90_DOUBLE

    CASE default

       e(1)='The following variable type is unrecognized by netCDF: ' &
            // TRIM(type)
       e(2)='Likely error in MIP input table'
       
       print*, 'CMOR error in cmor_type_to_nf90'
       print*, e(1)
       print*, e(2)
!!$       ierr = error(3001, terminate=.t., in='cmor_type_to_nf90', message=e)
       stop
    END SELECT

    RETURN

  END FUNCTION cmor_type_to_nf90



  FUNCTION cmor_setup(inpath, netcdf_file_action, set_verbosity,  &
       exit_control) RESULT(ierr)

    IMPLICIT NONE

    INTEGER :: ierr
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: inpath
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: netcdf_file_action
    INTEGER, INTENT(IN), OPTIONAL :: set_verbosity
    INTEGER, INTENT(IN), OPTIONAL :: exit_control
    
    CHARACTER(LEN=a128) :: e(3)=''
    INTEGER idx, i, j

    ierr = 0

    PRINT*, ' '
    PRINT*, 'CMOR is being initiated through cmor_setup.'
    PRINT*, ' '

    IF (PRESENT(inpath)) THEN
       IF (LEN_TRIM((ADJUSTL(inpath))) > a256-1) THEN
          e(1)='User-passed argument "inpath" is:'
          e(2)=TRIM(ADJUSTL(inpath))
          e(3)='and is too long.  Please reduce to fewer than 256 characters'
          PRINT*, 'Encountered error in CMOR: cmor_setup'
          PRINT*, e(1)
          PRINT*, e(2)
          PRINT*, e(3)
          STOP
!!$          ierr = -error(3010, in='cmor_setup', message=e)
          RETURN
       ELSE
          cmor_inpath=TRIM(ADJUSTL(inpath))
       END IF
    ELSE
       cmor_inpath='./'
    END IF

    cmor_inpath = TRIM(cmor_inpath) // '/'
    ! reduce double slashes '//' to '/'
    DO 
       idx = index(cmor_inpath, '//')
       IF (idx<1) EXIT
       cmor_inpath = cmor_inpath(1:idx) // inpath(idx+2:)
    END DO

    IF (PRESENT(netcdf_file_action)) THEN
       IF (similar(netcdf_file_action, 'preserve') .OR.    &
            similar(netcdf_file_action, 'append') .OR.     &
            similar(netcdf_file_action, 'replace')) THEN
          cmor_netcdf_file_action = netcdf_file_action
       ELSE
          e(1)='User-passed argument "netcdf_file_action" is: ' // &
               TRIM(ADJUSTL(netcdf_file_action))
          e(2)='but must have one of the following values: "preserve", ' // &
               '"append", or "replace"'
          PRINT*, 'Encountered error in CMOR: cmor_setup'
          PRINT*, e(1)
          PRINT*, e(2)
          STOP
         
!!$          ierr = -error(3011, in='cmor_setup', message=e)
       END IF
    ELSE
       cmor_netcdf_file_action = 'preserve'
    END IF

    IF (PRESENT(set_verbosity)) THEN
       IF ( (set_verbosity >= 0) .AND. (set_verbosity <=2) ) THEN
          cmor_set_verbosity=set_verbosity
       ELSE
          PRINT*, 'Encountered error in CMOR: cmor_setup'
          PRINT*, 'User-passed argument, "set_verbosity", must be in the range 0 to 1'
          PRINT*, 'You passed set_verbosity = ', set_verbosity
          STOP

       END IF
    ELSE
       cmor_set_verbosity=2
    END IF

    IF (PRESENT(exit_control)) THEN
       IF ((exit_control >= 0) .AND. (exit_control <=2)) THEN
          cmor_exit_control=exit_control
       ELSE
          write(e(1), '(i20)') exit_control
          e(1)='User-passed integer argument "exit_control" is: ' //  &
               ADJUSTL(e(1))
          e(2) = 'but must be 0 (only fatal), 1 (non-fatal & fatal), or 2' // &
               '(warnings, non-fatal & fatal)'
          PRINT*, 'Encountered error in CMOR: cmor_setup'
          PRINT*, e(1)
          PRINT*, e(2)
!!$          ierr = -error(3012, in='cmor_setup', message=e)
          STOP
       END IF
    ELSE
       cmor_exit_control=2
    END IF

    ! now initialize all variables

    num_mip_tables=0
    num_axes=0
    num_vars=0
    num_dims=0

    axis_arg_cntl = (/ 1, 1, 1, 3, 3, 3, 3 /)
    var_arg_cntl = (/ 1, 1, 1, 1, 3, 3, 3, 3, 3, 3 /)
    write_arg_cntl = (/ 1, 1, 3, 3, 3, 3, 3 /)

    mip_tbl = mip_table_info(&
         '',                 & ! cf_version
         0,                  & ! magic_number
         '',                 & ! project_id
         '',                 & ! table_id
         '',                 & ! table_date
         0.0,                & ! approx_interval
         init_undef_single,  & ! single_missing
         init_undef_double,  & ! double_missing
         init_undef_integer, & ! integer_missing
         '',                 & ! full_path
         SPREAD('',1,max_num_expts) ) ! expt_id_ok

    !   there seems to be a problem on the HP with initializing
    !   a character string array with the SPREAD function, so 
    !   we "reinitialize it here:

    DO i=1,max_num_expts
       mip_tbl%expt_id_ok(i) = ''
    END DO

    dset = dataset_info(     &
         '',                 & ! outpath
         '',                 & ! experiment_id
         '',                 & ! institution
         '',                 & ! source
         '',                 & ! calendar
         1,                  & ! realization
         '',                 & ! contact
         '',                 & ! history
         '',                 & ! comment
         '',                 & ! references
         init_undef_integer, & ! leap_year
         0,                  & ! leap_month
         SPREAD(0,1,12),     & ! month_lengths
         SPREAD(0,1,3)  )      ! error_count


    DO i=1,max_variables

       var(i)%mip_table          = ''
       var(i)%mip_table_idx      = 0
       var(i)%table_entry        = ''
       var(i)%arg_cntl           = var_arg_cntl
       var(i)%standard_name      = ''
       var(i)%out_units          = init_undef_char
       var(i)%cell_methods       = ''
       var(i)%long_name          = ''
       var(i)%comment            = ''
       var(i)%history            = ''
       var(i)%out_positive       = ''
       var(i)%rank               = 0
       !   var(i)%axis_names         = SPREAD('', 1, max_rank)

       !   there seems to be a problem on the HP with initializing
       !   a character string array with the SPREAD function, so 
       !   we "reinitialize it here:
       
       DO j=1,max_rank
          var(i)%axis_names(j) = ''
       END DO

       var(i)%axis_idxs          = SPREAD(0, 1, max_rank)
       var(i)%valid_min          = init_undef_single
       var(i)%valid_max          = init_undef_single
       var(i)%ok_min_mean_abs    = init_undef_single
       var(i)%ok_max_mean_abs    = init_undef_single
       var(i)%type               = 'real'
       var(i)%out_name           = ''
       var(i)%ncid               = 0
       var(i)%varid              = 0
       var(i)%nctimid            = 0
       var(i)%nctim_bndsid       = 0
       var(i)%full_outpath       = ''
       var(i)%file_status        = 'unopened'
       var(i)%file_suffix        = init_undef_char
       var(i)%dim_order          = SPREAD(0, 1, max_rank)
       var(i)%dim_idxs           = SPREAD(0, 1, max_rank)
       var(i)%scale_factor       = 1.0
       var(i)%offset             = 0.0
       var(i)%def_missing        = 0
       var(i)%users_missing      = init_undef_single
       var(i)%users_missing_dbl  = init_undef_double
       var(i)%users_missing_int  = init_undef_integer
       var(i)%miss_val_tolerance = 0.0
       var(i)%users_name         = ''
       var(i)%users_units        = init_undef_char
       var(i)%users_type         = ''
       var(i)%error_count        = SPREAD(0, 1, 9)
       var(i)%times_written      = 0
       var(i)%last_time_written  = init_undef_double
       var(i)%last_time_bound    = init_undef_double
       nullify(var(i)%zfac_times_written)
       nullify(var(i)%zfac_values)
       nullify(var(i)%zfac_values_dbl)
       nullify(var(i)%zfac_bounds)
       nullify(var(i)%zfac_bounds_dbl)
    END DO


    DO i = 1, max_axes
       axis(i)%mip_table         = ''
       axis(i)%table_entry       = ''
       axis(i)%arg_cntl          = axis_arg_cntl
       axis(i)%standard_name     = ''
       axis(i)%out_units         = init_undef_char
       axis(i)%axis              = ''
       axis(i)%bounds_attrib     = ''
       axis(i)%long_name         = ''
       axis(i)%positive_attrib   = ''
       axis(i)%out_direction     = 1
       axis(i)%valid_min         = init_undef_double
       axis(i)%valid_max         = init_undef_double
       axis(i)%type              = 'double'
       axis(i)%out_name          = ''
       axis(i)%coords_attrib     = ''
       axis(i)%tol_on_requests   = 1.e-3
       axis(i)%singleton_label   = ''
       axis(i)%singleton_bnds    = SPREAD(init_undef_double, 1, 2)
       axis(i)%formula           = ''
       axis(i)%convert_to        = ''
       nullify(axis(i)%requested_vals)
       nullify(axis(i)%requested_bounds)
       nullify(axis(i)%requested_labels)
       nullify(axis(i)%zfac_names)
       nullify(axis(i)%zfac_bnds_names)
    END DO

    do i =1, max_dimensions
       adim(i)%axis_idx       = 0
       adim(i)%users_units    = init_undef_char
       adim(i)%interval       = ''
       adim(i)%reverse        = .false.
       adim(i)%begin_out      = 1
       adim(i)%lendim         = 0
       adim(i)%scale_factor   = 1.
       adim(i)%offset         = 0.
       adim(i)%converted_from = ''
       adim(i)%error_count    = SPREAD(0, 1, 6)
!!$       adim(i)%zfac_names     = SPREAD(0, 1, max_zfactors)
!!$       adim(i)%zfac_bnds_names= SPREAD(0, 1, max_zfactors)
       nullify(adim(i)%zfac_idxs)
       nullify(adim(i)%zfac_bnds_idxs)
       nullify(adim(i)%values)
       nullify(adim(i)%bounds)
       nullify(adim(i)%labels)
    enddo
    called_setup = .true.

    RETURN
  END FUNCTION cmor_setup


  FUNCTION cmor_dataset(outpath, experiment_id, institution, source,   &
       calendar, realization, contact, history, comment, references, &
       leap_year, leap_month, month_lengths) RESULT(ierr)

    IMPLICIT NONE

    INTEGER :: ierr
    CHARACTER(len=*), INTENT(IN) :: outpath
    CHARACTER(len=*), INTENT(IN) :: experiment_id
    CHARACTER(len=*), INTENT(IN) :: institution 
    CHARACTER(len=*), INTENT(IN) :: source
    CHARACTER(len=*), INTENT(IN) :: calendar
    INTEGER, INTENT(IN), OPTIONAL :: realization
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: contact
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: history 
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: comment       
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: references
    INTEGER, INTENT(IN), OPTIONAL :: leap_year
    INTEGER, INTENT(IN), OPTIONAL :: leap_month
    INTEGER, INTENT(IN), DIMENSION(:), OPTIONAL :: month_lengths

    INTEGER :: i, idx
    CHARACTER(LEN=a128) :: e(5)=''

    PRINT*, 'CMOR is defining a "dataset" through cmor_dataset.'
    PRINT*, 'CMOR output will be placed in directory: ', TRIM(outpath)
    PRINT*, ' '

    ierr=0

    !  check that cmor_setup has already been called
    !   ??? the outputlog has not yet been defined at this point
    IF (.NOT. called_setup)  THEN
          PRINT*, 'Encountered error in CMOR: cmor_dataset'
          PRINT*, 'You must call cmor_setup before calling cmor_dataset'
!!$       ierr = -error(3020, in='cmor_dataset', &
!!$            message='You must call cmor_setup before calling cmor_dataset')
          stop
       RETURN
    END IF

    IF (LEN_TRIM(ADJUSTL(outpath))>a512-1) THEN
       PRINT*, 'Error encountered in CMOR: cmor_dataset'
       PRINT*, 'User-passed argument, "outpath", must be shorter than 512 characters'
       print*, 'You passed outpath = '
       print*, outpath
       STOP
    ELSE IF (LEN_TRIM(outpath)==0) THEN
       dset%outpath = './'
    ELSE

       dset%outpath = TRIM(ADJUSTL(outpath))
       dset%outpath = TRIM(dset%outpath) // '/'
       ! reduce double slashes '//' to '/'
       DO 
          idx = index(dset%outpath, '//')
          IF (idx<1) EXIT
          dset%outpath = dset%outpath(1:idx) // dset%outpath(idx+2:)
       END DO

    END IF

    ! open output log

    IF (LEN_TRIM(ADJUSTL(experiment_id))>a512) THEN
       PRINT*, 'Error encountered in CMOR: cmor_dataset'
       PRINT*, 'User-passed argument, "experiment_id", must be <= 512 characters'
       print*, 'You passed experiment_id =  '
       print*, experiment_id
       STOP
    ELSE IF (LEN_TRIM(experiment_id)==0) THEN
       PRINT*, 'Error encountered in CMOR: cmor_dataset'
       PRINT*, 'User-passed argument, "experiment_id"  must be > 0 characters'
       STOP
    ELSE
       dset%experiment_id = TRIM(ADJUSTL(experiment_id))
    END IF

    IF (LEN_TRIM(ADJUSTL(institution))>a512) THEN
       PRINT*, 'Error encountered in CMOR: cmor_dataset'
       PRINT*, 'User-passed argument, "institution",  must be <= 512 characters'
       print*, 'You passed institution = '
       print*, institution
       STOP
    ELSE IF (LEN_TRIM(institution)==0) THEN
       PRINT*, 'Error encountered in CMOR: cmor_dataset'
       PRINT*, 'user argument, "institution",  must be > 0 characters'
       STOP
    ELSE IF (INDEX(institution, '(') == 0) THEN
       PRINT*, 'Error encountered in CMOR: cmor_dataset'
       PRINT*, 'User-passed argument, "institution", must include information contained in parentheses.'
       PRINT*, 'For example: institution = "NCAR (National Center for Atmospheric Research, Boulder, CO. USA)'
       PRINT*, 'You passed institution = '
       PRINT*, institution
       STOP
    ELSE
       dset%institution = TRIM(ADJUSTL(institution))
    END IF

    IF (LEN_TRIM(ADJUSTL(source))>a512) THEN
       PRINT*, 'Error encountered in CMOR: cmor_dataset'
       PRINT*, 'User-passed argument, "source",  must be <= 512 characters'
       print*, 'You passed source = '
       PRINT*, source
       STOP
    ELSE IF (LEN_TRIM(source)==0) THEN
       PRINT*, 'Error encountered in CMOR: cmor_dataset'
       PRINT*, 'user argument, "source",  must be > 0 characters'
       STOP
    ELSE
       dset%source = TRIM(ADJUSTL(source))
    END IF

    IF (PRESENT(contact)) THEN
       IF (LEN_TRIM(ADJUSTL(contact))>a512) THEN
          PRINT*, 'Error encountered in CMOR: cmor_dataset'
          PRINT*, 'User-passed argument, "contact",  must be <= 512 characters'
          print*, 'You passed contact = '
          PRINT*, contact
          STOP
       ELSE IF (LEN_TRIM(contact)==0) THEN
          ! warning
       ELSE
          dset%contact = TRIM(ADJUSTL(contact))
       END IF
    END IF

    IF (PRESENT(history)) THEN
       IF (LEN_TRIM(ADJUSTL(history))>a2048) THEN
          PRINT*, 'Error encountered in CMOR: cmor_dataset'
          PRINT*, 'User-passed argument, "history",  must be <= 2048 characters'
          print*, 'You passed history = '
          PRINT*, history
          STOP
       ELSE IF (LEN_TRIM(history)==0) THEN
          ! warning
          PRINT*, 'Warning encountered in CMOR: cmor_dataset'
          PRINT*, 'User-passed argument, "history",  was an empty string'
          print*, 'If you so not wish to define this attribute, CMOR prefers'
          print*, '(but does not require) that you simply omit this argument.'

       ELSE
          dset%history = TRIM(ADJUSTL(history))
       END IF
    END IF
    
    IF (PRESENT(comment)) THEN
       IF (LEN_TRIM(ADJUSTL(comment))>a2048) THEN
          PRINT*, 'Error encountered in CMOR: cmor_dataset'
          PRINT*, 'User-passed argument, "comment",  must be <= 2048 characters'
          print*, 'You passed comment = '
          PRINT*, comment
          STOP

       ELSE IF (LEN_TRIM(comment)==0) THEN
          ! warning
          PRINT*, 'Warning encountered in CMOR: cmor_dataset'
          PRINT*, 'User-passed argument, "comment",  was an empty string'
          print*, 'If you so not wish to define this attribute, CMOR prefers'
          print*, '(but does not require) that you simply omit this argument.'
       ELSE
          dset%comment = TRIM(ADJUSTL(comment))
       END IF
    END IF

    IF (PRESENT(references)) THEN
       IF (LEN_TRIM(ADJUSTL(references))>a2048) THEN
          PRINT*, 'Error encountered in CMOR: cmor_dataset'
          PRINT*, 'User-passed argument, "references",  must be <= 2048 characters'
          print*, 'You passed references = '
          PRINT*, references
          STOP

       ELSE IF (LEN_TRIM(references)==0) THEN
          ! warning
          PRINT*, 'Warning encountered in CMOR: cmor_dataset'
          PRINT*, 'User-passed argument, "references",  was an empty string'
          print*, 'If you so not wish to define this attribute, CMOR prefers'
          print*, '(but does not require) that you simply omit this argument.'

       ELSE
          dset%references = TRIM(ADJUSTL(references))
       END IF
    END IF

    dset%realization = 1
    IF (PRESENT(realization)) THEN
       IF (realization > 0) THEN
          dset%realization=realization
       ELSE
          PRINT*, 'Error encountered in CMOR: cmor_dataset'
          print*, 'User-passed argument, "realization", has a negative value of ', &
              realization
          print*, 'which is not permitted'
          stop
       END IF
    END IF


    if_1: IF (LEN_TRIM(ADJUSTL(calendar))>a32) THEN
       PRINT*, 'Error encountered in CMOR: cmor_dataset'
       PRINT*, 'User-passed argument, "calendar",  must be <= 32 characters'
       print*, 'You passed calendar = '
       PRINT*, calendar
       STOP
    ELSE IF (LEN_TRIM(calendar)==0) THEN
       PRINT*, 'Error encountered in CMOR: cmor_dataset'
       PRINT*, 'User-passed and empty string as the "calendar" argument'
       print*, 'CMOR requires the calendar argument be defined'
       STOP
    ELSE

       dset%calendar=''
       IF (similar(calendar, 'gregorian')) THEN
          dset%calendar='gregorian'
       ELSE IF (similar(calendar, 'standard')) THEN
          dset%calendar='standard'
       ELSE IF (similar(calendar, 'proleptic_gregorian')) THEN
          dset%calendar='proleptic_gregorian'
       ELSE IF (similar(calendar, 'noleap')) THEN
          dset%calendar='noleap'
       ELSE IF (similar(calendar, '365_day')) THEN
          dset%calendar='365_day'
       ELSE IF (similar(calendar, '360_day')) THEN
          dset%calendar='360_day'
       ELSE IF (similar(calendar, 'julian')) THEN
          dset%calendar='julian'
       ELSE IF (similar(calendar, 'NONE')) THEN
          dset%calendar='NONE'
       END IF

       if_2: IF (LEN_TRIM(dset%calendar) /= 0) THEN

          !  standard calendar
          IF (PRESENT(month_lengths) .OR. PRESENT(leap_year) .OR. &
               PRESENT(leap_month)) THEN
             ! warn and ignore
             PRINT*, 'Warning encountered in CMOR: cmor_dataset'
             PRINT*, 'User-passed the following calendar defined by the CF-standard: ', &
                 dset%calendar
             print*, '   and should therefore not pass the following argument(s)'
             print*, '   which CMOR will ignore:'
             IF (PRESENT(month_lengths)) THEN
                PRINT*, 'month_lengths = ', month_lengths
             END IF
             IF (PRESENT(leap_year)) THEN
                PRINT*, 'leap_year = ', leap_year
             END IF
             IF (PRESENT(leap_month)) THEN
                PRINT*, 'leap_month = ', leap_month
             END IF
             PRINT*, 'Alternatively, the user should give a non-conventional name to ' 
             PRINT*, '      the calendar argument.'
          END IF

       ELSE
          !  non-standard calendar
          dset%calendar=TRIM(ADJUSTL(calendar))

          if_3: IF (PRESENT(month_lengths)) THEN

             if_4: IF (SIZE(month_lengths) < 12) THEN
                PRINT*, 'Error encountered in CMOR: cmor_dataset'
                PRINT*, 'User-passed argument, "month_lengths", must be length 12.'
                PRINT*, 'The user passed an array of length ', SIZE(month_lengths)
                PRINT*, 'containing the values: ', month_lengths
                STOP
             ELSE 
                IF (SIZE(month_lengths) > 12) THEN
                   PRINT*, 'Warning encountered in CMOR: cmor_dataset'
                   PRINT*, 'User-passed argument, "month_lengths", was dimensioned, ',&
                        SIZE(month_lengths), ', but should be 12.'
                   PRINT*, 'Only the first 12 elements of the array will be used:'
                   PRINT*, month_lengths(1:12)
                END IF

                DO i=1,12
                   IF ((month_lengths(i) < 20) .OR. (month_lengths(i)>50)) THEN
                      PRINT*, 'Warning encountered in CMOR: cmor_dataset'
                      PRINT*, 'One element of the user-passed argument, "month_lengths",'
                      PRINT*, '   was assigned an unusual value: ', month_lengths(i)
                      PRINT*, '   You should check whether this is correct, since months'
                      PRINT*, '   are usually about 30 days long.'
                   END IF
                   dset%month_lengths(i) = month_lengths(i)
                END DO

                IF (PRESENT(leap_year)) THEN
                   dset%leap_year = leap_year
                   IF (.NOT. PRESENT(leap_month)) THEN
                      PRINT*, 'Error encountered in CMOR: cmor_dataset'
                      PRINT*, 'The user indicated that his calendar includes leap years, '
                      PRINT*, '   but a leap_month was not defined. CMOR requires that'
                      PRINT*,  &
                          '   the argument leap_month be defined if leap_year is defined'
                      STOP
                   END IF
                END IF
                IF (PRESENT(leap_month)) THEN 
                   IF ((leap_month >= 1) .AND. (leap_month <= 12)) THEN
                      dset%leap_month = leap_month
                   ELSE
                      PRINT*, 'Error encountered in CMOR: cmor_dataset'
                      print*, 'The user-passed argument, "leap_month", must be in '
                      PRINT*, '   the range 1-12, but was given the value: ', leap_month
                      STOP
                   END IF
                   IF (.NOT. PRESENT(leap_year)) THEN
                      PRINT*, 'Error encountered in CMOR: cmor_dataset'
                      PRINT*, 'The user has indicated that his leap-month is: ', &
                           leap_month
                      PRINT*, '   but a leap_year was not defined. CMOR requires that'
                      PRINT*,  &
                          '   the argument leap_year be defined if leap_month is defined'
                      STOP
                   END IF
                END IF
             END IF  if_4
          ELSE

             PRINT*, 'Error encountered in CMOR: cmor_dataset'
             print*, 'The user indicated that a calendar with the name, ', calendar
             PRINT*, 'which is not included in the CF-standard, applies to the '
             print*, '   simulation, but the calendar was not defined.'
             PRINT*, 'The user should pass the "month_lengths" argument, and '
             PRINT*, '    (as appropriate) possibly: leap_year and leap_month.'
             STOP

          END IF  if_3
       END IF  if_2
    END IF  if_1

    RETURN
  END FUNCTION cmor_dataset


  
  FUNCTION table_path(table)
    
    IMPLICIT NONE
    
    CHARACTER(LEN=*), INTENT(IN) :: table
    CHARACTER(len=LEN(cmor_inpath)+LEN(table)) :: table_path
    
    INTEGER :: idx
    
    table_path = ''
    
    IF (LEN_TRIM(ADJUSTL(table))>a64) THEN 
       PRINT*, 'Error encountered in CMOR: table_path'
       PRINT*, 'A CMOR input table name exceeded the 64 characters allowed.'
       PRINT*, 'The user passed: ', table
       RETURN

    END IF
    
    IF (LEN_TRIM(table) == 0) THEN
       PRINT*, 'Error encountered in CMOR: table_path'
       PRINT*, 'The user passed an empty string, when the name of an input table '//&
                 'was expected by CMOR.'
       RETURN

    END IF
    
    table_path = TRIM(ADJUSTL(cmor_inpath)) // '/' //  TRIM(ADJUSTL(table))
    
    ! reduce any '//' to '/'
    DO 
       idx = index(table_path, '//')
       IF (idx<1) EXIT
       table_path = table_path(1:idx) // table_path(idx+2:)
    END DO
    
    RETURN
  END FUNCTION table_path
  
  
  SUBROUTINE axis_preamble(table, table_entry, itable,   &
              idim, iaxis, ierr)

    IMPLICIT NONE

    CHARACTER(len=*), INTENT(IN) :: table
    CHARACTER(len=*), INTENT(IN) :: table_entry
    INTEGER, INTENT(OUT) :: itable
    INTEGER, INTENT(OUT) :: idim
    INTEGER, INTENT(OUT) :: iaxis
    INTEGER, INTENT(OUT) :: ierr

    CHARACTER(len=LEN(cmor_inpath)+LEN(table)) :: pathfile
    INTEGER :: idx, i, n
    CHARACTER(LEN=a128) :: e(5)=''

    PRINT*, 'CMOR is defining axis ', TRIM(table_entry), &
         ' which appears in ', table

    ierr = 0
    idim = 0

    !  check that cmor_setup has already been called
    IF (.NOT. called_setup)  THEN
       PRINT*, 'Error encountered in CMOR: axis_preamble,'
       PRINT*, '   which is called by cmor_axis.'
       PRINT*, '   The function cmor_setup must be first called to initiate cmor.'
       PRINT*, 'You must invoke cmor_setup before invoking cmor_axis.'
       ierr = -1
       RETURN

    END IF

    pathfile = table_path(table)
    IF (LEN_TRIM(pathfile) == 0) THEN
       PRINT*, 'Correct the "table" argument passed to cmor_axis.'
       ierr = -2
       RETURN
    END IF

    ! check whether this file has already been read or not
    itable=0
    IF (num_mip_tables > 0) THEN
       DO n=1,num_mip_tables
          IF (TRIM(pathfile) == TRIM(mip_tbl(n)%full_path)) THEN
             itable=n
             EXIT
          END IF
       END DO
    END IF

    IF (itable == 0) THEN
       ! advance mip table counter and read table
       num_mip_tables=num_mip_tables+1
       IF (num_mip_tables > max_mip_tables) THEN
          PRINT*, 'Error encountered in CMOR: axis_preamble,'
          PRINT*, '   which is called by cmor_axis.'
          PRINT*, '   User is attempting to access too many MIP input tables.'
          PRINT*, '   The attempt to read ', pathfile
          PRINT*, '   failed.  Restructure program so that number of input tables'
          PRINT*, '   is fewer than ', max_mip_tables+1, ' or increase '//&
                     'parameter max_mip_tables.'
          ierr = -3
          RETURN
       END IF
       itable = num_mip_tables
       mip_tbl(itable)%full_path=pathfile
       ierr = read_table(filename=pathfile)
       IF (ierr /= 0) THEN
!!$          ierr = error(ierr, called_by='axis_preamble', terminate=.f.)
          IF (ierr > 0) RETURN
       END IF
    END IF

    idim = num_dims + 1
    IF (idim > max_dimensions) THEN
       e(1)='User (or CMOR) is attempting to define too many axes'
       WRITE(e(2), '(i20)') max_dimensions
       e(2)='The maximum number of axes allowed by CMOR is : ' //  &
            ADJUSTL(e(2))
       e(3)='If this is really too few, the user may increase the value of '//&
            'max_dimensions in module: cmor_internal_tables'
!!$       ierr=error(???, terminate=.f., in='axis_preamble', message=e)
       PRINT*, 'Error encountered in CMOR: axis_preamble,'
       PRINT*, '   which is called by cmor_axis.'
       PRINT*, e(1)
       PRINT*, e(2)
       PRINT*, e(3)
       ierr = -4
       RETURN
    END IF

  ! check whether table_entry is one of the allowable axes
    IF (LEN_TRIM(adjustl(table_entry)) == 0) THEN
       PRINT*, 'Error encountered in CMOR: axis_preamble,'
       PRINT*, '   which is called by cmor_axis.'
       PRINT*, '   User must pass a string containing the table_entry, but'
       PRINT*, '   an empty string was passed instead'
       stop
    END IF

    iaxis = 0
    DO i=1,num_axes 
       IF (similar(table_entry, axis(i)%table_entry) .AND. &
            similar(mip_tbl(itable)%table_id, axis(i)%mip_table)) THEN
          iaxis = i
          EXIT
       endif
    END DO

   IF (iaxis == 0) THEN
       PRINT*, 'Error encountered in CMOR: axis_preamble,'
       PRINT*, '   which is called by cmor_axis.'
       PRINT*, '   User is attempting to define table_entry: ', table_entry
       PRINT*, '   but this entry does not appear in table: ', axis(i)%mip_table
       STOP
    END IF
    adim(idim)%axis_idx = iaxis

    ! check whether table_entry indicates dimensionless vertical coordinate
    ! if so, replace any var%axis_idxs that are dimensionless with this
    !  axis coordinate
    ! also allocate space for zfactor indices and possible zfactor bounds 
    !  indices

    IF (associated(axis(iaxis)%zfac_names)) THEN
       ALLOCATE(adim(idim)%zfac_idxs(SIZE(axis(iaxis)%zfac_names)))
       adim(idim)%zfac_idxs = 0
       IF (associated(axis(iaxis)%zfac_bnds_names))  THEN
          ALLOCATE(   &
               adim(idim)%zfac_bnds_idxs(SIZE(axis(iaxis)%zfac_bnds_names)))
          adim(idim)%zfac_bnds_idxs = 0
       END IF
       ! cycle through variable table entries
       DO n=1,num_vars
          IF (similar(mip_tbl(itable)%table_id, axis(i)%mip_table)) THEN
             DO i=1,var(n)%rank
                IF (var(n)%axis_idxs(i)== -1) THEN
                   var(n)%axis_idxs(i) = iaxis
                END IF
             END DO
          END IF
       END DO
    END IF

    RETURN
  END SUBROUTINE axis_preamble




  FUNCTION cmor_axis_d2(table, table_entry, units, length, coord_vals,  &
       cell_bounds, interval) RESULT(idim)

    IMPLICIT NONE

    INTEGER :: idim
    CHARACTER(len=*), INTENT(IN) :: table
    CHARACTER(len=*), INTENT(IN) :: table_entry
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: units
    INTEGER, INTENT(IN), OPTIONAL :: length
    DOUBLE PRECISION, INTENT(IN), DIMENSION(:) :: coord_vals
    DOUBLE PRECISION, INTENT(IN), DIMENSION(:,:) :: cell_bounds
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: interval

    CHARACTER(LEN=a64) :: units_copy
    INTEGER :: itable, iaxis, lendim, ierr, istat
    DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:) :: values
    DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:,:) :: bounds


    CALL axis_preamble(table, table_entry, itable, idim, iaxis, ierr)
    IF (ierr /= 0 ) THEN
       PRINT*, 'You were calling cmor_axis and attempting to define ', TRIM(table_entry)
       PRINT*, '   in table: ', TRIM(table)
       STOP
    END IF
    ! check whether required arguments are present
    
    IF ((.not. PRESENT(units)) .and. (axis(iaxis)%arg_cntl(3) == 1)) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, 'The "units" argument is required when defining axis: ', TRIM(table_entry)
       PRINT*, '   in table: ', table
       STOP
       
    END IF
    IF ((axis(iaxis)%arg_cntl(4) == 1) .and. (.not. PRESENT(length))) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, 'The "length" argument is required when defining axis: ', TRIM(table_entry)
       PRINT*, '   in table: ', table
       STOP
       
    END IF
        
    IF (axis(iaxis)%arg_cntl(7) == 1) THEN
       IF (.not. PRESENT(interval)) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, 'The "interval" argument is required when defining axis: ', &
               TRIM(table_entry)
          PRINT*, '   in table: ', table
          STOP
          
       ELSE IF (LEN_TRIM(interval) == 0) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, 'The "interval" argument is required, but was passed as a null string'
          PRINT*, '   when defining axis: ', TRIM(table_entry), ' in table: ', table
          STOP
       END IF
    END IF    
        
    IF (PRESENT(units)) THEN
       units_copy = units
    ELSE
       units_copy = 'none'
    END IF
    
    IF (PRESENT(length)) THEN
       lendim = length
       
       IF (length > SIZE(coord_vals)) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, 'The length of the array passed as the "coord_vals" argument '
          PRINT*, '   must be >= to the value of the "length" argument.'
          PRINT*, '   You passed size(coord_vals) = ', SIZE(coord_vals)
          PRINT*, '   and length = ', length
          PRINT*, '   You were defining axis: ', TRIM(table_entry), ' in table: ', table
          STOP
       ELSE IF (length < 1) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, 'The "length" argument was passed with a value of 0, but should be '
          PRINT*, '   set to the number of coordinate values for the axis: ', &
               TRIM(table_entry)
          PRINT*, '   in table: ', table
          STOP    
       ELSE IF (length < SIZE(coord_vals)) THEN
          PRINT*, 'Warning encountered in CMOR: cmor_axis'
          PRINT*, 'The size of the coord_vals array passed to cmor_axis was '
          PRINT*, '   greater than the "length" argument'
          PRINT*, '   This is o.k., but only the first, ', length, ' values of ' 
          PRINT*, '   coord_vals will be used, although it is dimensioned ', &
               size(coord_vals) 
          PRINT*, '   To suppress this warning, dimension coord_vals ', length
          PRINT*, '   when defining axis: ', TRIM(table_entry), ' in table: ', table
       END IF
    ELSE       
       lendim = SIZE(coord_vals)
    END IF
    
    adim(idim)%lendim = lendim
    
    ALLOCATE(values(lendim), STAT=istat)
    
    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*,  &
            '   Unable to allocate memory for "values", which is dimensioned', &
            lendim
       stop
    END IF
    
    values = coord_vals(1:lendim)    
    
    IF (axis(iaxis)%arg_cntl(6) == 5) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, '   Cell bounds are forbidden for axis: ', &
            TRIM(axis(iaxis)%table_entry)
       PRINT*, '   found in table: ', TRIM(axis(iaxis)%mip_table)
       PRINT*, &
            '   User should omit the cell_bounds argument in calling cmor_axis'
       stop
    ELSE IF (axis(iaxis)%arg_cntl(6) == 4) THEN
       PRINT*, 'Warning encountered in CMOR: cmor_axis'
       PRINT*, '   Cell bounds are ignored for axis: ', &
            TRIM(axis(iaxis)%table_entry)
       PRINT*, '   found in table: ', TRIM(axis(iaxis)%mip_table)
       PRINT*, '   To suppress this warning, User should omit the' 
       PRINT*, '   cell_bounds argument in calling cmor_axis'
    ELSE IF (SIZE(cell_bounds,1) /= 2) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, '   When passing a 2-dimensional array containing cell_bounds'
       PRINT*, '   to cmor_axis, the first dimension must be length 2, but'
       PRINT*, '   was length: ', SIZE(cell_bounds,1)
       PRINT*, '   User was defining ', TRIM(axis(iaxis)%table_entry), &
            ' in table: ', TRIM(axis(iaxis)%mip_table)
       stop
    ELSE IF (SIZE(cell_bounds,2) < lendim) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, '   When passing a 2-dimensional array containing cell_bounds'
       PRINT*, '   to cmor_axis, the first dimension must be >= ', lendim, ' but'
       PRINT*, '   was length: ', SIZE(cell_bounds,2)
       PRINT*, '   User was defining ', TRIM(axis(iaxis)%table_entry), &
            ' in table: ', TRIM(axis(iaxis)%mip_table)
       stop
    ELSEIF (SIZE(cell_bounds,2) > lendim) THEN
       PRINT*, 'Warning encountered in CMOR: cmor_axis'
       PRINT*, 'The size of the second dimension of cell_bounds array which'
       PRINT*, '   was passed to cmor_axis was '
       PRINT*, '   greater than the "length" argument'
       PRINT*, '   This is o.k., but only the first, ', length, ' values of ' 
       PRINT*, '   cell_bounds will be used, although it is dimensioned ', &
            size(cell_bounds,2) 
       PRINT*, '   To suppress this warning, dimension cell_bounds 2,', length
       PRINT*, '   when defining axis: ', TRIM(table_entry), ' in table: ', table
    ELSE              
       
       ALLOCATE(bounds(2,lendim), STAT=istat)
       
       IF (istat /= 0 ) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, '   Unable to allocate memory for "bounds" (cell_bounds),'
          PRINT*, '    which is dimensioned 2,', lendim
          stop
       END IF
       
       bounds = cell_bounds(:,1:lendim)
       
    END IF
    
    IF (ALLOCATED(bounds)) THEN
       ierr = def_dim(itable, idim, iaxis, units_copy, lendim, values,   &
            bounds, interval) 
    ELSE
       ierr = def_dim(itable, idim, iaxis, units_copy, lendim, values,  &
            interval=interval) 
    END IF
    IF (ierr /= 0) THEN
       print*, 'CMOR error 20.5 '
       stop
    END IF
    
    DEALLOCATE (values, STAT=istat)
    
    IF (istat /= 0) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, '   Unable to deallocate memory for "values"'
       stop
    END IF
    
    IF (ALLOCATED(bounds)) THEN
       
       DEALLOCATE (bounds, STAT=istat)
       
       IF (istat /= 0) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, '   Unable to deallocate memory for "bounds"'
          stop
       END IF
    END IF
    
    RETURN
    
  END FUNCTION cmor_axis_d2
  

  FUNCTION cmor_axis_d1(table, table_entry, units, length, coord_vals,  &
       cell_bounds, interval) RESULT(idim)

    IMPLICIT NONE

    INTEGER :: idim
    CHARACTER(len=*), INTENT(IN) :: table
    CHARACTER(len=*), INTENT(IN) :: table_entry
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: units
    INTEGER, INTENT(IN), OPTIONAL :: length
    DOUBLE PRECISION, INTENT(IN), DIMENSION(:) :: coord_vals
    DOUBLE PRECISION, INTENT(IN), DIMENSION(:) :: cell_bounds
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: interval

    CHARACTER(LEN=a64) :: units_copy
    INTEGER :: itable, iaxis, lendim, ierr, istat, i
    DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:) :: values
    DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:,:) :: bounds


    CALL axis_preamble(table, table_entry, itable, idim, iaxis, ierr)
    IF (ierr /= 0 ) THEN
       PRINT*, 'You were calling cmor_axis and attempting to define ', TRIM(table_entry)
       PRINT*, '   in table: ', TRIM(table)
       STOP
    END IF
    ! check whether required arguments are present
    
    IF ((.not. PRESENT(units)) .and. (axis(iaxis)%arg_cntl(3) == 1)) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, 'The "units" argument is required when defining axis: ', TRIM(table_entry)
       PRINT*, '   in table: ', table
       STOP
       
    END IF
    IF ((axis(iaxis)%arg_cntl(4) == 1) .and. (.not. PRESENT(length))) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, 'The "length" argument is required when defining axis: ', TRIM(table_entry)
       PRINT*, '   in table: ', table
       STOP
       
    END IF
        
    IF (axis(iaxis)%arg_cntl(7) == 1) THEN
       IF (.not. PRESENT(interval)) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, 'The "interval" argument is required when defining axis: ', &
               TRIM(table_entry)
          PRINT*, '   in table: ', table
          STOP
          
       ELSE IF (LEN_TRIM(interval) == 0) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, 'The "interval" argument is required, but was passed as a null string'
          PRINT*, '   when defining axis: ', TRIM(table_entry), ' in table: ', table
          STOP
       END IF
    END IF    
        
    IF (PRESENT(units)) THEN
       units_copy = units
    ELSE
       units_copy = 'none'
    END IF
    
    IF (PRESENT(length)) THEN
       lendim = length
       
       IF (length > SIZE(coord_vals)) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, 'The length of the array passed as the "coord_vals" argument '
          PRINT*, '   must be >= to the value of the "length" argument.'
          PRINT*, '   You passed size(coord_vals) = ', SIZE(coord_vals)
          PRINT*, '   and length = ', length
          PRINT*, '   You were defining axis: ', TRIM(table_entry), ' in table: ', table
          STOP
       ELSE IF (length < 1) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, 'The "length" argument was passed with a value of 0, but should be '
          PRINT*, '   set to the number of coordinate values for the axis: ', &
               TRIM(table_entry)
          PRINT*, '   in table: ', table
          STOP    
       ELSE IF (length < SIZE(coord_vals)) THEN
          PRINT*, 'Warning encountered in CMOR: cmor_axis'
          PRINT*, 'The size of the coord_vals array passed to cmor_axis was '
          PRINT*, '   greater than the "length" argument'
          PRINT*, '   This is o.k., but only the first, ', length, ' values of ' 
          PRINT*, '   coord_vals will be used, although it is dimensioned ', &
               size(coord_vals) 
          PRINT*, '   To suppress this warning, dimension coord_vals ', length
          PRINT*, '   when defining axis: ', TRIM(table_entry), ' in table: ', table
       END IF
    ELSE       
       lendim = SIZE(coord_vals)
    END IF
    
    adim(idim)%lendim = lendim
    
    ALLOCATE(values(lendim), STAT=istat)
    
    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*,  &
            '   Unable to allocate memory for "values", which is dimensioned', &
            lendim
       stop
    END IF
    
    values = coord_vals(1:lendim)    
    
    IF (axis(iaxis)%arg_cntl(6) == 5) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, '   Cell bounds are forbidden for axis: ', &
            TRIM(axis(iaxis)%table_entry)
       PRINT*, '   found in table: ', TRIM(axis(iaxis)%mip_table)
       PRINT*, &
            '   User should omit the cell_bounds argument in calling cmor_axis'
       stop
    ELSE IF (axis(iaxis)%arg_cntl(6) == 4) THEN
       PRINT*, 'Warning encountered in CMOR: cmor_axis'
       PRINT*, '   Cell bounds are ignored for axis: ', &
            TRIM(axis(iaxis)%table_entry)
       PRINT*, '   found in table: ', TRIM(axis(iaxis)%mip_table)
       PRINT*, '   To suppress this warning, User should omit the' 
       PRINT*, '   cell_bounds argument in calling cmor_axis'
    ELSE IF (SIZE(cell_bounds) < lendim+1) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, '   When passing a 1-dimensional array containing cell_bounds'
       PRINT*, '   to cmor_axis, the dimension must be >= ', lendim+1, ' but'
       PRINT*, '   was length: ', SIZE(cell_bounds)
       PRINT*, '   User was defining ', TRIM(axis(iaxis)%table_entry), &
            ' in table: ', TRIM(axis(iaxis)%mip_table)
       stop
    ELSEIF (SIZE(cell_bounds) > lendim+1) THEN
       PRINT*, 'Warning encountered in CMOR: cmor_axis'
       PRINT*, 'The size of the dimension of cell_bounds array which'
       PRINT*, '   was passed to cmor_axis was '
       PRINT*, '   greater than the "length" argument'
       PRINT*, '   This is o.k., but only the first, ', lendim+1, ' values of ' 
       PRINT*, '   cell_bounds will be used, although it is dimensioned ', &
            size(cell_bounds) 
       PRINT*, '   To suppress this warning, dimension cell_bounds ', lendim+1
       PRINT*, '   when defining axis: ', TRIM(table_entry), ' in table: ', table
    ELSE              
       
       ALLOCATE(bounds(2,lendim), STAT=istat)
       
       IF (istat /= 0 ) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, '   Unable to allocate memory for "bounds" (cell_bounds),'
          PRINT*, '    which is dimensioned 2,', lendim
          stop
       END IF
       
       DO i=1,lendim
          bounds(1,i) = cell_bounds(i)
          bounds(2,i) = cell_bounds(i+1)
       END DO
       
    END IF
    
    IF (ALLOCATED(bounds)) THEN
       ierr = def_dim(itable, idim, iaxis, units_copy, lendim, values,   &
            bounds, interval) 
    ELSE
       ierr = def_dim(itable, idim, iaxis, units_copy, lendim, values,  &
            interval=interval) 
    END IF
    IF (ierr /= 0) THEN
       print*, 'CMOR error 20.5 '
       stop
    END IF
    
    DEALLOCATE (values, STAT=istat)
    
    IF (istat /= 0) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, '   Unable to deallocate memory for "values"'
       stop
    END IF
    
    IF (ALLOCATED(bounds)) THEN
       
       DEALLOCATE (bounds, STAT=istat)
       
       IF (istat /= 0) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, '   Unable to deallocate memory for "bounds"'
          stop
       END IF
    END IF
    
    RETURN
    
  END FUNCTION cmor_axis_d1




  FUNCTION cmor_axis_d0(table, table_entry, units, length, coord_vals,  &
       cell_bounds, interval) RESULT(idim)

    IMPLICIT NONE

    INTEGER :: idim
    CHARACTER(len=*), INTENT(IN) :: table
    CHARACTER(len=*), INTENT(IN) :: table_entry
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: units
    INTEGER, INTENT(IN), OPTIONAL :: length
    DOUBLE PRECISION, INTENT(IN), DIMENSION(:) :: coord_vals
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: cell_bounds
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: interval

    CHARACTER(LEN=a64) :: units_copy
    INTEGER :: itable, iaxis, lendim, ierr, istat, i
    DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:) :: values
    DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:,:) :: bounds


    CALL axis_preamble(table, table_entry, itable, idim, iaxis, ierr)
    IF (ierr /= 0 ) THEN
       PRINT*, 'You were calling cmor_axis and attempting to define ', TRIM(table_entry)
       PRINT*, '   in table: ', TRIM(table)
       STOP
    END IF
    ! check whether required arguments are present
    
    IF ((.not. PRESENT(units)) .and. (axis(iaxis)%arg_cntl(3) == 1)) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, 'The "units" argument is required when defining axis: ', TRIM(table_entry)
       PRINT*, '   in table: ', table
       STOP
       
    END IF
    IF ((axis(iaxis)%arg_cntl(4) == 1) .and. (.not. PRESENT(length))) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, 'The "length" argument is required when defining axis: ', TRIM(table_entry)
       PRINT*, '   in table: ', table
       STOP
       
    END IF
    
    IF ((axis(iaxis)%arg_cntl(6) == 1) .and. PRESENT(cell_bounds)) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, 'The "cell_bounds" argument is required when defining axis: ', &
            TRIM(table_entry)
       PRINT*, '   in table: ', table
       PRINT*, '   but you passed a string in its place with the value = ', &
            TRIM(cell_bounds)
       STOP
    END IF    
    
    IF (axis(iaxis)%arg_cntl(7) == 1) THEN
       IF (.not. PRESENT(interval)) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, 'The "interval" argument is required when defining axis: ', &
               TRIM(table_entry)
          PRINT*, '   in table: ', table
          STOP
          
       ELSE IF (LEN_TRIM(interval) == 0) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, 'The "interval" argument is required, but was passed as a null string'
          PRINT*, '   when defining axis: ', TRIM(table_entry), ' in table: ', table
          STOP
       END IF
    END IF    
        
    IF (PRESENT(units)) THEN
       units_copy = units
    ELSE
       units_copy = 'none'
    END IF
    
    IF (PRESENT(length)) THEN
       lendim = length
       
       IF (length > SIZE(coord_vals)) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, 'The length of the array passed as the "coord_vals" argument '
          PRINT*, '   must be >= to the value of the "length" argument.'
          PRINT*, '   You passed size(coord_vals) = ', SIZE(coord_vals)
          PRINT*, '   and length = ', length
          PRINT*, '   You were defining axis: ', TRIM(table_entry), ' in table: ', table
          STOP
       ELSE IF (length < 1) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, 'The "length" argument was passed with a value of 0, but should be '
          PRINT*, '   set to the number of coordinate values for the axis: ', &
               TRIM(table_entry)
          PRINT*, '   in table: ', table
          STOP    
       ELSE IF (length < SIZE(coord_vals)) THEN
          PRINT*, 'Warning encountered in CMOR: cmor_axis'
          PRINT*, 'The size of the coord_vals array passed to cmor_axis was '
          PRINT*, '   greater than the "length" argument'
          PRINT*, '   This is o.k., but only the first, ', length, ' values of ' 
          PRINT*, '   coord_vals will be used, although it is dimensioned ', &
               size(coord_vals) 
          PRINT*, '   To suppress this warning, dimension coord_vals ', length
          PRINT*, '   when defining axis: ', TRIM(table_entry), ' in table: ', table
       END IF
    ELSE       
       lendim = SIZE(coord_vals)
    END IF
    
    adim(idim)%lendim = lendim
    
    ALLOCATE(values(lendim), STAT=istat)
    
    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*,  &
            '   Unable to allocate memory for "values", which is dimensioned', &
            lendim
       stop
    END IF
    
    values = coord_vals(1:lendim)    
          
    IF (associated(axis(iaxis)%zfac_bnds_names)) THEN
       ALLOCATE(bounds(2,lendim), STAT=istat)
       
       IF (istat /= 0 ) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, '   Unable to allocate memory for "bounds" (zfactor bounds),'
          PRINT*, '    which is dimensioned 2,', lendim
          stop
       END IF
       
       bounds = 0.0
    END IF
    

    IF (ALLOCATED(bounds)) THEN
       ierr = def_dim(itable, idim, iaxis, units_copy, lendim, values,   &
            bounds, interval) 
    ELSE
       ierr = def_dim(itable, idim, iaxis, units_copy, lendim, values,  &
            interval=interval) 
    END IF
    IF (ierr /= 0) THEN
       print*, 'CMOR error 20.5 '
       stop
    END IF
    
    DEALLOCATE (values, STAT=istat)
    
    IF (istat /= 0) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, '   Unable to deallocate memory for "values"'
       stop
    END IF
    
    IF (ALLOCATED(bounds)) THEN
       
       DEALLOCATE (bounds, STAT=istat)
       
       IF (istat /= 0) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, '   Unable to deallocate memory for "bounds"'
          stop
       END IF
    END IF
    
    RETURN
    
  END FUNCTION cmor_axis_d0
  




  FUNCTION cmor_axis_r2(table, table_entry, units, length, coord_vals,  &
       cell_bounds, interval) RESULT(idim)

    IMPLICIT NONE

    INTEGER :: idim
    CHARACTER(len=*), INTENT(IN) :: table
    CHARACTER(len=*), INTENT(IN) :: table_entry
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: units
    INTEGER, INTENT(IN), OPTIONAL :: length
    REAL, INTENT(IN), DIMENSION(:) :: coord_vals
    REAL, INTENT(IN), DIMENSION(:,:) :: cell_bounds
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: interval

    CHARACTER(LEN=a64) :: units_copy
    INTEGER :: itable, iaxis, lendim, ierr, istat
    DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:) :: values
    DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:,:) :: bounds


    CALL axis_preamble(table, table_entry, itable, idim, iaxis, ierr)
    IF (ierr /= 0 ) THEN
       PRINT*, 'You were calling cmor_axis and attempting to define ', TRIM(table_entry)
       PRINT*, '   in table: ', TRIM(table)
       STOP
    END IF
    ! check whether required arguments are present
    
    IF ((.not. PRESENT(units)) .and. (axis(iaxis)%arg_cntl(3) == 1)) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, 'The "units" argument is required when defining axis: ', TRIM(table_entry)
       PRINT*, '   in table: ', table
       STOP
       
    END IF
    IF ((axis(iaxis)%arg_cntl(4) == 1) .and. (.not. PRESENT(length))) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, 'The "length" argument is required when defining axis: ', TRIM(table_entry)
       PRINT*, '   in table: ', table
       STOP
       
    END IF
        
    IF (axis(iaxis)%arg_cntl(7) == 1) THEN
       IF (.not. PRESENT(interval)) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, 'The "interval" argument is required when defining axis: ', &
               TRIM(table_entry)
          PRINT*, '   in table: ', table
          STOP
          
       ELSE IF (LEN_TRIM(interval) == 0) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, 'The "interval" argument is required, but was passed as a null string'
          PRINT*, '   when defining axis: ', TRIM(table_entry), ' in table: ', table
          STOP
       END IF
    END IF    
        
    IF (PRESENT(units)) THEN
       units_copy = units
    ELSE
       units_copy = 'none'
    END IF
    
    IF (PRESENT(length)) THEN
       lendim = length
       
       IF (length > SIZE(coord_vals)) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, 'The length of the array passed as the "coord_vals" argument '
          PRINT*, '   must be >= to the value of the "length" argument.'
          PRINT*, '   You passed size(coord_vals) = ', SIZE(coord_vals)
          PRINT*, '   and length = ', length
          PRINT*, '   You were defining axis: ', TRIM(table_entry), ' in table: ', table
          STOP
       ELSE IF (length < 1) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, 'The "length" argument was passed with a value of 0, but should be '
          PRINT*, '   set to the number of coordinate values for the axis: ', &
               TRIM(table_entry)
          PRINT*, '   in table: ', table
          STOP    
       ELSE IF (length < SIZE(coord_vals)) THEN
          PRINT*, 'Warning encountered in CMOR: cmor_axis'
          PRINT*, 'The size of the coord_vals array passed to cmor_axis was '
          PRINT*, '   greater than the "length" argument'
          PRINT*, '   This is o.k., but only the first, ', length, ' values of ' 
          PRINT*, '   coord_vals will be used, although it is dimensioned ', &
               size(coord_vals) 
          PRINT*, '   To suppress this warning, dimension coord_vals ', length
          PRINT*, '   when defining axis: ', TRIM(table_entry), ' in table: ', table
       END IF
    ELSE       
       lendim = SIZE(coord_vals)
    END IF
    
    adim(idim)%lendim = lendim
    
    ALLOCATE(values(lendim), STAT=istat)
    
    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*,  &
            '   Unable to allocate memory for "values", which is dimensioned', &
            lendim
       stop
    END IF
    
    values = coord_vals(1:lendim)    
    
    IF (axis(iaxis)%arg_cntl(6) == 5) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, '   Cell bounds are forbidden for axis: ', &
            TRIM(axis(iaxis)%table_entry)
       PRINT*, '   found in table: ', TRIM(axis(iaxis)%mip_table)
       PRINT*, &
            '   User should omit the cell_bounds argument in calling cmor_axis'
       stop
    ELSE IF (axis(iaxis)%arg_cntl(6) == 4) THEN
       PRINT*, 'Warning encountered in CMOR: cmor_axis'
       PRINT*, '   Cell bounds are ignored for axis: ', &
            TRIM(axis(iaxis)%table_entry)
       PRINT*, '   found in table: ', TRIM(axis(iaxis)%mip_table)
       PRINT*, '   To suppress this warning, User should omit the' 
       PRINT*, '   cell_bounds argument in calling cmor_axis'
    ELSE IF (SIZE(cell_bounds,1) /= 2) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, '   When passing a 2-dimensional array containing cell_bounds'
       PRINT*, '   to cmor_axis, the first dimension must be length 2, but'
       PRINT*, '   was length: ', SIZE(cell_bounds,1)
       PRINT*, '   User was defining ', TRIM(axis(iaxis)%table_entry), &
            ' in table: ', TRIM(axis(iaxis)%mip_table)
       stop
    ELSE IF (SIZE(cell_bounds,2) < lendim) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, '   When passing a 2-dimensional array containing cell_bounds'
       PRINT*, '   to cmor_axis, the first dimension must be >= ', lendim, ' but'
       PRINT*, '   was length: ', SIZE(cell_bounds,2)
       PRINT*, '   User was defining ', TRIM(axis(iaxis)%table_entry), &
            ' in table: ', TRIM(axis(iaxis)%mip_table)
       stop
    ELSEIF (SIZE(cell_bounds,2) > lendim) THEN
       PRINT*, 'Warning encountered in CMOR: cmor_axis'
       PRINT*, 'The size of the second dimension of cell_bounds array which'
       PRINT*, '   was passed to cmor_axis was '
       PRINT*, '   greater than the "length" argument'
       PRINT*, '   This is o.k., but only the first, ', length, ' values of ' 
       PRINT*, '   cell_bounds will be used, although it is dimensioned ', &
            size(cell_bounds,2) 
       PRINT*, '   To suppress this warning, dimension cell_bounds 2,', length
       PRINT*, '   when defining axis: ', TRIM(table_entry), ' in table: ', table
    ELSE              
       
       ALLOCATE(bounds(2,lendim), STAT=istat)
       
       IF (istat /= 0 ) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, '   Unable to allocate memory for "bounds" (cell_bounds),'
          PRINT*, '    which is dimensioned 2,', lendim
          stop
       END IF
       
       bounds = cell_bounds(:,1:lendim)
       
    END IF
    
    IF (ALLOCATED(bounds)) THEN
       ierr = def_dim(itable, idim, iaxis, units_copy, lendim, values,   &
            bounds, interval) 
    ELSE
       ierr = def_dim(itable, idim, iaxis, units_copy, lendim, values,  &
            interval=interval) 
    END IF
    IF (ierr /= 0) THEN
       print*, 'CMOR error 20.5 '
       stop
    END IF
    
    DEALLOCATE (values, STAT=istat)
    
    IF (istat /= 0) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, '   Unable to deallocate memory for "values"'
       stop
    END IF
    
    IF (ALLOCATED(bounds)) THEN
       
       DEALLOCATE (bounds, STAT=istat)
       
       IF (istat /= 0) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, '   Unable to deallocate memory for "bounds"'
          stop
       END IF
    END IF
    
    RETURN
    
  END FUNCTION cmor_axis_r2
  

  FUNCTION cmor_axis_r1(table, table_entry, units, length, coord_vals,  &
       cell_bounds, interval) RESULT(idim)

    IMPLICIT NONE

    INTEGER :: idim
    CHARACTER(len=*), INTENT(IN) :: table
    CHARACTER(len=*), INTENT(IN) :: table_entry
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: units
    INTEGER, INTENT(IN), OPTIONAL :: length
    REAL, INTENT(IN), DIMENSION(:) :: coord_vals
    REAL, INTENT(IN), DIMENSION(:) :: cell_bounds
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: interval

    CHARACTER(LEN=a64) :: units_copy
    INTEGER :: itable, iaxis, lendim, ierr, istat, i
    DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:) :: values
    DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:,:) :: bounds


    CALL axis_preamble(table, table_entry, itable, idim, iaxis, ierr)
    IF (ierr /= 0 ) THEN
       PRINT*, 'You were calling cmor_axis and attempting to define ', TRIM(table_entry)
       PRINT*, '   in table: ', TRIM(table)
       STOP
    END IF
    ! check whether required arguments are present
    
    IF ((.not. PRESENT(units)) .and. (axis(iaxis)%arg_cntl(3) == 1)) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, 'The "units" argument is required when defining axis: ', TRIM(table_entry)
       PRINT*, '   in table: ', table
       STOP
       
    END IF
    IF ((axis(iaxis)%arg_cntl(4) == 1) .and. (.not. PRESENT(length))) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, 'The "length" argument is required when defining axis: ', TRIM(table_entry)
       PRINT*, '   in table: ', table
       STOP
       
    END IF
        
    IF (axis(iaxis)%arg_cntl(7) == 1) THEN
       IF (.not. PRESENT(interval)) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, 'The "interval" argument is required when defining axis: ', &
               TRIM(table_entry)
          PRINT*, '   in table: ', table
          STOP
          
       ELSE IF (LEN_TRIM(interval) == 0) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, 'The "interval" argument is required, but was passed as a null string'
          PRINT*, '   when defining axis: ', TRIM(table_entry), ' in table: ', table
          STOP
       END IF
    END IF    
        
    IF (PRESENT(units)) THEN
       units_copy = units
    ELSE
       units_copy = 'none'
    END IF
    
    IF (PRESENT(length)) THEN
       lendim = length
       
       IF (length > SIZE(coord_vals)) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, 'The length of the array passed as the "coord_vals" argument '
          PRINT*, '   must be >= to the value of the "length" argument.'
          PRINT*, '   You passed size(coord_vals) = ', SIZE(coord_vals)
          PRINT*, '   and length = ', length
          PRINT*, '   You were defining axis: ', TRIM(table_entry), ' in table: ', table
          STOP
       ELSE IF (length < 1) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, 'The "length" argument was passed with a value of 0, but should be '
          PRINT*, '   set to the number of coordinate values for the axis: ', &
               TRIM(table_entry)
          PRINT*, '   in table: ', table
          STOP    
       ELSE IF (length < SIZE(coord_vals)) THEN
          PRINT*, 'Warning encountered in CMOR: cmor_axis'
          PRINT*, 'The size of the coord_vals array passed to cmor_axis was '
          PRINT*, '   greater than the "length" argument'
          PRINT*, '   This is o.k., but only the first, ', length, ' values of ' 
          PRINT*, '   coord_vals will be used, although it is dimensioned ', &
               size(coord_vals) 
          PRINT*, '   To suppress this warning, dimension coord_vals ', length
          PRINT*, '   when defining axis: ', TRIM(table_entry), ' in table: ', table
       END IF
    ELSE       
       lendim = SIZE(coord_vals)
    END IF
    
    adim(idim)%lendim = lendim
    
    ALLOCATE(values(lendim), STAT=istat)
    
    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*,  &
            '   Unable to allocate memory for "values", which is dimensioned', &
            lendim
       stop
    END IF
    
    values = coord_vals(1:lendim)    
    
    IF (axis(iaxis)%arg_cntl(6) == 5) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, '   Cell bounds are forbidden for axis: ', &
            TRIM(axis(iaxis)%table_entry)
       PRINT*, '   found in table: ', TRIM(axis(iaxis)%mip_table)
       PRINT*, &
            '   User should omit the cell_bounds argument in calling cmor_axis'
       stop
    ELSE IF (axis(iaxis)%arg_cntl(6) == 4) THEN
       PRINT*, 'Warning encountered in CMOR: cmor_axis'
       PRINT*, '   Cell bounds are ignored for axis: ', &
            TRIM(axis(iaxis)%table_entry)
       PRINT*, '   found in table: ', TRIM(axis(iaxis)%mip_table)
       PRINT*, '   To suppress this warning, User should omit the' 
       PRINT*, '   cell_bounds argument in calling cmor_axis'
    ELSE IF (SIZE(cell_bounds) < lendim+1) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, '   When passing a 1-dimensional array containing cell_bounds'
       PRINT*, '   to cmor_axis, the dimension must be >= ', lendim+1, ' but'
       PRINT*, '   was length: ', SIZE(cell_bounds)
       PRINT*, '   User was defining ', TRIM(axis(iaxis)%table_entry), &
            ' in table: ', TRIM(axis(iaxis)%mip_table)
       stop
    ELSEIF (SIZE(cell_bounds) > lendim+1) THEN
       PRINT*, 'Warning encountered in CMOR: cmor_axis'
       PRINT*, 'The size of the dimension of cell_bounds array which'
       PRINT*, '   was passed to cmor_axis was '
       PRINT*, '   greater than the "length" argument'
       PRINT*, '   This is o.k., but only the first, ', lendim+1, ' values of ' 
       PRINT*, '   cell_bounds will be used, although it is dimensioned ', &
            size(cell_bounds) 
       PRINT*, '   To suppress this warning, dimension cell_bounds ', lendim+1
       PRINT*, '   when defining axis: ', TRIM(table_entry), ' in table: ', table
    ELSE              
       
       ALLOCATE(bounds(2,lendim), STAT=istat)
       
       IF (istat /= 0 ) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, '   Unable to allocate memory for "bounds" (cell_bounds),'
          PRINT*, '    which is dimensioned 2,', lendim
          stop
       END IF
       
       DO i=1,lendim
          bounds(1,i) = cell_bounds(i)
          bounds(2,i) = cell_bounds(i+1)
       END DO
       
    END IF
    
    IF (ALLOCATED(bounds)) THEN
       ierr = def_dim(itable, idim, iaxis, units_copy, lendim, values,   &
            bounds, interval) 
    ELSE
       ierr = def_dim(itable, idim, iaxis, units_copy, lendim, values,  &
            interval=interval) 
    END IF
    IF (ierr /= 0) THEN
       print*, 'CMOR error 20.5 '
       stop
    END IF
    
    DEALLOCATE (values, STAT=istat)
    
    IF (istat /= 0) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, '   Unable to deallocate memory for "values"'
       stop
    END IF
    
    IF (ALLOCATED(bounds)) THEN
       
       DEALLOCATE (bounds, STAT=istat)
       
       IF (istat /= 0) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, '   Unable to deallocate memory for "bounds"'
          stop
       END IF
    END IF
    
    RETURN
    
  END FUNCTION cmor_axis_r1




  FUNCTION cmor_axis_r0(table, table_entry, units, length, coord_vals,  &
       cell_bounds, interval) RESULT(idim)

    IMPLICIT NONE

    INTEGER :: idim
    CHARACTER(len=*), INTENT(IN) :: table
    CHARACTER(len=*), INTENT(IN) :: table_entry
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: units
    INTEGER, INTENT(IN), OPTIONAL :: length
    REAL, INTENT(IN), DIMENSION(:) :: coord_vals
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: cell_bounds
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: interval

    CHARACTER(LEN=a64) :: units_copy
    INTEGER :: itable, iaxis, lendim, ierr, istat, i
    DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:) :: values
    DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:,:) :: bounds


    CALL axis_preamble(table, table_entry, itable, idim, iaxis, ierr)
    IF (ierr /= 0 ) THEN
       PRINT*, 'You were calling cmor_axis and attempting to define ', TRIM(table_entry)
       PRINT*, '   in table: ', TRIM(table)
       STOP
    END IF
    ! check whether required arguments are present
    
    IF ((.not. PRESENT(units)) .and. (axis(iaxis)%arg_cntl(3) == 1)) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, 'The "units" argument is required when defining axis: ', TRIM(table_entry)
       PRINT*, '   in table: ', table
       STOP
       
    END IF
    IF ((axis(iaxis)%arg_cntl(4) == 1) .and. (.not. PRESENT(length))) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, 'The "length" argument is required when defining axis: ', TRIM(table_entry)
       PRINT*, '   in table: ', table
       STOP
       
    END IF
    
    IF ((axis(iaxis)%arg_cntl(6) == 1) .and. PRESENT(cell_bounds)) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, 'The "cell_bounds" argument is required when defining axis: ', &
            TRIM(table_entry)
       PRINT*, '   in table: ', table
       PRINT*, '   but you passed a string in its place with the value = ', &
            TRIM(cell_bounds)
       STOP
    END IF    
    
    IF (axis(iaxis)%arg_cntl(7) == 1) THEN
       IF (.not. PRESENT(interval)) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, 'The "interval" argument is required when defining axis: ', &
               TRIM(table_entry)
          PRINT*, '   in table: ', table
          STOP
          
       ELSE IF (LEN_TRIM(interval) == 0) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, 'The "interval" argument is required, but was passed as a null string'
          PRINT*, '   when defining axis: ', TRIM(table_entry), ' in table: ', table
          STOP
       END IF
    END IF    
        
    IF (PRESENT(units)) THEN
       units_copy = units
    ELSE
       units_copy = 'none'
    END IF
    
    IF (PRESENT(length)) THEN
       lendim = length
       
       IF (length > SIZE(coord_vals)) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, 'The length of the array passed as the "coord_vals" argument '
          PRINT*, '   must be >= to the value of the "length" argument.'
          PRINT*, '   You passed size(coord_vals) = ', SIZE(coord_vals)
          PRINT*, '   and length = ', length
          PRINT*, '   You were defining axis: ', TRIM(table_entry), ' in table: ', table
          STOP
       ELSE IF (length < 1) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, 'The "length" argument was passed with a value of 0, but should be '
          PRINT*, '   set to the number of coordinate values for the axis: ', &
               TRIM(table_entry)
          PRINT*, '   in table: ', table
          STOP    
       ELSE IF (length < SIZE(coord_vals)) THEN
          PRINT*, 'Warning encountered in CMOR: cmor_axis'
          PRINT*, 'The size of the coord_vals array passed to cmor_axis was '
          PRINT*, '   greater than the "length" argument'
          PRINT*, '   This is o.k., but only the first, ', length, ' values of ' 
          PRINT*, '   coord_vals will be used, although it is dimensioned ', &
               size(coord_vals) 
          PRINT*, '   To suppress this warning, dimension coord_vals ', length
          PRINT*, '   when defining axis: ', TRIM(table_entry), ' in table: ', table
       END IF
    ELSE       
       lendim = SIZE(coord_vals)
    END IF
    
    adim(idim)%lendim = lendim
    
    ALLOCATE(values(lendim), STAT=istat)
    
    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*,  &
            '   Unable to allocate memory for "values", which is dimensioned', &
            lendim
       stop
    END IF
    
    values = coord_vals(1:lendim)    
          
    IF (associated(axis(iaxis)%zfac_bnds_names)) THEN
       ALLOCATE(bounds(2,lendim), STAT=istat)
       
       IF (istat /= 0 ) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, '   Unable to allocate memory for "bounds" (zfactor bounds),'
          PRINT*, '    which is dimensioned 2,', lendim
          stop
       END IF
       
       bounds = 0.0
    END IF
    

    IF (ALLOCATED(bounds)) THEN
       ierr = def_dim(itable, idim, iaxis, units_copy, lendim, values,   &
            bounds, interval) 
    ELSE
       ierr = def_dim(itable, idim, iaxis, units_copy, lendim, values,  &
            interval=interval) 
    END IF
    IF (ierr /= 0) THEN
       print*, 'CMOR error 20.5 '
       stop
    END IF
    
    DEALLOCATE (values, STAT=istat)
    
    IF (istat /= 0) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, '   Unable to deallocate memory for "values"'
       stop
    END IF
    
    IF (ALLOCATED(bounds)) THEN
       
       DEALLOCATE (bounds, STAT=istat)
       
       IF (istat /= 0) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, '   Unable to deallocate memory for "bounds"'
          stop
       END IF
    END IF
    
    RETURN
    
  END FUNCTION cmor_axis_r0
  



  
  FUNCTION cmor_axis_c(table, table_entry, units, length, coord_vals)  &
       RESULT(idim)

    IMPLICIT NONE

    INTEGER :: idim
    CHARACTER(len=*), INTENT(IN) :: table
    CHARACTER(len=*), INTENT(IN) :: table_entry
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: units
    INTEGER, INTENT(IN), OPTIONAL :: length
    CHARACTER(len=*), INTENT(IN), DIMENSION(:) :: coord_vals

    INTEGER :: itable, iaxis, lendim, ierr, istat, i, j
    CHARACTER(len=a32), ALLOCATABLE, DIMENSION(:) :: values
    LOGICAL :: found, reorder

    CALL axis_preamble(table, table_entry, itable, idim, iaxis, ierr)

    IF (ierr /= 0 ) THEN
       PRINT*, 'You were calling cmor_axis and attempting to define ', TRIM(table_entry)
       PRINT*, '   in table: ', TRIM(table)
       STOP
    END IF

    ! check whether required arguments are present
    
    IF (PRESENT(units)) THEN
       IF ((LEN_TRIM(units) /= 0) .and. (.not. similar(units, 'none'))) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, 'The "units" argument is forbidden when defining axis: ', TRIM(table_entry)
          PRINT*, '   in table: ', TRIM(table)
          PRINT*, '   The user passed units = ', TRIM(units)
          PRINT*, '   but this argument should either be omitted or set it to "none"'
          STOP
       END IF
    END IF
    
    IF ((axis(iaxis)%arg_cntl(4) == 1) .and. (.not. PRESENT(length))) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, 'The "length" argument was omitted but is required when defining axis: ', TRIM(table_entry)
       PRINT*, '   in table: ', TRIM(table)
       STOP
    END IF

    IF (PRESENT(length)) THEN
       lendim = length
       
       IF (length > SIZE(coord_vals)) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, 'The length of the array passed as the "coord_vals" argument '
          PRINT*, '   must be >= to the value of the "length" argument.'
          PRINT*, '   You passed size(coord_vals) = ', SIZE(coord_vals)
          PRINT*, '   and length = ', length
          PRINT*, '   You were defining axis: ', TRIM(table_entry), ' in table: ', table
          STOP
       ELSE IF (length < 1) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, 'The "length" argument was passed with a value of 0, but should be '
          PRINT*, '   set to the number of coordinate values for the axis: ', &
               TRIM(table_entry)
          PRINT*, '   in table: ', TRIM(table)
          STOP    
       ELSE IF (length < SIZE(coord_vals)) THEN
          PRINT*, 'Warning encountered in CMOR: cmor_axis'
          PRINT*, 'The size of the coord_vals array passed to cmor_axis was '
          PRINT*, '   greater than the "length" argument'
          PRINT*, '   This is o.k., but only the first, ', length, ' values of ' 
          PRINT*, '   coord_vals will be used, although it is dimensioned ', &
               size(coord_vals) 
          PRINT*, '   To suppress this warning, dimension coord_vals ', length
          PRINT*, '   when defining axis: ', TRIM(table_entry), ' in table: ', table
       END IF
    ELSE       
       lendim = SIZE(coord_vals)
    END IF
    
    adim(idim)%lendim = lendim
    
    ! check whether requested values are present
 
    IF (associated(axis(iaxis)%requested_labels)) THEN
       reorder = .false.
       DO i=1, SIZE(axis(iaxis)%requested_labels)
          found = .false.
          
          DO j=1,lendim
             
             IF (similar(coord_vals(j), axis(iaxis)%requested_labels(i))) THEN
                IF (i /= j) reorder = .true.
                found = .true.
                EXIT
             END IF
             
          END DO
          
          IF (.NOT. found) THEN
             ! warn or error exit ?
             PRINT*, 'Error encountered in CMOR: cmor_axis'
             PRINT*, '    In defining the axis: ', TRIM(table_entry) 
             PRINT*, '    found in table: ', TRIM(table)
             PRINT*, '    The user failed to pass the following requested coordinate value: ', &
                  axis(iaxis)%requested_labels(i)
             STOP
          END IF
          
       END DO

       IF (reorder) THEN
             PRINT*, 'Error encountered in CMOR: cmor_axis'
             PRINT*, '    In defining the axis: ', TRIM(table_entry) 
             PRINT*, '    found in table: ', TRIM(table)
             PRINT*, '    The user passed the coordinate values in the following order: ', &
                  (coord_vals(j), j=1,lendim)
             PRINT*, '    but they should be ordered: ', &
                  (axis(iaxis)%requested_labels(j), j=1,SIZE(axis(iaxis)%requested_labels))
             STOP
       END IF
       
    END IF

    ! check whether when specific coordinate values were requested, user 
    !    also provided additional values.
    IF (associated(axis(iaxis)%requested_labels)) THEN
       
       DO j=1,lendim
          found = .false.

          DO i=1, SIZE(axis(iaxis)%requested_labels)
             IF (similar(coord_vals(j), axis(iaxis)%requested_labels(i))) THEN
                found = .true.
                EXIT
             END IF
          END DO
          
          IF (.NOT. found) THEN
             ! warn or error exit:  extra coordinates found
             PRINT*, 'Error encountered in CMOR: cmor_axis'
             PRINT*, '    In defining the axis: ', TRIM(table_entry) 
             PRINT*, '    found in table: ', TRIM(table)
             PRINT*, '    The user passed the following unrequested coordinate value: ', &
                  axis(iaxis)%requested_labels(i)
             PRINT*, '    This coordinate value is not allowed.'
             STOP
          END IF
          
       END DO
  
    END IF
        
    ! set users units     
    adim(idim)%users_units = init_undef_char   
    
    ALLOCATE(adim(idim)%labels(lendim), STAT=istat)
    
    IF (istat /= 0 ) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*,  &
            '   Unable to allocate memory for "adim(idim)%labels", which is dimensioned', &
            lendim
       stop
    END IF
    adim(idim)%labels(1:lendim) = coord_vals(1:lendim)
    
    axis(iaxis)%type = 'character'
    
    num_dims = idim
    
    RETURN
    
  END FUNCTION cmor_axis_c



  
  FUNCTION cmor_axis_novals(table, table_entry, units, length, interval)  &
       RESULT(idim)
    
    ! treat case when no values or bounds are present
    !    this means time-dimension or dimensionless vertical coordinate

    IMPLICIT NONE

    INTEGER :: idim
    CHARACTER(len=*), INTENT(IN) :: table
    CHARACTER(len=*), INTENT(IN) :: table_entry
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: units
    INTEGER, INTENT(IN), OPTIONAL :: length
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: interval

    CHARACTER(len=a64) :: units_copy
    INTEGER :: itable, iaxis, ierr, m, n
    DOUBLE PRECISION :: dummy

    CALL axis_preamble(table, table_entry, itable, idim, iaxis, ierr)

    IF (ierr /= 0 ) THEN
       PRINT*, 'You were calling cmor_axis and attempting to define ', TRIM(table_entry)
       PRINT*, '   in table: ', TRIM(table)
       STOP
    END IF
    
    ! check whether required arguments are present

    IF ((.not. PRESENT(units)) .and. (axis(iaxis)%arg_cntl(3) == 1)) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, 'The "units" argument is required when defining axis: ', TRIM(table_entry)
       PRINT*, '   in table: ', table
       STOP 
    END IF

    IF ((axis(iaxis)%arg_cntl(4) == 1) .and. (.not. PRESENT(length))) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, 'The "length" argument is required when defining axis: ', TRIM(table_entry)
       PRINT*, '   in table: ', table
       STOP
    END IF

    IF (axis(iaxis)%arg_cntl(5) == 1) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, '   You are required to pass coordinate values when defining axis: ',  TRIM(table_entry)
       PRINT*, '   in table: ', table
       STOP 
    END IF

    IF (axis(iaxis)%arg_cntl(6) == 1) THEN
       PRINT*, 'Error encountered in CMOR: cmor_axis'
       PRINT*, '   You are required to pass cell bounds  when defining axis: ',  TRIM(table_entry)
       PRINT*, '   in table: ', table
       STOP 
    END IF
    
    IF (PRESENT(length)) THEN
       IF (length < 1) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, 'The "length" argument was passed with a value of 0, but should be '
          PRINT*, '   set to the number of coordinate values for the axis: ', &
               TRIM(table_entry)
          PRINT*, '   in table: ', table
          STOP    
       END IF
       adim(idim)%lendim = length
    END IF
    
    IF (axis(iaxis)%arg_cntl(7) == 1) THEN
       IF (.not. PRESENT(interval)) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, 'The "interval" argument is required when defining axis: ', &
               TRIM(table_entry)
          PRINT*, '   in table: ', table
          STOP
          
       ELSE IF (LEN_TRIM(interval) == 0) THEN
          PRINT*, 'Error encountered in CMOR: cmor_axis'
          PRINT*, 'The "interval" argument is required, but was passed as a null string'
          PRINT*, '   when defining axis: ', TRIM(table_entry), ' in table: ', table
          STOP
       END IF
    END IF    
    
    ! possibly scale coordinate values

    IF (PRESENT(units)) THEN
       units_copy = TRIM(ADJUSTL(units))
    ELSE
       units_copy = 'none'
    END IF

    IF (similar(axis(iaxis)%axis, 'T')) THEN

       adim(idim)%users_units = units_copy

       m = INDEX(axis(iaxis)%out_units, '?')
       IF ( m  > 0) THEN
          n = INDEX(units_copy, 'since')
          axis(iaxis)%out_units = TRIM(axis(iaxis)%out_units(1:m-1)) //' '// &
               TRIM(ADJUSTL(units_copy(n+5:LEN(units_copy))))
       END IF
       
    ELSE
       !  get scale-factor for axis (with no values passed)
       CALL get_scale(units_copy, axis(iaxis)%out_units, adim(idim)%users_units, &
            adim(idim)%scale_factor, dummy, ierr)
       
       IF (ierr /= 0) THEN 
          PRINT*, 'CMOR error 3.9'
          stop
       END IF
    END IF

    IF (PRESENT(interval)) THEN
       IF (LEN_TRIM(interval) > 0) THEN
       
          IF (axis(iaxis)%arg_cntl(7) == 5) THEN
             PRINT*, 'Error encountered in CMOR: cmor_axis'
             PRINT*, 'The "interval" argument is forbidden when defining axis: ', &
                  TRIM(table_entry)
             PRINT*, '   in table: ', table
             PRINT*, '   You should either omit this argument or set it to an empty string'
             STOP
          ELSE IF (axis(iaxis)%arg_cntl(7) == 4) THEN
             ! warn & ignore
             PRINT*, 'Warning encountered in CMOR: cmor_axis'
             PRINT*, 'The "interval" argument is ignored when defining axis: ', &
                  TRIM(table_entry)
             PRINT*, '   in table: ', table
             PRINT*, '   To suppress this message, you should either omit the "interval" argument '
             PRINT*, '   or set it to an empty string'
          ELSE
             IF (LEN_TRIM(ADJUSTL(interval)) > a64) THEN
                PRINT*, 'Error encountered in CMOR: cmor_axis'
                PRINT*, '   The "interval" argument must be <= 64 characters long.'
                PRINT*, '   You passed interval = ', TRIM(interval)
                PRINT*, '   which is too long.'
                PRINT*, '   You were defining axis, ',  TRIM(table_entry), &
                     ' in table: ', TRIM(table)
                STOP
             ELSE
                adim(idim)%interval = TRIM(ADJUSTL(interval))
             END IF
          END IF

       END IF
    END IF
    
    num_dims=idim
    
    RETURN

  END FUNCTION cmor_axis_novals

  function string_contains(string,sub) result(icontain)

    implicit none
    character(len=*) string,sub
    integer len_string, len_sub, icontain, i
    
    len_string=len(string)
    len_sub=len(trim(adjustl(sub)))
    
    icontain = 0
    
    if (len_sub.gt.len_string) return
    
    do i = 1, len_string-len_sub+1
       if (similar(string(i:i+len_sub),trim(adjustl(sub)))) icontain = 1
    enddo
    return
  end function string_contains

#ifdef CDMS
  subroutine cmor_time_and_bounds(cal, start, end, delta, delta_units, time, &
       bounds)
    
    character(len=*) cal, start, end, delta_units
    double precision, intent(OUT), dimension(:):: time
    double precision, optional, intent(out) :: bounds(:,:)
    double precision delta

    external fcdchar2comp, fcdchar2rel, fcdcomp2char
    external fcdcomp2rel, fcdrel2char, fcdrel2comp
    external fcdrel2rel
    
    integer CD_STANDARD, CD_JULIAN, CD_NOLEAP, CD_360, CD_CLIM
    integer CD_CLIMLEAP, CD_CLIM360, CD_MIXED
!!$    parameter(CD_STANDARD = '00001111'X)
!!$    parameter(CD_JULIAN = '00011111'X)
!!$    parameter(CD_MIXED = '00021111'X)
!!$    parameter(CD_NOLEAP = '00001011'X)
!!$    parameter(CD_360 = '00000011'X)
!!$    parameter(CD_CLIM = '00001000'X)
!!$    parameter(CD_CLIMLEAP = '00001100'X)
!!$    parameter(CD_CLIM360 = '00000000'X)

    parameter(CD_STANDARD = 4369)
    parameter(CD_JULIAN = 69905)
    parameter(CD_MIXED = 135441)
    parameter(CD_NOLEAP = 4113)
    parameter(CD_360 = 17)
    parameter(CD_CLIM = 4096)
    parameter(CD_CLIMLEAP = 4352)
    parameter(CD_CLIM360 = 0)

    integer ical,icount,icont
    
    double precision d1,d2
    character(len=64) units
    double precision val
    

    units= 'days since 1900'

    select case(trim(adjustl(cal)))
    case('gregorian')
       ical = CD_STANDARD
    case( 'standard')
       ical = CD_STANDARD
    case( 'proleptic_gregorian')
       ical = CD_MIXED
    case( 'noleap')
       ical = CD_NOLEAP
    case( '365_day')
       ical = CD_NOLEAP
    case( '360_day')
       ical = CD_360
    case( 'julian')
       ical = CD_JULIAN
    case( 'NONE')
       ical = CD_CLIM
    END select
    
    icount=1
    icont=1

    select case(trim(adjustl(delta_units)))
    case('months')
       call fcdchar2rel(ical, start, 'months since 1900', val)
    case('days')
       call fcdchar2rel(ical, start, 'days since 1900', val)
    case('hours')
       call fcdchar2rel(ical, start, 'hours since 1900', val)
    end select
    call fcdchar2rel(ical, end , units, d1)
    print*, end , units, d1, val,'****************************'
    do while(icont.eq.1)
    
       if (present(bounds)) then
          select case(trim(adjustl(delta_units)))
          case('months')
             call fcdrel2rel(ical, 'months since 1900',val, units, &
                  bounds(1,icount))
             call fcdrel2rel(ical, 'months since 1900',val+delta, &
                  units, bounds(2,icount))
          case('days')
             call fcdrel2rel(ical, 'days since 1900',val, units, &
                  bounds(1,icount))
             call fcdrel2rel(ical, 'days since 1900',val+delta, &
                  units, bounds(2,icount))
          case('hours')
             call fcdrel2rel(ical, 'hours since 1900',val, units, &
                  bounds(1,icount))
             call fcdrel2rel(ical, 'hours since 1900',val+delta, &
                  units, bounds(2,icount))
          end select
          time(icount) = (bounds(1,icount)+bounds(2,icount))/2.
       else
          select case(trim(adjustl(delta_units)))
          case('months')
             call fcdrel2rel(ical, 'months since 1900',val, units, &
                  time(icount))
          case('days')
             call fcdrel2rel(ical, 'days since 1900',val, units, time(icount))
          case('hours')
             call fcdrel2rel(ical, 'hours since 1900',val, units, time(icount))
          end select
       end if
       val=val+delta
       
       if (time(icount).gt.d1) then
          icont=0
       else
          icount=icount+1
       end if
    end do

    return
  end subroutine cmor_time_and_bounds

#endif
  
  SUBROUTINE get_scale(units, out_units, users_units, scale_factor, &
       offset, ierr)
    
    IMPLICIT NONE
    
    CHARACTER(LEN=*), INTENT(IN) :: units
    CHARACTER(LEN=*), INTENT(INOUT) :: out_units
    CHARACTER(LEN=*), INTENT(OUT) :: users_units
    DOUBLE PRECISION, INTENT(OUT) :: scale_factor
    DOUBLE PRECISION, INTENT(OUT) :: offset

    INTEGER, INTENT(OUT) :: ierr
#ifdef UDUNITS
    INTEGER :: unit1, outputunit
    DOUBLE PRECISION :: outslope, outintercept 
    INTEGER :: utmake, utdec
    INTEGER :: utopen,utcvt
#endif

    ierr = 0
    ! possibly scale values
    
    IF (LEN_TRIM(ADJUSTL(units)) > a64) THEN 
             PRINT*, 'Error encountered in CMOR: get_scale'
             PRINT*, '   The "units" argument must be <= 64 characters long.'
             PRINT*, '   You passed units = ', TRIM(units)
             PRINT*, '   which is too long.'
             ierr = -1
             RETURN
    ELSE IF ((LEN_TRIM(units) == 0) .or. similar(units, 'none')) THEN
       users_units='1'
    ELSE
       users_units = ADJUSTL(units)
       IF (similar(users_units, 'percent')) users_units='%'
    END IF

    IF (similar(out_units, 'none') .or.   &
         similar(out_units, init_undef_char)) THEN
       IF (similar(users_units, '1')) THEN
          out_units = init_undef_char
          scale_factor = 1.0
          offset = 0.0
       ELSE
          PRINT*, 'Error encountered in CMOR: get_scale'
          PRINT*, 'CMOR requires dimensionless units, but you passed: ', TRIM(users_units)
          PRINT*, '   You should pass either "1" or '
          ierr = -2
          PRINT*, 'Your units are incompatible with dimensionless units'
          STOP
       END IF
    
    ELSE IF (similar(users_units, out_units))  THEN

       users_units = out_units
       scale_factor = 1.0
       offset = 0.0

    ELSE  

#ifdef UDUNITS
!!$          if (utopen(trim(adjustl(UDUNITS))//'/etc/udunits.dat') /= 0) THEN
       if (utopen('') /= 0) THEN
          PRINT*, 'Error encountered in CMOR: get_scale'
          PRINT*, '   Error initializing udunits'
          STOP
       END IF
       
       unit1       = utmake()
       outputunit  = utmake()
       
       IF (utdec(users_units, unit1) /= 0 ) THEN
          PRINT*, 'Error encountered in CMOR: get_scale'
          PRINT*, '   User-specified the following units: ', TRIM(users_units)
          PRINT*, '   which are not identical to requested units: ', TRIM(out_units)
          PRINT*, '   and are also not recognized by UDUNITS'
          STOP
       END IF
       
       IF (utdec(out_units, outputunit) /= 0 ) THEN
          PRINT*, 'Error encountered in CMOR: get_scale'
          PRINT*, '   The following MIP table units are not recognized by UDUNITS: ', &
               TRIM(out_units)
          STOP
       END IF
       
       IF (utcvt(unit1, outputunit, outslope, outintercept) /= 0) THEN
          PRINT*, 'Error encountered in CMOR: get_scale'
          PRINT*, "   UDUNITS is unable to convert from user's units: ", TRIM(users_units)
          PRINT*, '   to ', TRIM(out_units)
          STOP
       END IF
       
       scale_factor = outslope
       offset       = outintercept
       
       CALL utfree(unit1)
       CALL utfree(outputunit)
       CALL utcls
#else
       PRINT*, 'Error encountered in CMOR: get_scale'
       PRINT*, "   User's units were not identical to those required"
       PRINT*, '   You passed units = ', TRIM(units)
       PRINT*, '   but CMOR requires ', TRIM(out_units)
       print*, '   Please scale your data and pass correct units or'
       PRINT*, '   build CMOR with the udunits units conversion package'
       STOP 
#endif

    END IF

    RETURN

  END SUBROUTINE get_scale

  SUBROUTINE scale_time(users_units, out_units, ntimes, time_vals, &
       time_bnds, ierr)
    
    IMPLICIT NONE
    
    CHARACTER(LEN=*), INTENT(IN) :: users_units
    CHARACTER(LEN=*), INTENT(INOUT) :: out_units
    INTEGER, INTENT(IN) :: ntimes
    DOUBLE PRECISION, INTENT(INOUT), DIMENSION(ntimes) :: time_vals
    DOUBLE PRECISION, INTENT(INOUT), DIMENSION(2,ntimes), OPTIONAL :: time_bnds

    INTEGER, INTENT(OUT) :: ierr

    INTEGER :: m, n

#ifdef CDMS
    external fcdchar2comp, fcdchar2rel, fcdcomp2char
    external fcdcomp2rel, fcdrel2char, fcdrel2comp
    external fcdrel2rel
    
    integer CD_STANDARD, CD_JULIAN, CD_NOLEAP, CD_360, CD_CLIM
    integer CD_CLIMLEAP, CD_CLIM360, CD_MIXED
    integer ical
!!$    c
!!$    c     Time types and parameters
!!$    c
!!$    parameter(CD_STANDARD = '00001111'X)
!!$    parameter(CD_JULIAN = '00011111'X)
!!$    parameter(CD_MIXED = '00021111'X)
!!$    parameter(CD_NOLEAP = '00001011'X)
!!$    parameter(CD_360 = '00000011'X)
!!$    parameter(CD_CLIM = '00001000'X)
!!$    parameter(CD_CLIMLEAP = '00001100'X)
!!$    parameter(CD_CLIM360 = '00000000'X)

    parameter(CD_STANDARD = 4369)
    parameter(CD_JULIAN = 69905)
    parameter(CD_MIXED = 135441)
    parameter(CD_NOLEAP = 4113)
    parameter(CD_360 = 17)
    parameter(CD_CLIM = 4096)
    parameter(CD_CLIMLEAP = 4352)
    parameter(CD_CLIM360 = 0)
#endif
    
    ierr = 0
    ! possibly scale values
    
    !    users_units = ADJUSTL(users_units)
    IF (LEN_TRIM(users_units) > a64) THEN 
       PRINT*, 'Error encountered in CMOR: scale_time'
       PRINT*, '   The "units" argument must be <= 64 characters long.'
       PRINT*, '   You passed units = ', TRIM(users_units)
       PRINT*, '   which is too long.'
       ierr = -1
       RETURN
    END IF

    m = INDEX(out_units, '?')
    IF ( m  > 0) THEN
       n = INDEX(users_units, 'since')
       out_units = TRIM(out_units(1:m-1)) // ' ' //   &
            TRIM(ADJUSTL(users_units(n+5:LEN(users_units))))
    END IF    

    IF (.NOT. similar(users_units, out_units))  THEN

       IF (string_contains(out_units,'since') == 1) THEN
#ifdef CDMS
          print*, 'cdms stuff'
          select case(trim(adjustl(dset%calendar)))
          case('gregorian')
             ical = CD_STANDARD
          case( 'standard')
             ical = CD_STANDARD
          case( 'proleptic_gregorian')
             ical = CD_MIXED
          case( 'noleap')
             ical = CD_NOLEAP
          case( '365_day')
             ical = CD_NOLEAP
          case( '360_day')
             ical = CD_360
          case( 'julian')
             ical = CD_JULIAN
          case( 'NONE')
             ical = CD_CLIM
          END select

          DO n=1, ntimes
             call fcdrel2rel(ical, users_units, time_vals(n), out_units, &
                  time_vals(n)) 
             IF (PRESENT(time_bnds)) THEN
                DO m=1,2
                    call fcdrel2rel(ical, users_units, time_bnds(m,n), &
                         out_units, time_bnds(m,n)) 
                END DO
              END IF
           END DO

#else
          PRINT*, 'Error encountered in CMOR: scale_time'
          PRINT*, '   Time units passed to CMOR were not identical to those required'
          PRINT*, '   You passed units = ', TRIM(users_units)
          PRINT*, '   but CMOR requires ', TRIM(out_units)
          print*, '   Please change your time coordinate or build CMOR with cdms package'
          STOP 
#endif
       ELSE
          PRINT*, 'Error encountered in CMOR: scale_time'
          PRINT*, '   Output time units specified in MIP input table do not'
          PRINT*, '   include the word "since", which is required.'
       END IF

    END IF
    
    IF (PRESENT(time_bnds)) THEN

       ! redefine time-coordinate to be mid-point of bounds:
       DO n=1,ntimes
          time_vals(n) = (time_bnds(1,n) + time_bnds(2,n))/2.
       END DO

    END IF
    
    RETURN

  END SUBROUTINE scale_time

  
  FUNCTION def_dim(itable, idim, iaxis, units, lendim, values,  &
       bounds, interval) RESULT(ierr)

    IMPLICIT NONE

    INTEGER :: ierr
    INTEGER, INTENT(IN) :: itable
    INTEGER, INTENT(IN) :: idim
    INTEGER, INTENT(IN) :: iaxis
    CHARACTER(len=*), INTENT(IN) :: units
    INTEGER, INTENT(IN) :: lendim
    DOUBLE PRECISION, INTENT(OUT), DIMENSION(:) :: values
    DOUBLE PRECISION, INTENT(OUT), OPTIONAL, DIMENSION(:,:) :: bounds
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: interval

    LOGICAL :: found
    INTEGER :: i, lon1
    INTEGER idx, j, k, istat, ipos, iwarn, n
    DOUBLE PRECISION :: eps, offset, dummy

    ierr = 0

    IF (similar(axis(iaxis)%axis, 'T')) THEN

       call scale_time(ADJUSTL(units), axis(iaxis)%out_units, lendim, &
            values, bounds, ierr)
       adim(idim)%users_units = units

    ELSE
       ! non time-dimension
       CALL get_scale(units, axis(iaxis)%out_units, adim(idim)%users_units, &
            adim(idim)%scale_factor, dummy, ierr)
       
       IF (ierr /= 0) THEN
          PRINT*, 'CMOR error 215 '
          stop
       END IF
       
       ! possibly scale dimension (whole array operation)
       IF (adim(idim)%scale_factor /= 1.0)  THEN
          values = adim(idim)%scale_factor*values
          IF (PRESENT(bounds)) bounds = adim(idim)%scale_factor*bounds
       END IF
    
    END IF

    ! Handle longitude as a special case:
    
    lon1 = 0
    
    IF (similar(axis(iaxis)%standard_name, 'longitude')) THEN
       
       !  add n*360 to get in correct longitude range
       
       IF ((axis(iaxis)%valid_max /= init_undef_double) .and. &
            (axis(iaxis)%valid_min /= init_undef_double)) THEN
          DO i=1,lendim
             offset = 360*FLOOR((values(i)-axis(iaxis)%valid_min)/360.)
             values(i) = values(i) - offset
             
             IF (PRESENT(bounds) .and. &
                  (axis(iaxis)%arg_cntl(6) /= 4)) &
                  bounds(:,i) = bounds(:,i) - offset
          END DO
          
          ! check that round-off problems didn't corrupt values
          ! if limits are 0 to 360, set coordinate value 360 to 0.
          IF ((axis(iaxis)%valid_min == 0.) .or.    &
               (axis(iaxis)%valid_max == 0.)) THEN
             
             
             eps = 1.d-3
             DO i=1,lendim
                IF (abs(values(i)-360.) < eps) THEN
                   values(i) = 0.0
                   IF (PRESENT(bounds) .and.            &
                        (axis(iaxis)%arg_cntl(6) /= 4)) &
                        bounds(:,i) = bounds(:,i) - 360.
                   
                   
                ELSE IF (abs(values(i)+360.) < eps) THEN
                   values(i) = 0.0
                   IF (PRESENT(bounds) .and.            &
                        (axis(iaxis)%arg_cntl(6) /= 4)) &
                        bounds(:,i) = bounds(:,i) + 360.
                END IF
             END DO
             
          END IF
          
          ! find the smallest longitude value
          lon1 = 1
          IF (lendim > 1) THEN
             dummy = values(1)
             DO i=2,lendim
                IF (values(i) < dummy) THEN
                   lon1 = i
                   dummy = values(i)
                END IF
             END DO
          END IF
          
       END IF
       
    END IF
    
    ! check whether requested values are present
 
    IF (associated(axis(iaxis)%requested_vals)) THEN
       DO i=1, SIZE(axis(iaxis)%requested_vals)
          found = .false.
          IF (i==1) THEN
             eps = 1.e-3*axis(iaxis)%tol_on_requests * &
                  ABS(axis(iaxis)%requested_vals(i))
          ELSE
             eps = axis(iaxis)%tol_on_requests *                   &
                  MIN(1.e-3*ABS(axis(iaxis)%requested_vals(i)),    &
                  ABS(axis(iaxis)%requested_vals(i) -              &
                  axis(iaxis)%requested_vals(i-1)))
          END IF
          
          DO j=1,lendim
             
             IF (abs(values(j) - axis(iaxis)%requested_vals(i)) <= eps) &
                  THEN
                found = .true.
                EXIT
             END IF
             
          END DO
          
          IF (.NOT. found) THEN
             ! warn or error exit ?
             PRINT*, 'Warning encountered in CMOR: def_dim, which was called by cmor_axis'
             PRINT*, '    In defining the axis: ', TRIM(axis(iaxis)%table_entry) 
             PRINT*, '    found in table: ', TRIM(axis(iaxis)%mip_table)
             PRINT*, '    The user failed to pass the following requested coordinate value: ', &
                  axis(iaxis)%requested_vals(i)
          END IF
          
       END DO
       
    END IF

    ! check whether users bounds are consistent with requested bounds
    
    IF (associated(axis(iaxis)%requested_bounds)) THEN
       DO i=1, SIZE(axis(iaxis)%requested_bounds, 2)
          
          eps = axis(iaxis)%tol_on_requests *          &
               ABS(axis(iaxis)%requested_bounds(2,i) - &
               axis(iaxis)%requested_bounds(1,i))
          
          found = .false.
          DO j=1,lendim
             
             IF (ABS(MAX(bounds(1,j), bounds(2,j)) -                &
                  MAX(axis(iaxis)%requested_bounds(1,i),            &
                  axis(iaxis)%requested_bounds(2,i))) < eps) THEN
                found = .true.
                IF (ABS(MIN(bounds(1,j), bounds(2,j)) -                &
                     MIN(axis(iaxis)%requested_bounds(1,i),            &
                     axis(iaxis)%requested_bounds(2,i))) > eps) THEN
                   ! warn or error exit ?
                   PRINT*, 'Warning encountered in CMOR: def_dim, which was called by cmor_axis'
                   PRINT*, '    In defining the axis: ', TRIM(axis(iaxis)%table_entry) 
                   PRINT*, '    found in table: ', TRIM(axis(iaxis)%mip_table)
                   PRINT*, '    the user passed the following bounds: ', bounds(1,j), bounds(2,j)
                   PRINT*, '    which are different from the bounds requested: ', &
                       axis(iaxis)%requested_bounds(1,i), axis(iaxis)%requested_bounds(2,i)
                   PRINT*, '    Please try to conform to the request.'
                END IF
                EXIT
             END IF
          END DO
          
          IF (.NOT. found) THEN
             ! warn or error exit ?
             PRINT*, 'Warning encountered in CMOR: def_dim, which was called by cmor_axis'
             PRINT*, '    In defining the axis: ', TRIM(axis(iaxis)%table_entry) 
             PRINT*, '    found in table: ', TRIM(axis(iaxis)%mip_table)
             PRINT*, '    the user passed the following bounds: ', bounds(1,j), bounds(2,j)
             PRINT*, '    which are different from the bounds requested: ', &
                  axis(iaxis)%requested_bounds(1,i), axis(iaxis)%requested_bounds(2,i)
             PRINT*, '    Please try to conform to the request.'
          END IF
          
       END DO
    END IF

    ! check whether when specific coordinate values were requested, user 
    !    also provided additional values.

    IF (associated(axis(iaxis)%requested_vals)) THEN
       
       DO j=1,lendim
          found = .false.
          DO i=1, SIZE(axis(iaxis)%requested_vals)
             IF (i==1) THEN
                eps = 1.e-3*axis(iaxis)%tol_on_requests*     &
                     ABS(axis(iaxis)%requested_vals(i))
             ELSE
                eps = axis(iaxis)%tol_on_requests*                    &
                     MIN(1.e-3*ABS(axis(iaxis)%requested_vals(i)),    &
                     ABS(axis(iaxis)%requested_vals(i) -              &
                     axis(iaxis)%requested_vals(i-1)))
             END IF
             IF (ABS(values(j)-axis(iaxis)%requested_vals(i)) <= eps) &
                  THEN
                found = .true.
                EXIT
             END IF
          END DO
          
          
          IF (.NOT. found) THEN
             ! warn of extra coordinates
             PRINT*, 'Warning encountered in CMOR: def_dim, which was called by cmor_axis'
             PRINT*, '    In defining the axis: ', TRIM(axis(iaxis)%table_entry) 
             PRINT*, '    found in table: ', TRIM(axis(iaxis)%mip_table)
             PRINT*, '    The user passed the following coordinate values: ', &
                  (values(k), k=1,lendim)
             PRINT*, '    which are not identical to those requested: ', &
                  (axis(iaxis)%requested_vals(k), k=1, SIZE(axis(iaxis)%requested_vals))

          END IF
          
       END DO
  
    END IF

    ALLOCATE(adim(idim)%values(lendim), STAT=istat)
    IF (istat /= 0) THEN
       print*, 'CMOR error 24 '
       PRINT*, '   error allocating coordinate values'
       stop
    END IF
    
    IF (PRESENT(bounds)) THEN
       IF ((axis(iaxis)%arg_cntl(6) /= 4) .or.    &
            (associated(axis(iaxis)%zfac_bnds_names))) &
            ALLOCATE(adim(idim)%bounds(2,lendim), STAT=istat)
       IF (istat /= 0) THEN 
          print*, 'CMOR error 25 '
          PRINT*, '   error allocating cell bounds'
          stop
       END IF
    END IF

    ierr = dimension_direction(iaxis, idim, lendim, lon1, PRESENT(bounds), &
         values, bounds)
    
    IF (ierr /= 0) THEN
       PRINT*, 'CMOR error  '
       stop
    END IF
        
    CALL check_coords(iaxis, idim, itable, lendim, PRESENT(bounds), values, &
         bounds, ierr)

    IF (ierr /= 0) THEN
       PRINT*, 'CMOR error 30.5'
       stop
    END IF

    IF (PRESENT(interval)) THEN
       IF (LEN_TRIM(interval) > 0) THEN
          
          IF (axis(iaxis)%arg_cntl(7) == 5) THEN
             PRINT*, 'Error encountered in CMOR: def_dim, which is called by cmor_axis'
             PRINT*, 'The "interval" argument is forbidden when defining axis: ', &
                  TRIM(axis(iaxis)%table_entry)
             PRINT*, '   in table: ', TRIM(axis(iaxis)%mip_table)
             PRINT*, '   You should either omit this argument or set it to an empty string'
             STOP
          ELSE IF (axis(iaxis)%arg_cntl(7) == 4) THEN
             ! warn & ignore
             PRINT*, 'Warning encountered in CMOR: def_dim, which is called by cmor_axis'
             PRINT*, 'The "interval" argument is ignored when defining axis: ', &
                  TRIM(axis(iaxis)%table_entry)
             PRINT*, '   in table: ',  TRIM(axis(iaxis)%mip_table)
             PRINT*, '   To suppress this message, you should either omit the "interval" argument '
             PRINT*, '   or set it to an empty string'
          ELSE
             IF (LEN_TRIM(ADJUSTL(interval)) > a64) THEN
                PRINT*, 'Error encountered in CMOR: def_dim, which is called by cmor_axis'
                PRINT*, '   The "interval" argument must be <= 64 characters long.'
                PRINT*, '   You passed interval = ', TRIM(interval)
                PRINT*, '   which is too long.'
                PRINT*, '   You were defining axis, ',  TRIM(axis(iaxis)%table_entry), &
                     ' in table: ',  TRIM(axis(iaxis)%mip_table)
                STOP
             ELSE
                adim(idim)%interval = TRIM(ADJUSTL(interval))
             END IF
          END IF

       END IF
    END IF
    
    num_dims=idim

    RETURN
    
  END FUNCTION def_dim

 
 
  FUNCTION dimension_direction(iaxis, idim, lendim, lon1, bnds_flag, &
       values, bounds) RESULT(ierr)
    
    IMPLICIT NONE
    
    INTEGER, INTENT(IN) :: iaxis
    INTEGER, INTENT(IN) :: idim
    INTEGER, INTENT(IN) :: lendim
    INTEGER, INTENT(INOUT) :: lon1
    LOGICAL, INTENT(IN) :: bnds_flag
    DOUBLE PRECISION, INTENT(IN), DIMENSION(:) :: values
    DOUBLE PRECISION, INTENT(IN), DIMENSION(:,:), OPTIONAL :: bounds
    
    INTEGER :: ierr
    
    INTEGER :: ipos, i, j
    DOUBLE PRECISION :: compare
    
    ! check whether axis values are ordered positively increasing or 
    !   decreasing
    
    ipos = 1
    IF (lendim > 3) THEN
       IF (lon1 < 3) THEN
          IF (values(4) < values(3)) ipos = -1
          
       ELSE
          IF (values(2) < values(1)) ipos = -1
       END IF
    ELSE IF (lendim == 3) THEN
       IF ((lon1 == 1) .or. (lon1 == 0)) THEN             
          IF (values(3) < values(2)) ipos = -1
       ELSE IF (lon1 == 2) THEN
          IF (abs(values(3)-values(2)) > abs(values(2)-values(1)))  &
               ipos = -1
       ELSE IF (lon1 == 3) THEN
          IF (values(2) < values(1)) ipos = -1
       END IF
    ELSE IF (lendim == 2) THEN
       IF ((values(2)-values(1)) < 0) ipos = -1
    END IF
    
    IF ((similar(axis(iaxis)%axis, 'T')) .and. (ipos < 0)) THEN
       PRINT*, 'Error encountered in CMOR: dimension_direction'
       PRINT*, '  The following time dimension must be stored in increasing order: ', &
              axis(iaxis)%table_entry
       STOP
    END IF

    adim(idim)%begin_out = 1

    IF (lendim > 1) THEN

       compare = values(1)
       IF (axis(iaxis)%out_direction > 0) THEN

       ! start at the smallest value
          DO i = 2, lendim
             IF (values(i) < compare) THEN
                compare = values(i)
                adim(idim)%begin_out = i
             ENDIF
          ENDDO
       
       ELSE

       ! start at the largest value
          DO i = 2, lendim
             IF (values(i) > compare) THEN
                compare = values(i)
                adim(idim)%begin_out = i
             ENDIF
          ENDDO

       ENDIF
    END IF
    
    lon1 = adim(idim)%begin_out

    IF (ipos*axis(iaxis)%out_direction > 0) THEN
       adim(idim)%reverse = .false.
       
       j = 0
       DO i = lon1, lendim
          j = j+1
          adim(idim)%values(j) = values(i)
          IF (bnds_flag) THEN
             IF ((axis(iaxis)%arg_cntl(6) /= 4) .or.   &
                  (associated(axis(iaxis)%zfac_bnds_names)))  THEN
                IF (axis(iaxis)%out_direction *    &
                     (bounds(2,i)-bounds(1,i)) > 0) THEN
                   adim(idim)%bounds(:,j) = bounds(:,i)
                ELSE
                   adim(idim)%bounds(1,j) = bounds(2,i)
                   adim(idim)%bounds(2,j) = bounds(1,i)
                END IF
             END IF
          END IF
       END DO
       
       IF (lon1 > 1) THEN
          
          DO i = 1, lon1-1
             j = j+1
             adim(idim)%values(j) = values(i)
             IF (bnds_flag) THEN
                IF ((axis(iaxis)%arg_cntl(6) /= 4) .or.   &
                     (associated(axis(iaxis)%zfac_bnds_names)))  THEN
                   IF (axis(iaxis)%out_direction *    &
                        (bounds(2,i)-bounds(1,i)) > 0) THEN
                      adim(idim)%bounds(:,j) = bounds(:,i)
                   ELSE
                      adim(idim)%bounds(1,j) = bounds(2,i)
                      adim(idim)%bounds(2,j) = bounds(1,i)
                   END IF
                END IF
             END IF
          END DO
          
       END IF
       
    ELSE
       
       adim(idim)%reverse = .true.
       
       j = 0
       DO i = lon1, 1, -1
          j = j+1
          adim(idim)%values(j) = values(i)
          IF (bnds_flag) THEN
             IF ((axis(iaxis)%arg_cntl(6) /= 4) .or.    &
                  (associated(axis(iaxis)%zfac_bnds_names))) THEN
                IF (axis(iaxis)%out_direction *    &
                     (bounds(2,i)-bounds(1,i)) > 0) THEN
                   adim(idim)%bounds(:,j) = bounds(:,i)
                ELSE
                   adim(idim)%bounds(1,j) = bounds(2,i)
                   adim(idim)%bounds(2,j) = bounds(1,i)
                END IF
             END IF
          END IF
          
       END DO
       
       IF (lon1 < lendim) THEN
          
          DO i = lendim, lon1+1, -1
             j = j+1
             adim(idim)%values(j) = values(i)
             IF (bnds_flag) THEN
                IF ((axis(iaxis)%arg_cntl(6) /= 4) .or.   &
                     (associated(axis(iaxis)%zfac_bnds_names)))  THEN
                   IF (axis(iaxis)%out_direction *    &
                        (bounds(2,i)-bounds(1,i)) > 0) THEN
                      adim(idim)%bounds(:,j) = bounds(:,i)
                   ELSE
                      adim(idim)%bounds(1,j) = bounds(2,i)
                      adim(idim)%bounds(2,j) = bounds(1,i)
                   END IF
                END IF
             END IF
          END DO
       END IF
       
    END IF
    
    ierr = 0
    
    RETURN
  END FUNCTION dimension_direction
  



  SUBROUTINE check_coords(iaxis, idim, itable, lendim, bnds_flag,  values,  &
       bounds, jerr)
    
    IMPLICIT NONE
    
    INTEGER, INTENT(IN) :: iaxis
    INTEGER, INTENT(IN) :: idim
    INTEGER, INTENT(IN) :: itable
    INTEGER, INTENT(IN) :: lendim
    LOGICAL, INTENT(IN) :: bnds_flag
    DOUBLE PRECISION, INTENT(IN), DIMENSION(:) :: values
    DOUBLE PRECISION, INTENT(IN), DIMENSION(:,:), OPTIONAL :: bounds
    INTEGER, INTENT(OUT) :: jerr
    
    INTEGER :: i, j, ierr, iwarn
    DOUBLE PRECISION :: eps
    
    jerr = 0
    ! check that values fall within accepted range
    
    IF (axis(iaxis)%valid_max /= init_undef_double) THEN
       DO i = 1, lendim
          IF (adim(idim)%values(i) > axis(iaxis)%valid_max) THEN
             PRINT*, 'Error encountered in CMOR: check_coords'
             PRINT*, '  Coordinate value, ', adim(idim)%values(i), ', exceeds '
             PRINT*, '  maximum acceptable value: ', axis(iaxis)%valid_max
             PRINT*, '  for axis: ', axis(iaxis)%table_entry, ' found in table: ', &
                 axis(iaxis)%mip_table
             STOP
          END IF
       END DO
    END IF
    
    IF (axis(iaxis)%valid_min /= init_undef_double) THEN
       DO i = 1, lendim
          IF (adim(idim)%values(i) < axis(iaxis)%valid_min)  THEN
             PRINT*, 'Error encountered in CMOR: check_coords'
             PRINT*, '  Coordinate value, ', adim(idim)%values(i), &
             ', is less than '
             PRINT*, '  minimum acceptable value: ', axis(iaxis)%valid_min
             PRINT*, '  for axis: ', axis(iaxis)%table_entry, ' found in table: ', &
                 axis(iaxis)%mip_table
             STOP
          END IF
       END DO
    END IF
    
    ! check whether all coordinate ordering is monotonic
    
    IF (lendim > 1) THEN
       DO i = 2, lendim
          IF (axis(iaxis)%out_direction *      &
               (adim(idim)%values(i)-adim(idim)%values(i-1))<0.) THEN
             PRINT*, 'Error encountered in CMOR: check_coords'
             PRINT*, '  Coordinate values must be monotonically increasing' 
             PRINT*, '  or monotonically decreasing, but are not: '
             PRINT*, '  user passed: ', (values(j), j=1, lendim)
             PRINT*, '  for axis: ', axis(iaxis)%table_entry, ' found in table: ', &
                 axis(iaxis)%mip_table
             STOP
          END IF
       END DO
    END IF
    
    IF (bnds_flag .and. ((axis(iaxis)%arg_cntl(6) /= 4) .or.   &
         (associated(axis(iaxis)%zfac_bnds_names))) )  THEN
       
       ! check that bounds satisfy requirements:
       !     fall between or on nodes
       !     do not overlap
       !     warn if non-contiguous
       
       
       ierr = 0
       iwarn = 0
       
       IF (axis(iaxis)%out_direction > 0) THEN
          DO i=1,lendim
             IF (adim(idim)%bounds(1,i) > adim(idim)%values(i)) &
                  ierr=ierr+1
             IF (adim(idim)%bounds(2,i) < adim(idim)%values(i)) &
                  ierr=ierr+1
             IF (i < lendim) THEN
                eps = abs(adim(idim)%values(i+1) -     &
                     adim(idim)%values(i))*1.e-4
                IF (adim(idim)%bounds(2,i) > (  &
                     adim(idim)%bounds(1,i+1) + eps))  THEN
                   ierr = ierr + 1
                ELSE IF (adim(idim)%bounds(2,i) /=     &
                     (adim(idim)%bounds(1,i+1))) THEN
                   iwarn = iwarn + 1
                END IF
             END IF
          END DO
       ELSE
          ! coords stored in decending order
          DO i=1,lendim
             IF (adim(idim)%bounds(1,i) < adim(idim)%values(i))  &
                  ierr=ierr+1
             IF (adim(idim)%bounds(2,i) > adim(idim)%values(i))  &
                  ierr=ierr+1
             
             IF (i < lendim) THEN
                eps = abs(adim(idim)%values(i+1) -     &
                     adim(idim)%values(i))*1.e-4
                IF (adim(idim)%bounds(2,i) <    &
                     (adim(idim)%bounds(1,i+1)-eps)) THEN
                   ierr = ierr + 1
                ELSE IF (adim(idim)%bounds(2,i) /=    &
                     (adim(idim)%bounds(1,i+1))) THEN
                   iwarn = iwarn + 1
                END IF
             END IF
          END DO
       END IF
         
       IF (iwarn > 0) THEN
           PRINT*, 'Warning encountered in CMOR: check_coords'
           PRINT*, '  Coordinate cell bounds for non-contiguous cells'
           PRINT*, '  Please check that the following bounds are correct: '
           PRINT*, '   ', (bounds(1,i), bounds(2,i), i=1, lendim)
           PRINT*, '  for axis: ', axis(iaxis)%table_entry, ' found in table: ', &
               axis(iaxis)%mip_table
       END IF
       
       
       IF (ierr > 0) THEN
           PRINT*, 'Error encountered in CMOR: check_coords'
           PRINT*, '  Ordering of coordinate cell bounds is incorrect. '
           PRINT*, '  Something wrong with the following: ', &
                            (bounds(1,i), bounds(2,i), i=1, lendim)
           PRINT*, '  for axis: ', axis(iaxis)%table_entry, ' found in table: ', &
               axis(iaxis)%mip_table
          STOP
       END IF
       
    END IF

    ! if a time dimension, check whether interval is about right
    
    IF ((similar(axis(iaxis)%axis, 'T')) .and. (lendim > 1)) THEN
       IF (mip_tbl(itable)%approx_interval > 0.0) THEN
          eps = 0.1*mip_tbl(itable)%approx_interval
          DO i=2, lendim
             IF (abs(abs(adim(idim)%values(i)-adim(idim)%values(i-1))-mip_tbl(itable)%approx_interval) > eps) THEN
                PRINT*, 'Error encountered in CMOR: check_coords'
                PRINT*, '  The interval between time-samples passed to CMOR should be approximately ', &
                     mip_tbl(itable)%approx_interval, TRIM(axis(iaxis)%out_units)
                PRINT*, '  but user passed the following successive coordinate values: ', &
                     adim(idim)%values(i-1), adim(idim)%values(i)
                PRINT*, '  for axis: ', axis(iaxis)%table_entry, ' found in table: ', &
                     axis(iaxis)%mip_table
                STOP
             END IF
          END DO
       ENDIF
    END IF

    RETURN
  END SUBROUTINE CHECK_COORDS



  SUBROUTINE var_preamble(pathfile, table_entry, itable, ivar, ierr)

    IMPLICIT NONE

    CHARACTER(len=*), INTENT(IN) :: pathfile
    CHARACTER(len=*), INTENT(IN) :: table_entry
    INTEGER, INTENT(OUT) :: itable
    INTEGER, INTENT(OUT) :: ivar
    INTEGER, INTENT(OUT) :: ierr

    INTEGER :: idx, i, n

    PRINT*, 'CMOR is defining variable ', trim(table_entry), &
         ' which appears in ',  TRIM(pathfile)

    ierr = 0

      !  check that cmor_setup has already been called
    IF (.NOT. called_setup)  THEN
       PRINT*, 'Error encountered in CMOR: var_preamble,'
       PRINT*, 'You must invoke function cmor_setup before invoking function cmor_variable.'
       STOP
    END IF

    ! check whether this file has already been read or not
    itable=0
    IF (num_mip_tables > 0) THEN
       DO n=1,num_mip_tables
          IF (TRIM(pathfile) == TRIM(mip_tbl(n)%full_path)) THEN
             itable=n
             EXIT
          END IF
       END DO
    END IF

    IF (itable == 0) THEN  
       PRINT*, 'Error encountered in CMOR: var_preamble'
       PRINT*, '   User is attempting to access a MIP input table should first be'
       PRINT*, '   accessed in a call to cmor_axis.'
       PRINT*, '   The attempt to read ', pathfile
       PRINT*, '   failed.  Define axes before defining variables.'
       STOP
    END IF

    ! check whether table_entry is one of the allowable variables

    IF (LEN_TRIM(table_entry) == 0) THEN
       PRINT*, 'Error encountered in CMOR: var_preamble'
       PRINT*, '   User has passed a null string instead of a table_entry'
       PRINT*, '   in a call to cmor_variable or cmor_zfactor'
       STOP
    END IF

    ivar = 0
    DO i=1,num_vars
       IF (similar(table_entry, var(i)%table_entry) .AND. &
            similar(mip_tbl(itable)%table_id, var(i)%mip_table)) THEN
          ivar = i
          EXIT
       ENDIF
    END DO

    IF (ivar == 0) THEN
       PRINT*, 'Error encountered in CMOR: var_preamble,'
       PRINT*, '   User is attempting to define table_entry: ', table_entry
       PRINT*, '   but this entry does not appear in table: ', var(i)%mip_table
       STOP
    END IF

    RETURN
  END SUBROUTINE var_preamble
  
  
  FUNCTION cmor_variable_i(table, table_entry, units, axis_ids,       &
       missing_value, tolerance, positive, original_name, history,    &
       comment) RESULT(ivar)
    
    IMPLICIT NONE


    INTEGER :: ivar
    CHARACTER(len=*), INTENT(IN) :: table
    CHARACTER(len=*), INTENT(IN) :: table_entry
    CHARACTER(len=*), INTENT(IN) :: units
    INTEGER, INTENT(IN), DIMENSION(:) :: axis_ids
    INTEGER, INTENT(IN) :: missing_value
    REAL, INTENT(IN), OPTIONAL :: tolerance
    CHARACTER(len=*) , INTENT(IN), OPTIONAL :: positive
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: original_name
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: history
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: comment

    INTEGER :: itable, ierr

    call var_preamble(table_path(table), table_entry, itable, ivar, ierr)

    IF (ierr /= 0) THEN
       print*, 'CMOR error 45.5 '
       stop
    END IF

    ! handle missing value

    IF (var(ivar)%arg_cntl(5) == 5) THEN
       print*, 'CMOR error 69 '
       stop
    ELSE IF (var(ivar)%arg_cntl(5) == 4) THEN
       ! warn and continue
       var(ivar)%def_missing = 0
    ElSE
       var(ivar)%def_missing = 3
       var(ivar)%users_missing_int = missing_value
    END IF
    
    ! do main work of this subroutine
    
    ierr = def_var(itable, ivar, units, axis_ids, tolerance,   &
         positive, original_name, history, comment)
    
    RETURN
  END FUNCTION cmor_variable_i

  FUNCTION cmor_variable_r(table, table_entry, units, axis_ids,       &
       missing_value, tolerance, positive, original_name, history,    &
       comment) RESULT(ivar)
    
    IMPLICIT NONE


    INTEGER :: ivar
    CHARACTER(len=*), INTENT(IN) :: table
    CHARACTER(len=*), INTENT(IN) :: table_entry
    CHARACTER(len=*), INTENT(IN) :: units
    INTEGER, INTENT(IN), DIMENSION(:) :: axis_ids
    REAL, INTENT(IN) :: missing_value
    REAL, INTENT(IN), OPTIONAL :: tolerance
    CHARACTER(len=*) , INTENT(IN), OPTIONAL :: positive
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: original_name
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: history
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: comment

    INTEGER :: itable, ierr

    call var_preamble(table_path(table), table_entry, itable, ivar, ierr)

    IF (ierr /= 0) THEN
       IF (ierr == -1) THEN
          PRINT*, &
              'Check "table" argument passed to function cmor_variable for variable: ', &
              table_entry
          STOP
       ELSE
       print*, 'CMOR error 45.5 '
       stop
       END IF
    END IF

    ! handle missing value

    IF (var(ivar)%arg_cntl(5) == 5) THEN
       print*, 'CMOR error 69 '
       stop
    ELSE IF (var(ivar)%arg_cntl(5) == 4) THEN
       ! warn and continue
       var(ivar)%def_missing = 0
    ElSE
       var(ivar)%def_missing = 1
       var(ivar)%users_missing = missing_value
    END IF
    
    ! do main work of this subroutine
    
    ierr = def_var(itable, ivar, units, axis_ids, tolerance,   &
         positive, original_name, history, comment)
    
    RETURN
  END FUNCTION cmor_variable_r



  FUNCTION cmor_variable_d(table, table_entry, units, axis_ids,       &
       missing_value, tolerance, positive, original_name, history,    &
       comment) RESULT(ivar)
    
    IMPLICIT NONE


    INTEGER :: ivar
    CHARACTER(len=*), INTENT(IN) :: table
    CHARACTER(len=*), INTENT(IN) :: table_entry
    CHARACTER(len=*), INTENT(IN) :: units
    INTEGER, INTENT(IN), DIMENSION(:) :: axis_ids
    DOUBLE PRECISION, INTENT(IN) :: missing_value
    REAL, INTENT(IN), OPTIONAL :: tolerance
    CHARACTER(len=*) , INTENT(IN), OPTIONAL :: positive
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: original_name
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: history
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: comment

    INTEGER :: itable, ierr

    call var_preamble(table_path(table), table_entry, itable, ivar, ierr)

    IF (ierr /= 0) THEN
       IF (ierr == -1) THEN
          PRINT*, 'Check "table" argument passed to cmor_variable for variable: ', &
              table_entry
          STOP
       ELSE
          print*, 'CMOR error 45.5 '
       END IF
    END IF

    ! handle missing value

    IF (var(ivar)%arg_cntl(5) == 5) THEN
       print*, 'CMOR error 69 '
       stop
    ELSE IF (var(ivar)%arg_cntl(5) == 4) THEN
       ! warn and continue
       var(ivar)%def_missing = 0
    ElSE
       var(ivar)%def_missing = 2
       var(ivar)%users_missing_dbl = missing_value
    END IF

    ! do main work of this subroutine
    
    ierr = def_var(itable, ivar, units, axis_ids, tolerance,   &
         positive, original_name, history, comment)
    
    RETURN

  END FUNCTION cmor_variable_d


  
  FUNCTION cmor_variable_no_missing(table, table_entry, units, axis_ids,   &
       missing_value, tolerance, positive, original_name, history,         &
       comment) RESULT(ivar)
    
    IMPLICIT NONE


    INTEGER :: ivar
    CHARACTER(len=*), INTENT(IN) :: table
    CHARACTER(len=*), INTENT(IN) :: table_entry
    CHARACTER(len=*), INTENT(IN) :: units
    INTEGER, INTENT(IN), DIMENSION(:) :: axis_ids
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: missing_value
    REAL, INTENT(IN), OPTIONAL :: tolerance
    CHARACTER(len=*) , INTENT(IN), OPTIONAL :: positive
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: original_name
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: history
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: comment

    INTEGER :: itable, ierr

    call var_preamble(table_path(table), table_entry, itable, ivar, ierr)

    IF (ierr /= 0) THEN
       IF (ierr == -1) THEN
          PRINT*, 'Check "table" argument passed to cmor_variable for variable: ', &
              table_entry
          STOP
       ELSE
          print*, 'CMOR error 45.5 '
       END IF
    END IF

    ! handle missing value

    IF (var(ivar)%arg_cntl(5) == 1) THEN
       print*, 'CMOR error 48 '
       stop
    END IF

    var(ivar)%def_missing = 0

    ! do main work of this subroutine

    ierr = def_var(itable, ivar, units, axis_ids, tolerance,   &
         positive, original_name, history, comment)

    RETURN

  END FUNCTION cmor_variable_no_missing

  
  FUNCTION def_var(itable, ivar, units, passed_dim_idxs, tolerance,    &
       positive, original_name, history, comment)
    
    IMPLICIT NONE


    INTEGER :: def_var
    INTEGER, INTENT(IN) :: itable
    INTEGER, INTENT(IN) :: ivar
    CHARACTER(len=*), INTENT(IN) :: units
    INTEGER, INTENT(IN), DIMENSION(:) :: passed_dim_idxs
    REAL, INTENT(IN), OPTIONAL :: tolerance
    CHARACTER(len=*) , INTENT(IN), OPTIONAL :: positive
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: original_name
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: history
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: comment

    INTEGER :: ierr, i, iax, inrank, iout, j, jin
    LOGICAL :: found
    LOGICAL, DIMENSION(max_rank) :: singleton

    def_var = 0

    ! check for required arguments

    IF ((var(ivar)%arg_cntl(6) == 1) .and. (.not. PRESENT(tolerance))) THEN
       print*, 'CMOR error 49 '
       stop
    END IF

    IF ((var(ivar)%arg_cntl(7) == 1) .and. (.not. PRESENT(positive))) THEN
       PRINT*, 'Error encountered in CMOR: def_var'
       PRINT*, '  The "positive" argument must be defined for variable: ', TRIM(var(ivar)%table_entry)
       PRINT*, '  found in table ', TRIM(var(ivar)%mip_table)
       STOP
    END IF

    IF (var(ivar)%arg_cntl(8) == 1) THEN
       IF (.not. PRESENT(original_name)) THEN
          print*, 'CMOR errr 51 '
          stop
       ELSE IF (LEN_TRIM(original_name) == 0) THEN
          print*, 'CMOR error 52 '
          stop
       END IF
    END IF

    IF (var(ivar)%arg_cntl(9) == 1) THEN
       IF (.not. PRESENT(history)) THEN
          print*, 'CMOR error 53 '
          stop
       ELSE IF (LEN_TRIM(history) == 0) THEN
          print*, 'CMOR error 54 '
          stop
       END IF
    END IF

    IF (var(ivar)%arg_cntl(10) == 1) THEN
       IF (.not. PRESENT(comment)) THEN
          print*, 'CMOR error 55 '
          stop
       ELSE IF (LEN_TRIM(comment) == 0) THEN
          print*, 'CMOR error 56 '
stop
       END IF
    END IF

    IF ((TRIM(var(ivar)%file_status) == 'write mode') .or. &
         (TRIM(var(ivar)%file_status) == 'define mode')) THEN
       print*, 'CMOR error 58 '
       stop
    END IF

    ! check that experiment_id is one of acceptable values
    found = .false.
    DO i=1,max_num_expts
       IF (similar(dset%experiment_id, mip_tbl(itable)%expt_id_ok(i))) THEN
          dset%experiment_id = mip_tbl(itable)%expt_id_ok(i)
          found = .true.
          EXIT
       END IF
    END DO

    IF (.not. found) THEN
       PRINT*, 'Error encountered in CMOR: def_var'
       PRINT*, '  The "experiment_id" argument of function cmor_dataset was passed as: '
       PRINT*, '   ', TRIM(dset%experiment_id), ' but it should be assigned'
       PRINT*, '  one of the following values: ', &
                  (mip_tbl(itable)%expt_id_ok(i), i=1,max_num_expts)
       STOP
    END IF

    var(ivar)%mip_table_idx = itable

    ! determine scale factor for variable

    CALL get_scale(units, var(ivar)%out_units, var(ivar)%users_units, &
         var(ivar)%scale_factor, var(ivar)%offset, ierr)
    
    IF (ierr /= 0) THEN
       PRINT*, 'CMOR error 62 '
       stop
    END IF

    ! determine rank of array, check whether passed_dim_idxs are acceptable
    inrank = 0
    DO i=1,MIN(SIZE(passed_dim_idxs), max_rank)
       IF ((passed_dim_idxs(i) > num_dims) .or.  (passed_dim_idxs(i) < 0)) THEN
          PRINT*, 'Error encountered in CMOR: def_var'
          PRINT*, '   The axis_ids passed by the user to cmor_variable or  '
          PRINT*, '   cmor_zfactor have one or more unacceptible values'
          PRINT*, '   User passed axis_ids = ', passed_dim_idxs
          STOP
       ELSE IF (passed_dim_idxs(i) == 0) THEN
          CYCLE
       ELSE
          inrank = inrank + 1
          IF (inrank < i) THEN
             PRINT*, 'Error encountered in CMOR: def_var'
             PRINT*, '   One of the axis_ids passed by the user to cmor_variable or  '
          PRINT*, '   cmor_zfactor was set to zero, which is illegal.'
             ! (a non-trailing axis id was set to 0)
          STOP
          END IF
       END IF
    END DO

    ! check that there is a one-to-one correspondence between dimensions
    !  passed (or singleton dimension) and dimensions expected.
    !  also map from users order to output dimension order

    ! initialize order 
    ! dim_order(iin) will point from position of input dimension (inn) to 
    !    position of output dimension (omitting singleton dimensions that 
    !    the user has omitted from passed_dim_idxs)

    var(ivar)%dim_order = 0

    ! dim_idxs(iout) will point from position of output dimension (iout) to 
    !       dimension table index (including singleton dimensions)

    !  loop over expected dimensions and set pointers
    !    singleton(iout) = .t. or .f., which indicates whether output 
    !    dimension iout is an official  singleton dimension (Note: all 
    !    singleton dimensions should be listed last in the MIP table).
 
    !  ??? should check in cmor_write that # of non-zero dim_order elements
    !  matches rank of array passed [but array may have one or more extra 
    !  singleton dimensions; also array rank may be one if elements are 
    !  stored contiguously


    !  ??? following assumes a scalar dimension will not have a singleton 
    !       dimension
    IF (inrank == 0) THEN
       IF (var(ivar)%rank /= 0) THEN
          PRINT*, 'Error encountered in CMOR: def_var'
          PRINT*, '   Rank of a variable defined by cmor_variable or '
          PRINT*, '   cmor_zfactor is specified by the user as zero'
          PRINT*, '   but CMOR expects it to be rank ', var(ivar)%rank
          STOP
       END IF

    ELSE
       
       DO iout=1,var(ivar)%rank
          iax = var(ivar)%axis_idxs(iout)
          IF (iax == -1) THEN
             PRINT*, 'Error encountered in CMOR: def_var'
             PRINT*, '   User must define a dimensionless vertical axis'
             PRINT*, '   for this variable: ', var(ivar)%table_entry
             STOP
             ! user needs to define dimensionless vertical axis
          END IF
          singleton(iout) = .false.
          IF (LEN_TRIM(axis(iax)%singleton_label) /= 0) &
               singleton(iout) = .true.
          
          ! find user defined dimension that matches output axis
          
          found=.false.
          DO j=1,inrank
             IF (adim(passed_dim_idxs(j))%axis_idx == iax) THEN
                var(ivar)%dim_order(j) = iout
                var(ivar)%dim_idxs(iout) = passed_dim_idxs(j)
                
                found=.true.
                EXIT
             END IF
          END DO
          IF (.NOT. found) THEN
             IF (.NOT. singleton(iout)) THEN
                
                ! this is not an official singleton dimension, so the user 
                !  must not omit it
                PRINT*, 'Error encountered in CMOR: def_var'
                PRINT*, '   You must define axis: ', TRIM(axis(iax)%table_entry) 
                PRINT*, '   for the following variable you are defining through'
                PRINT*, '   cmor_variable or cmor_zfactor: ', &
                     TRIM(var(ivar)%table_entry)
                PRINT*, '  found in table ', TRIM(var(ivar)%mip_table)
                STOP
             ELSE  
                DO j = 1, num_dims
                   IF (adim(j)%axis_idx == iax) THEN
                      var(ivar)%dim_idxs(iout) = j
                      found=.true.
                      EXIT
                   END IF
                END DO
                
                IF (.NOT. found) THEN
                   PRINT*, 'Error encountered in CMOR: def_var'
                   PRINT*, '   User must define the following axis: ', &
                        TRIM(axis(iax)%table_entry)
                   PRINT*, '   before defining varialbe: ', &
                        TRIM(var(ivar)%table_entry)
                   STOP
                END IF
                
             END IF
          END IF
          
          
       END DO
       
       ! loop through each input dimension and check whether it has been 
       !      associated with an output dimension
       DO jin=1,inrank
          IF (var(ivar)%dim_order(jin) == 0) THEN
                PRINT*, 'Error encountered in CMOR: def_var'
                PRINT*, '   User defined an axis: axis(iax)%table_entry '
                PRINT*, '   that is not allowed for variable: ', &
                    TRIM(var(ivar)%table_entry)
                PRINT*, '  found in table ', TRIM(var(ivar)%mip_table)
                STOP
          END IF
       END DO
       
    END IF
    
    IF (PRESENT(tolerance)) THEN
       IF (var(ivar)%arg_cntl(6) == 5) THEN
          print*, 'CMOR error 70 '
          stop
       ELSE IF (var(ivar)%arg_cntl(6) == 4) THEN
          ! warn and continue
       ElSE
          IF (var(ivar)%def_missing == 0) THEN
             ! warn and continue
          ELSE
             var(ivar)%miss_val_tolerance=tolerance
          END IF
       END IF
    ELSE
       IF ((var(ivar)%def_missing == 1) .or.   &
            (var(ivar)%def_missing == 2)) THEN
          var(ivar)%miss_val_tolerance = 1.e-4
       ELSE IF (var(ivar)%def_missing == 3) THEN
          var(ivar)%miss_val_tolerance = 0.
! ??? check following
!!$       ELSE
!!$          print*, 'warning unknown def missing with tolerance not defined:',var(ivar)%def_missing
!!$          PRINT*, 'CMOR error 74.5' 
       END IF
    END IF
    
    IF (PRESENT(positive)) THEN
       IF (LEN_TRIM(positive) > 0) THEN
          IF (var(ivar)%arg_cntl(7) == 5) THEN
             PRINT*, 'Error encountered in CMOR: def_var'
             PRINT*, '   The "positive" argument is forbidden when defining ', &
                  TRIM(var(ivar)%table_entry)
             STOP
          ELSE IF (var(ivar)%arg_cntl(7) == 4) THEN 
             PRINT*, 'Warning encountered in CMOR: def_var'
             PRINT*, '   The "positive" argument is ignored when defining ', &
                  TRIM(var(ivar)%table_entry)
             
             ! warn and ignore 
          ELSE IF ((.NOT. similar(positive, 'up')) .AND.    &
               (.NOT. similar(positive, 'down'))) THEN
             PRINT*, 'Error encountered in CMOR: def_var'
             PRINT*, '   The "positive" argument was be either assigned "up" or "down"'
             PRINT*, '   when defining ',  TRIM(var(ivar)%table_entry)
             STOP
          ELSE IF (.NOT. similar(var(ivar)%out_positive, positive)) THEN
             var(ivar)%scale_factor = -var(ivar)%scale_factor
          END IF
       END IF
    END IF

    IF (PRESENT(original_name)) THEN
       IF (var(ivar)%arg_cntl(8) == 5) THEN
          print*, 'CMOR error 72 '
          stop
       ELSE IF (var(ivar)%arg_cntl(8) == 4) THEN 
          ! warn and ignore 
       ELSE
          var(ivar)%users_name = original_name
       END IF
    END IF

    IF (PRESENT(history)) THEN
       IF (var(ivar)%arg_cntl(9) == 5) THEN
          print*, 'CMOR error 73 '
          stop
       ELSE IF (var(ivar)%arg_cntl(9) == 4) THEN 
          ! warn and ignore 
       ELSE
          var(ivar)%history = history
       END IF
    END IF

    IF (PRESENT(comment)) THEN
       IF (var(ivar)%arg_cntl(10) == 5) THEN
          print*, 'CMOR error 74 '
          stop
       ELSE IF (var(ivar)%arg_cntl(10) == 4) THEN 
          ! warn and ignore 
       ELSE
          var(ivar)%comment = comment
       END IF
    END IF


    RETURN
  END FUNCTION def_var

  function write_history(itable, ivar, non_single) result (ierr)

    IMPLICIT none

    INTEGER :: def_write_history
    INTEGER, INTENT(IN) :: itable
    INTEGER, INTENT(IN) :: ivar
    INTEGER, INTENT(IN) :: non_single
    INTEGER ierr

    integer i, iout
    CHARACTER(len=a1024) :: str, str1,str3,str2
     
    ! generate history information
    str1 = ''
    
    IF (TRIM(var(ivar)%type) == 'real') THEN
       write(str3, '(ES12.5)') mip_tbl(itable)%single_missing
    ELSE IF (TRIM(var(ivar)%type) == 'double') THEN
       write(str3, '(ES12.5)') mip_tbl(itable)%double_missing
    ELSE IF (TRIM(var(ivar)%type) == 'integer') THEN
       write(str3, '(i12)') mip_tbl(itable)%INTEGER_missing
    END IF
    
    IF (TRIM(var(ivar)%type) /= TRIM(var(ivar)%users_type)) &
         str1 = TRIM(str1) // ' converted from type "' //   &
         TRIM(var(ivar)%users_type) // '" to type "'//      &
         TRIM(var(ivar)%type) // '";'
    
    !  if user doesn't supply a missing value, then the following info. will
    !   not be written to history
    IF (var(ivar)%def_missing == 1) THEN
       IF (var(ivar)%users_missing /= mip_tbl(itable)%single_missing) THEN
          write(str2, '(ES12.5)') var(ivar)%users_missing
          str1 = TRIM(str1) // ' replaced missing value flag (' &
               // TRIM(ADJUSTL(str2)) // &
               ') with standard missing value (' // TRIM(ADJUSTL(str3)) // ');'
       END IF
    ELSE IF (var(ivar)%def_missing == 2) THEN
       IF (var(ivar)%users_missing_dbl /= mip_tbl(itable)%double_missing) THEN
          write(str2, '(ES12.5)') var(ivar)%users_missing_dbl
          str1 = TRIM(str1) // ' replaced missing value flag (' // TRIM(ADJUSTL(str2)) // &
               ') with standard missing value (' // TRIM(ADJUSTL(str3)) // ');'
       END IF
    ELSE IF (var(ivar)%def_missing == 3) THEN
       IF (var(ivar)%users_missing_int/= mip_tbl(itable)%INTEGER_missing) THEN
          write(str2, '(I13)') var(ivar)%users_missing_int
          str1 = TRIM(str1) // ' replaced missing value flag (' // TRIM(ADJUSTL(str2)) // &
               ') with standard value (' // TRIM(ADJUSTL(str3)) // ');'
       END IF
    END IF
    
    IF (var(ivar)%offset /= .0) THEN
       WRITE(str2, '(ES12.5)') var(ivar)%offset
       
       str1 = TRIM(str1) // ' added ' // TRIM(ADJUSTL(str2)) // &
            ' to yield output units;'
    END IF
    
    IF (var(ivar)%scale_factor /= 1.0) THEN
       WRITE(str2, '(ES12.5)') var(ivar)%scale_factor
       str1 = TRIM(str1) // ' multiplied by ' // TRIM(ADJUSTL(str2)) //  &
            ' to yield output units;'
    END IF
    
    DO i = 1, var(ivar)%rank
       IF (LEN_TRIM(adim(var(ivar)%dim_idxs(i))%converted_from) > 0) THEN
          str1 = TRIM(str1) // ' Converted ' //    &
               TRIM(adim(var(ivar)%dim_idxs(i))%converted_from) //    &
               ' (dimensionless vertical coordinate) to ' //    &
               TRIM(axis(var(ivar)%axis_idxs(i))%table_entry) // ';'
       END IF
    END DO

    IF (non_single > 1) THEN
       DO iout = 2,non_single
          IF (var(ivar)%dim_order(iout) < var(ivar)%dim_order(iout-1)) THEN
             str1 = TRIM(str1) // ' Dimension order was changed;'
             EXIT
          END IF
       END DO
    END IF
    
    DO iout = 1,non_single
       IF (adim(var(ivar)%dim_idxs(iout))%reverse) THEN 
          str1 = TRIM(str1) // '  ' //   &
               TRIM(axis(var(ivar)%axis_idxs(iout))%out_name) // &
               ' dimension direction was reversed;'
       END IF
    END DO
    
    DO iout = 1,non_single
       IF (((.not.adim(var(ivar)%dim_idxs(iout))%reverse) .and.  &
            (adim(var(ivar)%dim_idxs(iout))%begin_out /= 1))     &
            .or. &
            ((adim(var(ivar)%dim_idxs(iout))%reverse) .and.      &
            (adim(var(ivar)%dim_idxs(iout))%begin_out /=         &
            adim(var(ivar)%dim_idxs(iout))%lendim))) THEN 
          str1 = TRIM(str1) // &
               '  Cyclical dimension was output starting at a different ' // &
               TRIM(axis(var(ivar)%axis_idxs(iout))%out_name) // ';'
       END IF
    END DO

    ! ??? mention also conversion of vertical dimensionless coordinate here
    
    IF (LEN_TRIM(str1) > 0) THEN
       str1 = ' At  ' // ' ' // cmor_getTime() // ' on ' // cmor_getDate() // &
            
            ': CMOR altered the data in the following ways:' // TRIM(str1)
    END IF
    IF (LEN_TRIM(var(ivar)%history // str1) > 0) THEN
       ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=var(ivar)%varid, &
            name='history',                                            &
            values=TRIM(var(ivar)%history) // TRIM(str1)  )  
       IF (ierr /= NF90_NOERR) THEN
          print*, 'CMOR error 115 '
          stop
       END IF
    END IF
  end function write_history


  FUNCTION initialize_file(ivar, file_suffix) RESULT(ierr)
           
    IMPLICIT NONE
    
    INTEGER, INTENT(IN) :: ivar
    CHARACTER(len=*), INTENT(IN) :: file_suffix
    
    INTEGER :: ierr
    
    INTEGER :: iout, i, j, k, m, n, iax, idim, non_single, file_action, ilen
    INTEGER :: bnds_id, str_id, itable
    INTEGER, DIMENSION(max_rank) :: ncdimids, nczids, coord_id, &
         coord_bnds_id
    CHARACTER(len=a1024) :: str, str1,str3,str2
    INTEGER :: max_str_len
    CHARACTER(len=a128) :: coords, std_name_copy
    LOGICAL, DIMENSION(max_rank) :: singleton
    LOGICAL :: found
    INTEGER :: dimensionless, izfac, vvid, dimid, ntimes


    ierr = 0
    itable = var(ivar)%mip_table_idx

    ! return if file suffix has been defined, but is now omitted
    IF (similar(file_suffix, init_undef_char) .and.                   &
         (.not. similar(var(ivar)%file_suffix, init_undef_char)))      RETURN

    ! return unless file suffix differs from an already defined suffix
    IF ((similar(file_suffix, var(ivar)%file_suffix)) .and. &
         (.not. similar(file_suffix, init_undef_char)))                RETURN

    ! if file for this variable is already open, close it.
    
    IF ((var(ivar)%file_status == 'define') .or.   &
         (var(ivar)%file_status == 'write')) THEN 
       ierr = cmor_close(ivar)
       IF (ierr /= 0) THEN
          PRINT*, 'CMOR error 65.2'
          stop
       END IF
    END IF

    ! construct full path/ filename
    i = INDEX(mip_tbl(itable)%table_id, ' ')
    IF ((LEN_TRIM(file_suffix) > 0) .and.  &
         (.not. similar(file_suffix, init_undef_char))) THEN
       IF (LEN_TRIM(file_suffix) > a64) THEN
          PRINT*, 'Error encountered in CMOR: initialize_file'
          PRINT*, '   The file_suffix must be <= 64 characters long, ' 
          PRINT*, '   but the following string was passed: ', TRIM(file_suffix)
          STOP
       END IF
       ! construct path/file with suffix
       var(ivar)%file_suffix = file_suffix
       IF (LEN(TRIM(dset%outpath) // TRIM(ADJUSTL(var(ivar)%table_entry)) //  &
            '_' //  TRIM(ADJUSTL(mip_tbl(itable)%table_id(i:)))// '_' //      &
            TRIM(ADJUSTL(file_suffix))) > a256) THEN
          PRINT*, 'Error encountered in CMOR: initialize_file'
          PRINT*, '   The full path/filename must be <= 256 characters long, ' 
          PRINT*, '   Shorten the path: ', TRIM(dset%outpath)
          PRINT*, '   or shorten the file_suffix: ',  TRIM(file_suffix)
          STOP
       ELSE
          ! extract table number
          var(ivar)%full_outpath = TRIM(dset%outpath) //                &
               TRIM(ADJUSTL(var(ivar)%table_entry)) // '_' //           &
               TRIM(ADJUSTL(mip_tbl(itable)%table_id(i:)))//            &
               '_' // TRIM(ADJUSTL(file_suffix)) // '.nc' 
       END IF
    ELSE
       var(ivar)%file_suffix = ''
       ! construct path/file without suffix
       IF (LEN(TRIM(dset%outpath) // TRIM(ADJUSTL(var(ivar)%table_entry)) //  &
            '_' //  TRIM(ADJUSTL(mip_tbl(itable)%table_id(i:)))) > a256) THEN

          PRINT*, 'Error encountered in CMOR: initialize_file'
          PRINT*, '   The full path/filename must be <= 256 characters long, ' 
          PRINT*, '   Shorten the path: ', TRIM(dset%outpath)
          STOP
       ELSE
          var(ivar)%full_outpath = TRIM(dset%outpath) //           &
               TRIM(ADJUSTL(var(ivar)%table_entry)) // '_' //      &
               TRIM(ADJUSTL(mip_tbl(itable)%table_id(i:))) // '.nc'
       END IF
    END IF

!!$    ! check whether any bounds will be stored for zfactors
!!$    
!!$    do_bounds = .false.
!!$    DO i=1, var(ivar)%rank
!!$       iax = var(ivar)%axis_idxs(i) 
!!$       idim = var(ivar)%dim_idxs(i)
!!$       IF ((axis(iax)%table_entry, 'zlevel') > 0) THEN
!!$          ! dimensionless vertical coordinate found 
!!$          DO j=1, max_zfactors
!!$             izfac = adim(idim)%zfac_idxs(j)
!!$             IF (izfac == 0) EXIT
!!$             IF ((var(izfac)%rank == 1) .and.   &
!!$                  (INDEX(var(izfac)%axis_names(1), 'zlevel') > 0))  &
!!$                  do_bounds(j) = .true.
!!$          END DO
!!$       END IF
!!$    END DO

    ! now open netCDF file and define variable, attributes, axes, etc.
    
    IF (TRIM(cmor_netcdf_file_action) == 'append')  THEN 

       ! assume here that all variables and attributes have already been
       !    defined.

       ierr = nf90_open(path=TRIM(var(ivar)%full_outpath), mode=NF90_WRITE, &
            ncid=var(ivar)%ncid)

       IF (ierr /= NF90_NOERR) THEN
          PRINT*, 'Error encountered in CMOR: initialize_file'
          PRINT*, '   Problem opening netCDF file: '
          PRINT*, '   ', var(ivar)%full_outpath
          STOP
       END IF

       ierr = nf90_inquire_variable(ncid=var(ivar)%ncid,    &
            name=var(ivar)%out_name, varid=var(ivar)%varid)

       !  check if a function of time; if so, determine times_written

       IF (nf90_inq_dimid(ncid=var(ivar)%ncid, name='time', dimid=dimid) ==  &
            NF90_NOERR) THEN
          ierr = nf90_inquire_dimension(ncid=var(ivar)%ncid, dimid=dimid, &
               len=ntimes)

          IF (ierr /= NF90_NOERR) THEN
             PRINT*, 'Error encountered in CMOR: initialize_file'
             PRINT*, '   Problem reading length of time dimension from'
             PRINT*, '   existing netCDF file: '
             PRINT*, '   ', var(ivar)%full_outpath
             STOP
          END IF
          

          var(ivar)%times_written = ntimes
          IF (ASSOCIATED(var(ivar)%zfac_times_written))    &
               var(ivar)%zfac_times_written = ntimes

       END IF

       var(ivar)%file_status = 'write mode'

       RETURN

    ELSE

       IF (TRIM(cmor_netcdf_file_action) == 'preserve')        &
            file_action = NF90_NOCLOBBER
       IF (TRIM(cmor_netcdf_file_action) == 'replace')       &
            file_action = NF90_CLOBBER
       ierr = nf90_create(path=TRIM(var(ivar)%full_outpath),   &
            cmode=file_action, ncid=var(ivar)%ncid)

       IF (ierr /= NF90_NOERR) THEN
          PRINT*, 'Error encountered in CMOR: initialize_file'
          PRINT*, '   Problem creating file:  ', &
               TRIM(var(ivar)%full_outpath)
          STOP
       END IF
    END IF
    var(ivar)%file_status='define mode'

    ! define global attributes

    j = INDEX(dset%institution, '(')
    ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=NF90_GLOBAL,  &
         name='title', values=dset%institution(1:j-1) //   &
         ' model output prepared for ' // TRIM(mip_tbl(itable)%project_id)&
         // ' ' //  TRIM(dset%experiment_id))
    IF (ierr /= NF90_NOERR) THEN 
          PRINT*, 'Error encountered in CMOR: initialize_file'
          PRINT*, '   Problem writing "title" global attribute to file: ', &
               TRIM(var(ivar)%full_outpath)
          STOP
    END IF

    IF (LEN_TRIM(dset%institution) /= 0) THEN
       ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=NF90_GLOBAL, &
            name='institution', values=TRIM(dset%institution))
       IF (ierr /= NF90_NOERR) THEN
          PRINT*, 'Error encountered in CMOR: initialize_file'
          PRINT*, '   Problem writing "institution" global attribute to file: ', &
               TRIM(var(ivar)%full_outpath)
          STOP
       END IF
    END IF
    IF (LEN_TRIM(dset%source) /= 0) THEN
       ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=NF90_GLOBAL, &
            name='source', values=TRIM(dset%source))
       IF (ierr /= NF90_NOERR) THEN
          PRINT*, 'Error encountered in CMOR: initialize_file'
          PRINT*, '   Problem writing "source" global attribute to file: ', &
               TRIM(var(ivar)%full_outpath)
          STOP
       END IF
    END IF

 
    IF (LEN_TRIM(dset%contact) /= 0) THEN
       ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=NF90_GLOBAL, &
            name='contact',                                     &
            values=TRIM(dset%contact))
       IF (ierr /= NF90_NOERR) THEN
          PRINT*, 'Error encountered in CMOR: initialize_file'
          PRINT*, '   Problem writing "contact" global attribute to file: ', &
               TRIM(var(ivar)%full_outpath)
          STOP
       END IF
    END IF

    IF (LEN_TRIM(mip_tbl(itable)%project_id) /= 0) THEN
       ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=NF90_GLOBAL, &
            name='project_id',                                     &
            values=TRIM(mip_tbl(itable)%project_id))
       IF (ierr /= NF90_NOERR) THEN
           PRINT*, 'Error encountered in CMOR: initialize_file'
          PRINT*, '   Problem writing "project_id" global attribute to file: ', &
               TRIM(var(ivar)%full_outpath)
          STOP
       END IF
    END IF

    IF (LEN_TRIM(mip_tbl(itable)%table_id) /= 0) THEN
       ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=NF90_GLOBAL, &
            name='table_id', &
            values=TRIM(mip_tbl(itable)%table_id) // ' (' //       &
            TRIM(mip_tbl(itable)%table_date) // ')' )
       IF (ierr /= NF90_NOERR) THEN
          PRINT*, 'Error encountered in CMOR: initialize_file'
          PRINT*, '   Problem writing "table_id" global attribute to file: ', &
               TRIM(var(ivar)%full_outpath)
          STOP
       END IF
    END IF

    IF (LEN_TRIM(dset%experiment_id) /= 0) THEN
       ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=NF90_GLOBAL, &
            name='experiment_id',                                  &
            values=TRIM(dset%experiment_id))

       IF (ierr /= NF90_NOERR) THEN
          PRINT*, 'Error encountered in CMOR: initialize_file'
          PRINT*, '   Problem writing "experiment_id" global attribute to file: ', &
               TRIM(var(ivar)%full_outpath)
          STOP
       END IF
    END IF
    IF (dset%realization /= 0) THEN
       ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=NF90_GLOBAL, &
            name='realization', values=dset%realization)
       IF (ierr /= NF90_NOERR) THEN
          PRINT*, 'Error encountered in CMOR: initialize_file'
          PRINT*, '   Problem writing "realization" global attribute to file: ', &
               TRIM(var(ivar)%full_outpath)
          STOP
       END IF
    END IF

    ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=NF90_GLOBAL, &
         name='cmor_version', &
         values=cmor_version)
    IF (ierr /= NF90_NOERR) THEN
       PRINT*, 'Error encountered in CMOR: initialize_file'
       PRINT*, '   Problem writing "cmor_version" global attribute to file: ', &
            TRIM(var(ivar)%full_outpath)
       STOP
    END IF

    IF (LEN_TRIM(mip_tbl(itable)%cf_version) /= 0) THEN
       str = 'CF-' // TRIM(ADJUSTL(mip_tbl(itable)%cf_version))
       ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=NF90_GLOBAL,     &
            name='Conventions',                                        &
            values='CF-' // TRIM(ADJUSTL(mip_tbl(itable)%cf_version)))
       IF (ierr /= NF90_NOERR) THEN
          PRINT*, 'Error encountered in CMOR: initialize_file'
          PRINT*, '   Problem writing "Conventions" global attribute to file: ', &
               TRIM(var(ivar)%full_outpath)
          STOP
       END IF
    END IF

   ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=NF90_GLOBAL,    &
         name='history',                                          &
         values=TRIM(ADJUSTL(dset%history)) // '  At ' //         &
         TRIM(ADJUSTL(cmor_getTime())) // ' on ' //               &
         TRIM(ADJUSTL(cmor_getDate())) //                         &
         ', CMOR rewrote data to comply with CF standards and '// &
         TRIM(mip_tbl(itable)%project_id)  // ' requirements')

    IF (ierr /= NF90_NOERR) THEN
          PRINT*, 'Error encountered in CMOR: initialize_file'
          PRINT*, '   Problem writing "history" global attribute to file: ', &
               TRIM(var(ivar)%full_outpath)
          STOP
    END IF

    IF (LEN_TRIM(dset%references) /= 0) THEN
       
       ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=NF90_GLOBAL,   &
            name='references',                                       &
            values=TRIM(ADJUSTL(dset%references)))
       IF (ierr /= NF90_NOERR) THEN
          PRINT*, 'Error encountered in CMOR: initialize_file'
          PRINT*, '   Problem writing "references" global attribute to file: ', &
               TRIM(var(ivar)%full_outpath)
          STOP
       END IF
    END IF


    IF (LEN_TRIM(dset%comment) /= 0) THEN
       ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=NF90_GLOBAL, &
            name='comment', values=TRIM(dset%comment))
       IF (ierr /= NF90_NOERR) THEN
          PRINT*, 'Error encountered in CMOR: initialize_file'
          PRINT*, '   Problem writing "comment" global attribute to file: ', &
               TRIM(var(ivar)%full_outpath)
          STOP
       END IF
    END IF

    !  Write to netCDF lengths of dimensions

    ! number of non-singleton dimensions
    non_single=0

    ! define dimensions
    ! loop over output dimensions
    DO iout=1,var(ivar)%rank

       iax = var(ivar)%axis_idxs(iout)
       idim = var(ivar)%dim_idxs(iout)
       singleton(iout) = .false.
       IF (LEN_TRIM(axis(iax)%singleton_label) /= 0) &
            singleton(iout) = .true.

       ! ignore if official singleton dimension
       IF (.NOT. singleton(iout)) THEN
          non_single=non_single+1
          IF (similar(axis(iax)%standard_name, 'time')) THEN
             ilen = NF90_UNLIMITED
          ELSE
             idim = var(ivar)%dim_idxs(iout)
             ilen = adim(idim)%lendim
          END IF
          ierr = nf90_def_dim(ncid=var(ivar)%ncid,    &
               name=TRIM(axis(iax)%out_name),         &
               len=ilen,                              &
               dimid=ncdimids(iout))
          IF (ierr /= NF90_NOERR) THEN
             PRINT*, 'Error encountered in CMOR: initialize_file'
             PRINT*, '   Problem defining dimension: ', axis(iax)%table_entry
             PRINT*, '   to file: ',  TRIM(var(ivar)%full_outpath)
             STOP
          END IF
       END IF
    END DO

    ! check whether any bounds will be stored

    DO iout=1,var(ivar)%rank

       iax = var(ivar)%axis_idxs(iout)
       IF (LEN_TRIM(axis(iax)%bounds_attrib) > 0) THEN 
          ierr = nf90_def_dim(ncid=var(ivar)%ncid,    &
               name='bnds',                           &
               len=2  ,                               &
               dimid=bnds_id)
          IF (ierr /= NF90_NOERR) THEN
             PRINT*, 'Error encountered in CMOR: initialize_file'
             PRINT*, '   Problem defining bnds: in file: ', &
                  TRIM(var(ivar)%full_outpath)
             STOP
          END IF
          EXIT
       END IF
       
    END DO

    ! check whether any coordinates are labels, and if so, find max string 
    ! length
    max_str_len = 0
    DO iout = 1, var(ivar)%rank
       iax=var(ivar)%axis_idxs(iout)
       idim = var(ivar)%dim_idxs(iout)
       
       IF (TRIM(axis(iax)%type) == 'character') THEN
          DO i = 1, adim(idim)%lendim
             max_str_len = MAX(LEN_TRIM(adim(idim)%labels(i)), max_str_len)
          END DO
       END IF
    END DO

    IF (max_str_len > 0) THEN
       ierr = nf90_def_dim(ncid=var(ivar)%ncid,      &
            name='strlen',                           &
            len=max_str_len,                         &
            dimid=str_id)
       IF (ierr /= NF90_NOERR) THEN
          PRINT*, 'Error encountered in CMOR: initialize_file'
          PRINT*, '   Problem defining bnds: in file: ', &
               TRIM(var(ivar)%full_outpath)
          PRINT*, 'CMOR error 96'
          STOP
       END IF
    END IF
       
    ! define coordinate variables (including singleton dimensions)

    dimensionless = 0

    DO i=1, var(ivar)%rank
       iax=var(ivar)%axis_idxs(i)
       idim = var(ivar)%dim_idxs(i)

       std_name_copy = axis(iax)%standard_name
       IF (LEN_TRIM(axis(iax)%convert_to) > 0)   &
            std_name_copy = axis(iax)%convert_to

       ! treat labels different from numerical coordinates
       IF (TRIM(axis(iax)%type) == 'character') THEN

          ! label "coordinate"
          IF (singleton(i)) THEN
             ! singleton dimensions: define scalar variable
             ierr = nf90_def_var(ncid=var(ivar)%ncid,       &    
                  xtype=cmor_type_to_nf90(axis(iax)%type),  &
                  name=TRIM(axis(iax)%out_name),            &
                  dimids=str_id,                            &
                  varid=coord_id(i))
          ELSE
             ! regular dimensions
!!$             ierr = nf90_def_var(ncid=var(ivar)%ncid,       &
!!$                  xtype=cmor_type_to_nf90(axis(iax)%type),  &
!!$                  name=TRIM(axis(iax)%out_name),            &
!!$                  dimids=(/ str_id, ncdimids(i) /),         &  
!!$                  varid=coord_id(i))

             ierr = nf90_def_var(ncid=var(ivar)%ncid,       &
                  xtype=cmor_type_to_nf90(axis(iax)%type),  &
                  name=TRIM(axis(iax)%coords_attrib),       &
                  dimids=(/ str_id, ncdimids(i) /),         &  
                  varid=label_id)
          END IF

       ELSE

          ! numeric coordinate
          IF (singleton(i)) THEN
             ! singleton dimensions: define scalar variable
             ierr = nf90_def_var(ncid=var(ivar)%ncid,       &    
                  xtype=cmor_type_to_nf90(axis(iax)%type),  &
                  name=TRIM(axis(iax)%out_name),            &
                  varid=coord_id(i))
          ELSE
             ! regular dimensions
             ierr = nf90_def_var(ncid=var(ivar)%ncid,       &
                  xtype=cmor_type_to_nf90(axis(iax)%type),  &
                  name=TRIM(axis(iax)%out_name),            &
                  dimids=ncdimids(i),                     &  
                  varid=coord_id(i))
          END IF

       END IF
       IF (ierr /= NF90_NOERR) THEN
          print*, 'CMOR error 91 '
          stop
       END IF

       IF (similar(axis(iax)%axis, 'T')) THEN
          var(ivar)%nctimid = coord_id(i)
       END IF

       IF (LEN_TRIM(axis(iax)%bounds_attrib) > 0)  THEN

          IF (singleton(i)) THEN
             ! singleton dimensions: define bounds variable for scalar 
             !     dimension
!!$print*, TRIM(axis(iax)%bounds_attrib)
!!$print*, i, 'singleton = ', singleton(i)
!!$print*, cmor_type_to_nf90(axis(iax)%type)
!!$print*, bnds_id
!!$print*, coord_bnds_id(i)

             ierr = nf90_def_var(ncid=var(ivar)%ncid,       &
                  xtype=cmor_type_to_nf90(axis(iax)%type),  &
                  name=TRIM(axis(iax)%bounds_attrib),       &
                  dimids=bnds_id,                           &
                  varid=coord_bnds_id(i))
          ELSE
             ! bounds for regular dimensions
             ierr = nf90_def_var(ncid=var(ivar)%ncid,       &
                  xtype=cmor_type_to_nf90(axis(iax)%type),  &
                  name=TRIM(axis(iax)%bounds_attrib),       &
                  dimids=(/bnds_id, ncdimids(i)/),        &
                  varid=coord_bnds_id(i))
          END IF
          IF (ierr /= NF90_NOERR) THEN
             print*, 'CMOR error 92 '
             stop
          END IF

          IF (similar(axis(iax)%axis, 'T')) THEN

             var(ivar)%nctim_bndsid = coord_bnds_id(i)
          END IF

       END IF

       !  write coordinate variable attributes

       IF ((TRIM(axis(iax)%type) == 'character') .and. &
            (.NOT. singleton(i))) THEN

          ! we have a non-singleton label dimension
          vvid = label_id

       ELSE

          vvid = coord_id(i)

       END IF

       ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=vvid, &
            name='standard_name',                                  &
            values=TRIM(std_name_copy))
       IF (ierr /= NF90_NOERR) THEN
          print*, 'CMOR error 93 '
          stop
       END IF


       IF (LEN_TRIM(var(ivar)%long_name) > 0) THEN
          ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=vvid, &
               name='long_name',                                      &
               values=TRIM(axis(iax)%long_name))
          IF (ierr /= NF90_NOERR) THEN
             print*, 'CMOR error 94 '
             stop
          END IF
       END IF

       IF (.not. similar(axis(iax)%out_units, init_undef_char)) THEN
          ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=vvid, &
               name='units', values=TRIM(axis(iax)%out_units))
          IF (ierr /= NF90_NOERR) THEN
             print*, 'CMOR error 95 '
             stop
          END IF
       END IF

       IF (LEN_TRIM(axis(iax)%axis) > 0) THEN
          ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=vvid, &
               name='axis', values=TRIM(axis(iax)%axis))
          IF (ierr /= NF90_NOERR) THEN
             print*, 'CMOR error 96 '
             stop
          END IF
       END IF

       IF (similar(axis(iax)%axis, 'T')) THEN

          ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=vvid, &
               name='calendar', values=TRIM(dset%calendar))
          IF (ierr /= NF90_NOERR) THEN
             print*, 'CMOR error 97 '
             stop
          END IF

          IF (dset%leap_year /= init_undef_integer) THEN
             ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=vvid, &
                  name='leap_year', values=dset%leap_year)
             IF (ierr /= NF90_NOERR) THEN
                print*, 'CMOR error 98 '
                stop
             END IF
          END IF

          IF (dset%leap_month /= 0) THEN
             ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=vvid, &
                  name='leap_month', values=dset%leap_month)
             IF (ierr /= NF90_NOERR) THEN
                print*, 'CMOR error 99 '
                stop
             END IF
          END IF

          IF (dset%month_lengths(1) /= 0) THEN
             ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=vvid, &
                  name='month_lengths', values=dset%month_lengths)
             IF (ierr /= NF90_NOERR) THEN
                print*, 'CMOR error 100 '
                stop
             END IF
          END IF


       END IF

       IF (LEN_TRIM(axis(iax)%positive_attrib) > 0) THEN
          ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=vvid, &
               name='positive',                                       &
               values=TRIM(axis(iax)%positive_attrib))
          IF (ierr /= NF90_NOERR) THEN
             print*, 'CMOR error 101 '
             stop
          END IF
       END IF

       IF (LEN_TRIM(axis(iax)%bounds_attrib) > 0) THEN
          ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=vvid, &
               name='bounds',                                         &
               values=TRIM(axis(iax)%bounds_attrib))
          IF (ierr /= NF90_NOERR) THEN
             print*, 'CMOR error 102 '
             stop
          END IF
       END IF

       IF (.NOT. similar(adim(idim)%users_units, axis(adim(idim)%axis_idx)%out_units)) THEN
!  .and.   &
!             (.not. similar(adim(idim)%users_units, init_undef_char))) THEN
          ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=vvid, &
               name='original_units',                                 &
               values=TRIM(adim(idim)%users_units))
          IF (ierr /= NF90_NOERR) THEN
             print*, 'CMOR error 103 '
             stop
          END IF
       END IF

       ! if dimensionless vertical coordinate, write formula_terms and formula

       IF (associated(axis(iax)%zfac_names)) THEN

          dimensionless = idim

          str = ''
          DO j=1,SIZE(axis(iax)%zfac_names)
             str2 = TRIM(axis(iax)%zfac_names(j))
             IF (LEN_TRIM(str2) == 0) EXIT
             str = TRIM(str) // ' ' // TRIM(str2) // ': ' // TRIM(str2)
          END DO
          ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=vvid, &
               name='formula',                                 &
               values=TRIM(axis(adim(dimensionless)%axis_idx)%formula) )
          IF (ierr /= NF90_NOERR) THEN
             print*, 'CMOR error 103.4 '
             stop
          END IF

         ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=vvid, &
               name='formula_terms',                                 &
               values= TRIM(ADJUSTL(str)))
          IF (ierr /= NF90_NOERR) THEN
             print*, 'CMOR error 103.4 '
             stop
          END IF

          IF (LEN_TRIM(axis(iax)%bounds_attrib) > 0)  THEN

             ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=coord_bnds_id(i), &
                  name='standard_name',                                  &
                  values=TRIM(std_name_copy))
             IF (ierr /= NF90_NOERR) THEN
                print*, 'CMOR error 93 '
                stop
             END IF

             str = ''
             DO j=1,SIZE(axis(iax)%zfac_bnds_names)
                str2 = TRIM(axis(iax)%zfac_bnds_names(j))
                IF (LEN_TRIM(str2) == 0) EXIT
                str = TRIM(str) // ' ' // TRIM(axis(iax)%zfac_names(j)) //  &
                     ': ' // TRIM(str2)
             END DO
             ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=coord_bnds_id(i), &
                  name='formula',                                 &
                  values=TRIM(axis(adim(dimensionless)%axis_idx)%formula))
             IF (ierr /= NF90_NOERR) THEN
                print*, 'CMOR error 103.4 '
                stop
             END IF

             ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=coord_bnds_id(i), &
                  name='formula_terms',                                 &
                  values= TRIM(ADJUSTL(str)))
             IF (ierr /= NF90_NOERR) THEN
                print*, 'CMOR error 103.4 '
                stop
             END IF
             
          END IF

       END IF
       
    END DO


    ! loop over output dimensions and define any zfactors
    DO i=1, var(ivar)%rank
       iax=var(ivar)%axis_idxs(i)
       idim = var(ivar)%dim_idxs(i)
       IF (associated(axis(iax)%zfac_names)) THEN

          IF (associated(axis(iax)%zfac_bnds_names)) THEN
             ierr = def_zfacs(adim(idim)%zfac_bnds_idxs, bnds_id=bnds_id, &
                  ivar=ivar, ncdimids=ncdimids)
             IF (ierr /= 0) THEN
                print*, 'CMOR error 103.65 '
                stop
             END IF
          END IF

          ierr = def_zfacs(adim(idim)%zfac_idxs, bnds_id=0, ivar=ivar,   &
               ncdimids=ncdimids)
          IF (ierr /= 0) THEN
             print*, 'CMOR error 103.6  '
             stop
          END IF
          
       END IF
       
    END DO
       
    ! define variable

    ierr = nf90_def_var(ncid=var(ivar)%ncid,            &
         xtype=cmor_type_to_nf90(var(ivar)%type),       &
         name=TRIM(var(ivar)%out_name),                 &
         dimids=ncdimids(1:non_single),               &
         varid=var(ivar)%varid)

    IF (ierr /= NF90_NOERR) THEN
       print*, 'CMOR error 104a '
       stop
    END IF

    ! define variable attributes
    IF (LEN_TRIM(var(ivar)%standard_name) > 0) THEN
       ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=var(ivar)%varid, &
            name='standard_name',                                      &
            values=TRIM(var(ivar)%standard_name))
       IF (ierr /= NF90_NOERR) THEN
          print*, 'CMOR error 105 '
          stop
       END IF
    END IF

    IF (LEN_TRIM(var(ivar)%long_name) > 0) THEN
       ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=var(ivar)%varid, &
            name='long_name',                                          &
            values=TRIM(var(ivar)%long_name))
       IF (ierr /= NF90_NOERR) THEN
          print*, 'CMOR error 106 '
          stop
       END IF
    END IF

    IF (.not. similar(var(ivar)%out_units, init_undef_char)) THEN
       ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=var(ivar)%varid, &
            name='units', values=TRIM(var(ivar)%out_units))
       IF (ierr /= NF90_NOERR) THEN
          print*, 'CMOR error 107 '
          stop
       END IF
    END IF

    IF (LEN_TRIM(var(ivar)%cell_methods) > 0) THEN
       DO iout=1,var(ivar)%rank
          idim = var(ivar)%dim_idxs(iout)
          
          IF (LEN_TRIM(adim(idim)%interval) > 0) THEN
             j=INDEX(var(ivar)%cell_methods, &
                  TRIM(axis(adim(idim)%axis_idx)%out_name))
             IF (j > 0) THEN
                k=INDEX(var(ivar)%cell_methods(j:), ':')
                IF (k > 0) THEN
                   m=VERIFY(var(ivar)%cell_methods(j+k:), ' ' )-1
                   IF (m >= 0) THEN 
                      n=INDEX(var(ivar)%cell_methods(j+k+m:), ' ')-1
                      
                      IF (LEN_TRIM(var(ivar)%cell_methods(1:j+k+m+n-1) //    &
                           ' (interval: ' //                                  &
                           TRIM(adim(idim)%interval) // ') ' //     &
                           TRIM(var(ivar)%cell_methods(j+k+m+n:))) > a256) &
                           THEN
                        
                         print*, 'CMOR error 108 '
                         stop
                         
                      ELSE
                         var(ivar)%cell_methods =                           &
                              ! var(ivar)%cell_methods(1:j+k+m+n+1) //        &
                              var(ivar)%cell_methods(1:j+k+m+n-1) //        &
                              ' (interval: ' //                              &
                              TRIM(adim(idim)%interval) // ') ' // &
                              TRIM(var(ivar)%cell_methods(j+k+m+n:))
                              ! TRIM(var(ivar)%cell_methods(j+k+m+n+2:))
                         
                      END IF
                   END IF
                END IF
             END IF
          END IF
       END DO
       
       
       ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=var(ivar)%varid, &
            name='cell_methods',                                       &
            values=TRIM(var(ivar)%cell_methods))
       IF (ierr /= NF90_NOERR) THEN
          print*, 'CMOR error 109 '
          stop
       END IF
    END IF
        
    ! generate list of singleton dimensions
    coords=''
    DO iout=1,var(ivar)%rank
       IF (singleton(iout)) coords = TRIM(coords) // ' ' //   &
            TRIM(axis(var(ivar)%axis_idxs(iout))%out_name)
    END DO
    
    ! check whether any of the dimensions is a "region" dimension (i.e., has
    !    labels).
    
    DO i=1,var(ivar)%rank
       IF (TRIM(axis(var(ivar)%axis_idxs(i))%type) == 'character')    &
            coords = TRIM(coords) // ' ' //                           &
            TRIM(axis(var(ivar)%axis_idxs(i))%coords_attrib)
       !           TRIM(axis(var(ivar)%axis_idxs(i))%out_name)
    END DO
    
    IF (LEN_TRIM(coords) > 0) THEN
       ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=var(ivar)%varid, &
            name='coordinates', values=TRIM(ADJUSTL(coords)))
       IF (ierr /= NF90_NOERR) THEN
          print*, 'CMOR error 110 '
          stop
       END IF
    END IF
    
    IF (var(ivar)%def_missing /= 0) THEN
       select case(trim(var(ivar)%type))
       case("real")
          ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=var(ivar)%varid, &
               name='_FillValue', values=mip_tbl(itable)%single_missing)
          ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=var(ivar)%varid, &
               name='missing_value',                                      &
               values=mip_tbl(itable)%single_missing)
       case("double")
          ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=var(ivar)%varid, &
               name='_FillValue', values=mip_tbl(itable)%double_missing)
          ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=var(ivar)%varid, &
               name='missing_value',                                      &
               values=mip_tbl(itable)%double_missing)
       case("integer")
          ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=var(ivar)%varid, &
               name='_FillValue', values=mip_tbl(itable)%INTEGER_missing)
          ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=var(ivar)%varid, &
               name='missing_value',                                      &
               values=mip_tbl(itable)%INTEGER_missing)
       end select
       
       IF (ierr /= NF90_NOERR) THEN
          print*, 'CMOR error 111 '
          stop
       END IF
    END IF
    
    IF (LEN_TRIM(var(ivar)%users_name) > 0) THEN
       ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=var(ivar)%varid, &
            name='original_name',                                      &
            values=TRIM(var(ivar)%users_name))
       IF (ierr /= NF90_NOERR) THEN
          
          print*, 'CMOR error 112 '
          stop
       END IF
    END IF
    IF (.NOT. similar(var(ivar)%users_units, var(ivar)%out_units)) THEN
       ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=var(ivar)%varid, &
            name='original_units',                                     &
            values=TRIM(var(ivar)%users_units))    
       IF (ierr /= NF90_NOERR) THEN
          print*, 'CMOR error 113 '
          stop
       END IF
    END IF
    IF (LEN_TRIM(var(ivar)%comment) > 0) THEN
       ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=var(ivar)%varid, &
            name='comment', values=TRIM(var(ivar)%comment))   
       IF (ierr /= NF90_NOERR) THEN
          print*, 'CMOR error 114 '
          stop
       END IF
    END IF
    

    ierr = write_history(itable, ivar, non_single)
    
    
    ! output coordinate values and bounds

    ! leave netCDF define mode
    ierr = nf90_enddef(ncid=var(ivar)%ncid)
    IF (ierr /= NF90_NOERR) THEN
       print*, 'CMOR error 115.5 '
       stop
    END IF
    
    var(ivar)%file_status = 'write mode'
    
    DO iout=1, var(ivar)%rank
       
       iax=var(ivar)%axis_idxs(iout)
       idim = var(ivar)%dim_idxs(iout)
       iax = adim(idim)%axis_idx

       IF (singleton(iout)) THEN
          
          
          ! singleton dimension 
          ! put value 
          
          IF (TRIM(axis(iax)%type) == 'character') THEN
             ierr = nf90_put_var(ncid=var(ivar)%ncid, varid=coord_id(iout),  &
                  values=adim(idim)%labels(1))
          ELSE
             ierr = nf90_put_var(ncid=var(ivar)%ncid, varid=coord_id(iout),  &
                  values=adim(idim)%values(1))
          END IF
          IF (ierr /= NF90_NOERR) THEN
             print*, 'CMOR error116 '
             stop
          END IF
          IF (LEN_TRIM(axis(iax)%bounds_attrib) > 0) THEN
             ! put bounds
             ierr = nf90_put_var(ncid=var(ivar)%ncid,          &
                  varid=coord_bnds_id(iout),                   &
                  values=adim(idim)%bounds, count=(/ 2, 1 /) )
             IF (ierr /= NF90_NOERR) THEN
                print*, 'CMOR error 117 '
                stop
             END IF
          END IF
       ELSE 
          
          !  non-singleton dimension
          ! if not an unlimited dimension (i.e., time), 
          ! put values
          
          IF (TRIM(axis(iax)%type) == 'character') THEN
           ! ierr = nf90_put_var(ncid=var(ivar)%ncid, varid=coord_id(iout),  &
             ierr = nf90_put_var(ncid=var(ivar)%ncid, varid=label_id,        &
                  values =                                                   &
                  adim(idim)%labels(1:adim(idim)%lendim)(1:max_str_len),     &
                  count=(/ max_str_len, adim(idim)%lendim /))
             IF (ierr /= NF90_NOERR) THEN
                print*, 'CMOR error 117.5 '
                stop
             END IF
          ELSE IF (.not. similar(axis(adim(idim)%axis_idx)%standard_name,    &
               'time')) THEN
             
            ierr = nf90_put_var(ncid=var(ivar)%ncid, varid=coord_id(iout),  &
                  values=adim(idim)%values, count=(/ adim(idim)%lendim /))
             IF (ierr /= NF90_NOERR) THEN
                print*, 'CMOR error 118 '
                stop
             END IF
             
             IF (LEN_TRIM(axis(iax)%bounds_attrib) > 0) THEN
                ! put bounds
                ierr = nf90_put_var(ncid=var(ivar)%ncid,     &
                     varid=coord_bnds_id(iout),              &
                     values=adim(idim)%bounds,               &
                     count=(/ 2, adim(idim)%lendim /))
                IF (ierr /= NF90_NOERR) THEN
                   print*, 'CMOR error 119 '
                   stop
                END IF
             END IF
          END IF
       END IF
       
    END DO

    IF (ierr == NF90_NOERR) ierr = 0
    

    IF (dimensionless /= 0) THEN

       ! output z-factors that are available
       DO i=1, SIZE(adim(dimensionless)%zfac_idxs) 
          izfac = adim(dimensionless)%zfac_idxs(i)
          ! note the following is a recursive call to cmor_write (because 
          ! we already are in a subroutine called by cmor_write).
          ! To prevent an endless loop, make sure that cmor_write
          !  doesn't call initialize_file.
          IF (associated(var(izfac)%zfac_values)) THEN
             ! ??? does following work for a scalar???
             ierr= cmor_write(izfac,  var(izfac)%zfac_values, &
                  store_with=ivar)
             ! old version
             ! ierr= cmor_write(izfac, &
             !      RESHAPE(var(izfac)%zfac_values,  &
             !      (/ SIZE(var(izfac)%zfac_values) /)), &
             !      store_with=ivar)
             IF (ierr /= 0) THEN
                
                print*, 'CMOR error 119.2'
                stop
             END IF
          ELSE IF (associated(var(izfac)%zfac_values_dbl)) THEN

             ierr= cmor_write(izfac, var(izfac)%zfac_values_dbl, &
                  store_with=ivar)
          END IF
       END DO
       IF (associated(axis(adim(dimensionless)%axis_idx)%zfac_bnds_names)) THEN
          DO i=1, SIZE(adim(dimensionless)%zfac_bnds_idxs) 
             izfac = adim(dimensionless)%zfac_bnds_idxs(i)
             ! Check whether this field has already been stored
             found = .false.
             DO j=1,SIZE(adim(dimensionless)%zfac_idxs)
                IF (izfac == adim(dimensionless)%zfac_idxs(j)) THEN
                   found = .true.
                   EXIT
                END IF
             END DO

             IF (found) CYCLE

             IF (associated(var(izfac)%zfac_bounds)) THEN
                ! ??? does following work for a scalar???
                ierr = nf90_put_var(ncid=var(ivar)%ncid,          &
                     varid=var(izfac)%varid,                      &
                     values=var(izfac)%zfac_bounds,  &
                     count=(/ 2, SIZE(var(izfac)%zfac_bounds)/2 /) )

                ! old version
                ! ierr = nf90_put_var(ncid=var(ivar)%ncid,          &
                !      varid=var(izfac)%varid,                      &
                !      values=var(izfac)%zfac_bounds,  &
                !      count=(/ 2, SIZE(var(izfac)%zfac_bounds)/2 /) )
                IF (ierr /= NF90_NOERR) THEN
                   print*, 'CMOR error 117.3 '
                   stop
                END IF

             ELSE IF (associated(var(izfac)%zfac_bounds_dbl)) THEN
                ierr = nf90_put_var(ncid=var(ivar)%ncid,          &
                     varid=var(izfac)%varid,                      &
                     values=var(izfac)%zfac_bounds_dbl,  &
                     count=(/ 2, SIZE(var(izfac)%zfac_bounds_dbl)/2 /) )
                IF (ierr /= NF90_NOERR) THEN
                   print*, 'CMOR error 119.5'
                   stop
                END IF
             END IF
          END DO

       END IF

    END IF

    RETURN

  END FUNCTION initialize_file



  FUNCTION def_zfacs(zfac_idxs, bnds_id, ivar, ncdimids)  RESULT(ierr)
    
    IMPLICIT NONE
    
    INTEGER, INTENT(IN), DIMENSION(:) :: zfac_idxs
    INTEGER, INTENT(IN) :: bnds_id
    INTEGER, INTENT(IN) :: ivar
    INTEGER, INTENT(IN), DIMENSION(:) :: ncdimids

    INTEGER ierr

    INTEGER :: n, k, j, izfac
    INTEGER, DIMENSION(max_rank) :: nczids
    LOGICAL :: found, zfound
    
    ierr = 0
    ! cycle through z-factors
    DO n=1,SIZE(zfac_idxs)
       izfac = zfac_idxs(n)
       
       IF (var(izfac)%rank == 0) THEN
          IF (bnds_id == 0) THEN
             
             ! scalar factor
             ierr = nf90_def_var(ncid=var(ivar)%ncid,            &
                  xtype=cmor_type_to_nf90(var(izfac)%type),      &
                  name=TRIM(var(izfac)%out_name),                &
                  varid=var(izfac)%varid)
          END IF
          IF (ierr /= NF90_NOERR) THEN
             print*, 'CMOR error 104b '
             stop
          END IF
          
       ELSE
          
          ! array of factors
          ! cycle through output dimensions and match with zfactor dimensions
          found = .false.
          zfound = .false.

          DO k=1,var(izfac)%rank
             DO j=1,var(ivar)%rank
                IF (var(ivar)%dim_idxs(j) == var(izfac)%dim_idxs(k)) THEN
                   nczids(k) = ncdimids(j)
                   found = .true.
                   IF (similar(axis(var(izfac)%axis_idxs(k))%axis, 'z')) &
                        zfound = .true.
                END IF
             END DO
             IF (.not. found) THEN
                PRINT*, 'Error encountered in CMOR: def_zfacs'
                PRINT*, '   One of the z-factor axes for z-factor: ', &
                     TRIM(var(izfac)%table_entry)
                PRINT*, '   is not found in the list of dimensions '
                PRINT*, '   for variable: ', TRIM(var(ivar)%table_entry)
                PRINT*, '   z-factor dimension not found: ', &
                     TRIM(axis(var(izfac)%axis_idxs(k))%table_entry)
                STOP
             END IF
          END DO
         
          ! zfound indicates that THIS z-factor is a function of z

          IF ((bnds_id == 0) .or. zfound) THEN
             ! define z factor

             IF (bnds_id > 0) THEN
                ierr = nf90_def_var(ncid=var(ivar)%ncid,             &
                     xtype=cmor_type_to_nf90(var(izfac)%type),       &
                     name=TRIM(var(izfac)%out_name),                 &
                     dimids=(/ bnds_id, nczids(1:var(izfac)%rank) /),&
                     varid=var(izfac)%varid)
             ELSE
                ierr = nf90_def_var(ncid=var(ivar)%ncid,             &
                     xtype=cmor_type_to_nf90(var(izfac)%type),       &
                     name=TRIM(var(izfac)%out_name),                 &
                     dimids=nczids(1:var(izfac)%rank),               &
                     varid=var(izfac)%varid)
             END IF
             
             IF (ierr /= NF90_NOERR) THEN
                print*, 'CMOR error 104c '
                print*, TRIM(nf90_strerror(ierr))
                stop
             END IF

          END IF
       END IF
       
       IF ((bnds_id == 0) .or. zfound) THEN
          ! define standard-name, long-name and units attributes for the factor
          IF (LEN_TRIM(var(izfac)%standard_name) > 0) THEN
             ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=var(izfac)%varid, &
                  name='standard_name',                                           &
                  values=TRIM(var(izfac)%standard_name))
             IF (ierr /= NF90_NOERR) THEN
                print*, 'CMOR error 106.1 '
                stop
             END IF
          END IF

          IF (LEN_TRIM(var(izfac)%long_name) > 0) THEN
             ierr = nf90_put_att(ncid=var(ivar)%ncid, varid=var(izfac)%varid, &
                  name='long_name',                                           &
                  values=TRIM(var(izfac)%long_name))
             IF (ierr /= NF90_NOERR) THEN
                print*, 'CMOR error 106 '
                stop
             END IF
          END IF

          IF (.not. similar(var(izfac)%out_units, init_undef_char)) THEN
             ierr = nf90_put_att(ncid=var(ivar)%ncid,    &
                  varid=var(izfac)%varid,                &
                  name='units', values=TRIM(var(izfac)%out_units))
             IF (ierr /= NF90_NOERR) THEN
                print*, 'CMOR error 107 '
                stop
             END IF
          END IF

       END IF
    END DO
    
    RETURN
  END FUNCTION def_zfacs
     
     
  
  SUBROUTINE zfac_preamble_sub(zaxis_id, zfactor_name, units, axis_ids, &
       itable, zfactor_id, izax, ierr)
    
    IMPLICIT NONE
    
    INTEGER, INTENT(IN) :: zaxis_id
    CHARACTER(len=*), INTENT(IN) :: zfactor_name
    CHARACTER(len=*), INTENT(IN) :: units
    INTEGER, INTENT(IN), DIMENSION(:), OPTIONAL :: axis_ids
    INTEGER, INTENT(OUT) :: itable
    INTEGER, INTENT(OUT) :: zfactor_id
    INTEGER, INTENT(OUT) :: izax
    INTEGER, INTENT(OUT) :: ierr
    
    CHARACTER(len=(a256+a64+1)) :: pathfile
    INTEGER :: i

    ierr = 0

!  check that zaxis_id is in range 1 to num_dims ???

    izax = adim(zaxis_id)%axis_idx
    IF (.not. associated(axis(izax)%zfac_names)) THEN
       PRINT*, 'Error encountered in CMOR: zfac_preamble_sub'
       PRINT*, '   According to CMOR, there are no z-factors associated with axis: ', &
            TRIM(axis(izax)%table_entry)
       PRINT*, '   appearing in table ', TRIM(axis(izax)%mip_table)
       STOP
    END IF

    ! find full path to table
    
    Do i=1,num_mip_tables
       IF (similar(mip_tbl(i)%table_id, &
            axis(adim(zaxis_id)%axis_idx)%mip_table)) THEN
          pathfile = mip_tbl(i)%full_path
          EXIT
       END IF
    END DO
    
    call var_preamble(pathfile, zfactor_name, itable, zfactor_id, ierr)
    
    IF (ierr /= 0) THEN
       print*, 'CMOR error 45.5 '
       stop
    END IF
    
    ! check whether axis_ids required
    
    IF ((var(zfactor_id)%arg_cntl(4) == 1) .and.  &
         (.not. PRESENT(axis_ids))) THEN
       PRINT*, 'Error encountered in CMOR: zfac_preamble_sub'
       PRINT*, '   The z-factor you are attempting to define should be a function'
       PRINT*, '   of 1 or more axes, but you failed to pass the axis_ids argument'
       PRINT*, '   in calling cmor_zfactor for variable: ', TRIM(zfactor_name)
       STOP
    END IF
    
    ! copy variable table entry so that if the variable will also
    !  be written independently (not as a zfactor), CMOR won't
    !  get confused
    
    IF (num_vars < max_variables) THEN
       num_vars = num_vars + 1
       var(num_vars) = var(zfactor_id)
       zfactor_id = num_vars
    ELSE
       PRINT*, 'Error encountered in CMOR: zfac_preamble_sub'
       PRINT*, '  You are requiring CMOR to define too many variables.'
       PRINT*, '  The maximum number of variables allowed by CMOR is: ', max_variables
       PRINT*, '  If this is really too few, the user may increase the value of '//&
            'max_variables in module: cmor_internal_tables'
       STOP
    END IF
    
    var(zfactor_id)%def_missing = 0
    
    ! do main work of this subroutine
    
    IF (PRESENT(axis_ids)) THEN
       ierr = def_var(itable, zfactor_id, units, axis_ids)
    ELSE
       ierr = def_var(itable, zfactor_id, units, passed_dim_idxs=(/ 0 /))
    END IF
    
    IF (ierr .ne. 0) THEN
       PRINT*, 'CMOR error 48.7'
       stop
    END IF
    
    
    RETURN
  END SUBROUTINE zfac_preamble_sub



  SUBROUTINE zfac_preamble(zaxis_id, zfactor_name, units, axis_ids, &
       itable, zfactor_id, zfactor_bnds_id, izax, alldone, ierr)
    
    IMPLICIT NONE
    
    INTEGER, INTENT(IN) :: zaxis_id
    CHARACTER(len=*), INTENT(IN) :: zfactor_name
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: units
    INTEGER, INTENT(IN), DIMENSION(:), OPTIONAL :: axis_ids
    INTEGER, INTENT(OUT) :: itable
    INTEGER, INTENT(OUT) :: zfactor_id
    INTEGER, INTENT(OUT) :: zfactor_bnds_id
    INTEGER, INTENT(OUT) :: izax
    INTEGER, INTENT(OUT) :: ierr
    LOGICAL, INTENT(OUT) :: alldone
    
    CHARACTER(len=a64) :: units_copy
    LOGICAL :: found
    INTEGER :: i
    
    PRINT*, 'CMOR is defining a z-factor: ',  TRIM(zfactor_name) 

    IF (PRESENT(units)) THEN
       units_copy = units
    ELSE
       units_copy = 'none'
    END IF

    ! check that the table entry is o.k.
    ! also check whether this is the last factor that needs to be defined
    ! for this dimension

    CALL zfac_preamble_sub(zaxis_id, zfactor_name, units_copy, axis_ids, &
         itable, zfactor_id, izax, ierr)
    
    zfactor_bnds_id = 0
    found = .false.
    alldone = .true.

    DO i=1,SIZE(axis(izax)%zfac_names)
       IF (similar(zfactor_name, axis(izax)%zfac_names(i))) THEN
          adim(zaxis_id)%zfac_idxs(i) = zfactor_id

          IF (associated(axis(izax)%zfac_bnds_names)) THEN
             IF (axis(izax)%zfac_names(i) /= axis(izax)%zfac_bnds_names(i))  &
                  THEN
                CALL zfac_preamble_sub(zaxis_id,      &
                     axis(izax)%zfac_bnds_names(i),   &
                     units_copy, axis_ids, itable,         &
                     zfactor_bnds_id,                 &
                     izax, ierr)
                adim(zaxis_id)%zfac_bnds_idxs(i) = zfactor_bnds_id
             ELSE
                adim(zaxis_id)%zfac_bnds_idxs(i) = zfactor_id
             ENDIF
          ENDIF
          found = .true.
       END IF
       IF (adim(zaxis_id)%zfac_idxs(i) <= 0) alldone = .false.
    END DO
    
    IF (.not. found) THEN
       PRINT*, 'Error encountered in CMOR: zfac_preamble'
       PRINT*, '   User is attempting to define a z-factor named ', TRIM(zfactor_name)
       PRINT*, '   which is not one of the following: ', &
          (axis(izax)%zfac_names(i), i=1,SIZE(axis(izax)%zfac_names))
       PRINT*, '   which are associated with axis: ', axis(izax)%table_entry
       PRINT*, '   found in table ', axis(izax)%mip_table
       STOP
    END IF

    RETURN
  END SUBROUTINE zfac_preamble
  

 SUBROUTINE zfac_alldone(zaxis_id)

    IMPLICIT NONE

    INTEGER, INTENT(IN) :: zaxis_id

    INTEGER :: izax
    REAL :: ptop
    INTEGER :: i, izaxnew, ps_id, p0_id, a_id, b_id, a_bnds_id, b_bnds_id
    INTEGER :: itable, ierr, ptop_id, lon1, ap_id, ap_bnds_id, j, ipos
    INTEGER :: lendim
    DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:) :: sigma
    DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:) :: sigma_bnds
    DOUBLE PRECISION, DIMENSION(2) :: ddum2
    DOUBLE PRECISION :: ddum

    izax = adim(zaxis_id)%axis_idx
    izaxnew = izax

    IF (LEN_TRIM(axis(izax)%convert_to) > 0) THEN
       SELECT CASE(axis(izax)%convert_to)

       CASE('standard_hybrid_sigma')

          ! find in axis table, standard_hybrid_sigma entry:
          DO i=1, num_axes
             IF (similar(axis(i)%mip_table, axis(izax)%mip_table) &
                  .and.   &
                  similar(axis(i)%table_entry, 'standard_hybrid_sigma')) &
                  THEN
                izaxnew = i
                EXIT
             END IF
          END DO
          
          SELECT CASE(axis(izax)%table_entry)

          CASE('standard_sigma')
           ! convert from standard_sigma to standard_hybrid_sigma coordinates

             ! copy z-factors needed to convert

             Allocate(sigma(  &
                  SIZE(var(adim(zaxis_id)%zfac_idxs(2))%zfac_values_dbl)))
             
             ptop_id = adim(zaxis_id)%zfac_idxs(1)
             
             ptop = var(ptop_id)%scale_factor*var(ptop_id)%zfac_values(1) +  &
                  var(ptop_id)%offset
             sigma = var(adim(zaxis_id)%zfac_idxs(2))%zfac_values_dbl
             ps_id = adim(zaxis_id)%zfac_idxs(3)

            call zfac_preamble_sub(        &
                 zaxis_id=zaxis_id,  &
                 zfactor_name='p0',   &
                 units='Pa', axis_ids=(/ 0 /), itable=itable,  &
                 zfactor_id=p0_id, &
                 izax=izax, ierr=ierr)
            call zfac_preamble_sub(       &
                 zaxis_id=zaxis_id,   &
                 zfactor_name='a',   &
                 units='none', axis_ids=(/ zaxis_id /),  &
                 itable=itable, zfactor_id=a_id,   &
                 izax=izax, ierr=ierr)
            call zfac_preamble_sub(     &
                 zaxis_id=zaxis_id,   &
                 zfactor_name='b',   &
                 units='none', axis_ids=(/ zaxis_id /),  &
                 itable=itable, zfactor_id=b_id,   &
                 izax=izax, ierr=ierr)
             ALLOCATE(var(p0_id)%zfac_values(1), STAT=ierr)
             var(p0_id)%zfac_values(1) = 1.d5
             ALLOCATE(var(a_id)%zfac_values_dbl(adim(zaxis_id)%lendim), &
                  STAT=ierr)
             var(a_id)%zfac_values_dbl =    &
                  (1.-sigma)*ptop/var(p0_id)%zfac_values(1)
             ALLOCATE(var(b_id)%zfac_values_dbl(adim(zaxis_id)%lendim), &
                  STAT=ierr)
             var(b_id)%zfac_values_dbl = sigma

             DEALLOCATE(sigma)
             DEALLOCATE(adim(zaxis_id)%zfac_idxs)
             ALLOCATE(adim(zaxis_id)%zfac_idxs(4))
             adim(zaxis_id)%zfac_idxs(1) = p0_id
             adim(zaxis_id)%zfac_idxs(2) = a_id
             adim(zaxis_id)%zfac_idxs(3) = b_id
             adim(zaxis_id)%zfac_idxs(4) = ps_id

             IF (associated(axis(izax)%zfac_bnds_names)) THEN
                ALLOCATE(sigma_bnds(   &
                     2*SIZE(var(adim(zaxis_id)%zfac_idxs(2))%zfac_values_dbl)))

                sigma_bnds = var(adim(zaxis_id)%zfac_bnds_idxs(2))%zfac_bounds_dbl
                call zfac_preamble_sub(     &
                     zaxis_id=zaxis_id,   &
                     zfactor_name='a_bnds',   &
                     units='none', axis_ids=(/ zaxis_id /),  &
                     itable=itable, zfactor_id=a_bnds_id,   &
                     izax=izax, ierr=ierr)
                call zfac_preamble_sub(     &
                     zaxis_id=zaxis_id,   &
                     zfactor_name='b_bnds',   &
                     units='none', axis_ids=(/ zaxis_id /),  &
                     itable=itable, zfactor_id=b_bnds_id,   &
                     izax=izax, ierr=ierr)
                ALLOCATE(  &
                     var(a_bnds_id)%zfac_bounds_dbl(2*adim(zaxis_id)%lendim), &
                     STAT=ierr)
                var(a_bnds_id)%zfac_bounds_dbl =    &
                     (1.-sigma_bnds)*ptop/var(p0_id)%zfac_values(1)
                ALLOCATE(  &
                     var(b_bnds_id)%zfac_bounds_dbl(2*adim(zaxis_id)%lendim), &
                     STAT=ierr)
                var(b_bnds_id)%zfac_bounds_dbl = sigma_bnds
                
                DEALLOCATE(sigma_bnds)
                DEALLOCATE(adim(zaxis_id)%zfac_bnds_idxs)
                ALLOCATE(adim(zaxis_id)%zfac_bnds_idxs(4))
                adim(zaxis_id)%zfac_bnds_idxs(1) = p0_id
                adim(zaxis_id)%zfac_bnds_idxs(2) = a_bnds_id
                adim(zaxis_id)%zfac_bnds_idxs(3) = b_bnds_id
                adim(zaxis_id)%zfac_bnds_idxs(4) = ps_id
                
             END IF

          CASE('alternate_hybrid_sigma')

             ! convert from alternat_hybrid_sigma to standard_hybrid_sigma 
             !        coordinates

            call zfac_preamble_sub(       &
                 zaxis_id=zaxis_id,   &
                 zfactor_name='a',   &
                 units='none', axis_ids=(/ zaxis_id /),  &
                 itable=itable, zfactor_id=a_id,   &
                 izax=izax, ierr=ierr)

             ALLOCATE(var(a_id)%zfac_values_dbl(adim(zaxis_id)%lendim), &
                  STAT=ierr)
             ap_id = adim(zaxis_id)%zfac_idxs(2)
             p0_id = adim(zaxis_id)%zfac_idxs(1)
             var(a_id)%zfac_values_dbl =   &
                  (var(ap_id)%zfac_values_dbl*var(ap_id)%scale_factor +   &
                  var(ap_id)%offset) /   &
                  (var(p0_id)%zfac_values(1)*var(p0_id)%scale_factor +   &
                  var(p0_id)%offset)

             adim(zaxis_id)%zfac_idxs(2) = a_id

             IF (associated(axis(izax)%zfac_bnds_names)) THEN

                call zfac_preamble_sub(     &
                     zaxis_id=zaxis_id,   &
                     zfactor_name='a_bnds',   &
                     units='none', axis_ids=(/ zaxis_id /),  &
                     itable=itable, zfactor_id=a_bnds_id,   &
                     izax=izax, ierr=ierr)

                ALLOCATE(  &
                     var(a_bnds_id)%zfac_bounds_dbl(2*adim(zaxis_id)%lendim), &
                     STAT=ierr)
                ap_bnds_id = adim(zaxis_id)%zfac_bnds_idxs(2)
                var(a_bnds_id)%zfac_bounds_dbl =   &
                     (var(ap_bnds_id)%zfac_bounds_dbl*   &
                     var(ap_bnds_id)%scale_factor +   &
                     var(ap_bnds_id)%offset) /   &
                     (var(p0_id)%zfac_values(1)*var(p0_id)%scale_factor +   &
                     var(p0_id)%offset)
                
                adim(zaxis_id)%zfac_bnds_idxs(2) = a_bnds_id
                
             END IF
             
          CASE default
             
             PRINT*, 'CMOR unable to convert z-factor from ',   &
                  TRIM(axis(izax)%table_entry), &
                  ' to ', TRIM(axis(izax)%convert_to)
             
          END SELECT

       CASE default
          
          PRINT*, 'CMOR unable to convert z-factor from ',   &
               TRIM(axis(izax)%table_entry), &
               ' to ', TRIM(axis(izax)%convert_to)
          
       END SELECT
       
       ! loop through variables and for those that are functions of the
       ! vertical dimensionless coordinate, change axis_names and 
       ! axis_idx

       adim(zaxis_id)%converted_from = axis(izax)%table_entry
       adim(zaxis_id)%axis_idx = izaxnew
       DO i=1, num_vars

          IF (var(i)%rank > 0) THEN
             DO j=1, var(i)%rank
                IF (var(i)%axis_idxs(j) == izax) THEN
                   var(i)%axis_idxs(j) = izaxnew
                END IF
             END DO
          END IF
       END DO
       
    END IF

    ! generate coordinate values
       adim(zaxis_id)%values =    &
            var(adim(zaxis_id)%zfac_idxs(2))%zfac_values_dbl +   &
            var(adim(zaxis_id)%zfac_idxs(3))%zfac_values_dbl
!  old version:
!!$    adim(zaxis_id)%values =    &
!!$         var(adim(zaxis_id)%zfac_idxs(2))%zfac_values +   &
!!$         var(adim(zaxis_id)%zfac_idxs(3))%zfac_values
    IF (associated(axis(izax)%zfac_bnds_names)) THEN
!  old version:
!!$       adim(zaxis_id)%bounds = RESHAPE(   &
!!$            (var(adim(zaxis_id)%zfac_bnds_idxs(2))%zfac_bounds +   &
!!$            var(adim(zaxis_id)%zfac_bnds_idxs(3))%zfac_bounds), &
!!$            (/ 2, adim(zaxis_id)%lendim /) )
       adim(zaxis_id)%bounds = RESHAPE(   &
            (var(adim(zaxis_id)%zfac_bnds_idxs(2))%zfac_bounds_dbl +   &
            var(adim(zaxis_id)%zfac_bnds_idxs(3))%zfac_bounds_dbl), &
            (/ 2, adim(zaxis_id)%lendim /) )
    END IF
    
    lon1 = 0
    
    !  check whether or not to reverse dimension direction:
    lendim=adim(zaxis_id)%lendim
    IF (lendim > 1) THEN

       IF (adim(zaxis_id)%values(2) > adim(zaxis_id)%values(1)) THEN 
          ipos = 1
       ELSE
          ipos = -1
       ENDIF

       adim(zaxis_id)%reverse=.false.
       adim(zaxis_id)%begin_out=1

       IF (ipos*axis(izaxnew)%out_direction < 0) THEN

          ! reverse direction
          adim(zaxis_id)%reverse=.true.
          adim(zaxis_id)%begin_out=lendim

          DO i=1,lendim/2
             ddum = adim(zaxis_id)%values(i)
             adim(zaxis_id)%values(i) =  adim(zaxis_id)%values(lendim+1-i)
             adim(zaxis_id)%values(lendim+1-i) = ddum
          ENDDO

          IF (associated(axis(izax)%zfac_bnds_names)) THEN
             DO i=1,lendim/2
                ddum2 = adim(zaxis_id)%bounds(:,i)
                adim(zaxis_id)%bounds(:,i) =adim(zaxis_id)%bounds(:,lendim+1-i)
                adim(zaxis_id)%bounds(:,lendim+1-i) = ddum2
             ENDDO
             DO i=1,lendim
                ddum = adim(zaxis_id)%bounds(1,i)
                adim(zaxis_id)%bounds(1,i) = adim(zaxis_id)%bounds(2,i)
                adim(zaxis_id)%bounds(2,i) = ddum
             ENDDO
          ENDIF

          SELECT CASE(axis(izaxnew)%table_entry)

          CASE('standard_hybrid_sigma', 'alternate_hybrid_sigma')

             IF (associated(axis(izax)%zfac_bnds_names)) THEN

                a_id = adim(zaxis_id)%zfac_bnds_idxs(2)
                b_id = adim(zaxis_id)%zfac_bnds_idxs(3)

                DO i=1,lendim
                   ddum = var(a_id)%zfac_bounds_dbl(i)
                   var(a_id)%zfac_bounds_dbl(i) =    &
                        var(a_id)%zfac_bounds_dbl(2*lendim+1-i)
                   var(a_id)%zfac_bounds_dbl(2*lendim+1-i) = ddum
                   ddum = var(b_id)%zfac_bounds_dbl(i)
                   var(b_id)%zfac_bounds_dbl(i) =    &
                        var(b_id)%zfac_bounds_dbl(2*lendim+1-i)
                   var(b_id)%zfac_bounds_dbl(2*lendim+1-i) = ddum
                ENDDO

             ENDIF

          CASE('standard_sigma')

             IF (associated(axis(izax)%zfac_bnds_names)) THEN

                a_id = adim(zaxis_id)%zfac_bnds_idxs(2)

                DO i=1,lendim/2
                   ddum = var(a_id)%zfac_bounds_dbl(i)
                   var(a_id)%zfac_bounds_dbl(i) =    &
                        var(a_id)%zfac_bounds_dbl(2*lendim+1-i)
                   var(a_id)%zfac_bounds_dbl(2*lendim+1-i) = ddum
                ENDDO

             ENDIF

          END SELECT

       ENDIF

    ENDIF

    call check_coords(izaxnew, zaxis_id, adim(zaxis_id)%lendim, lon1, &
         associated(axis(izax)%zfac_bnds_names), adim(zaxis_id)%values, &
         adim(zaxis_id)%bounds, ierr)

    RETURN

  END SUBROUTINE zfac_alldone


     
  FUNCTION cmor_zfactor_r_1d_b2d(zaxis_id, zfactor_name, axis_ids, &
      units, zfactor_values, zfactor_bounds) RESULT(zfactor_id)
    
    IMPLICIT NONE
  
    INTEGER, INTENT(IN) :: zaxis_id
    CHARACTER(len=*), INTENT(IN) :: zfactor_name
    INTEGER, INTENT(IN), DIMENSION(:), OPTIONAL :: axis_ids
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: units
    REAL, INTENT(IN), DIMENSION(:) :: zfactor_values
    REAL, INTENT(IN), DIMENSION(:,:) :: zfactor_bounds

    INTEGER :: zfactor_id

    INTEGER :: ierr, zfactor_bnds_id, itable, i, izax, izaxnew
    INTEGER :: p0_id, a_id, b_id
    INTEGER :: ps_id, isize
    LOGICAL :: alldone
    REAL :: ptop

    ierr = 0
    call zfac_preamble(zaxis_id, zfactor_name, units, axis_ids,  &
         itable, zfactor_id, zfactor_bnds_id, izax, alldone, ierr)

    IF (ierr /= 0) THEN
       PRINT*, 'error in zfactor'
       zfactor_id = ierr
       stop
    END IF
    
    ! if necessary, allocate space for zfac_values
    
    ! find size needed   
    isize = 1
    IF (PRESENT(axis_ids)) THEN
       DO i=1,SIZE(axis_ids)
          IF (axis_ids(i) > 0) THEN
             isize = isize*adim(axis_ids(i))%lendim
          ELSE
             EXIT
          END IF
       END DO
    END IF
    IF (similar(var(zfactor_id)%type, 'real')) THEN
       ALLOCATE(var(zfactor_id)%zfac_values(isize), STAT=ierr)
       var(zfactor_id)%zfac_values = zfactor_values
    ELSE IF (similar(var(zfactor_id)%type, 'double')) THEN
       ALLOCATE(var(zfactor_id)%zfac_values_dbl(isize), STAT=ierr)
       var(zfactor_id)%zfac_values_dbl = zfactor_values
    ENDIF
    
    IF (ierr /= 0) THEN
       PRINT*, 'Error encountered in CMOR: cmor_zfactor'
       PRINT*, '   Not able to allocate memory for zfac_values '
       STOP
    END IF
    
    IF (zfactor_bnds_id > 0) THEN
       
       IF (similar(var(zfactor_bnds_id)%type, 'real')) THEN
          ALLOCATE(var(zfactor_bnds_id)%zfac_bounds(2*isize), STAT=ierr)
          var(zfactor_bnds_id)%zfac_bounds = RESHAPE(zfactor_bounds, &
               (/ 2*isize /))
       ELSE IF (similar(var(zfactor_bnds_id)%type, 'double')) THEN
          ALLOCATE(var(zfactor_bnds_id)%zfac_bounds_dbl(2*isize), STAT=ierr)
          var(zfactor_bnds_id)%zfac_bounds_dbl = RESHAPE(zfactor_bounds, &
               (/ 2*isize /))
       ENDIF
       IF (ierr /= 0) THEN
          PRINT*, 'Error encountered in CMOR: cmor_zfactor'
          PRINT*, '   Not able to allocate memory for zfac_bounds '
          STOP
          
       ENDIF
    ENDIF

    IF (alldone) CALL zfac_alldone(zaxis_id)

    RETURN

  END FUNCTION cmor_zfactor_r_1d_b2d
  
  FUNCTION cmor_zfactor_r_1d_b1d(zaxis_id, zfactor_name, axis_ids, &
      units, zfactor_values, zfactor_bounds) RESULT(zfactor_id)
    
    IMPLICIT NONE
  
    INTEGER, INTENT(IN) :: zaxis_id
    CHARACTER(len=*), INTENT(IN) :: zfactor_name
    INTEGER, INTENT(IN), DIMENSION(:), OPTIONAL :: axis_ids
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: units
    REAL, INTENT(IN), DIMENSION(:) :: zfactor_values
    REAL, INTENT(IN), DIMENSION(:) :: zfactor_bounds

    INTEGER :: zfactor_id

    INTEGER :: ierr, zfactor_bnds_id, itable, i, izax, izaxnew
    INTEGER :: p0_id, a_id, b_id
    INTEGER :: ps_id, isize
    LOGICAL :: alldone
    REAL :: ptop

    ierr = 0
    call zfac_preamble(zaxis_id, zfactor_name, units, axis_ids,  &
         itable, zfactor_id, zfactor_bnds_id, izax, alldone, ierr)

    IF (ierr /= 0) THEN
       PRINT*, 'error in zfactor'
       zfactor_id = ierr
       stop
    END IF
    
    ! if necessary, allocate space for zfac_values
    
    ! find size needed   
    isize = 1
    IF (PRESENT(axis_ids)) THEN
       DO i=1,SIZE(axis_ids)
          IF (axis_ids(i) > 0) THEN
             isize = isize*adim(axis_ids(i))%lendim
          ELSE
             EXIT
          END IF
       END DO
    END IF
    IF (similar(var(zfactor_id)%type, 'real')) THEN
       ALLOCATE(var(zfactor_id)%zfac_values(isize), STAT=ierr)
       var(zfactor_id)%zfac_values = zfactor_values
    ELSE IF (similar(var(zfactor_id)%type, 'double')) THEN
       ALLOCATE(var(zfactor_id)%zfac_values_dbl(isize), STAT=ierr)
       var(zfactor_id)%zfac_values_dbl = zfactor_values
    ENDIF
    
    IF (ierr /= 0) THEN
       PRINT*, 'Error encountered in CMOR: cmor_zfactor'
       PRINT*, '   Not able to allocate memory for zfac_values '
       STOP
    END IF
    
    IF (zfactor_bnds_id > 0) THEN
       
       IF (similar(var(zfactor_bnds_id)%type, 'real')) THEN
          ALLOCATE(var(zfactor_bnds_id)%zfac_bounds(2*isize), STAT=ierr)
          DO i=1,isize
             var(zfactor_bnds_id)%zfac_bounds(2*i-1) = zfactor_bounds(i)
             var(zfactor_bnds_id)%zfac_bounds(2*i) = zfactor_bounds(i+1)
          END DO
       ELSE IF (similar(var(zfactor_bnds_id)%type, 'double')) THEN
          ALLOCATE(var(zfactor_bnds_id)%zfac_bounds_dbl(2*isize), STAT=ierr)
          DO i=1,isize
             var(zfactor_bnds_id)%zfac_bounds_dbl(2*i-1) = zfactor_bounds(i)
             var(zfactor_bnds_id)%zfac_bounds_dbl(2*i) = zfactor_bounds(i+1)
          END DO
       ENDIF

       IF (ierr /= 0) THEN
          PRINT*, 'Error encountered in CMOR: cmor_zfactor'
          PRINT*, '   Not able to allocate memory for zfac_bounds '
          STOP
          
       ENDIF
    ENDIF
    IF (alldone) CALL zfac_alldone(zaxis_id)

    RETURN

  END FUNCTION cmor_zfactor_r_1d_b1d
  
  FUNCTION cmor_zfactor_r_1d_none(zaxis_id, zfactor_name, axis_ids, &
      units, zfactor_values, zfactor_bounds) RESULT(zfactor_id)
    
    IMPLICIT NONE
  
    INTEGER, INTENT(IN) :: zaxis_id
    CHARACTER(len=*), INTENT(IN) :: zfactor_name
    INTEGER, INTENT(IN), DIMENSION(:), OPTIONAL :: axis_ids
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: units
    REAL, INTENT(IN), DIMENSION(:) :: zfactor_values
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: zfactor_bounds

    INTEGER :: zfactor_id

    INTEGER :: ierr, zfactor_bnds_id, itable, i, izax, izaxnew
    INTEGER :: p0_id, a_id, b_id
    INTEGER :: ps_id, isize
    LOGICAL :: alldone
    REAL :: ptop

    ierr = 0

    call zfac_preamble(zaxis_id, zfactor_name, units, axis_ids,  &
         itable, zfactor_id, zfactor_bnds_id, izax, alldone, ierr)

    IF (ierr /= 0) THEN
       PRINT*, 'error in zfactor'
       zfactor_id = ierr
       stop
    END IF

    ! if necessary, allocate space for zfac_values

    ! find size needed   
    isize = 1
    IF (PRESENT(axis_ids)) THEN
       DO i=1,SIZE(axis_ids)
          IF (axis_ids(i) > 0) THEN
             isize = isize*adim(axis_ids(i))%lendim
          ELSE
             EXIT
          END IF
       END DO
    END IF
    IF (similar(var(zfactor_id)%type, 'real')) THEN
       ALLOCATE(var(zfactor_id)%zfac_values(isize), STAT=ierr)
       var(zfactor_id)%zfac_values = zfactor_values
    ELSE IF (similar(var(zfactor_id)%type, 'double')) THEN
       ALLOCATE(var(zfactor_id)%zfac_values_dbl(isize), STAT=ierr)
       var(zfactor_id)%zfac_values_dbl = zfactor_values
    ENDIF

    IF (ierr /= 0) THEN
       PRINT*, 'Error encountered in CMOR: cmor_zfactor'
       PRINT*, '   Not able to allocate memory for zfac_values '
       STOP
    END IF

    IF (alldone) CALL zfac_alldone(zaxis_id)

    RETURN

  END FUNCTION cmor_zfactor_r_1d_none
  


  FUNCTION cmor_zfactor_r_0d_none(zaxis_id, zfactor_name, &
      units, zfactor_values, zfactor_bounds) RESULT(zfactor_id)
    
    IMPLICIT NONE
  
    INTEGER, INTENT(IN) :: zaxis_id
    CHARACTER(len=*), INTENT(IN) :: zfactor_name
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: units
    REAL, INTENT(IN) :: zfactor_values
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: zfactor_bounds

    INTEGER :: zfactor_id

    INTEGER :: ierr, zfactor_bnds_id, itable, i, izax, izaxnew
    INTEGER :: p0_id, a_id, b_id
    INTEGER :: ps_id, isize
    LOGICAL :: alldone
    REAL :: ptop

    ierr = 0
    call zfac_preamble(zaxis_id, zfactor_name, units,  &
         itable=itable, zfactor_id=zfactor_id, &
         zfactor_bnds_id=zfactor_bnds_id, izax=izax, &
         alldone=alldone, ierr=ierr)

    IF (ierr /= 0) THEN
       PRINT*, 'error in zfactor'
       zfactor_id = ierr
       stop
    END IF

    ! if necessary, allocate space for zfac_values

    ! find size needed   
    isize = 1
    IF (similar(var(zfactor_id)%type, 'real')) THEN
       ALLOCATE(var(zfactor_id)%zfac_values(isize), STAT=ierr)
       var(zfactor_id)%zfac_values = zfactor_values
    ELSE IF (similar(var(zfactor_id)%type, 'double')) THEN
       ALLOCATE(var(zfactor_id)%zfac_values_dbl(isize), STAT=ierr)
       var(zfactor_id)%zfac_values_dbl = zfactor_values
    ENDIF

    IF (ierr /= 0) THEN
       PRINT*, 'Error encountered in CMOR: cmor_zfactor'
       PRINT*, '   Not able to allocate memory for zfac_values '
       STOP
    END IF

    IF (alldone) CALL zfac_alldone(zaxis_id)

    RETURN

  END FUNCTION cmor_zfactor_r_0d_none
  


  FUNCTION cmor_zfactor_d_1d_b2d(zaxis_id, zfactor_name, axis_ids, &
      units, zfactor_values, zfactor_bounds) RESULT(zfactor_id)
    
    IMPLICIT NONE
  
    INTEGER, INTENT(IN) :: zaxis_id
    CHARACTER(len=*), INTENT(IN) :: zfactor_name
    INTEGER, INTENT(IN), DIMENSION(:), OPTIONAL :: axis_ids
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: units
    DOUBLE PRECISION, INTENT(IN), DIMENSION(:) :: zfactor_values
    DOUBLE PRECISION, INTENT(IN), DIMENSION(:,:) :: zfactor_bounds

    INTEGER :: zfactor_id

    INTEGER :: ierr, zfactor_bnds_id, itable, i, izax, izaxnew
    INTEGER :: p0_id, a_id, b_id
    INTEGER :: ps_id, isize
    LOGICAL :: alldone
    REAL :: ptop

    ierr = 0
    call zfac_preamble(zaxis_id, zfactor_name, units, axis_ids,  &
         itable, zfactor_id, zfactor_bnds_id, izax, alldone, ierr)

    IF (ierr /= 0) THEN
       PRINT*, 'error in zfactor'
       zfactor_id = ierr
       stop
    END IF
    
    ! if necessary, allocate space for zfac_values
    
    ! find size needed   
    isize = 1
    IF (PRESENT(axis_ids)) THEN
       DO i=1,SIZE(axis_ids)
          IF (axis_ids(i) > 0) THEN
             isize = isize*adim(axis_ids(i))%lendim
          ELSE
             EXIT
          END IF
       END DO
    END IF
    IF (similar(var(zfactor_id)%type, 'real')) THEN
       ALLOCATE(var(zfactor_id)%zfac_values(isize), STAT=ierr)
       var(zfactor_id)%zfac_values = zfactor_values
    ELSE IF (similar(var(zfactor_id)%type, 'double')) THEN
       ALLOCATE(var(zfactor_id)%zfac_values_dbl(isize), STAT=ierr)
       var(zfactor_id)%zfac_values_dbl = zfactor_values
    ENDIF
    
    IF (ierr /= 0) THEN
       PRINT*, 'Error encountered in CMOR: cmor_zfactor'
       PRINT*, '   Not able to allocate memory for zfac_values '
       STOP
    END IF
    
    IF (zfactor_bnds_id > 0) THEN
       
       IF (similar(var(zfactor_bnds_id)%type, 'real')) THEN
          ALLOCATE(var(zfactor_bnds_id)%zfac_bounds(2*isize), STAT=ierr)
          var(zfactor_bnds_id)%zfac_bounds = RESHAPE(zfactor_bounds, &
               (/ 2*isize /))
       ELSE IF (similar(var(zfactor_bnds_id)%type, 'double')) THEN
          ALLOCATE(var(zfactor_bnds_id)%zfac_bounds_dbl(2*isize), STAT=ierr)
          var(zfactor_bnds_id)%zfac_bounds_dbl = RESHAPE(zfactor_bounds, &
               (/ 2*isize /))
       ENDIF
       IF (ierr /= 0) THEN
          PRINT*, 'Error encountered in CMOR: cmor_zfactor'
          PRINT*, '   Not able to allocate memory for zfac_bounds '
          STOP
          
       ENDIF
    ENDIF

    IF (alldone) CALL zfac_alldone(zaxis_id)

    RETURN

  END FUNCTION cmor_zfactor_d_1d_b2d
  
  FUNCTION cmor_zfactor_d_1d_b1d(zaxis_id, zfactor_name, axis_ids, &
      units, zfactor_values, zfactor_bounds) RESULT(zfactor_id)
    
    IMPLICIT NONE
  
    INTEGER, INTENT(IN) :: zaxis_id
    CHARACTER(len=*), INTENT(IN) :: zfactor_name
    INTEGER, INTENT(IN), DIMENSION(:), OPTIONAL :: axis_ids
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: units
    DOUBLE PRECISION, INTENT(IN), DIMENSION(:) :: zfactor_values
    DOUBLE PRECISION, INTENT(IN), DIMENSION(:) :: zfactor_bounds

    INTEGER :: zfactor_id

    INTEGER :: ierr, zfactor_bnds_id, itable, i, izax, izaxnew
    INTEGER :: p0_id, a_id, b_id
    INTEGER :: ps_id, isize
    LOGICAL :: alldone
    REAL :: ptop

    ierr = 0
    call zfac_preamble(zaxis_id, zfactor_name, units, axis_ids,  &
         itable, zfactor_id, zfactor_bnds_id, izax, alldone, ierr)

    IF (ierr /= 0) THEN
       PRINT*, 'error in zfactor'
       zfactor_id = ierr
       stop
    END IF
    
    ! if necessary, allocate space for zfac_values
    
    ! find size needed   
    isize = 1
    IF (PRESENT(axis_ids)) THEN
       DO i=1,SIZE(axis_ids)
          IF (axis_ids(i) > 0) THEN
             isize = isize*adim(axis_ids(i))%lendim
          ELSE
             EXIT
          END IF
       END DO
    END IF
    IF (similar(var(zfactor_id)%type, 'real')) THEN
       ALLOCATE(var(zfactor_id)%zfac_values(isize), STAT=ierr)
       var(zfactor_id)%zfac_values = zfactor_values
    ELSE IF (similar(var(zfactor_id)%type, 'double')) THEN
       ALLOCATE(var(zfactor_id)%zfac_values_dbl(isize), STAT=ierr)
       var(zfactor_id)%zfac_values_dbl = zfactor_values
    ENDIF
    
    IF (ierr /= 0) THEN
       PRINT*, 'Error encountered in CMOR: cmor_zfactor'
       PRINT*, '   Not able to allocate memory for zfac_values '
       STOP
    END IF
    
    IF (zfactor_bnds_id > 0) THEN
       
       IF (similar(var(zfactor_bnds_id)%type, 'real')) THEN
          ALLOCATE(var(zfactor_bnds_id)%zfac_bounds(2*isize), STAT=ierr)
          DO i=1,isize
             var(zfactor_bnds_id)%zfac_bounds(2*i-1) = zfactor_bounds(i)
             var(zfactor_bnds_id)%zfac_bounds(2*i) = zfactor_bounds(i+1)
          END DO
       ELSE IF (similar(var(zfactor_bnds_id)%type, 'double')) THEN
          ALLOCATE(var(zfactor_bnds_id)%zfac_bounds_dbl(2*isize), STAT=ierr)
          DO i=1,isize
             var(zfactor_bnds_id)%zfac_bounds_dbl(2*i-1) = zfactor_bounds(i)
             var(zfactor_bnds_id)%zfac_bounds_dbl(2*i) = zfactor_bounds(i+1)
          END DO
       ENDIF
       IF (ierr /= 0) THEN
          PRINT*, 'Error encountered in CMOR: cmor_zfactor'
          PRINT*, '   Not able to allocate memory for zfac_bounds '
          STOP
          
       ENDIF
    ENDIF

    IF (alldone) CALL zfac_alldone(zaxis_id)

    RETURN

  END FUNCTION cmor_zfactor_d_1d_b1d
  
  FUNCTION cmor_zfactor_d_1d_none(zaxis_id, zfactor_name, axis_ids, &
      units, zfactor_values, zfactor_bounds) RESULT(zfactor_id)
    
    IMPLICIT NONE
  
    INTEGER, INTENT(IN) :: zaxis_id
    CHARACTER(len=*), INTENT(IN) :: zfactor_name
    INTEGER, INTENT(IN), DIMENSION(:), OPTIONAL :: axis_ids
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: units
    DOUBLE PRECISION, INTENT(IN), DIMENSION(:) :: zfactor_values
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: zfactor_bounds

    INTEGER :: zfactor_id

    INTEGER :: ierr, zfactor_bnds_id, itable, i, izax, izaxnew
    INTEGER :: p0_id, a_id, b_id
    INTEGER :: ps_id, isize
    LOGICAL :: alldone
    REAL :: ptop

    ierr = 0

    call zfac_preamble(zaxis_id, zfactor_name, units, axis_ids,  &
         itable, zfactor_id, zfactor_bnds_id, izax, alldone, ierr)

    IF (ierr /= 0) THEN
       PRINT*, 'error in zfactor'
       zfactor_id = ierr
       stop
    END IF

    ! if necessary, allocate space for zfac_values

    ! find size needed   
    isize = 1
    IF (PRESENT(axis_ids)) THEN
       DO i=1,SIZE(axis_ids)
          IF (axis_ids(i) > 0) THEN
             isize = isize*adim(axis_ids(i))%lendim
          ELSE
             EXIT
          END IF
       END DO
    END IF
    IF (similar(var(zfactor_id)%type, 'real')) THEN
       ALLOCATE(var(zfactor_id)%zfac_values(isize), STAT=ierr)
       var(zfactor_id)%zfac_values = zfactor_values
    ELSE IF (similar(var(zfactor_id)%type, 'double')) THEN
       ALLOCATE(var(zfactor_id)%zfac_values_dbl(isize), STAT=ierr)
       var(zfactor_id)%zfac_values_dbl = zfactor_values
    ENDIF

    IF (ierr /= 0) THEN
       PRINT*, 'Error encountered in CMOR: cmor_zfactor'
       PRINT*, '   Not able to allocate memory for zfac_values '
       STOP
    END IF

    IF (alldone) CALL zfac_alldone(zaxis_id)

    RETURN

  END FUNCTION cmor_zfactor_d_1d_none
  
      

  FUNCTION cmor_zfactor_d_0d_none(zaxis_id, zfactor_name, &
      units, zfactor_values, zfactor_bounds) RESULT(zfactor_id)
    
    IMPLICIT NONE
  
    INTEGER, INTENT(IN) :: zaxis_id
    CHARACTER(len=*), INTENT(IN) :: zfactor_name
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: units
    DOUBLE PRECISION, INTENT(IN) :: zfactor_values
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: zfactor_bounds

    INTEGER :: zfactor_id

    INTEGER :: ierr, zfactor_bnds_id, itable, i, izax, izaxnew
    INTEGER :: p0_id, a_id, b_id
    INTEGER :: ps_id, isize
    LOGICAL :: alldone
    REAL :: ptop

    ierr = 0
    call zfac_preamble(zaxis_id, zfactor_name, units,  &
         itable=itable, zfactor_id=zfactor_id, &
         zfactor_bnds_id=zfactor_bnds_id, izax=izax, &
         alldone=alldone, ierr=ierr)

    IF (ierr /= 0) THEN
       PRINT*, 'error in zfactor'
       zfactor_id = ierr
       stop
    END IF

    ! if necessary, allocate space for zfac_values

    ! find size needed   
    isize = 1
    IF (similar(var(zfactor_id)%type, 'real')) THEN
       ALLOCATE(var(zfactor_id)%zfac_values(isize), STAT=ierr)
       var(zfactor_id)%zfac_values = zfactor_values
    ELSE IF (similar(var(zfactor_id)%type, 'double')) THEN
       ALLOCATE(var(zfactor_id)%zfac_values_dbl(isize), STAT=ierr)
       var(zfactor_id)%zfac_values_dbl = zfactor_values
    ENDIF

    IF (ierr /= 0) THEN
       PRINT*, 'Error encountered in CMOR: cmor_zfactor'
       PRINT*, '   Not able to allocate memory for zfac_values '
       STOP
    END IF

    IF (alldone) CALL zfac_alldone(zaxis_id)

    RETURN

  END FUNCTION cmor_zfactor_d_0d_none
  
      
  FUNCTION cmor_zfactor_none_none(zaxis_id, zfactor_name, axis_ids, &
      units, zfactor_values, zfactor_bounds) RESULT(zfactor_id)
    
    IMPLICIT NONE
  
    INTEGER, INTENT(IN) :: zaxis_id
    CHARACTER(len=*), INTENT(IN) :: zfactor_name
    INTEGER, INTENT(IN), DIMENSION(:), OPTIONAL :: axis_ids
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: units
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: zfactor_values
    CHARACTER(len=*), INTENT(IN), OPTIONAL :: zfactor_bounds

    INTEGER :: zfactor_id

    INTEGER :: ierr, zfactor_bnds_id, itable, i, izax, izaxnew
    INTEGER :: p0_id, a_id, b_id
    INTEGER :: ps_id, isize
    LOGICAL :: alldone
    REAL :: ptop

    ierr = 0

    call zfac_preamble(zaxis_id, zfactor_name, units, axis_ids,  &
         itable, zfactor_id, zfactor_bnds_id, izax, alldone, ierr)

    IF (ierr /= 0) THEN
       PRINT*, 'error in zfactor'
       zfactor_id = ierr
       stop
    END IF

    ! field is time-dependent and will be written through cmor_write
    ALLOCATE(var(zfactor_id)%zfac_times_written(max_variables), STAT=ierr)

    IF (ierr /= 0) THEN
       PRINT*, 'Error encountered in CMOR: cmor_zfactor'
       PRINT*, '   Not able to allocate memory for var(zfactor_id)%zfac_times_written '
       STOP
    END IF

    var(zfactor_id)%zfac_times_written = 0.0
  
    IF (alldone) CALL zfac_alldone(zaxis_id)

    RETURN

  END FUNCTION cmor_zfactor_none_none
  
 
 

  FUNCTION cmor_close(var_id)

    IMPLICIT NONE

    INTEGER :: cmor_close,ierr,i
    INTEGER, OPTIONAL, INTENT(IN) :: var_id


    PRINT*, ' '

    cmor_close = 0
    !  check that cmor_setup has already been called
    IF (.NOT. called_setup)  THEN
       PRINT*, 'Encountered error in CMOR: cmor_close'
       PRINT*, 'You must call cmor_setup before calling cmor_close'
       STOP
    END IF

    IF (PRESENT(var_id)) THEN
       IF ((var_id <= 0) .or. (var_id > num_vars)) THEN
          PRINT*, 'Encountered error in CMOR: cmor_close'
          PRINT*, '   User passed a value for var_id that is < 1 or > ', num_vars
          PRINT*, '   Value passed = ', var_id,  ' which is unacceptable'
          STOP
       ELSE IF (similar(var(var_id)%file_status, 'write mode') .or. &
                 similar(var(var_id)%file_status, 'define mode')) THEN

          ! ??? check when possible whether all expected times have
          !     been written.
          ! ??? send error summary for this variable to the log file

          ierr = nf90_close(ncid = var(var_id)%ncid)

          IF (ierr /= NF90_NOERR) THEN
             print*, 'CMOR error 203 '
             stop
          END IF

          var(var_id)%file_status = 'closed'

          PRINT*, 'CMOR has closed the netCDF file for variable ',  &
               var(var_id)%table_entry

       ELSE 
          PRINT*, 'Encountered error in CMOR: cmor_close'
          PRINT*, '   User is attempting to close a file that has not yet '
          PRINT*, '   been written to.  This is illegal.'
          PRINT*, '   The following variable must first be written: ', &
               TRIM(var(var_id)%table_entry)
       END IF

    ELSE

       ! close all files opened by cmor
       
       DO i=1,num_vars
          IF (similar(var(i)%file_status, 'write mode') .or. &
               similar(var(i)%file_status, 'define mode')) THEN
             

             ! ??? check when possible whether all expected times have
             !     been written.
             ! ??? send error summary for this variable to the log file
             
             ierr = nf90_close(ncid = var(i)%ncid)
             IF (ierr /= NF90_NOERR) THEN
                print*, 'CMOR error  '
                stop
             END IF
             
             var(i)%file_status = 'closed'

             PRINT*, 'CMOR has closed the netCDF file for variable ',  &
                  var(i)%table_entry

          ENDIF
       END DO
       
       ! ??? close log and summary files
       ! check for error
       
       ! deallocate z-factor times written, value (double or real), and 
       !    bounds (double or real)

       DO i=1,num_vars
          IF (ASSOCIATED(var(i)%zfac_times_written)) THEN
             DEALLOCATE(var(i)%zfac_times_written, STAT=ierr)
             IF (ierr /= 0) THEN
                print*, 'CMOR error '
                stop
             END IF
          END IF
          IF (ASSOCIATED(var(i)%zfac_values)) THEN
             DEALLOCATE(var(i)%zfac_values, STAT=ierr)
             IF (ierr /= 0) THEN
                print*, 'CMOR error '
                stop
             END IF
          END IF
          IF (ASSOCIATED(var(i)%zfac_values_dbl)) THEN
             DEALLOCATE(var(i)%zfac_values_dbl, STAT=ierr)
             IF (ierr /= 0) THEN
                print*, 'CMOR error '
                stop
             END IF
          END IF
          IF (ASSOCIATED(var(i)%zfac_bounds)) THEN
             DEALLOCATE(var(i)%zfac_bounds, STAT=ierr)
             IF (ierr /= 0) THEN
                print*, 'CMOR error '
                stop
             END IF
          END IF
          IF (ASSOCIATED(var(i)%zfac_bounds_dbl)) THEN
             DEALLOCATE(var(i)%zfac_bounds_dbl, STAT=ierr)
             IF (ierr /= 0) THEN
                print*, 'CMOR error '
                stop
             END IF
          END IF
       END DO 

       ! deallocate coordinate value vectors and bounds vector,
       ! z-factor indices, and z-factor bounds indices

       DO i=1,num_dims
          IF (ASSOCIATED(adim(i)%values)) THEN
             DEALLOCATE(adim(i)%values, STAT=ierr)
             IF (ierr /= 0) THEN
                print*, 'CMOR error '
                stop
             END IF
          END IF
          IF (ASSOCIATED(adim(i)%bounds)) THEN
             DEALLOCATE(adim(i)%bounds, STAT=ierr)
             IF (ierr /= 0) THEN
                print*, 'CMOR error '
                stop
             END IF
          END IF
          IF (ASSOCIATED(adim(i)%labels)) THEN
             DEALLOCATE(adim(i)%labels, STAT=ierr)
             IF (ierr /= 0) THEN
                print*, 'CMOR error '
                stop
             END IF
          END IF
          IF (ASSOCIATED(adim(i)%zfac_idxs)) THEN
             DEALLOCATE(adim(i)%zfac_idxs, STAT=ierr)
             IF (ierr /= 0) THEN
                print*, 'CMOR error '
                stop
             END IF
          END IF
          IF (ASSOCIATED(adim(i)%zfac_bnds_idxs)) THEN
             DEALLOCATE(adim(i)%zfac_bnds_idxs, STAT=ierr)
             IF (ierr /= 0) THEN
                print*, 'CMOR error '
                stop
             END IF
          END IF
       END DO 

       ! deallocate requested_labels, requested_vals, and requested_bounds
       !    z-factor names, and z-factor bounds names

       DO i=1,num_axes

          IF (ASSOCIATED(axis(i)%requested_vals)) THEN
             DEALLOCATE(axis(i)%requested_vals, STAT=ierr)
             IF (ierr /= 0) THEN
                print*, 'CMOR error '
                stop
             END IF
          END IF
 
          IF (ASSOCIATED(axis(i)%requested_bounds)) THEN
             DEALLOCATE(axis(i)%requested_bounds, STAT=ierr)
             IF (ierr /= 0) THEN
                print*, 'CMOR error '
                stop
             END IF
          END IF
 
          IF (ASSOCIATED(axis(i)%requested_labels)) THEN
             DEALLOCATE(axis(i)%requested_labels, STAT=ierr)
             IF (ierr /= 0) THEN
                print*, 'CMOR error '
                stop
             END IF
          END IF

          IF (ASSOCIATED(axis(i)%zfac_names)) THEN
             DEALLOCATE(axis(i)%zfac_names, STAT=ierr)
             IF (ierr /= 0) THEN
                print*, 'CMOR error '
                stop
             END IF
          END IF

          IF (ASSOCIATED(axis(i)%zfac_bnds_names)) THEN
             DEALLOCATE(axis(i)%zfac_bnds_names, STAT=ierr)
             IF (ierr /= 0) THEN
                print*, 'CMOR error '
                stop
             END IF
          END IF
 
       END DO

       called_setup = .false.
       
    END IF
    
    RETURN

  END FUNCTION cmor_close

END MODULE cmor_users_functions
